package com.ibm.nzna.projects.qit.promail ;
/*-----------------------------------------------------------*/
/*   ProMailPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProMailPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel holds information for the user to preview and manipulate proactive eMail 
 *
 *
 * @version 1.0, 9/3/1999 
 * @author  David Lentz
 */
import java.util.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import java.sql.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.qit.doc.docedit.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProMailPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public final class ProMailPanel extends JPanel implements 
ProMailConst,
AppConst, 
Authority,
QuestPanel,
ActionListener,
SelectDocListener, 
ListSelectionListener,
Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_SEARCH         = "Search" ;
   private static final String THREAD_REFRESHPREVIEW = "RP" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Container Titles                                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] listTitle = { 
      " ",
      Str.getStr ( STR_INTERNAL_NUM ),
      Str.getStr ( STR_TITLE ),
      Str.getStr ( STR_LOTUS_NOTES_ID ),
      Str.getStr ( STR_EMAILABLE_DATE ),
      Str.getStr ( STR_LAST_TOUCHED_ON ),
      Str.getStr ( STR_LAST_TOUCHED_BY )} ;                

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle               st_TITLE        = null ;
   private DocumentTable        documents         = null ;
   private JSplitPane           splitPane       = null ;
   private ProMailPreviewPane   previewPane     = null ;
   private ActionButton         pb_SEND         = null ;
   private ActionButton         pb_CANCEL       = null ;
   private ActionButton         pb_EDIT         = null ;
   private ActionButton         pb_CLOSE        = null ;
   private JMenuItem            mnu_EDIT_FOOTER = null ;
   private JMenuItem            mnu_EDIT_HEADER = null ;
   private JSeparator           mnu_SEPARATOR   = new JSeparator () ;
   private ToolBarButton        pb_FIND         = null ;
   private JToolBar             toolBar         = null ;
   private ProMailNavPanel      proMailNavPanel = null ;
   private DocumentRow          previewDocument = null ;
   private AppDefaultWin        parentWin       = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean inThread      = false ;
   private int     view          = 0 ;
   private String  startDate     = new CDate ( CDate.DATE_DATE ).today() ;
   private String  stopDate      = CDate.addDaysToDate ( startDate, 30, CDate.DATE_DATE ) ;
   private String  docId         = "" ;



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize and create our
    *  GUI, it will return TRUE if everything was
    *  created Ok
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE        = new JTitle ( "" ) ;
      documents       = new DocumentTable() ;
      previewPane     = new ProMailPreviewPane () ;
      splitPane       = new JSplitPane ( JSplitPane.VERTICAL_SPLIT ) ;
      pb_CLOSE        = new ActionButton ( Str.getStr ( STR_CLOSE  ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_CANCEL       = new ActionButton ( Str.getStr ( STR_CANCEL ), ImageSystem.getImageIcon ( this, ImageSystem.NEW ),   Str.getStr ( STR_CANCEL_PROMAIL ) ) ;      
      pb_SEND         = new ActionButton ( Str.getStr ( STR_SEND   ), ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SEND_PROMAIL ) ) ;      
      pb_EDIT         = new ActionButton ( Str.getStr ( STR_EDIT   ), ImageSystem.getImageIcon ( this, ImageSystem.EDIT ),  Str.getStr ( STR_EDIT_PROMAIL ) ) ;                 
      mnu_EDIT_FOOTER = new JMenuItem ( Str.getStr ( STR_CHANGE_FOOTER ) ) ;
      mnu_EDIT_HEADER = new JMenuItem ( Str.getStr ( STR_CHANGE_HEADER ) ) ;        
      pb_FIND         = new ToolBarButton ( Str.getStr ( STR_FIND ), ImageSystem.getImageIcon ( this, ImageSystem.SEARCH ), Str.getStr ( STR_FIND ) ) ;
      toolBar         = new JToolBar () ;
      proMailNavPanel = new ProMailNavPanel ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE.setFont ( FontSystem.largeTitleFont ) ;
      documents.setColumnTitle ( 3, Str.getStr ( STR_EMAILABLE_DATE ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener        ( this ) ;
      pb_SEND.addActionListener         ( this ) ;
      pb_CANCEL.addActionListener       ( this ) ;
      pb_EDIT.addActionListener         ( this ) ;       
      mnu_EDIT_FOOTER.addActionListener ( this ) ;
      mnu_EDIT_HEADER.addActionListener ( this ) ;
      documents.addListSelectionListener  ( this ) ;
      documents.addActionListener         ( this ) ;
      pb_FIND.addActionListener         ( this ) ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the Toolbar components                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.add ( new ToolBarSeparator () ) ;
      toolBar.add ( pb_FIND ) ;
      toolBar.setBorder ( null ) ;
      toolBar.setOpaque ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      splitPane.setTopComponent    ( documents ) ;
      splitPane.setBottomComponent ( previewPane ) ;
      add ( st_TITLE,  BorderLayout.NORTH ) ;
      add ( splitPane, BorderLayout.CENTER ) ;


      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will start our panel. This will be
    *  called once we are showing and need to refresh
    *  our data
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_SEND ) ;
      parentWin.addActionComponent ( this, pb_EDIT ) ;
      parentWin.addActionComponent ( this, pb_CANCEL ) ;
      parentWin.getAppDefaultMenuBar().addEditMenuItem ( mnu_SEPARATOR ) ;
      parentWin.getAppDefaultMenuBar().addEditMenuItem ( mnu_EDIT_HEADER ) ;
      parentWin.getAppDefaultMenuBar().addEditMenuItem ( mnu_EDIT_FOOTER ) ;

      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 1 ) ;                                        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Do this last, because it is the only way it works       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      splitPane.setDividerLocation ( (double) .60 ) ;

   }

   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is sent to us.
    *  
    *
    *  @param event The Action Event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent event ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( event.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin ( this ).closePanel ( this, null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_HEADER                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( event.getSource() == mnu_EDIT_HEADER )
         editHeader() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_FOOTER                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( event.getSource() == mnu_EDIT_FOOTER )
         editFooter() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_FIND                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( event.getSource() == pb_FIND )
         new SelectDocDlg ( ).getInstance().addSelectDocListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( event.getSource() == pb_EDIT ) ||
                ( event.getSource() == documents ) )
         edit() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( event.getSource() == pb_CANCEL )
         cancel() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SEND                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( event.getSource() == pb_SEND )
         send() ;


   }

   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      Vector selection = documents.getSelection() ;

      if ( ( selection != null ) && ( selection.size() > 0 ) ) {
         int i = 0 ;
         int size = selection.size() ;

         while ( i < size ) {
            Bookmark bookmark = new Bookmark() ;
            bookmark.setDescript ( ((Document)selection.elementAt(i)).getTitle() ) ;
            bookmark.setObject ( new Integer ( ((Document)selection.elementAt(i)).getDocInd() ),
                                 BOOKMARK_DOCUMENT ) ;
            bookmark.updateRecStatus ( bookmark.REC_INSERT ) ;
            new AddBookmarkDlg ( GUISystem.getParentDefWin ( this ), bookmark ) ;
            i++ ;
         }

      } else
         GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_MUST_SELECT_ITEM_TO_BOOKMARK ) ) ;

      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called for us to do some
    *  clean up work
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      if ( !inThread ) {
         removeAll() ;
         panelDeselected() ;
         return( true ) ;
      }

      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the Quest
    *  application wants to shutdown, we will return
    *  TRUE if we are not in a thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( !inThread ) ;
   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.PROMAIL_SMALL ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default
    *  navigation panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel ( ) {
      return( proMailNavPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* selectDocComplete
    */
   /**
    *    This method will be called when a document
    *  is selected from the 'Select A Doc' window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectDocComplete ( Vector vec ) {
      if ( vec != null ) {
         int    i    = 0 ;
         int    size = vec.size() ;
         Vector docVec = new Vector ( size ) ;                                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, loop through and grab all of the docs only       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is a DocRec, just add it                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( vec.elementAt(i) instanceof Document )
               docVec.addElement ( vec.elementAt(i) ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is a Document Bookmark, create the DocRec       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( vec.elementAt(i) instanceof Bookmark ) {
               Bookmark bookmark = (Bookmark)vec.elementAt(i) ;

               //if ( bookmark.getBookmarkType() == BOOKMARK_DOCUMENT )
               //   docVec.addElement ( new DocumentRow ( ((Integer)bookmark.getObject()).intValue()) ) ;
            }

            i++ ;
         } /* while  */

         setDocList ( docVec ) ;
      } /* if  */

   }





   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when we have
    *  been selected to show from the Main Window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
      AppDefaultWin parent = GUISystem.getParentDefWin ( this ) ;

      parent.getAppDefaultMenuBar().addEditMenuItem ( mnu_SEPARATOR ) ;
      parent.getAppDefaultMenuBar().addEditMenuItem ( mnu_EDIT_HEADER ) ;
      parent.getAppDefaultMenuBar().addEditMenuItem ( mnu_EDIT_FOOTER ) ;

      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 1 ) ;
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when we are
    *  hidden because another panel has popped up
    *  in front of us
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      AppDefaultWin parent = GUISystem.getParentDefWin ( this ) ;

      parent.getAppDefaultMenuBar().removeEditMenuItem ( mnu_SEPARATOR ) ;
      parent.getAppDefaultMenuBar().removeEditMenuItem ( mnu_EDIT_HEADER ) ;
      parent.getAppDefaultMenuBar().removeEditMenuItem ( mnu_EDIT_FOOTER ) ;
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
   }




   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }



   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will kick off the thread that will
    *  refresh our list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      setView ( view ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocList
    */
   /**
    *    This method will set the list of documents being
    *  displayed
    *  
    *
    *  @param vector The Vector of DocRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocList ( Vector vector ) {
      documents.removeAll() ;
      documents.setData ( vector ) ;
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name of the
    *  QuestPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_PROMAIL ) ) ;
   } /* getName  */





   /*------------------------------------------------------------*/
   /* editHeader
    */
   /**
    *    This method will edit the header of the
    *  eMail
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editHeader () {
      if ( UserSystem.hasAuthority ( AUTH_PROACTIVEEMAILADMIN ) ) {
         SingleMLEDlg editWin   = new SingleMLEDlg ( GUISystem.getParentDefWin ( this ),
                                                     Str.getStr ( STR_CHANGE_HEADER ),
                                                     Str.getStr ( STR_PLEASE_ENTER_HEADER ),
                                                     ProMailSystem.getHeader() ) ;
         String       newHeader = editWin.getResult() ;

         if ( ( newHeader != null ) && ( newHeader.length() > 0 ) ) {
            if ( ProMailSQL.saveHeader ( newHeader ) )
               GUISystem.printBox ( STR_INFO, STR_HEADER_CHANGED ) ;
         }

      } else
         GUISystem.printBox ( STR_INFO, STR_NEED_PROACTIVE_ADMIN_AUTH ) ;

   } /* editHeader  */


   /*------------------------------------------------------------*/
   /* editFooter
    */
   /**
    *    This method will edit the Footer of the
    *  eMail
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editFooter () {
      if ( UserSystem.hasAuthority ( AUTH_PROACTIVEEMAILADMIN ) ) {
         SingleMLEDlg editWin   = new SingleMLEDlg ( GUISystem.getParentDefWin ( this ),
                                                     Str.getStr ( STR_CHANGE_FOOTER ),
                                                     Str.getStr ( STR_PLEASE_ENTER_FOOTER ),
                                                     ProMailSystem.getFooter() ) ;
         String       newFooter = editWin.getResult() ;

         if ( ( newFooter != null ) && ( newFooter.length() > 0 ) ) {
            if ( ProMailSQL.saveFooter ( newFooter ) )
               GUISystem.printBox ( STR_INFO, STR_FOOTER_CHANGED ) ;
         }

      } else
         GUISystem.printBox ( STR_INFO, STR_NEED_PROACTIVE_ADMIN_AUTH ) ;
   } /* editFooter  */





   /*------------------------------------------------------------*/
   /* setView
    */
   /**
    *    This method will set the view to the view
    *  specified. This view must be defined in ProMailConst
    *  as one of the VIEW_XXX views
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setView ( int view ) {
      this.view = view ;
      if ( view > 0 )
         new Thread ( this, THREAD_SEARCH ).start () ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will kick off one of our threads. The
    *  thread that is kicked off depends on the name
    *  of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;    

      inThread = true ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_SEARCH                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_SEARCH ) )
         search() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESHPREVIEW                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( operation.equals ( THREAD_REFRESHPREVIEW ) )
         refreshPreview ( (DocumentRow) documents.getSelectedItem() ) ;


      inThread = false ;
   }





   /*------------------------------------------------------------*/
   /* search
    */
   /**
    *    This method will gather information from the
    *  user for a search, and then will perform the
    *  search. The search done depends on what the 'view' property
    *  is set to.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void search () {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      refreshPreview ( null ) ;

      switch ( view ) {
         
         case VIEW_ALL :
            parentWin.setStatus ( STR_PLEASE_WAIT ) ;
            setDocList ( ProMailSQL.readAllProMail ( proMailNavPanel.getBrand() ) ) ;
            parentWin.setStatus ( null ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   View documents by eMailable Date                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case VIEW_EMAILABLE_DATE :
            String eMailDate = new SingleDateEntryDlg ( GUISystem.getParentDefWin ( this ),
                                                        Str.getStr ( STR_EMAILABLE_DATE ),
                                                        Str.getStr ( STR_EMAILABLE_DATE ),
                                                        startDate ).getResult() ;                              

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Only continue if they gave a valid date                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( eMailDate != null ) && ( eMailDate.length() > 0 ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Only continue if the date is truly valid                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( CDate.checkDate ( eMailDate, CDate.DATE_DATE ) == 0 ) {

                  parentWin.setStatus ( STR_PLEASE_WAIT ) ;
                  startDate = eMailDate ;
                  setDocList ( ProMailSQL.readProMailFromDate ( eMailDate, proMailNavPanel.getBrand() ) ) ;
                  parentWin.setStatus ( null ) ;

               } else
                  GUISystem.printBox ( STR_INFO, STR_DATE_ENTERED_WRONG ) ;
            }
            break ;                    


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   View Document in a date range                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case VIEW_DATE_RANGE :
            DateRangeEntryDlg rangeDlg = new DateRangeEntryDlg ( GUISystem.getParentDefWin ( this ),
                                                                 Str.getStr ( STR_EMAILABLE_DATE ),
                                                                 Str.getStr ( STR_EMAILABLE_DATE ),
                                                                 startDate,
                                                                 stopDate ) ;

            if ( rangeDlg.getResult() ) {

               parentWin.setStatus ( STR_PLEASE_WAIT ) ;
               startDate = rangeDlg.getFromDate() ;
               stopDate   = rangeDlg.getToDate() ;
               setDocList ( ProMailSQL.readProMailFromDateRange ( startDate, 
                                                                  stopDate, 
                                                                  proMailNavPanel.getBrand() ) ) ;
               parentWin.setStatus ( null ) ;

            }
            break ;              


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   View by a single doc id                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case VIEW_DOCID :
            String docId = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                                Str.getStr ( STR_LOTUS_NOTES_ID ),
                                                Str.getStr ( STR_LOTUS_NOTES_ID ),
                                                this.docId,
                                                "VVVV-VVVVV" ).getResult() ;

            if ( ( docId != null ) && ( docId.length() > 0 ) ) {

               parentWin.setStatus ( STR_PLEASE_WAIT ) ;
               this.docId = docId ;
               setDocList ( ProMailSQL.readProMailFromDocId ( docId,
                                                              proMailNavPanel.getBrand() ) ) ;
               parentWin.setStatus ( null ) ;
            }
            break ;                        


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   View for eMails going out tonight                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case VIEW_EMAIL_TONIGHT :
            parentWin.setStatus ( STR_PLEASE_WAIT ) ;
            startDate = new CDate ( CDate.DATE_DATE ).today() ;
            setDocList ( ProMailSQL.readProMailFromDateRange ( startDate, startDate, proMailNavPanel.getBrand() ) ) ;
            parentWin.setStatus ( null ) ;
            break;



      }

   }






   /*------------------------------------------------------------*/
   /* refreshPreview
    */
   /**
    *    This method will refresh the preview with 
    *  whatever is selected in the documents
    *  
    *  +
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshPreview ( DocumentRow document ) {
      parentWin.setStatus ( STR_READING ) ;
      previewPane.setDocument ( document ) ;
      this.previewDocument = document ;
      parentWin.setStatus ( null ) ;
   }




   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when an item in
    *  a list is selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent event ) {
      if ( event.getValueIsAdjusting() ) {
         refreshPreview ( null ) ;
         new Thread ( this, THREAD_REFRESHPREVIEW ).start() ;
      }
   }





   /*------------------------------------------------------------*/
   /* edit
    */
   /**
    *    This method will edit the currently selected
    *  document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void edit () {
      if ( documents.getSelectedItem() != null )
         EditDocument.editDocument ( ((DocumentRow)documents.getSelectedItem() ) ) ;
      else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;
   }




   /*------------------------------------------------------------*/
   /* cancel
    */
   /**
    *    This method will cancel a proactive
    *  email that is selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void cancel () {
      Vector docVec = documents.getSelection() ;

      if ( ( docVec != null ) &&
           ( docVec.size() > 0 ) &&
           ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_CANCEL_EMAIL ) ) ) {
         int    i       = 0 ;
         int    size    = docVec.size() ;
         String comment = new SingleMLEDlg ( parentWin, 
                                             Str.getStr ( STR_COMMENT ), 
                                             Str.getStr ( STR_COMMENT_CANCEL_PROMAIL ),
                                             "" ).getResult() ;

         while ( i < size ) {
            cancelDoc ( (DocumentRow)docVec.elementAt(i), comment ) ;
            i++ ;
         }
      }

   }


   /*------------------------------------------------------------*/
   /* send
    */
   /**
    *    This method will send a proactive
    *  email that is selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void send () {
      Vector docVec = documents.getSelection() ;

      if ( ( docVec != null ) &&
           ( docVec.size() > 0 ) &&
           ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_SEND_MAIL ) ) ) {
         int i    = 0 ;
         int size = docVec.size() ;

         while ( i < size ) {
            sendDoc ( (DocumentRow)docVec.elementAt(i) ) ;
            i++ ;
         }
      }

      documents.repaint() ;

   }

   /*------------------------------------------------------------*/
   /* cancelDoc
    */
   /**
    *    This method will cancel a document's
    *  emailable flag.
    *  
    *
    *  @param docRec The DocRec to mark
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void cancelDoc ( DocumentRow documentRow, String comment ) {
      StatusWin     statusWin   = new StatusWin ( GUISystem.getParentDefWin ( this ) ) ;
      int           rc          = 0 ;

      statusWin.setText ( Str.getStr ( STR_CANCELLING )+" '"+documentRow.getTitle() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, create a Draft for this object                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rc = documentRow.readDocument ( true ) ;
      if ( rc == 0 ) {
         DocumentDraft draft = EditDocument.createDraft ( documentRow.getDocument() ) ;

         if ( draft != null ) {

            EditDocument.setDefaultValues ( draft ) ;
            draft.setWorkRequired ( TypeList.WORKREQ_EDIT ) ;
            draft.setEMailable ( false ) ;

            statusWin.setText ( Str.getStr ( STR_SAVING )+" "+documentRow.getTitle() ) ;
            if ( !DocumentSaver.saveDraft ( draft, null ) ) {
               GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            } else
               cancelPublishedDoc ( documentRow ) ;


         }
      } else {
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
         LogSystem.log ( 1, "Could not read document. rc:"+rc ) ;
      }

      statusWin.dispose() ;
   }





   /*------------------------------------------------------------*/
   /* cancelPublishedDoc
    */
   /**
    *    This method will cancel a published document only
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void cancelPublishedDoc ( DocumentRow documentRow ) {
      String        rtnName   = "cancelDoc" ;
      SQLMethod     sql       = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      try {
         if ( sql != null ) {
            Statement stmt = sql.createStatement() ;

            stmt.executeUpdate ( "UPDATE TIGRIS.DOCUMENTS "+
                                 "SET   EMAILABLE   = 'N', "+
                                 "      DBUSER      = '"+UserSystem.getUserId()+"', "+
                                 "      CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE DOCIND      = "+documentRow.getDocumentInd() ) ;

            documents.remove ( documentRow ) ;

            if ( previewDocument == documentRow )
               previewPane.setDocument ( null ) ;

         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }
      sql.close() ;
   }

   /*------------------------------------------------------------*/
   /* sendDoc
    */
   /**
    *    This method will set the eMailable date to today for the
    * document passed in
    *  
    *
    *  @param docRec The DocRec to mark
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void sendDoc ( DocumentRow documentRow ) {
      String        rtnName   = "sendDoc" ;
      SQLMethod     sql       = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      parentWin.setStatus ( Str.getStr ( STR_SENDING_MAIL )+" '"+documentRow.getTitle()+"'" ) ;
      try {
         if ( sql != null ) {
            String    today = new CDate ( CDate.DATE_DATE ).today() ;
            Statement stmt  = sql.createStatement() ;

            stmt.executeUpdate ( "UPDATE TIGRIS.DOCUMENTS "+
                                 "SET   EMAILABLE     = 'Y', "+
                                 "      EMAILABLEDATE = '"+today+"', "+
                                 "      DBUSER        = '"+UserSystem.getUserId()+"', "+
                                 "      CHANGEDTIME   = CURRENT TIMESTAMP "+
                                 "WHERE DOCIND        = "+documentRow.getDocumentInd() ) ;

            documentRow.getDocument().setLastTouchedBy ( UserSystem.getUserId() ) ;
            documentRow.getDocument().setEMailableDate ( today ) ;

            if ( documentRow == previewDocument ) {
               previewPane.setDocument ( null ) ;
               previewPane.setDocument ( documentRow ) ;
            }

         }
      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sql.close() ;
      parentWin.setStatus ( null ) ;
   }



} /* ProMailPanel */
