package com.ibm.nzna.projects.qit.promail ;
/*-----------------------------------------------------------*/
/*   ProMailPreviewPane                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProMailPreviewPane                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel holds the following information about a Proactive eMail 
 *
 *
 * @version 1.0, 10/25/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   ProMailPreviewPane                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProMailPreviewPane extends JPanel implements Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESHDATA = "REFRESH" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea    mle_MAIL    = null ;
   private JScrollPane  scr_MAIL    = null ;
   private JLabel       st_IMAGE    = null ;
   private JLabel       st_SENTBY   = null ;
   private JLabel       stt_SENTBY  = null ;
   private JLabel       st_SENTON   = null ;
   private JLabel       stt_SENTON  = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DocumentRow  document = null ;
   private Dimension    prefSize = new Dimension ( 100, 100 ) ;
                       




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the blank preview
    *  pane
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProMailPreviewPane () {
      createControls() ;
   }




   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and attach all
    *  of the GUI controls we will use
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createControls () {   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mle_MAIL    = new JTextArea ( Str.getStr ( STR_NO_DOC_SELECTED ) ) ;
      scr_MAIL    = new JScrollPane ( mle_MAIL ) ;
      st_IMAGE    = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.PROMAIL_LARGE ) ) ;
      st_SENTBY   = new JLabel ( Str.getStr ( STR_FROM ) ) ;
      stt_SENTBY  = new JLabel ( "-" ) ;
      st_SENTON   = new JLabel ( Str.getStr ( STR_MAIL_SENT ) ) ;
      stt_SENTON  = new JLabel ( "-" ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust some properties                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      stt_SENTBY.setForeground ( Color.blue ) ;
      stt_SENTON.setForeground ( Color.blue ) ;
      mle_MAIL.setFont ( new Font ( "Courier", Font.PLAIN, 12 ) ) ;
      mle_MAIL.setEditable ( false ) ;
      setBackground ( new Color ( 230, 230, 230 ) ) ;
      scr_MAIL.setBorder ( null ) ;

      setLayout ( null ) ;
      add ( scr_MAIL ) ;
      add ( st_IMAGE ) ;
      add ( st_SENTON ) ;
      add ( stt_SENTON ) ;
      add ( st_SENTBY ) ;
      add ( stt_SENTBY ) ;

   }





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred size
    *  of this component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 0 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;
      st_IMAGE.setBounds ( x, y, 45, 45 ) ;
      st_SENTBY.setBounds ( x+50, y, 100, rowHeight ) ;
      stt_SENTBY.setBounds ( x+150, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_SENTON.setBounds ( x+50, y, 100, rowHeight ) ;
      stt_SENTON.setBounds ( x+150, y, 200, rowHeight ) ;
      y+=rowHeight+10 ;

      scr_MAIL.setBounds ( 0, y, size.width, size.height-y ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocument
    */
   /**
    *    This method will set the DocRec that
    *  we are currently displaying
    *  
    *
    *  @param document The Document to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocument ( DocumentRow document ) {
      if ( document != null ) {
         if ( this.document != document ) {
            this.document = document ;
            new Thread ( this, THREAD_REFRESHDATA ).start() ;
         }
      } else {
         stt_SENTBY.setText ( "-" ) ;
         stt_SENTON.setText ( "-" ) ;
         mle_MAIL.setText ( "" ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our operations inside of
    *  a thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESHDATA                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_REFRESHDATA ) )
         refreshData () ;
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh what doc we
    *  are currently showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      if ( document != null ) {
         String sentTime = ProMailSQL.getSentTime ( document.getFilename() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the sent time correctly                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( sentTime == null ) || ( sentTime.length() == 0 ) )
            sentTime = Str.getStr ( STR_NOT_SENT_YET ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Refresh the displayed fields                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stt_SENTON.setText ( sentTime ) ;
         stt_SENTBY.setText ( UserSystem.getNameFromUserId ( document.getLastTouchedBy() ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the pro mail preview                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( document.readDocument ( false ) == 0 ) {
            String preview      = null ;

            preview = ProMailSystem.createEMailPreview ( document.getDocument() ) ;

            if ( ( preview != null ) && ( preview.length() > 0 ) ) {
               mle_MAIL.setText ( preview ) ; 
            } else
               mle_MAIL.setText ( Str.getStr ( STR_NO_PREVIEW_INFO ) ) ;
         } else
            mle_MAIL.setText ( "Cannot read document" ) ;
      }

      invalidate() ;
      revalidate() ;
   }


} /* ProMailPreviewPane */
