package com.ibm.nzna.projects.qit.promail ;
/*-----------------------------------------------------------*/
/*   ProMailSQL                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProMailSQL                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This routine simply holds static methods for accessing information about proactive eMail 
 *
 *
 * @version 1.0, 9/2/1999 
 * @author  David Lentz
 */
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   ProMailSQL                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProMailSQL extends Object implements DocConst, AppConst {





   /*------------------------------------------------------------*/
   /* getHeader
    */
   /**
    *    This method will return the current header
    *  that should show on the eMails
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getHeader() {
      String    rtnName   = "getHeader" ;
      SQLMethod sql       = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5  ) ;
      String    retString = "" ;

      MainWindow.getInstance().setStatus ( Str.getStr ( STR_READING_PROMAIL_HEADER ) ) ;
      try {
         if ( sql != null ) {
            Statement stmt = sql.createStatement() ;
            ResultSet rs   = stmt.executeQuery ( "SELECT TEXT "+
                                                 "FROM CLIO.CONTENTQUEUE "+
                                                 "WHERE CONTENTNUM=1 FOR FETCH ONLY" ) ;

            if ( rs.next() )
               retString = rs.getString(1) ;

            retString = retString.trim() ;

            rs.close() ;
         }
      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sql.close() ;
      MainWindow.getInstance().setStatus ( null ) ;

      return( retString ) ;
   }

   /*------------------------------------------------------------*/
   /* getFooter
    */
   /**
    *    This method will return the current footer
    *  that should show on the eMails
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getFooter() {
      String    rtnName   = "getFooter" ;
      SQLMethod sql       = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;
      String    retString = "" ;

      MainWindow.getInstance().setStatus ( Str.getStr ( STR_READING_PROMAIL_FOOTER ) ) ;
      try {
         if ( sql != null ) {
            Statement stmt = sql.createStatement() ;
            ResultSet rs   = stmt.executeQuery ( "SELECT TEXT "+
                                                 "FROM CLIO.CONTENTQUEUE "+
                                                 "WHERE CONTENTNUM=2 FOR FETCH ONLY" ) ;

            if ( rs.next() )
               retString = rs.getString(1) ;

            retString = retString.trim() ;

            rs.close() ;
         }
      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sql.close() ;
      MainWindow.getInstance().setStatus ( null ) ;

      return( retString ) ;
   }





   /*------------------------------------------------------------*/
   /* saveHeader
    */
   /**
    *    This method will save the header passed
    *  in to the CLIO database
    *  
    *
    *  @param header The header to save
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean saveHeader ( String header ) {
      boolean   rc      = false ;
      String    rtnName = "saveHeader" ;
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;

      if ( sql != null ) {

         try {
            Statement stmt = sql.createStatement() ;

            stmt.executeUpdate ( "UPDATE CLIO.CONTENTQUEUE "+
                                 "SET TEXT        = '"+header+"', "+
                                 "    DBUSER      = '"+UserSystem.getUserId()+"', "+
                                 "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE CONTENTNUM = 1 " ) ;

            rc = true ;

         } catch ( Exception e ) {
            sql.rollBack () ;
            LogSystem.log ( 1, e ) ;
         }

      } /* if sql  */
      ProMailSystem.clearHeaderFooter() ;
      sql.close() ;

      return( rc ) ;
   }

   /*------------------------------------------------------------*/
   /* saveFooter
    */
   /**
    *    This method will save the footer passed
    *  in to the CLIO database
    *  
    *
    *  @param footer The footer to save
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean saveFooter ( String footer ) {
      boolean   rc      = false ;
      String    rtnName = "saveFooter" ;
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;

      if ( sql != null ) {

         try {
            Statement stmt = sql.createStatement() ;

            stmt.executeUpdate ( "UPDATE CLIO.CONTENTQUEUE "+
                                 "SET TEXT        = '"+footer+"', "+
                                 "    DBUSER      = '"+UserSystem.getUserId()+"', "+
                                 "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE CONTENTNUM = 2 " ) ;

            rc = true ;

         } catch ( Exception e ) {
            sql.rollBack () ;
            LogSystem.log ( 1, e ) ;
         }

      } /* if sql  */

      sql.close() ;
      ProMailSystem.clearHeaderFooter() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getSentTime
    */
   /**
    *    This method will return the time a message
    *  was sent.
    *  
    *
    *  @param notesId The Lotus Notes Id of the message to check
    *  @param None
    *  @param None
    *
    *  @return String - The Time, or NULL if error
    *//*---------------------------------------------------------*/
   public static String getSentTime ( String notesId ) {
      String    rtnName  = "getSentTime" ;
      SQLMethod sql      = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;
      String    sentTime = null ;

      if ( sql != null ) {
         try {
            Statement stmt = sql.createStatement() ;
            ResultSet rs   = stmt.executeQuery ( "SELECT MAX(A.SENTTIME) "+
                                                 "FROM CLIO.HISTMAIL A, "+
                                                 "     CLIO.HISTMAILCONTENT B, "+
                                                 "     CLIO.HISTCONTENT C "+
                                                 "WHERE C.DOCID = '"+notesId+"' AND "+
                                                 "      B.CONTENTNUM = C.CONTENTNUM AND "+
                                                 "      A.MESSAGEID  = B.MESSAGEID "+
                                                 "FOR FETCH ONLY" ) ;

            try {
               if ( rs.next() )
                  sentTime = rs.getString ( 1 ).trim () ;
            } catch ( Exception e ) {
               sentTime = "" ;
            }


         } catch ( Exception e ) {
            sql.rollBack() ;
            LogSystem.log ( 1, e ) ;
         }
      } /* if */

      sql.close() ;

      return( sentTime ) ;
   } /* getSentTime */






   /*------------------------------------------------------------*/
   /* search
    */
   /**
    *    This method will perform a search and returns
    *  DocumentRow objects. The search must be defined
    *  in ProMailConst as one of the VIEW_XXX vies
    *
    *  @param view The View to search
    *  @param brandRec The Brand to restrict to, NULL if no restriction
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static Vector search ( int view, TypeBrandRec brandRec ) {
      Vector     retVec = new Vector ( 10, 10 ) ;
      SQLMethod  sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "ProMailSQL.search", 10 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
         }


      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }


      sqlMethod.close() ;
      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* readProMailFromDate
    */
   /**
    *    This method will read all of the proactive
    *  eMail that is scheduled to go out, or has gone
    *  out since the date passed in.
    *
    *  @param eMaildate The date to start pulling eMails from
    *  @param brand The Brand to pull with, this is optional. A NULL passed pulls all brands
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readProMailFromDate ( String eMaildate, TypeCategoryRec brandRec ) {
      Vector retVec = new Vector ( 10, 10 ) ;
      String sqlStmt = "" ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the brand is NULL, do an optimized query             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( brandRec == null )
         sqlStmt = "SELECT DISTINCT DOCIND,  TITLE,       DOCCLASSIND,   LNDOCID, "+
                   "                DBUSER,  CHANGEDTIME, EMAILABLEDATE, OWNER "+
                   "FROM TIGRIS.DOCUMENTS "+
                   "WHERE EMAILABLEDATE = '"+eMaildate+"' FOR FETCH ONLY" ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, perform a query with the brand               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else
         sqlStmt = "SELECT DISTINCT A.DOCIND,  A.TITLE,       A.DOCCLASSIND,   A.LNDOCID, "+
                   "                A.DBUSER,  A.CHANGEDTIME, A.EMAILABLEDATE, A.OWNER "+
                   "FROM TIGRIS.DOCUMENTS A, "+
                   "     TIGRIS.CATEGORY  B, "+
                   "     PRODUCT.TYPECATCHILD C "+
                   "WHERE A.EMAILABLEDATE = '"+eMaildate+"' AND "+
                   "      A.DOCIND        =  B.DOCIND AND "+
                   "      B.TYPECATIND    =  C.TYPECATIND AND "+
                   "      C.PARENTCATIND  = "+brandRec.getInd()+" "+
                   "ORDER BY A.EMAILABLEDATE DESC "+
                   "FOR FETCH ONLY" ;

      retVec = readProMail ( sqlStmt ) ;

      return( retVec ) ;
   } /* readProMailFromDate */





   /*------------------------------------------------------------*/
   /* readProMailFromDate
    */
   /**
    *    This method will read all of the proactive
    *  eMail that is scheduled to go out, or has gone
    *  out since the date passed in.
    *
    *  @param startDate The date to start pulling documents from
    *  @param stopDate  The date to stop pulling documents from
    *  @param brand The Brand to pull with, this is optional. A NULL passed pulls all brands
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readProMailFromDateRange ( String startDate, String stopDate, TypeCategoryRec brandRec ) {
      Vector     retVec = new Vector ( 10, 10 ) ;
      String      sqlStmt = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the brand is NULL, do an optimized query             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( brandRec == null )
         sqlStmt = "SELECT DISTINCT DOCIND,  TITLE,       DOCCLASSIND,   LNDOCID, "+
                   "                DBUSER,  CHANGEDTIME, EMAILABLEDATE, OWNER "+
                   "FROM TIGRIS.DOCUMENTS "+
                   "WHERE EMAILABLEDATE >= '"+startDate+"' AND EMAILABLEDATE <= '"+stopDate+"' FOR FETCH ONLY" ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, perform a query with the brand               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else
         sqlStmt = "SELECT DISTINCT A.DOCIND,  A.TITLE,       A.DOCCLASSIND,   A.LNDOCID, "+
                   "                A.DBUSER,  A.CHANGEDTIME, A.EMAILABLEDATE, A.OWNER "+
                   "FROM TIGRIS.DOCUMENTS A, "+
                   "     TIGRIS.CATEGORY  B, "+
                   "     PRODUCT.TYPECATCHILD C "+
                   "WHERE A.EMAILABLEDATE >= '"+startDate+"' AND "+
                   "      A.EMAILABLEDATE <= '"+stopDate+"' AND "+
                   "      A.DOCIND        =  B.DOCIND AND "+
                   "      B.TYPECATIND    =  C.TYPECATIND AND "+
                   "      C.PARENTCATIND  = "+brandRec.getInd()+" "+
                   "ORDER BY A.EMAILABLEDATE DESC "+
                   "FOR FETCH ONLY" ;




      retVec = readProMail ( sqlStmt ) ;
      return( retVec ) ;
   } /* readProMailFromDate */




   /*------------------------------------------------------------*/
   /* readProMailFromDocId
    */
   /**
    *    This method will read a single document matching
    *  the doc id passed in. This single document is passed back in
    *  a Vector since all other 'readProMailFromXXX' methods return a vector
    *
    *  @param docId The doc id to look for
    *  @param brand The Brand to pull with, this is optional. A NULL passed pulls all brands
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readProMailFromDocId ( String docId, TypeCategoryRec brandRec ) {
      Vector     retVec = new Vector ( 10, 10 ) ;
      String      sqlStmt = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the brand is NULL, do an optimized query             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlStmt = "SELECT DISTINCT DOCIND,  TITLE,       DOCCLASSIND,   LNDOCID, "+
                "                DBUSER,  CHANGEDTIME, EMAILABLEDATE, OWNER "+
                "FROM TIGRIS.DOCUMENTS "+
                "WHERE LNDOCID = '"+docId+"' FOR FETCH ONLY" ;

      retVec = readProMail ( sqlStmt ) ;

      return( retVec ) ;
   } /* readProMailFromDate */



   /*------------------------------------------------------------*/
   /* readAllProMail
    */
   /**
    *    This method will read all of the proactive
    *  eMail that is scheduled to go out for the brand
    *
    *  @param brand The Brand to pull with, this is optional. A NULL passed pulls all brands
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readAllProMail ( TypeCategoryRec brandRec ) {
      Vector     retVec = new Vector ( 10, 10 ) ;
      String      sqlStmt = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, perform a query with the brand               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( brandRec != null ) {
         sqlStmt = "SELECT DISTINCT A.DOCIND,  A.TITLE,       A.DOCCLASSIND,   A.LNDOCID, "+
                   "                A.DBUSER,  A.CHANGEDTIME, A.EMAILABLEDATE, A.OWNER "+
                   "FROM TIGRIS.DOCUMENTS A, "+
                   "     TIGRIS.DOCBRAND  B  "+
                   "WHERE A.DOCIND        =  B.DOCIND AND "+
                   "      B.TYPECATIND    =  C.TYPECATIND AND "+
                   "      C.PARENTCATIND  = "+brandRec.getInd()+" "+
                   "ORDER BY A.EMAILABLEDATE DESC "+
                   "FOR FETCH ONLY" ;
      } else {
         sqlStmt = "SELECT DISTINCT A.DOCIND,  A.TITLE,       A.DOCCLASSIND,   A.LNDOCID, "+
                   "                A.DBUSER,  A.CHANGEDTIME, A.EMAILABLEDATE, A.OWNER "+
                   "FROM TIGRIS.DOCUMENTS A "+
                   "WHERE A.EMAILABLE     = 'Y' AND "+
                   "      A.EMAILABLEDATE IS NOT NULL AND "+
                   "      A.PUBLISH = 'Y' "+
                   "ORDER BY A.EMAILABLEDATE DESC "+
                   "FOR FETCH ONLY" ;
      }

      retVec = readProMail ( sqlStmt ) ;

      return( retVec ) ;
   } /* readProMailFromDate */





   /**
     * This method will run all Queries that the views in proactive eMail can produce.
     *
     * @param query The String query.
     *
     * @returns Vector of DocumentRow objects
     **/
   public static Vector readProMail ( String query ) {
      Vector     retVec = new Vector ( 10, 10 ) ;
      SQLMethod  sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readProMail", 10 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet   rs      = null ;
            DocumentRow readDoc = null ;


            rs = stmt.executeQuery ( query ) ;

            while ( rs.next() ) {
               readDoc = new DocumentRow ( ""+rs.getInt(1),
                                           ""+rs.getInt(1),
                                           rs.getString(2).trim(),
                                           rs.getString(8).trim(),
                                           rs.getInt(3),
                                           rs.getString(4),
                                           0,
                                           rs.getString(5).trim(),
                                           DateSystem.prettyDateFromStamp ( rs.getString(6).trim() ),
                                           DateSystem.prettyDateFromStamp ( rs.getString(6).trim() ),
                                           false ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the column data in it                               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               readDoc.setEmailableDate ( DateSystem.prettyDateFromDBDate ( rs.getString(7) ) ) ;
               readDoc.setColumnOrder ( 1, DocumentRow.COL_DOCUMENTIND ) ;
               readDoc.setColumnOrder ( 2, DocumentRow.COL_TITLE ) ;
               readDoc.setColumnOrder ( 3, DocumentRow.COL_FILENAME ) ;
               readDoc.setColumnOrder ( 4, DocumentRow.COL_EMAILABLE_DATE ) ;

               readDoc.setColumnOrder ( 5, DocumentRow.COL_LASTTOUCHED ) ;
               readDoc.setColumnOrder ( 6, DocumentRow.COL_LASTTOUCHEDBY ) ;

               retVec.addElement ( readDoc ) ;
            }

         } /* if the Statement object is good */
         else
            LogSystem.log ( 1, "Could not create SQLStatement for method 'readProMailFromDate'" ) ;


      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }


      sqlMethod.close() ;
      return( retVec ) ;
   } /* readProMailFromDate */


} /* ProMailSQL */
