package com.ibm.nzna.projects.qit.recyclebin ;  
/*-----------------------------------------------------------*/
/*   RecycleSQL                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: RecycleSQL                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import java.io.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.qit.app.*;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
/**
 * This class holds static methods used by the RecycleBin Applet 
 *
 *
 * @version 1.0, 4/10/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   RecycleSQL                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class RecycleSQL extends Object implements AppConst {





   /*------------------------------------------------------------*/
   /* readRecycledProducts
    */
   /**
    *    This method will read in all of the recycled
    *  products
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector. NULL if none found or error
    *//*---------------------------------------------------------*/
   protected static Vector readRecycledProducts () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readRecycledProducts", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement  stmt            = sqlMethod.createStatement() ;
         ProductRow readRec         = null ;
         boolean    containsOptions = false ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT PRODUCTIND, EXISTINGPRODUCTIND, PRODNUM, DESCRIPTION, CREATEDON, CREATEDBY, LASTTOUCHEDON, LASTTOUCHEDBY "+
                                               "FROM PRODRAFT.PRODUCT "+
                                               "WHERE RECYCLED = 'Y' "+
                                               "ORDER BY LASTTOUCHEDON FOR FETCH ONLY" );

            while ( rs.next() ) {
               readRec = new ProductRow ( rs.getInt(1),
                                          rs.getInt(2),
                                          rs.getString(3),
                                          rs.getString(4).trim(),
                                          rs.getString(5).trim(), //Created On
                                          rs.getString(6).trim(), //Created By
                                          "",
                                          "",
                                          rs.getString(7).trim(),
                                          rs.getString(8).trim(),
                                          true ) ;

               if ( ( rs.getString(3) == null ) || ( rs.getString(3).trim().equals ( "NULL" ) ) ) 
                  containsOptions = true ;

               returnVec.addElement ( readRec ) ;
            } /* while */

            rs.close() ;

         } /* if stmt */


         if ( containsOptions )
            readPartNumbers(returnVec) ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;


   } /* readRecycledProducts  */






   /*------------------------------------------------------------*/
   /* readRecycledDocuments
    */
   /**
    *    This method will read in all of the recycled
    *  documents in QUEST.DOCUMENTS
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static Vector readRecycledDocuments () {
      Vector    retVec  = new Vector ( 1 )  ;
      String    rtnName = "readRecycle" ;
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      int       sqlCode = 0 ;

      try {
         Statement   stmt    = sql.createStatement() ;
         DocumentRow docRow  = null ;
         ResultSet   rs      = stmt.executeQuery ( "SELECT DOCIND,  EXISTINGDOCIND, TITLE, OWNER, DOCCLASSIND, "+
                                                   "       LNDOCID, WORKIND,        CHANGEDTIME,  DBUSER "+
                                                   "FROM QUEST.DOCUMENTS "+
                                                   "WHERE RECYCLED = 'Y' ORDER BY CHANGEDTIME" ) ;

         while ( rs.next() ) {
            docRow = new DocumentRow ( ""+rs.getInt(1),
                                       ""+rs.getInt(2),
                                       rs.getString(3).trim(),
                                       rs.getString(4),
                                       rs.getInt(5),
                                       rs.getString(6),
                                       rs.getInt(7),
                                       rs.getString(8).trim(),
                                       rs.getString(8).trim(),
                                       rs.getString(9).trim(),
                                       true ) ;


            retVec.addElement ( docRow ) ;
         }

         adjustDocumentRows ( retVec ) ;

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sql.close() ;
      System.gc() ;

      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* restoreProducts
    */
   /**
    *    This method will restore a set of products
    *  
    *  
    *
    *  @param vec A Vector containing ProductRec
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void restoreProducts ( Vector vec ) {
      int     i      = 0 ;
      int     size   = vec.size() ;
      String  sqlStr = "UPDATE PRODRAFT.PRODUCT SET RECYCLED = 'N' WHERE PRODUCTIND IN ( " ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL String to execute                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((ProductRow)vec.elementAt(i)).getProductInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sqlStr+")" ) ;

   }




   /*------------------------------------------------------------*/
   /* restoreQuestions
    */
   /**
    *    This method will restore a set of questions
    *  
    *  
    *
    *  @param vec A Vector containing QuestionDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void restoreQuestions ( Vector vec ) {
      int     i      = 0 ;
      int     size   = vec.size() ;
      String  sqlStr = "UPDATE OADRAFT.QUESTION SET RECYCLED = 'N' WHERE QUESTIONIND IN ( " ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL String to execute                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((QuestionDisplayRec)vec.elementAt(i)).getQuestionInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sqlStr+")" ) ;

   }


   /*------------------------------------------------------------*/
   /* restoreLinkGroups
    */
   /**
    *    This method will restore a set of questions
    *  
    *  
    *
    *  @param vec A Vector containing LinkGroupDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void restoreLinkGroups ( Vector vec ) {
      int     i      = 0 ;
      int     size   = vec.size() ;
      String  sqlStr = "UPDATE OADRAFT.LINKGROUP SET RECYCLED = 'N' WHERE LINKGROUPIND IN ( " ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL String to execute                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((LinkGroupDisplayRec)vec.elementAt(i)).getLinkGroupInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sqlStr+")" ) ;

   }

   /*------------------------------------------------------------*/
   /* restoreLinks
    */
   /**
    *    This method will restore a set of questions
    *  
    *  
    *
    *  @param vec A Vector containing LinkDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void restoreLinks ( Vector vec ) {
      int     i      = 0 ;
      int     size   = vec.size() ;
      String  sqlStr = "UPDATE OADRAFT.LINK SET RECYCLED = 'N' WHERE LINKIND IN ( " ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL String to execute                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((LinkDisplayRec)vec.elementAt(i)).getLinkInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sqlStr+")" ) ;

   }

   /*------------------------------------------------------------*/
   /* restoreSymptoms
    */
   /**
    *    This method will restore a set of questions
    *  
    *  
    *
    *  @param vec A Vector containing SymptomDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void restoreSymptoms ( Vector vec ) {
      int     i      = 0 ;
      int     size   = vec.size() ;
      String  sqlStr = "UPDATE OADRAFT.SYMPTOM SET RECYCLED = 'N' WHERE SYMPTOMIND IN ( " ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL String to execute                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((SymptomDisplayRec)vec.elementAt(i)).getSymptomInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sqlStr+")" ) ;

   }


   /*------------------------------------------------------------*/
   /* restoreActions
    */
   /**
    *    This method will restore a set of questions
    *  
    *  
    *
    *  @param vec A Vector containing ActionDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void restoreActions ( Vector vec ) {
      int     i      = 0 ;
      int     size   = vec.size() ;
      String  sqlStr = "UPDATE OADRAFT.ACTION SET RECYCLED = 'N' WHERE ACTIONIND IN ( " ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL String to execute                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((ActionDisplayRec)vec.elementAt(i)).getActionInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sqlStr+")" ) ;

   }



   /*------------------------------------------------------------*/
   /* restoreDocuments
    */
   /**
    *    This method will restore a set of documents
    *  
    *  
    *
    *  @param vec A Vector containing DocRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void restoreDocuments ( Vector vec ) {
      int     i      = 0 ;
      int     size   = vec.size() ;
      String  sqlStr = "UPDATE QUEST.DOCUMENTS SET RECYCLED = 'N' WHERE DOCIND IN ( " ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL String to execute                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((DocumentRow)vec.elementAt(i)).getDocumentInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sqlStr+")" ) ;

   }





   /*------------------------------------------------------------*/
   /* deleteProducts
    */
   /**
    *    This method will delete the products
    *  listed in the vec passed in
    *  
    *
    *  @param vec A vector of ProductRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void deleteProducts ( Vector vec ) {
      String sqlStr    = "" ;
      int    i         = 0 ;
      int    size      = vec.size() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((ProductRow)vec.elementAt(i)).getProductInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      if ( sqlStr.length() > 0 ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute our update                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         QITSqlRunner.sqlRunner ( "DELETE FROM PRODRAFT.DATES        WHERE PRODUCTIND IN ("+sqlStr+")"+QITSqlRunner.END_DELIM+
                                  "DELETE FROM PRODRAFT.COUNTRY      WHERE PRODUCTIND IN ("+sqlStr+")"+QITSqlRunner.END_DELIM+
                                  "DELETE FROM PRODRAFT.GEO          WHERE PRODUCTIND IN ("+sqlStr+")"+QITSqlRunner.END_DELIM+
                                  "DELETE FROM PRODRAFT.XREF         WHERE PRODUCTIND IN ("+sqlStr+")"+QITSqlRunner.END_DELIM+
                                  "DELETE FROM PRODRAFT.CATEGORY     WHERE PRODUCTIND IN ("+sqlStr+")"+QITSqlRunner.END_DELIM+
                                  "DELETE FROM PRODRAFT.TYPECATCHILD WHERE TYPECATIND IN (SELECT TYPECATIND FROM PRODRAFT.TYPECATEGORY WHERE CATEGORYLEVEL in ( 'model', 'partnumber' ) and typecatind in (select typecatind from prodraft.category where PRODUCTIND IN ("+sqlStr+")))"+QITSqlRunner.END_DELIM+
                                  "DELETE FROM PRODRAFT.TYPECATEGORY WHERE TYPECATIND IN (SELECT TYPECATIND FROM PRODRAFT.TYPECATEGORY WHERE CATEGORYLEVEL in ( 'model', 'partnumber' ) and typecatind in (select typecatind from prodraft.category where PRODUCTIND IN ("+sqlStr+")))"+QITSqlRunner.END_DELIM+
                                  "DELETE FROM PRODRAFT.PRODUCT      WHERE PRODUCTIND IN ("+sqlStr+")"+QITSqlRunner.END_DELIM ) ;

      }
   }


   /*------------------------------------------------------------*/
   /* deleteDocuments
    */
   /**
    *    This method will delete the documents
    *  listed in the vec passed in
    *  
    *
    *  @param vec A vector of DocRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void deleteDocuments ( Vector vec ) {
      String sqlStr       = "" ;
      String masterSqlStr = "" ;
      int    i            = 0 ;
      int    size         = vec.size() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((DocumentRow)vec.elementAt(i)).getDocumentInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      masterSqlStr = "DELETE FROM QUEST.DOCBRAND       WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.DOCCATEGORY    WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.CATEGORY       WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.DOCBODY        WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.DOCBODYHISTORY WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.DOCCOUNTRY     WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.DOCGRAPHIC     WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.DOCMEDIA       WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.DOCSTATUS      WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.PRODDOCS       WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM+
                     "DELETE FROM QUEST.DOCUMENTS      WHERE DOCIND IN ( "+sqlStr+")"+QITSqlRunner.END_DELIM ;
      QITSqlRunner.sqlRunner ( masterSqlStr ) ;
   }

   /*------------------------------------------------------------*/
   /* deleteSymptoms
    */
   /**
    *    This method will delete the questions
    *  listed in the vec passed in
    *  
    *
    *  @param vec A vector of SymptomDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void deleteSymptoms ( Vector vec ) {
      String sqlStr       = "" ;
      String masterSqlStr = "" ;
      int    i            = 0 ;
      int    size         = vec.size() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((SymptomDisplayRec)vec.elementAt(i)).getSymptomInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      masterSqlStr = "UPDATE OADRAFT.SYMPTOM SET STOPDATE = CURRENT DATE WHERE SYMPTOMIND IN ( "+sqlStr+" )"+QITSqlRunner.END_DELIM ;

      QITSqlRunner.sqlRunner ( masterSqlStr ) ;
   }


   /*------------------------------------------------------------*/
   /* deleteLinkGroups
    */
   /**
    *    This method will delete the questions
    *  listed in the vec passed in
    *  
    *
    *  @param vec A vector of LinkGroupDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void deleteLinkGroups ( Vector vec ) {
      String sqlStr       = "" ;
      String masterSqlStr = "" ;
      int    i            = 0 ;
      int    size         = vec.size() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((LinkGroupDisplayRec)vec.elementAt(i)).getLinkGroupInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      masterSqlStr = "UPDATE OADRAFT.LINKGROUP SET STOPDATE = CURRENT DATE WHERE LINKGROUPIND IN ( "+sqlStr+" )"+QITSqlRunner.END_DELIM ;

      QITSqlRunner.sqlRunner ( masterSqlStr ) ;
   }

   /*------------------------------------------------------------*/
   /* deleteLinks
    */
   /**
    *    This method will delete the questions
    *  listed in the vec passed in
    *  
    *
    *  @param vec A vector of LinkDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void deleteLinks ( Vector vec ) {
      String sqlStr       = "" ;
      String masterSqlStr = "" ;
      int    i            = 0 ;
      int    size         = vec.size() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((LinkDisplayRec)vec.elementAt(i)).getLinkInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      masterSqlStr = "UPDATE OADRAFT.LINK SET STOPDATE = CURRENT DATE WHERE LINKIND IN ( "+sqlStr+" )"+QITSqlRunner.END_DELIM ;

      QITSqlRunner.sqlRunner ( masterSqlStr ) ;
   }

   /*------------------------------------------------------------*/
   /* deleteActions
    */
   /**
    *    This method will delete the questions
    *  listed in the vec passed in
    *  
    *
    *  @param vec A vector of ActionDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void deleteActions ( Vector vec ) {
      String sqlStr       = "" ;
      String masterSqlStr = "" ;
      int    i            = 0 ;
      int    size         = vec.size() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((ActionDisplayRec)vec.elementAt(i)).getActionInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      masterSqlStr = "UPDATE OADRAFT.ACTION SET STOPDATE = CURRENT DATE WHERE ACTIONIND IN ( "+sqlStr+" )"+QITSqlRunner.END_DELIM ;

      QITSqlRunner.sqlRunner ( masterSqlStr ) ;
   }

   /*------------------------------------------------------------*/
   /* deleteQuestions
    */
   /**
    *    This method will delete the questions
    *  listed in the vec passed in
    *  
    *
    *  @param vec A vector of QuestionDisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void deleteQuestions ( Vector vec ) {
      String sqlStr       = "" ;
      String masterSqlStr = "" ;
      int    i            = 0 ;
      int    size         = vec.size() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         sqlStr+=""+((QuestionDisplayRec)vec.elementAt(i)).getQuestionInd() ;
         if ( i != (size-1) )
            sqlStr+="," ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute our update                                      */         
      /*                                                           */
      /*-----------------------------------------------------------*/
      masterSqlStr = "UPDATE OADRAFT.QUESTION SET STOPDATE = CURRENT DATE WHERE QUESTIONIND IN ( "+sqlStr+" )"+QITSqlRunner.END_DELIM ;

      QITSqlRunner.sqlRunner ( masterSqlStr ) ;
   }

   /*------------------------------------------------------------*/
   /* readRecycledQuestions
    */
   /**
    *    This method will read in all of the recycled questions
    *  and returns a Vector of QuestionDisplayRecs
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readRecycledQuestions () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readRecycledQuestions", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.DOCCLASSIND, B.TITLE, A.CHANGEDTIME, A.DBUSER "+
                                               "FROM OADRAFT.QUESTION A,     "+
                                               "     OADRAFT.QUESTIONTITLE B "+
                                               "WHERE A.RECYCLED = 'Y' AND "+
                                               "      A.QUESTIONIND = B.QUESTIONIND AND "+
                                               "      B.SORTORDER IN (SELECT MAX(SORTORDER) FROM OADRAFT.QUESTIONTITLE WHERE QUESTIONIND = A.QUESTIONIND) "+
                                               "ORDER BY CHANGEDTIME FOR FETCH ONLY" );

            while ( rs.next() ) {
               returnVec.addElement ( new QuestionDisplayRec ( rs.getInt(1),
                                                               rs.getInt(2),
                                                               rs.getString(3).trim(),
                                                               rs.getString(4).trim(),
                                                               rs.getString(5).trim() ) ) ;
            } /* while */

            rs.close() ;

         } /* if stmt */

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readRecycledActions
    */
   /**
    *    This method will read in all of the recycled actions
    *  and returns a Vector of ActionDisplayRecs
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readRecycledActions () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readRecycledActions", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT A.ACTIONIND, A.DOCCLASSIND, B.TITLE, A.CHANGEDTIME, A.DBUSER "+
                                               "FROM OADRAFT.ACTION A,     "+
                                               "     OADRAFT.ACTIONTITLE B "+
                                               "WHERE A.RECYCLED = 'Y' AND "+
                                               "      A.ACTIONIND = B.ACTIONIND AND "+
                                               "      B.SORTORDER IN (SELECT MAX(SORTORDER) FROM OADRAFT.ACTIONTITLE WHERE ACTIONIND = A.ACTIONIND) "+
                                               "ORDER BY CHANGEDTIME FOR FETCH ONLY" );

            while ( rs.next() ) {
               returnVec.addElement ( new ActionDisplayRec ( rs.getInt(1),
                                                             rs.getInt(2),
                                                             rs.getString(3).trim(),
                                                             rs.getString(4).trim(),
                                                             rs.getString(5).trim() ) ) ;
            } /* while */

            rs.close() ;

         } /* if stmt */

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readRecycledSymptoms
    */
   /**
    *    This method will read in all of the recycled symptoms
    *  and returns a Vector of SymptomDisplayRecs
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readRecycledSymptoms () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readRecycledSymptoms", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT A.SYMPTOMIND, A.STATUSIND, A.DOCCLASSIND, B.TITLE, A.CHANGEDTIME, A.DBUSER "+
                                               "FROM OADRAFT.SYMPTOM A,     "+
                                               "     OADRAFT.SYMPTOMTITLE B "+
                                               "WHERE A.RECYCLED = 'Y' AND "+
                                               "      A.SYMPTOMIND = B.SYMPTOMIND AND "+
                                               "      B.SORTORDER IN (SELECT MAX(SORTORDER) FROM OADRAFT.SYMPTOMTITLE WHERE SYMPTOMIND = A.SYMPTOMIND) "+
                                               "ORDER BY CHANGEDTIME FOR FETCH ONLY" );

            while ( rs.next() ) {
               returnVec.addElement ( new SymptomDisplayRec ( rs.getInt(1),
                                                              rs.getInt(2),
                                                              rs.getInt(3),
                                                              rs.getString(4).trim(),
                                                              rs.getString(5).trim(),
                                                              rs.getString(6).trim() ) ) ;
            } /* while */

            rs.close() ;

         } /* if stmt */

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readRecycledLinks
    */
   /**
    *    This method will read in all of the recycled links
    *  and returns a Vector of LinkDisplayRecs
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readRecycledLinks () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readRecycledLinks", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT A.LINKIND, B.TITLE, A.VALID, A.LINKTYPEIND, A.DBUSER, A.CHANGEDTIME "+
                                               "FROM OADRAFT.LINK A,     "+
                                               "     OADRAFT.LINKTITLE B "+
                                               "WHERE A.RECYCLED = 'Y' AND "+
                                               "      A.LINKIND = B.LINKIND AND "+
                                               "      B.SORTORDER IN (SELECT MAX(SORTORDER) FROM OADRAFT.LINKTITLE WHERE LINKIND = A.LINKIND) "+
                                               "ORDER BY CHANGEDTIME FOR FETCH ONLY" );


            while ( rs.next() ) {
               returnVec.addElement ( new LinkDisplayRec ( rs.getInt(1),                        // Link Ind
                                                           rs.getString(2).trim(),              // Title
                                                           rs.getString(3).trim().equals("Y"),  // Valid  
                                                           rs.getInt(4),                        // Link Type Ind
                                                           0,                                   // Doc Ind
                                                           "",                                  // Doc Filename
                                                           "",                                  // URL
                                                           rs.getString(5).trim(),              // DB User
                                                           rs.getString(6).trim() ) ) ;         // Changed Time
            } /* while */

            rs.close() ;

         } /* if stmt */

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readRecycledLinkGroups
    */
   /**
    *    This method will read in all of the recycled linkGroups
    *  and returns a Vector of LinkGroupDisplayRecs
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readRecycledLinkGroups () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readRecycledLinkGroups", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT A.LINKGROUPIND, B.TITLE, A.CHANGEDTIME, A.DBUSER "+
                                               "FROM OADRAFT.LINKGROUP A,     "+
                                               "     OADRAFT.LINKGROUPTITLE B "+
                                               "WHERE A.RECYCLED = 'Y' AND "+
                                               "      A.LINKGROUPIND = B.LINKGROUPIND AND "+
                                               "      B.SORTORDER IN (SELECT MAX(SORTORDER) FROM OADRAFT.LINKGROUPTITLE WHERE LINKGROUPIND = A.LINKGROUPIND) "+
                                               "ORDER BY CHANGEDTIME FOR FETCH ONLY" );

            while ( rs.next() ) {
               returnVec.addElement ( new LinkGroupDisplayRec ( rs.getInt(1),
                                                                rs.getString(3).trim(),
                                                                rs.getString(4).trim(),
                                                                rs.getString(5).trim() ) ) ;
            } /* while */

            rs.close() ;

         } /* if stmt */

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   }


   /**
     *  This method will go through all of the DocumentRow objects in the
     *  vector and will add the Image to the Document Classification, set
     *  the full name of the user.
     *  column.
     **/
   private static void adjustDocumentRows ( Vector documents ) {
      int         i           = 0 ;
      int         size        = documents.size() ;
      int         docClassInd = 0 ;
      DocumentRow docRow      = null ;

      while ( i < size ) {
         docRow = (DocumentRow)documents.elementAt(i) ;

         docClassInd = docRow.getDocClassInd() ;

         switch ( docClassInd ) {
            case TypeList.DOCCLASS_IBM_CONFIDENTIAL :
               docRow.setDocClassImage ( ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ) ;
               break ;

            case TypeList.DOCCLASS_DEALER_CONFIDENTIAL :
               docRow.setDocClassImage ( ImageSystem.getImage ( MainWindow.getInstance(),  ImageSystem.IBM_DEALER_CONFIDENTIAL ) )  ;
               break ;

            case TypeList.DOCCLASS_UNCLASSIFIED :
               docRow.setDocClassImage ( ImageSystem.getImage ( MainWindow.getInstance(),  ImageSystem.IBM_UNCLASSIFIED ) ) ;
               break ;
         }

         docRow.setLastTouchedBy ( UserSystem.getNameFromUserId ( docRow.getLastTouchedBy() ) ) ;
         i++ ;
      }
   }


   /**
     * This method will read in all the option part numbers to display
     **/
   private static void readPartNumbers ( Vector models ) {
      int       i         = 0 ;
      int       size      = models.size() ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readOptionPartNumbers", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = null ;
         String    query = "SELECT B.PRODUCTIND, A.DESCRIPTION FROM PRODRAFT.TYPECATEGORY A, PRODRAFT.CATEGORY B WHERE A.TYPECATIND = B.TYPECATIND AND PRODUCTIND IN (" ;
         int       actualCount = 0 ;

         while ( i < size ) {
            if ( ( ((ProductRow)models.elementAt(i)).getProductNumber() == null ) ||
                 ( ((ProductRow)models.elementAt(i)).getProductNumber().trim().equals ( "NULL" ) ) ) {
               query+=((ProductRow)models.elementAt(i)).getProductInd()+"," ;
               actualCount++ ;
            }
            i++ ;
         }

         if ( actualCount > 0 ) {
            query=query.substring ( 0, query.length()-1 )+") FOR FETCH ONLY" ;

            rs = stmt.executeQuery ( query ) ;

            while ( rs.next() ) {
               ProductRow row = getProductRowFromInd ( rs.getInt ( 1 ), models ) ;

               if ( row != null )
                  row.setProdNum ( rs.getString(2).trim() ) ;
            }
         }

         
      } catch ( Exception e) {
         LogSystem.log ( 1, e, false ) ;
      }

   }


   /**
     * This method will find a ProductRow from the product indicator
     **/
   private static ProductRow getProductRowFromInd ( int productInd, Vector products ) {
      ProductRow productRow = null ;
      int        i          = 0 ;
      int        size       = products.size() ;

      while ( ( i < size ) && ( productRow == null ) ) {
         if ( ((ProductRow)products.elementAt(i)).getProductInd() == productInd )
            productRow = (ProductRow)products.elementAt(i) ;
         i++ ;
      }

      return( productRow ) ;
   }

} /* RecycleSQL */
