package com.ibm.nzna.projects.qit.sample ;
/*-----------------------------------------------------------*/
/*   Sample                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Sample                              */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This is a 'Sample' Quest Applet. This applet has the following functionality :<br>
 * 1. Has an Icon on the Quest Workspace           <br>
 * 2. Launches a QuestPanel with a NavPanel        <br>
 * 3. Performs a simple DB/2 Query                 <br>
 * 4. Performs a Close                             <br>
 * 5. Has a Help page                              <br><br><br>
 *
 * This class implements QuestApplet. The QuestApplet is the base interface to implement
 * if you wish an Object to be considered a QuestApplet and wish the object to appear as
 * an Icon on the Quest Workspace.<br><br>
 *
 * @version 1.0, 2/7/2001
 * @author  David Lentz
 */
import javax.swing.* ;                          // Import the Swing libraries
import com.ibm.nzna.projects.qit.app.* ;        // Application Base Imports including AppConst



/*-----------------------------------------------------------*/
/*                                                           */
/*   Sample                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class Sample extends Object implements
AppConst,  //Default Application Constants
Authority, // For all Authorities in the Application
QuestApplet {




    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This default constructor for all QuestApplets
     *  should do nothing.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Sample () {
    }





    /*------------------------------------------------------------*/
    /* getSmallIcon
     */
    /**
     *    This method should return a reduced 
     *  size image of the getIcon method. This is used
     *  when displaying this QuestApplet in any List
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return ImageIcon
     *//*---------------------------------------------------------*/
    public ImageIcon getSmallIcon () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Here we will ask the ImageSystem to grab an image for   */
        /* us. The ImageSystem is the preferred way to obtain an     */
        /* Image or ImageIcon because it caches these images for     */
        /* easy and compact management                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        return( ImageSystem.getImageIcon ( MainWindow.getInstance(),    // The ImageSystem requires a Component, 
                                           // The Main Window is always available.
                                           ImageSystem.DOCUMENT_SMALL ) ) ;
    }


    /*------------------------------------------------------------*/
    /* getIcon
     */
    /**
     *    This method will return the Icon that is displayed
     *  on the Quest Workspace.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return ImageIcon
     *//*---------------------------------------------------------*/
    public ImageIcon getIcon () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Here we will ask the ImageSystem to grab an image for   */
        /* us. The ImageSystem is the preferred way to obtain an     */
        /* Image or ImageIcon because it caches these images for     */
        /* easy and compact management                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        return( ImageSystem.getImageIcon ( MainWindow.getInstance(),    // The ImageSystem requires a Component, 
                                           // The Main Window is always available.
                                           ImageSystem.DOCUMENT_LARGE ) ) ;
    }





    /*------------------------------------------------------------*/
    /* getName
     */
    /**
     *    This method will return the name of our
     *  QuestApplet. This name is displayed under
     *  the Icon in the Quest Workspace.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String getName () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Ask the Str object to obtain a String from the language */
        /* file. This is the ONLY way anyone should use a String     */
        /* for output in Quest. This method provides the NLS         */
        /* capability needed.                                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        return( Str.getStr ( STR_SAMPLE ) ) ;       // The Constant STR_SAMPLE is stored in AppConst.java
    }





    /*------------------------------------------------------------*/
    /* canChat
     */
    /**
     *    Future versions of Quest will support chat. This
     *  method should return TRUE if this QuestApplet
     *  will Chat. Currently you should return FALSE
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return FALSE
     *//*---------------------------------------------------------*/
    public boolean canChat () {
        return( false ) ;
    }





    /*------------------------------------------------------------*/
    /* getAuthority
     */
    /**
     *    This method is very important because it
     *  tells Quest what authority level is needed for
     *  the user to launch this Quest Applet.<br><br>
     *
     *  Authority levels are set in the TIGRIS.TYPEAUTH
     * table and should be reflected here. There are constants
     * set up for each authority in the AppConst class. 
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public int getAuthority () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Since we really require no authority, return NO_AUTH    */
        /*                                                           */
        /*-----------------------------------------------------------*/
        return( AUTH_NOAUTHREQ ) ;

    }






    /*------------------------------------------------------------*/
    /* start
     */
    /**
     *    This is the starting place. This method is called
     *  once the users selects this QuestApplet's Icon
     *  from the Main Window. Any panels/messages
     *  should be shown here
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void start () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Ok, lets put up our panel                               */                
        /*                                                           */
        /*-----------------------------------------------------------*/
        WindowSystem.createPanel ( new SamplePanel () ) ;

    }




    /*------------------------------------------------------------*/
    /* initialize
     */
    /**
     *    This method will be called once when your
     *  QuestApplet is loaded into the Workspace. Please
     *  do not do any time consuming tasks during
     *  initialization as it slows down the Quest Workspace.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return TRUE if everything was initialized, FALSE if not
     *//*---------------------------------------------------------*/
    public boolean initialize () {

        return( true ) ;
    }





    /*------------------------------------------------------------*/
    /* getDescription
     */
    /**
     *    This method should return a detailed single line
     *  description of the Quest Applet. This description
     *  is shown in any list containing the Quest Applet
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String getDescription () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Ask the Str object to obtain a String from the language */
        /* file. This is the ONLY way anyone should use a String     */
        /* for output in Quest. This method provides the NLS         */
        /* capability needed.                                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        return( Str.getStr ( STR_SAMPLE_LONG_DESCRIPTION ) ) ;       // The Constant STR_SAMPLE_LONG_DESCRIPTION is 
                                                                     // stored in AppConst.java
    }


} /* Sample */
