package com.ibm.nzna.projects.qit.sample ;       
/*-----------------------------------------------------------*/
/*   SampleNavPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SampleNavPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;

/**
 * This is the NavPanel that our SamplePanel uses! Notice how this
 * object is a JPanel and implements NavPanel
 *
 *
 * @version 1.0, 2/12/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SampleNavPanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class SampleNavPanel extends JPanel implements ActionListener, NavPanel, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       headingLabel      = null ;
   private HotLinkLabel pb_REFRESH        = null ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private SamplePanel samplePanel = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Usually the NavPanel of a QuestPanel
    *  takes the QuestPanel in the constructor
    *  so that when the nav panel needs to call a 
    *  method or something in the QuestPanel it can
    *
    *  @param samplePanel The SamplePanel that is using us as it's NavPanel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SampleNavPanel ( SamplePanel samplePanel ) {
      this.samplePanel = samplePanel ;
   }






   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    Just like a QuestPanel and QuestApplet, the
    *  initialize method is called before anything else and
    *  before this NavPanel is shown. Initialize the GUI here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      headingLabel = new JLabel       ( Str.getStr ( STR_SAMPLE ) ) ;
      pb_REFRESH   = new HotLinkLabel ( Str.getStr ( STR_REFRESH ) ) ;                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action listeners                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_REFRESH.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( headingLabel, BorderLayout.NORTH ) ;
      add ( pb_REFRESH, BorderLayout.CENTER ) ;

   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    Just like the QuestPanel, the start
    *  method is called once we are showing. We
    *  could use this time to refresh data etc..
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
   }


   /*------------------------------------------------------------*/
   /* getTItle
    */
   /**
    *    This method will return the name of this
    *  NavPanel. The name is displayed above
    *  the nav panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_VIEWS ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    Since a NavPanel can be implemented by
    *  anything, the main window requires the
    *  JComponent that actually represents the NavPanel's GUI
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ; // We are the GUI, so return ourselves!
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when we have
    *  been closed. Cleanup work is done here
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when a GUI
    *  control of ours fires an ActionEvent
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REFRESH                                              */ 
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_REFRESH )
         samplePanel.refresh() ;
      
   }

} /* SampleNavPanel */
