package com.ibm.nzna.projects.qit.sample ;   
/*-----------------------------------------------------------*/
/*   SamplePanel                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SamplePanel                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.sql.* ;
import javax.swing.* ;
import java.awt.event.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;     // MultiList, DButton etc..
import com.ibm.nzna.shared.util.* ;    // LogSystem, Text etc,,
import com.ibm.nzna.shared.db.* ;      // DatabaseSystem, SQLMethod
/**
 * This is a sample QuestPanel that has a NavPanel and a list of information.<br><br
 * This sample shows you how to:<br>
 * 1. Create a basic QuestPanel
 * 2. Create a basic NavPanel
 * 3. Select data from the database
 * 4. Show data in a MultiList
 * 5. Respond to Quest Events
 *
 *
 * @version 1.0, 2/8/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SamplePanel                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class SamplePanel extends JPanel implements Runnable, ActionListener, AppConst, QuestPanel {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String THREAD_REFRESH_DATA    = "REFRESH_DATA" ; 

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Title data. Each string in this array is a title        */
   /* to a column in the MultiList                              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String[] titles = {
      Str.getStr ( STR_INDICATOR ),
      Str.getStr ( STR_TITLE )
   };


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private MultiList       documentList         = null ;
   private ActionButton    pb_SAVE              = null ;
   private ActionButton    pb_CLOSE             = null ;
   private ToolBarButton   pb_PROPERTY          = null ;
   private JToolBar        toolBar              = null ;      
   private SampleNavPanel  navPanel             = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private AppDefaultWin parentWin              = null ;





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method is called first before anything
    *  else. We are not showing at this time so it is
    *  a good idea to create our GUI now
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if we initialize, FALSE if we do not
    *//*---------------------------------------------------------*/
   public boolean initialize () {
      boolean rc = false ;              


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI Controls                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navPanel     = new SampleNavPanel ( this ) ;
      documentList = new MultiList ( GUISystem.getFontUtil() ) ;
      pb_SAVE      = new ActionButton ( Str.getStr ( STR_SAVE ),                          // Button Text
                                        ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),      // Image on the button
                                        Str.getStr ( STR_SAVE ) ) ;                       // Flyover Help
      pb_CLOSE     = new ActionButton ( Str.getStr ( STR_CLOSE ),                         // Button Text
                                        ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),     // Image on the button
                                        Str.getStr ( STR_CLOSE ) ) ;                      // Flyover Help
      pb_PROPERTY  = new ToolBarButton ( Str.getStr ( STR_PROPERTY ),                     // Button Text
                                         ImageSystem.getImageIcon ( this, ImageSystem.PROPERTY ), //Image on the button
                                         Str.getStr ( STR_PROPERTY ) ) ;                  // Fly over help
      toolBar      = new JToolBar () ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.setOpaque ( false ) ;                   // Always make your toolbars see-through
      toolBar.setBorder ( GUISystem.emptyBorder ) ;   // Always make a blank border on your toolbar
      documentList.setColumnHeadings ( titles ) ;     // Tell our MultiList what titles to put where
      documentList.setColumnWidth ( 0, 150 ) ;        // Make column 1 in our list 150 pixels wide
      documentList.setColumnWidth ( 1, 500 ) ;        // Make column 2 in our list 500 pixels wide


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action listeners                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SAVE.addActionListener     ( this ) ; // See how all of the parameters on these
      pb_CLOSE.addActionListener    ( this ) ; // lines all line up. Pretty Sweet!
      pb_PROPERTY.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our toolbar buttons                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.add ( pb_PROPERTY ) ;
      toolBar.add ( new ToolBarSeparator () ) ;  // Always put a separator at the end!


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout () ) ;
      add ( documentList, BorderLayout.CENTER ) ;

      rc = true ;

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are added
    *  to a Window and we are showing.
    *  We use this time to add our toolbars, action buttons, etc..
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get our parent window                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Action Buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;     // Always put Close on first
      parentWin.addActionComponent ( this, pb_SAVE ) ;                        // Always put Save next to close


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put our tool bar up on the window                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      showToolbar() ;

   }





   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when this panel
    *  has once again gained focus on the main window.
    *  This happens when the user clicks off of this panel
    *  and then clicks back on.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {

      /* We are now showing again, show our toolbar buttons  */
      showToolbar() ;
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when our panel
    *  is clicked off of. We will take our toolbar
    *  off
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      parentWin.removeToolbarButton ( toolBar ) ;
   }





   /*------------------------------------------------------------*/
   /* showToolbar
    */
   /**
    *    This method will add and show our toolbar
    *  on our parent AppDefaultWin
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showToolbar () {
      parentWin.addToolbarButton ( toolBar, 2 ) ;
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method should return the name
    *  of this panel that is displayed in the top left
    *  of the main window
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_SAMPLE ) ) ;
   }





   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called when the user
    *  wants to bookmark something on our panel.
    *  We don't support bookmarking here so we'll return FALSE
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean bookmark () {                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up a small message to the user                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.printBox ( "Hey Hey Hey", "This is a sample and doesn't support bookmarks!" ) ;

      return( false ) ;

   }






   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called when the user
    *  has selected the 'Refresh' toolbar button
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      GUISystem.printBox ( "Hey Hey Hey", "You just selected 'Refresh' button! We will now put data in our list" ) ;

      new Thread ( this, THREAD_REFRESH_DATA ).start() ;
   }





   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method should copy text from this
    *  panel. We do nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method should cut text from this
    *  panel. We do nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method should paste text onto this
    *  panel. We do nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    Since QuestPanels can really be made
    *  of anything, we need to return the actual
    *  JComponent that represents our QuestPanel.
    *  However, since our QuestPanel is actually a JPanel,
    *  just return an instance of ourselves
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the user
    *  wants to shut down the app. Return FALSE from
    *  this if it is Not OK to shutdown and shutdown will stop.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {

      return( true ) ; // We return TRUE Since we are Ok to shutdown, however
                       // If you had unsaved data on this window, you could use
                       // this time to save it or something
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the user
    *  wants to close this panel. You can return FALSE
    *  from this method and you are not closed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {

      return( true ) ; // We return TRUE Since we are Ok to close, however
                       // If you had unsaved data on this window, you could use
                       // this time to save it or something
   }






   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default
    *  navigation panel. This NavPanel is shown
    *  on the left side of us. It is Ok to return a NULL. If you
    *  return a NULL, your panel will take the full width of the window
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NavPanel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( navPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method should return a small icon
    *  that represents this panel. This icon is displayed
    *  in the task bar of the tool
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remember that you should always use the                 */
      /* ImageSystem to get images. Also, the ImageSystem.DOCUMENT_SMALL   */
      /* is defined in AppConst.java                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      return( ImageSystem.getImageIcon ( this, 
                                         ImageSystem.DOCUMENT_SMALL ) ) ;
   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an ActionEvent
    *  from our GUI is fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE ) {
         /* Close our panel  */
         parentWin.closePanel ( this, null ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SAVE                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE ) {

         /* Put up a message to the user   */
         GUISystem.printBox ( "Hey Hey Hey", "You pushed the Save! Button!" ) ;

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_PROPERTY                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_PROPERTY ) {

         /* Put up a message again!  */
         GUISystem.printBox ( "Properties", "You just selected to show the properties of this thing!" ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread can perform a variety of operations.
    *  The operation performed is picked from the name
    *  of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH_DATA                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_REFRESH_DATA ) )
         refreshData() ;

   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    If you notice, this method is only called from
    *  the run method. This is because all queries
    *  and anything that takes more than 1 second should be in a thread!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,    // Get a connection to Quest
                                            "refreshData",           // Our method name
                                            5 ) ;                    // How many seconds we might run for

      try {
         Statement stmt       = sqlMethod.createStatement() ;                              // Ask the SQLMethod for a statement
         Vector    dataVector = new Vector () ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Always check that your Statement object                 */
         /* is good. If a connection to the database could not be     */
         /* made then the stmt returned from SQLMethod is NULL        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( stmt != null ) {
            ResultSet rs = null ;       


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Select some data                                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs = stmt.executeQuery ( "SELECT DOCIND, TITLE "+
                                     "FROM TIGRIS.DOCUMENTS "+
                                     "WHERE CHANGEDTIME > '2001-02-04-07.00.00.000000'" ) ;                                    


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, walk down through the results, creating a           */
            /* SampleRec for each item                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( rs.next() ) {
               dataVector.addElement ( new SampleRec ( rs.getInt(1),
                                                       rs.getString(2).trim() ) ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Always close your result set!                           */                    
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs.close() ;                       


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now set the data in the list                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            documentList.removeAll() ;
            documentList.add ( dataVector ) ;


         }


      } catch ( Exception e ) {

         /* Always roll back on any exception  */
         sqlMethod.rollBack() ;

         /* Always Log ALL exceptions  */
         LogSystem.log ( 1,        // Severity 1 log
                         e,        // The Exception to log
                         false ) ; // Do not show the Exception in a Window

         /* Now tell the user something bad just happened  */
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always, no matter what, close your SQLMethod            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;

   }


} /* SamplePanel */
