package com.ibm.nzna.projects.qit.setup ;        
/*-----------------------------------------------------------*/
/*   SelectLangStep                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectLangStep                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import java.util.* ;   
import java.awt.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This step will allow the user to specify their default language 
 *
 *
 * @version 1.0, 1/21/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectLangStep                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectLangStep extends WizardStep implements AppConst, PropertyListener {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private JLabel   st_SELECTLANG  = null ;
    private JList    lb_LANGUAGES   = null ;
    private Vector   langVec        = null ;


    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create our step
     *  
     *
     *  @param message The Message to display to the user
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public SelectLangStep ( ) {                

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add languages to our list                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        langVec = new Vector ( 10, 1 ) ;
        langVec.addElement ( Str.getStr ( STR_ENGLISH ) ) ;
        langVec.addElement ( Str.getStr ( STR_FRENCH ) ) ;
        langVec.addElement ( Str.getStr ( STR_GERMAN ) ) ;          
        langVec.addElement ( Str.getStr ( STR_ITALIAN ) ) ;
        langVec.addElement ( Str.getStr ( STR_SPANISH ) ) ;
        langVec.addElement ( Str.getStr ( STR_JAPANESE ) ) ;
        langVec.addElement ( Str.getStr ( STR_CHINESE ) ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create our controls                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        st_SELECTLANG = new JLabel ( ) ;
        lb_LANGUAGES  = new JList  ( langVec ) ;                            

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust properties                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setText() ;
        lb_LANGUAGES.setBorder ( GUISystem.loweredBorder ) ;
        st_SELECTLANG.setHorizontalAlignment ( SwingConstants.CENTER ) ;
        try {
            lb_LANGUAGES.setSelectedIndex ( langVec.indexOf ( PropertySystem.getString ( PropertySystem.PROPERTY_LANGUAGE ) ) ) ;
        } catch ( Exception e ) {
            lb_LANGUAGES.setSelectedIndex ( 0 ) ;
        }


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add listeners                                           */
        /*                                                           */
        /*-----------------------------------------------------------*/
        PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our controls to the panel                           */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setLayout ( null ) ;
        add ( st_SELECTLANG ) ;
        add ( lb_LANGUAGES ) ;
    }




    /*------------------------------------------------------------*/
    /* doLayout
     */
    /**
     *    This method will position and size all
     *  of our controls
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void doLayout () {
        Dimension size      = getSize() ;
        int       rowHeight = GUISystem.getRowHeight() ;

        st_SELECTLANG.setBounds ( 0, 30, size.width, rowHeight ) ;
        lb_LANGUAGES.setBounds  ( 10, 32+rowHeight, size.width-10, 32+rowHeight+(rowHeight*6) ) ;
    }





    /*------------------------------------------------------------*/
    /* saveInfo
     */
    /**
     *    This method will save the information
     *  and return TRUE if a language was
     *  selected.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean saveInfo () {
        boolean rc = false ;

        try {
            String  language        = (String)lb_LANGUAGES.getSelectedValue() ;
            String  currentLanguage = (String)PropertySystem.getString ( PropertySystem.PROPERTY_LANGUAGE ) ;

            if ( language == null )
                GUISystem.printBox ( STR_ERROR, STR_MUST_SELECT_LANGUAGE ) ;
            else {
                rc = true ;

                if ( language.equals ( Str.getStr ( STR_ENGLISH ) ) ) {
                    PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGE,  "english" ) ;
                } else if ( language.equals ( Str.getStr ( STR_FRENCH ) ) ) {
                    PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGE,  "french" ) ;
                } else if ( language.equals ( Str.getStr ( STR_ITALIAN ) ) ) {
                    PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGE,  "italian" ) ;
                } else if ( language.equals ( Str.getStr ( STR_SPANISH ) ) ) {
                    PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGE,  "spanish" ) ;
                } else if ( language.equals ( Str.getStr ( STR_GERMAN ) ) ) {
                    PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGE,  "german" ) ;
                } else if ( language.equals ( Str.getStr ( STR_JAPANESE ) ) ) {
                    PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGE,  "japanese" ) ;
                } else if ( language.equals ( Str.getStr ( STR_CHINESE ) ) ) {
                    PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGE,  "chinese" ) ;
                }
            }
        } catch ( Exception e ) {
            rc = false ;
            LogSystem.log ( 1, e ) ;
        }

        return( rc ) ;
    }

    public void close () {}




    /*------------------------------------------------------------*/
    /* This method will set the text of the
     */
    /**
     *    panel
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void setText () {
        st_SELECTLANG.setText ( Str.getStr ( STR_SELECT_LANGUAGE ) ) ;
    }


    /*------------------------------------------------------------*/
    /* propertyChanged
     */
    /**
     *    Called when a property we are listening
     *  for has been fired
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void propertyChanged ( int property, Object value ) {
        if ( property == PropertySystem.PROPERTY_LANGUAGE ) {
            setText() ;
        }
    }



} /* SelectLangStep */
