package com.ibm.nzna.projects.qit.setup ;     
/*-----------------------------------------------------------*/
/*   SetupWizard                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SetupWizard                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.projects.qit.app.appProp.* ;
import com.ibm.nzna.projects.qit.app.userProp.* ;
import javax.swing.* ;
import java.awt.* ;
import java.io.* ;
/**
 * The Setup Wizard will walk the user through the steps 
 * of installing the Quest Application.
 *
 * Step 1 - Just a welcome message to the user
 * Step 2 - Asks the user for a temp dir. Default = (curdir)/data
 * Step 3 - Ask the user for their preferred Brand
 * Step 4 - Ask the user for their fav. countries
 * Step 5 - Ask the user for their Skin
 * Step 6 - Internet Setup
 * Step 7 - Ask the user for their word processor
 * Step 8 - Finished Msg.
 *
 *
 * @version 1.0, 1/21/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SetupWizard                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class SetupWizard extends JDialog implements AppConst, WizardListener, PropertyListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Wizard               wizardPanel          = null ;
   private boolean              rc                   = false ;
   private InfoStep             welcomeStep          = null ;
   private TempDirStep          tempDirStep          = null ;
   private BrandFamilyPage      brandFamilyStep      = null ;
   private FavoriteCountryPage  favoriteCountryStep  = null ;
   private SkinsPage            skinsStep            = null ;
   private InfoStep             finishStep           = null ;
   private InternetPage         internetStep         = null ;
   private WordProcPage         wordProcStep         = null ;
   private Frame                parentFrame          = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  all of our controls. It will not show the dialog.
    *  Please use the 'getResult' method to show and process the dialog
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SetupWizard () {
      super ( new Frame(), "", true ) ;          

      parentFrame = (Frame) getParent() ;
      CreateWizardWin statusWin = new CreateWizardWin ( parentFrame, 9 ) ;

      WinUtil.centerWindow ( statusWin ) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Wizard control                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         wizardPanel = new Wizard ( ImageSystem.getImageIcon ( this, "skins"+
                                                               File.separator+
                                                               PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                                                               File.separator+
                                                               "setup_wizard.gif" ) ) ;                     
         statusWin.setValue ( 1 ) ;
      } catch ( Exception e ) {
         wizardPanel = new Wizard () ;
      }




      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our steps                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      welcomeStep = new InfoStep ( "", Str.getStr ( STR_SETUP_TITLE ) ) ;                      
      statusWin.setValue ( 2 ) ;
      tempDirStep = new TempDirStep() ;
      statusWin.setValue ( 3 ) ;
      brandFamilyStep = new BrandFamilyPage() ;
      statusWin.setValue ( 4 ) ;
      favoriteCountryStep = new FavoriteCountryPage () ;
      statusWin.setValue ( 5 ) ;
      skinsStep = new SkinsPage () ;
      statusWin.setValue ( 6 ) ;
      internetStep = new InternetPage () ;
      statusWin.setValue ( 7 ) ;
      wordProcStep = new WordProcPage() ;
      statusWin.setValue ( 8 ) ;
      finishStep = new InfoStep ( "", Str.getStr ( STR_WIZARD_FINISH ) ) ;
      statusWin.setValue ( 9 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties/Add Listeners                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;
      wizardPanel.addWizardListener ( this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our window                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( new BorderLayout () ) ;
      getContentPane().add ( wizardPanel ) ;
      statusWin.setValue ( 6 ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      setSize ( 495, 375 ) ;
      statusWin.dispose() ;
      WinUtil.centerWindow ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will return the results
    *  from the user processing this wizard. This
    *  method will show the wizard and will not return unless Finish or Cancel were pressed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Finish was pressed, FALSE = Cancel was pressed
    *//*---------------------------------------------------------*/
   public boolean getResult () {

      wizardPanel.setCurrentStep ( welcomeStep ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;

      setVisible ( true ) ;
      parentFrame.dispose() ;
      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* cancelPerformed
    */
   /**
    *    This method will be called if the user
    *  hit the 'Cancel' button on the Wizard
    *  
    *
    *  @param e The wizard event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cancelPerformed ( WizardEvent e ) {
      rc = false ;
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* finishPerformed
    */
   /**
    *    This method will be called when the
    *  user presses the finish button
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finishPerformed ( WizardEvent e) {
      rc = true ;
      PropertySystem.putBool ( PropertySystem.PROPERTY_APPLICATIONSETUP, true ) ;
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* nextPerformed
    */
   /**
    *    This method will be called when the
    *  next button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void nextPerformed ( WizardEvent e ) {
      WizardStep step = wizardPanel.getCurrentStep() ;


      wizardPanel.setVisible ( false ) ;
      if ( step == welcomeStep )
         wizardPanel.setCurrentStep ( tempDirStep ) ;
      else if ( step == tempDirStep )
         wizardPanel.setCurrentStep ( brandFamilyStep ) ;
      else if ( step == brandFamilyStep )
         wizardPanel.setCurrentStep ( favoriteCountryStep ) ;
      else if ( step == favoriteCountryStep )
         wizardPanel.setCurrentStep ( skinsStep ) ;
      else if ( step == skinsStep )
         wizardPanel.setCurrentStep ( internetStep ) ;
      else if ( step == internetStep )
         wizardPanel.setCurrentStep ( wordProcStep ) ;
      else if ( step == wordProcStep )
         wizardPanel.setCurrentStep ( finishStep ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      wizardPanel.setVisible ( true ) ;
      adjustButtons () ;
   }


   /*------------------------------------------------------------*/
   /* backPerformed
    */
   /**
    *    This method will be called when the
    *  back button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void backPerformed ( WizardEvent e ) {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == tempDirStep )
         wizardPanel.setCurrentStep ( welcomeStep ) ;
      else if ( step == brandFamilyStep )
         wizardPanel.setCurrentStep ( tempDirStep ) ;
      else if ( step == favoriteCountryStep )
         wizardPanel.setCurrentStep ( brandFamilyStep ) ;
      else if ( step == skinsStep )
         wizardPanel.setCurrentStep ( favoriteCountryStep ) ;
      else if ( step == internetStep )
         wizardPanel.setCurrentStep ( skinsStep ) ;
      else if ( step == wordProcStep )
         wizardPanel.setCurrentStep ( internetStep ) ;
      else if ( step == finishStep )
         wizardPanel.setCurrentStep ( wordProcStep ) ;

      adjustButtons ( ) ;
   }





   /*------------------------------------------------------------*/
   /* adjustButtons
    */
   /**
    *    This method will look at what step is
    *  currently showing and will decide which
    *  buttons to show in the wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void adjustButtons () {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == welcomeStep )
         wizardPanel.showButtons ( true, false, true, false ) ;
      else if ( step == finishStep )
         wizardPanel.showButtons ( false, true, false, true ) ;
      else
         wizardPanel.showButtons ( true, true, true, false ) ;
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    Called when a property we are listening
    *  for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE ) {
         setText() ;
      }
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on all
    *  buttons and InfoStep panels of the 
    *  wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      wizardPanel.setNextText   ( Str.getStr ( STR_NEXT ) ) ;
      wizardPanel.setBackText   ( Str.getStr ( STR_BACK ) ) ;
      wizardPanel.setFinishText ( Str.getStr ( STR_FINISH ) ) ;
      wizardPanel.setCancelText ( Str.getStr ( STR_CANCEL ) ) ;
      setTitle ( Str.getStr ( STR_SETUP ) ) ;
      welcomeStep.setMessage ( Str.getStr ( STR_SETUP_WELCOME ) ) ;                      
      finishStep.setMessage ( Str.getStr ( STR_SETUP_COMPLETE ) ) ;
   }

} /* SetupWizard */
