package com.ibm.nzna.projects.qit.setup ;        
/*-----------------------------------------------------------*/
/*   TempDirStep                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TempDirStep                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import java.util.* ;   
import java.io.* ;
import java.awt.* ;
/**
 * This step will allow the user to specify their default language 
 *
 *
 * @version 1.0, 1/21/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TempDirStep                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class TempDirStep extends WizardStep implements Runnable, AppConst, PropertyListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel     st_ENTERTEMPDIR = null ;
   private JTextField ef_TEMPDIR      = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our step
    *  
    *
    *  @param message The Message to display to the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TempDirStep( ) {                          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_ENTERTEMPDIR = new JLabel ( "" ) ;
      ef_TEMPDIR      = new JTextField ( "" ) ;               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_ENTERTEMPDIR.setVerticalAlignment ( SwingConstants.TOP ) ;
      setText() ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_ENTERTEMPDIR ) ;
      add ( ef_TEMPDIR ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh the data in
    *  the temp directory entryfield
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshInfo () {
      new Thread ( this ).start() ;
   }



   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will position and size all
    *  of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      st_ENTERTEMPDIR.setBounds ( 0, 0, size.width, rowHeight*5 ) ;
      ef_TEMPDIR.setBounds ( 10, rowHeight*6, size.width-20, rowHeight ) ;
   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save the information
    *  and return TRUE if a language was
    *  selected.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      boolean rc          = false ;
      String  tempDir     = ef_TEMPDIR.getText() ;
      File    tempDirFile = new File ( tempDir ) ;

      if ( !tempDirFile.exists() ) {
         if ( GUISystem.printBox ( STR_QUESTION, STR_DIRECTORY_DOES_NOT_EXIST_CREATE ) ) {
            tempDirFile.mkdirs() ;
            rc=true ;
         }
      } else
         rc = true ;

      if ( rc )
         PropertySystem.put ( PropertySystem.PROPERTY_TEMP_DIR, tempDir ) ;

      return( rc ) ;
   }

   public void close () {}





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    Called when a property we are listening
    *  for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE ) {
         setText() ;
      }
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on all of
    *  our labels
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      st_ENTERTEMPDIR.setText ( Str.getStr ( STR_ENTER_TEMP_DIR ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method returns our title
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_SETUP_TEMP_DIR_TITLE ) ) ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will refresh our information and
    *  give focus to the entry field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      Properties p       = System.getProperties () ;
      String     homeDir = p.getProperty ( "user.dir" ) ;

      ef_TEMPDIR.setText ( homeDir+File.separator+"temp_data" ) ;
      ef_TEMPDIR.requestFocus() ;
      invalidate() ;
   }
} /* TempDirStep*/
