package com.ibm.nzna.projects.qit.storedProc.logonUser ;   
/*-----------------------------------------------------------*/
/*   LogonUser                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LogonUser                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.BrandMgrRec ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will log on a user for the Quest Application. 
 *
 * 
 * Main Execution Point: logonUser
 *
 * Param Number   Descript
 * -------------- -------------------------------------------
 * 1              userId - The User Id to Log on
 * 2              password - The Encrypted password to check for
 * 3              StoredProcRec ( Allocated and returned by LogonUser )
 *
 * 
 *
 * @version 1.0, 2/23/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LogonUser                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class LogonUser extends StoredProc {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private StoredProcRec returnRec         = new StoredProcRec () ;
    private LogonUserRec  logonUserRec      = null ;
    private Connection    con               = null ;
    private String        userId            = null ;
    private String        password          = null ;


    /*------------------------------------------------------------*/
    /* logonUser
     */
    /**
     *    This method will log on the user. This method
     *  will be called by DB/2 when this stored
     *  procedure is invoked
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void logonUser ( String userId, String password, Blob retLogonUserRec, Blob storedProcRetRec ) {

        try {
            try {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Get our connection to DB/2                              */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( con == null )
                    con = getConnection() ;

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Initialize our Logon User Rec                           */
                /*                                                           */
                /*-----------------------------------------------------------*/
                logonUserRec = new LogonUserRec () ;
                this.userId   = userId ;
                this.password = password ;      

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Log the User in                                         */
                /*                                                           */
                /*-----------------------------------------------------------*/
                logonUser () ;                 

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Write out our LogonUserRec                              */
                /*                                                           */
                /*-----------------------------------------------------------*/
                retLogonUserRec = Lob.newBlob() ;
                StoredProcUtil.setObjectInBlob ( retLogonUserRec, logonUserRec );
                set ( 3, retLogonUserRec ) ;


            } catch ( Exception e ) {
                StringWriter    sw      = new StringWriter () ;
                PrintWriter     pw      = new PrintWriter ( sw ) ;

                /* Get the stackTrace */
                e.printStackTrace ( pw ) ;

                String retStr = sw.toString() ;

                StoredProcUtil.writeException ( e, returnRec ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Commit                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            con.commit() ;
            con.close() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Write out our return information                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            storedProcRetRec = Lob.newBlob() ;
            StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

            set ( 4, storedProcRetRec ) ;
        } catch ( Throwable e ) {
            StringWriter    sw      = new StringWriter () ;
            PrintWriter     pw      = new PrintWriter ( sw ) ;

            /* Get the stackTrace */
            e.printStackTrace ( pw ) ;

            String retStr = sw.toString() ;
            FileUtil.saveFile ( "UserLogon.err", retStr ) ;

        }
    }






    /*------------------------------------------------------------*/
    /* logonUser
     */
    /**
     *    This method will actually log the user
     *  on. It will check the password and id and
     *  will update the 'Attemps' field accordingly
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void logonUser () throws Exception {
        Statement stmt = con.createStatement() ;
        String    sql  = null ;
        ResultSet rs   = null ;


        sql = "SELECT A.PASSWORD, A.NAME, A.ATTEMPTS, B.LANGUAGEIND "+
              "FROM TIGRIS.USERID A, "+
              "     TIGRIS.USERLANGUAGE B "+
              "WHERE A.USERID = '"+userId+"' AND "+
              "      A.USERID = B.USERID "+
              "FOR FETCH ONLY" ;

        rs = stmt.executeQuery ( sql ) ;

        if ( rs.next() ) {
            String dbPassword = rs.getString(1).trim() ;
            int    attempts   = rs.getInt(3) ;                                      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If attempts < 5 continue, otherwise throw error         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( attempts < 5 ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Compare passwords                                       */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( dbPassword.equals ( password ) ) {


                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Ok, everything is good, now copy our info over          */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    logonUserRec.userId = userId ;
                    logonUserRec.name = rs.getString(2).trim() ;
                    logonUserRec.language = rs.getInt(4) ;

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Read in the rest of the information for this user       */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    readAuthorities ( logonUserRec ) ;
                    readTeams       ( logonUserRec ) ;
                    readCountry     ( logonUserRec ) ;
                    readBrands      ( logonUserRec ) ;


                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   If the attempts is > 0, set them back to 0              */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( attempts > 0 ) {
                        setAttempts ( 0 ) ;
                    }

                }

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   The password was wrong, return the error and update     */
                /* the amount of attempts to login                           */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else {
                    logonUserRec.errorCode = LogonUserRec.ERROR_PASSWORD_WRONG ;
                    setAttempts ( attempts+1 ) ;
                }


            } else
                logonUserRec.errorCode = LogonUserRec.ERROR_TOO_MANY_ATTEMPTS ;

        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Otherwise the user has not been found                   */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else
            logonUserRec.errorCode = LogonUserRec.ERROR_USER_NOT_FOUND ;


        rs.close() ;
        stmt.close() ;
    }





    /*------------------------------------------------------------*/
    /* setAttempts
     */
    /**
     *    This method will update the 'ATTEMPTS'
     *  column in 'TIGRIS.USERID' to the amount
     *  of attempts passed in
     *
     *  @param attempts The new amount of attempts
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void setAttempts ( int attempts ) throws Exception {
        Statement stmt = con.createStatement() ;

        stmt.executeUpdate ( "UPDATE TIGRIS.USERID "+
                             "SET ATTEMPTS = "+attempts+" "+
                             "WHERE USERID = '"+userId+"'" ) ;

        stmt.close() ;
    }





    /*------------------------------------------------------------*/
    /* readAuthorities
     */
    /**
     *    This method will read in all of the authories
     *  granted to this user in all of the teams
     *  that the user belongs to
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void readAuthorities ( LogonUserRec userRec ) throws Exception {
        Statement stmt    = con.createStatement() ;
        ResultSet rs      = null ;
        String    sql     = null ;
        Vector    readVec = new Vector ( 1, 1 ) ;
        int       i       = 0 ;     

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Set up our SQL                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        sql = "SELECT A.AUTHIND "+
              "FROM TIGRIS.USERGROUPAUTH A,                   "+
              "     TIGRIS.USERIDUSERGROUP B                  "+
              "WHERE B.USERID = '"+userId+"' AND "+
              "      A.USERGROUP = B.USERGROUP                "+
              "FOR FETCH ONLY                                 " ;      


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Execute the SQL                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        rs = stmt.executeQuery ( sql ) ;                       


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Loop through and read our values                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        userRec.auths = new Vector ( 1 ) ;
        while ( rs.next() )
            userRec.auths.add ( new Integer ( rs.getInt(1) ) ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Close our cursor and exit                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        rs.close() ;
        stmt.close() ;
    }


    /*------------------------------------------------------------*/
    /* readTeams
     */
    /**
     *    This method will read in all of the teams
     *  granted to this user 
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void readTeams ( LogonUserRec userRec ) throws Exception {
        Statement stmt    = con.createStatement() ;
        ResultSet rs      = null ;
        String    sql     = null ;
        Vector    readVec = new Vector ( 1, 1 ) ;
        int       i       = 0 ;     

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Set up our SQL                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        sql = "SELECT USERGROUP                         "+
              "FROM TIGRIS.USERIDUSERGROUP              "+
              "WHERE USERID = '"+userId+"' "+
              "FOR FETCH ONLY                           " ;      


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Execute the SQL                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        rs = stmt.executeQuery ( sql ) ;                       


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Loop through and read our values                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        userRec.teams = new Vector ( 1 ) ;
        while ( rs.next() )
            userRec.teams.addElement ( new Integer ( rs.getInt(1) ) ) ;                


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Close our cursor and exit                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        rs.close() ;
        stmt.close() ;
    }


    /*------------------------------------------------------------*/
    /* readCountry
     */
    /**
     *    This method will read in all of the user's
     *  countries.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void readCountry ( LogonUserRec userRec ) throws Exception {
        Statement stmt = con.createStatement() ;
        String    sql  = null ;
        ResultSet rs   = null ;

        sql = "SELECT COUNTRYCODEIND FROM TIGRIS.USERIDCOUNTRY WHERE USERID = '"+userRec.userId+"'" ;
        rs  = stmt.executeQuery ( sql ) ;

        userRec.countries = new Vector ( 3 ) ;
        while ( rs.next() )
            userRec.countries.addElement ( new Integer ( rs.getInt(1) ) ) ;

        rs.close() ;
        stmt.close() ;
    }


    /*------------------------------------------------------------*/
    /* readBrands
     */
    /**
     *    This method will read in all of the user's
     *  brands.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void readBrands ( LogonUserRec userRec ) throws Exception {
        Statement stmt = con.createStatement() ;
        String    sql  = null ;
        ResultSet rs   = null ;

        sql = "SELECT BRANDIND, TYPEBRANDMGR FROM TIGRIS.USERIDBRAND WHERE USERID = '"+userRec.userId+"'" ;
        rs  = stmt.executeQuery ( sql ) ;

        userRec.brands = new Vector ( 1 ) ;
        while ( rs.next() )
            userRec.brands.addElement ( new BrandMgrRec ( rs.getString(2).trim(), rs.getInt(1) ) );

        rs.close() ;
        stmt.close() ;
    }


} /* LogonUser */
