package com.ibm.nzna.projects.qit.storedProc.sqlRunner ; 
/*-----------------------------------------------------------*/
/*   QITSqlRunner                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QITSqlRunner                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class subclasses the SqlRunner Stored Procedure to provide a
 * convient method to call the stored procedure.
 *
 * @version 1.0, 9/15/1999 
 * @author  David Lentz
 */
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import java.sql.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   SqlRunner                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class QITSqlRunner extends SqlRunner implements AppConst {

   /*------------------------------------------------------------*/
   /* sqlRunner
    */
   /**
    *    This method will take a string and sends it to
    *  the sqlRunner stored procedure.
    *  
    *
    *  @param String the String to send
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean sqlRunner ( String sqlRunnerString ) {
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.QUEST, "sqlRunner", 5 ) ;
      int       sqlCode = 0 ;

      Qit.setEnabled ( false ) ;

      try {
         CallableStatement stmt        = sql.createStoredProc ( "QUEST.sqlRunner", 2 ) ;
         String            debugString = Text.replaceAllStrInStr ( sqlRunnerString, END_DELIM, "\n" ) ;

         LogSystem.log ( 1, "Starting to send statements to EXT SQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the Callable Stmt                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlRunnerString.getBytes() ) ;      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute the SQL                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log( 2, "Running SQLRunner:" ) ;
         LogSystem.log( 2, Text.replaceAllStrInStr ( sqlRunnerString, END_DELIM, "\n    " ) ) ;
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
            sqlCode = retRec.sqlCode ;
         }

         stmt.close() ;

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e ) ;
         sqlCode=-1 ;
      }

      sql.close() ;
      Qit.setEnabled ( true ) ;

      return( sqlCode == 0 ) ;
   }

   public static int sqlRunnerCode ( String sqlRunnerString ) {
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.QUEST, "sqlRunner", 5 ) ;
      int       sqlCode = 0 ;

      Qit.setEnabled ( false ) ;

      try {
         CallableStatement stmt        = sql.createStoredProc ( "QUEST.sqlRunner", 2 ) ;
         String            debugString = Text.replaceAllStrInStr ( sqlRunnerString, END_DELIM, "\n" ) ;

         LogSystem.log ( 1, "Starting to send statements to EXT SQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the Callable Stmt                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlRunnerString.getBytes() ) ;      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute the SQL                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log( 2, "Running SQLRunner:" ) ;
         LogSystem.log( 2, Text.replaceAllStrInStr ( sqlRunnerString, END_DELIM, "\n    " ) ) ;
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            sqlCode = retRec.sqlCode ;
         }

         stmt.close() ;

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         sqlCode=-1 ;
      }

      sql.close() ;
      Qit.setEnabled ( true ) ;

      return( sqlCode ) ;
   }


} /* SqlRunner */
