package com.ibm.nzna.projects.qit.type ;
/*-----------------------------------------------------------*/
/*   TypeList                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeList                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * A collection of static methods that allow easy manipulation of TypeLists 
 *
 *
 * @version 1.0, 3/5/1998 
 * @author  David Lentz
 */
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
import java.awt.* ;
import java.sql.* ;
import javax.swing.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeList                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class QITTypeList extends TypeList implements java.io.Serializable, AppConst, PropertyListener {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static int     languageInd = 0 ;
   private static boolean initialized = false ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Simply adds us as a property listener
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QITTypeList () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   We most instantiate this class only once                */                               
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !initialized ) {

         setCacheDir ( CacheSystem.getCacheDir() ) ;
         super.initialize ( PropertySystem.getBool ( PropertySystem.PROPERTY_CACHE_TYPE_LISTS ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         languageInd = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;

         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
         setInstance ( (TypeList)this ) ;
         initialized = true ;

      }
   }

   public static TypeList getInstance () {
      return( instance ) ;
   }


   /*-----------------------------------------------------------*/
   /* readStartupLists
    */
   /**
    *    This method will read all of the TypeLists we
    *  read when the application initializes
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean readStartupLists () {
      boolean rc = false ;

      Qit.setEnabled ( false ) ;
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( Str.getStr ( STR_READING_STARTUP_TYPE_LISTS ) ) ;

      rc = readStartupLists ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ) ;

      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      Qit.setEnabled ( true ) ;
      return( rc ) ;
   }


   /*-----------------------------------------------------------*/
   /* readTypeList
    */
   /**
    *    This method will read the specified TypeList.
    *  
    *  
    *
    *  @param lock = Lock the application while the list is read
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void readTypeList ( TypeListRec typeListRec, boolean lock ) {
      if ( lock )
         Qit.setEnabled ( false ) ;
      MainWindow.getInstance().setStatus ( Str.getStr ( STR_READING )+" "+typeListRec.toString() ) ;

      readTypeList ( typeListRec ) ;

      if ( lock )
         Qit.setEnabled ( true ) ;
      MainWindow.getInstance().setStatus ( null ) ;
   }



   /*------------------------------------------------------------*/
   /* PropertyChanged
    */
   /**
    *    THis method will be called when a property
    *  value we are listening for has changed
    *  
    *
    *  @param propertyInd The property that changed
    *  @param property The new value
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int propertyInd, Object property ) {                                            

      try {

         if ( languageInd != PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ) {
            Vector typeLists = getTypeListVec() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the language changes, we need to re-read our lists   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            int i = 0 ;
            int size = typeLists.size() ;

            while ( i < size ) {
               ((TypeListRec)typeLists.elementAt(i)).invalidate() ;
               i++ ;
            }

            languageInd = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
            setLanguage ( languageInd ) ;
            readStartupLists() ;
         }
      } catch ( Exception e ) {
      }

   }
}
