package com.ibm.nzna.projects.qit.web ;                
/*-----------------------------------------------------------*/
/*   WebBrowserNavPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WebBrowserNavPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import java.awt.* ;       
import java.awt.event.* ;       
/**
 * This is the navigation panel for the Web Browser 
 *
 *
 * @version 1.0, 2/3/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WebBrowserNavPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class WebBrowserNavPanel extends JPanel implements AppConst, NavPanel, TreeSelectionListener, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private NavList         navList         = null ;
   private WebBrowserPanel webPanel        = null ;
   private BookmarkTree    bookmarkTree    = null ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will simply assign local
    *  variables. No initialization will be done. The
    *  initialization is done in the initilize method
    *  which will be called by the PanelSystem
    *
    *  @param webPanel The panel that we are tied to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WebBrowserNavPanel ( WebBrowserPanel webPanel ) {
      this.webPanel = webPanel ;
   }



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel. This
    *  is done before anything is shown to the
    *  user, so it is a good time to create GUIs
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return nothing
    *//*---------------------------------------------------------*/
   public void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navList = new NavList () ;
      navList.add ( bookmarkTree = new BookmarkTree() ) ;
      setText() ;        // This call will create all of our text based GUI

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      navList.setOpaque ( false ) ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      bookmarkTree.addTreeSelectionListener ( this ) ;




      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( new JScrollPane ( navList ), BorderLayout.CENTER ) ;
   }



   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we
    *  have been initialized and are now showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when we
    *  are needed to cease operations. Clean
    *  up time!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
   }



   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title of this
    *  navigation panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_WEB_NAV ) ) ;
   }



   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of
    *  this class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set our text in all of our GUI Components
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one
    *  of our labels are hit
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() instanceof HotLinkLabel ) {
         String text = ((HotLinkLabel)e.getSource()).getText() ;

         if ( text.equals ( Str.getStr ( STR_ESUPPORT ) ) )
            webPanel.gotoURL ( "http://www.pc.ibm.com/support" ) ;
         else if ( text.equals ( Str.getStr ( STR_INFOTIPS ) ) )
            webPanel.gotoURL ( "http://w3.pc.ibm.com/helpcenter/infotips" ) ;
         else if ( text.equals ( Str.getStr ( STR_BUSINESS_PARTNER ) ) )
            webPanel.gotoURL ( "http://www.pc.ibm.com/ipartner/infotips" ) ;
         else if ( text.equals ( Str.getStr ( STR_INTRANET ) ) )
            webPanel.gotoURL ( "http://w3.ibm.com" ) ;
         else if ( text.equals ( Str.getStr ( STR_CALLUP ) ) )
            webPanel.gotoURL ( "http://w3.ibm.com" ) ;
         else if ( text.equals ( Str.getStr ( STR_YAHOO ) ) )
            webPanel.gotoURL ( "http://www.yahoo.com" ) ;
         else if ( text.equals ( Str.getStr ( STR_LYCOS ) ) )
            webPanel.gotoURL ( "http://www.lycos.com" ) ;
         else if ( text.equals ( Str.getStr ( STR_HOTBOT ) ) )
            webPanel.gotoURL ( "http://www.hotbot.com" ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when the user
    *  hits one of the hot links
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( TreeSelectionEvent e ) {
      try {
         StringTreeNode selectedObject = (StringTreeNode)bookmarkTree.getSelectionPath().getLastPathComponent() ;                                         

         if ( selectedObject != null ) {
            Bookmark bookmark = (Bookmark) selectedObject.getData() ;

            if ( bookmark.getObject() instanceof String )
               webPanel.gotoURL ( (String) bookmark.getObject() ) ;
         }
      } catch ( Exception ex ) {
      }
   }

} /* WebBrowserNavPanel */
