package com.ibm.nzna.projects.qit.web ;          
/*-----------------------------------------------------------*/
/*   WebBrowserPanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WebBrowserPanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;    
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;

/* The following imports are only for the ability to edit Docs from the web,
   delete these if docs are no longer supported  */
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.docmaint.* ;
/**
 * This panel holds the content and navigation for the web browser 
 *
 *
 * @version 1.0, 2/1/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WebBrowserPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class WebBrowserPanel extends JPanel implements BrowserListener, Authority, QuestPanel, AppConst, ActionListener, DocConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Browser            browser     = null ;
   private JTextField         ef_URL      = null ;
   private DButton            pb_GO       = null ;
   private WebBrowserNavPanel navPanel    = null ;
   private AppDefaultWin      parentWin   = null ;
   private String             prevStatus  = null ;
   private JToolBar           toolBar     = null ;
   private ToolBarButton      pb_BACK     = null ;
   private ToolBarButton      pb_FORWARD  = null ;
   private ToolBarButton      pb_HOME     = null ;
   private ActionButton       pb_EDIT     = null ;
   private ActionButton       pb_CLOSE    = null ;                                        



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   The following variables are only for editing Docs       */
   /* from the web, delete these if docs are no longer here     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean      lastSearchDidRequests = false ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will do nothing, like all
    *  QuestPanels should do
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WebBrowserPanel () {
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel, 
    *  including any variables and GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything went Ok
    *//*---------------------------------------------------------*/
   public boolean initialize () {
      JLabel st_URL   = null ;
      JPanel urlPanel = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE   = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_EDIT    = new ActionButton ( Str.getStr ( STR_EDIT ),  ImageSystem.getImageIcon ( this, ImageSystem.EDIT ),  Str.getStr ( STR_EDIT ) ) ;
      browser    = GUISystem.createBrowser() ;
      ef_URL     = new JTextField () ;           
      pb_GO      = new DButton ( Str.getStr ( STR_GO ) ) ;
      urlPanel   = new JPanel () ;
      st_URL     = new JLabel ( Str.getStr ( STR_URL ) ) ;
      navPanel   = new WebBrowserNavPanel ( this ) ;
      toolBar    = new JToolBar() ;
      pb_BACK    = new ToolBarButton ( Str.getStr ( STR_BACK ),    ImageSystem.getImageIcon ( this, ImageSystem.BACK ),    Str.getStr ( STR_BACK_HELP ) ) ;
      pb_FORWARD = new ToolBarButton ( Str.getStr ( STR_FORWARD ), ImageSystem.getImageIcon ( this, ImageSystem.FORWARD ), Str.getStr ( STR_FORWARD_HELP ) ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_URL.addActionListener     ( this ) ;
      pb_GO.addActionListener      ( this ) ;        
      browser.addBrowserListener   ( this ) ;
      pb_BACK.addActionListener    ( this ) ;
      pb_FORWARD.addActionListener ( this ) ;
      pb_CLOSE.addActionListener   ( this ) ;
      pb_EDIT.addActionListener    ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.setBorderPainted ( false ) ;
      toolBar.setOpaque ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout(4,4)  ) ;
      urlPanel.setLayout ( new BorderLayout ( 4, 4 ) ) ;
      add ( urlPanel, BorderLayout.NORTH ) ;
      urlPanel.add ( ef_URL,  BorderLayout.CENTER ) ;
      urlPanel.add ( st_URL,  BorderLayout.WEST ) ;
      urlPanel.add ( pb_GO,   BorderLayout.EAST ) ;
      add ( browser.getComponent(),  BorderLayout.CENTER ) ;

      toolBar.add ( pb_BACK, 0 ) ;
      toolBar.add ( pb_FORWARD, 1 ) ;
      toolBar.add ( new ToolBarSeparator (), 2 ) ;

      return( true ) ;
   }






   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will copy text from the
    *  browser
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will just call the
    *  copy
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* paste
    */
   /**
    *    This method will attempt to paste
    *  information into our URL field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      ef_URL.setText ( "Pasted!" ) ;
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh our
    *  page
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      browser.reload() ;
   }






   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance
    *  of ourselves
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }



   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_WEBBROWSER ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return the default
    *  navigation panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( navPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when we
    *  need to shutdown. We will simply return
    *  TRUE here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will stop and free our
    *  resources
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      removeAll() ;
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
      browser = null ;
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* gotoURL
    */
   /**
    *    This method will load the specified URL
    *  
    *
    *  @param url The URL to load
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void gotoURL ( String url ) {
      ef_URL.setText ( url ) ;
      loadURL() ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will start our panel. The
    *  initialize method should have taken care
    *  of everything, but this method will set the cursor focus
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      parentWin = GUISystem.getParentDefWin ( this ) ;

      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_EDIT ) ;

      addToolBarButtons() ;

      ef_URL.requestFocus () ;              
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name
    *  of this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_WEBBROWSER ) ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an
    *  action event has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_GO                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == ef_URL ) || ( e.getSource() == pb_GO ) )
         gotoURL ( ef_URL.getText() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_BACK                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_BACK )
         browser.back() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_FORWARD                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_FORWARD )
         browser.forward() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin ( this ).closePanel ( this, null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT )
         editPage() ;




   }







   /*------------------------------------------------------------*/
   /* fixURL
    */
   /**
    *    This method will attempt to fix URLS
    *  by placing 'http' and 'www' in front
    *  of bad looking URLs. Very simple. If they want more,
    *  we will give it to them, but this is out of the kindness
    *  of my heart.
    *
    *  @param url The URL to fix
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String fixURL ( String url ) {

      return( url ) ;
      //if ( url.indexOf ( "/" ) == -1 ) {
      //    if ( url.indexOf ( ".com" ) == -1 )
      //        url=url+".com" ;
      //}

      //if ( url.startsWith ( "w3" ) ) {
      //    url = "http://"+url ;
      //}

      //if ( ( !url.startsWith ( "www." ) ) &&
      //     ( !url.startsWith ( "ftp." ) ) )
      //    url = "www."+url ;

      //if ( url.startsWith ( "ftp." ) )
      //    url = "ftp://"+url ;
      //if ( url.startsWith ( "www." ) )
      //    url = "http://"+url ;

      //return( url ) ;
   }




   /*------------------------------------------------------------*/
   /* loadURL
    */
   /**
    *    This method will load whatever URL
    *  is displayed in the URL Location Field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void loadURL () {
      browser.loadURL ( ef_URL.getText() ) ;
   }





   /*------------------------------------------------------------*/
   /* pageStarted
    */
   /**
    *    This method will be called once a page
    *  has begun to load. We will display information
    *  to the user here
    *
    *  @param url The url that is loading
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void pageStarted ( String url ) {
      ef_URL.setText ( url ) ;
      parentWin.setStatus ( Str.getStr ( STR_LOADING )+" "+url ) ;
      pb_BACK.setEnabled ( browser.canGoBack() ) ;
      pb_FORWARD.setEnabled ( browser.canGoForward() ) ;
   }





   /*------------------------------------------------------------*/
   /* pageStopped
    */
   /**
    *    This method will be called when a page
    *  has stopped loading
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void pageStopped ( ) {
      if ( parentWin != null )
         parentWin.setStatus ( prevStatus ) ;
      pb_BACK.setEnabled ( browser.canGoBack() ) ;
      pb_FORWARD.setEnabled ( browser.canGoForward() ) ;
   }




   /*------------------------------------------------------------*/
   /* mouseLinkExit
    */
   /**
    *    This method will be called when a mouse
    *  has left a link in the page
    *  
    *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseLinkExit () {
      if ( parentWin != null )
         parentWin.setStatus ( prevStatus, false ) ;
      prevStatus = null ;
   }





   /*------------------------------------------------------------*/
   /* mouseOverLink
    */
   /**
    *    This method will be called when the mouse
    *  has moved over a link
    *  
    *
    *  @param url The URL link that the mouse is over
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseOverLink ( String url ) {
      prevStatus = parentWin.getStatus() ;
      parentWin.setStatus ( url ) ;
   }



   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the
    *  user has selected our panel from a
    *  tabbed pane. If we are in MDI mode, this method is never called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
      addToolBarButtons() ;
   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel onto another. This method
    *  will never be called in the MDI mode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
   }





   /*------------------------------------------------------------*/
   /* addToolBarButtons
    */
   /**
    *    This method will add all of our specific
    *  toolbar buttons
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addToolBarButtons () {
      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 0 ) ;

   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      String url = ef_URL.getText() ;

      if ( ( url != null ) && ( url.length() > 0 ) ) {
         Bookmark bookmark = new Bookmark() ;

         bookmark.setObject ( url, BOOKMARK_URL );
         bookmark.updateRecStatus ( bookmark.REC_INSERT ) ;
         new AddBookmarkDlg ( GUISystem.getParentDefWin ( this ), bookmark ) ;
      }
      return( true ) ;
   }

   /*------------------------------------------------------------*/
   /* editPage
    */
   /**
    *    This method will edit the current document
    *  in the page
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void editPage () {
      String currentUrl = ef_URL.getText() ;

      if ( ( UserSystem.hasAuthority ( AUTH_DOCMAINT ) ) &&
           ( ( UserSystem.hasAuthority ( AUTH_REQUESTOR ) ) ||
             ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) ) ) {
         try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Don't edit if this is not an IBM Site                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( currentUrl.indexOf ( "pc.ibm.com" ) >= 0 ) ||
                 ( currentUrl.indexOf ( "w3.ibm.com" ) >= 0 ) ) {
               int    htmlFileIndex    = currentUrl.indexOf ( ".html" )+5 ;
               int    htmlEndFileIndex = htmlFileIndex ;
               String htmlFile         = null ;          

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Pull out the Doc Id                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               htmlFileIndex = currentUrl.lastIndexOf ( "/", htmlEndFileIndex )+1 ;
               htmlFile = currentUrl.substring ( htmlFileIndex, htmlEndFileIndex ) ;                                           

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If this is a true HTML Document, search for the doc id  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( htmlFile.indexOf ( "html" ) != -1 ) {
                  editDocument ( Text.replaceInStr ( htmlFile, ".html", "" ) ) ;
               } else
                  GUISystem.printBox (STR_INFO, STR_MUST_NAVIGATE_TO_DOC_TO_EDIT ) ;

            } /* if */
            else
               GUISystem.printBox ( STR_INFO, STR_MUST_NAVIGATE_TO_DOC_TO_EDIT ) ;

         } catch ( Exception e ) {
            e.printStackTrace() ;
            GUISystem.printBox (STR_INFO, STR_MUST_NAVIGATE_TO_DOC_TO_EDIT ) ;
         } /* catch */
      } /* if the user has authority */

   } /* editPage */





   /*------------------------------------------------------------*/
   /* editDocument
    */
   /**
    *    This method will attempt to edit the document
    *  passed in through the notesId param
    *  
    *
    *  @param notesId The Lotus Notes id to edit
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editDocument ( String notesId ) {
      //VOYSQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "WebBrowserPanel", 5 ) ;
      //VOYDocRec    readRec   = null ;

      //VOYtry {
      //VOY   Statement stmt = sqlMethod.createStatement() ;
      //VOY   ResultSet rs   = null ;

      //VOY   rs = stmt.executeQuery ( "SELECT DOCIND,  DOCIND,  TITLE,  OWNER,  DOCCLASSIND,   "+
      //VOY                            "       LNDOCID, WORKIND, DBUSER, CHANGEDTIME    "+
      //VOY                            "FROM TIGRIS.DOCUMENTS "+
      //VOY                            "WHERE LNDOCID  = '"+notesId+"' AND "+
      //VOY                            "      PUBLISH  = 'Y' AND "+
      //VOY                            "      RECYCLED = 'N' FOR FETCH ONLY" ) ;


      //VOY   if ( rs.next() ) {
      //VOY      readRec = new DocRec () ;
      //VOY      readRec.setData ( false, rs ) ;
      //VOY   } else
      //VOY      GUISystem.printBox (STR_INFO, STR_DOCUMENT_NOT_FOUND ) ;

      //VOY   rs.close() ;

      //VOY} catch ( Exception e ) {
      //VOY   sqlMethod.rollBack() ;
      //VOY   LogSystem.log ( 1, e ,false ) ;
      //VOY   GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      //VOY   readRec = null ;
      //VOY}

      //VOYsqlMethod.close() ;

      //VOYif ( readRec != null )
      //VOY   readRec.edit() ;
      GUISystem.printBox ( "ERROR", "NOT IMPLEMENTED! - WebBrowserPanel.editDocument" ) ;
   }
   public boolean linkClicked ( String string ) {
      ef_URL.setText ( string ) ;
      return( true ) ;
   }


   public void statusChange ( String status, int value, int max ) {
      parentWin.setStatus ( status ) ;
      parentWin.setValueMax ( max ) ;
      parentWin.setValue ( value ) ;
   }

} /* WebBrowserPanel */
