package com.ibm.nzna.projects.qit.wordproc ;       
/*-----------------------------------------------------------*/
/*   SpellCheckDlg                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SpellCheckDlg                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import javax.swing.border.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.spell.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog will prompt the user with information on a misspelled word. 
 * The user will have a chance to select a suggestion. This dialog will
 * return from it's getErrorInfo method when the user selects an option.
 *
 *
 * @version 1.0, 10/5/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SpellCheckDlg                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class SpellCheckDlg extends JDialog implements ActionListener, AppConst, ListSelectionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int PB_STOP        = 1 ;
   public static final int PB_IGNORE      = 2 ;
   public static final int PB_IGNOREALL   = 3 ;
   public static final int PB_LEARN       = 4 ;
   public static final int PB_REPLACE     = 4 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_SUGGESTIONS         = null ;
   private JList        lb_SUGGESTIONS         = null ;
   private JTextField   ef_WORD                = null ;
   private DButton      pb_REPLACE             = null ;
   private DButton      pb_IGNORE              = null ;
   private DButton      pb_IGNOREALL           = null ;
   private DButton      pb_LEARN               = null ;
   private DButton      pb_STOP                = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JSpellErrorInfo errorInfo    = null ;
   private JSpellToolkit   spellToolkit = null ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create and populate our SpellCheckDlg
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public SpellCheckDlg ( Frame parent, JSpellErrorInfo errorInfo ) {
      super ( parent, Str.getStr ( STR_SPELL ), true ) ;
      spellToolkit = new SpellChecker().getToolkit() ;
      createControls() ;
      this.errorInfo = errorInfo ;
      setSize ( 300, 230 ) ;
      WinUtil.centerWindow ( this ) ;
   }



   /*-----------------------------------------------------------*/
   /* getErrorInfo
    */
   /**
    *    This method will return the error information
    *  that the user populates
    *  
    *             
    *  @param newErrorInfo The error info to get data from
    *  @param 
    *  @param 
    *
    *  @return JSpellErrorInfo the errorInfo record
    */
   public JSpellErrorInfo getErrorInfo ( JSpellErrorInfo newErrorInfo) {
      String[] wordArray = errorInfo.getSuggestions() ;

      this.errorInfo = newErrorInfo ;
      if ( ( wordArray != null ) && ( wordArray.length > 0 ) )
         lb_SUGGESTIONS.setListData ( wordArray ) ;
      else
         ((DefaultListModel)lb_SUGGESTIONS.getModel()).clear() ;

      ef_WORD.setText ( errorInfo.getWord() ) ;
      setVisible ( true ) ;
      return( errorInfo ) ;
   }



   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create our controls
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void createControls () {
      ef_WORD         = new JTextField() ;
      st_SUGGESTIONS  = new JLabel ( Str.getStr ( STR_SUGGESTIONS ) ) ;
      lb_SUGGESTIONS  = new JList () ;

      pb_REPLACE   = new DButton ( Str.getStr ( STR_REPLACE ) ) ;
      pb_IGNORE    = new DButton ( Str.getStr ( STR_IGNORE ) ) ;
      pb_IGNOREALL = new DButton ( Str.getStr ( STR_IGNOREALL ) ) ;
      pb_LEARN     = new DButton ( Str.getStr ( STR_LEARN ) ) ;
      pb_STOP      = new DButton ( Str.getStr ( STR_STOP ) ) ;

      lb_SUGGESTIONS.setBorder ( new BevelBorder ( BevelBorder.LOWERED ) ) ;
      lb_SUGGESTIONS.addListSelectionListener ( this ) ;
      pb_REPLACE.addActionListener     ( this ) ;
      pb_IGNORE.addActionListener      ( this ) ;
      pb_IGNOREALL.addActionListener   ( this ) ;
      pb_LEARN.addActionListener       ( this ) ;
      pb_STOP.addActionListener        ( this ) ;

      getContentPane().setLayout ( null ) ;
      getContentPane().add ( ef_WORD ) ;
      getContentPane().add ( st_SUGGESTIONS ) ;
      getContentPane().add ( lb_SUGGESTIONS ) ;
      getContentPane().add ( pb_REPLACE ) ;
      getContentPane().add ( pb_IGNORE ) ;
      getContentPane().add ( pb_IGNOREALL ) ;
      getContentPane().add ( pb_LEARN ) ;
      getContentPane().add ( pb_STOP ) ;
   }




   /*-----------------------------------------------------------*/
   /* layoutControls
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void doLayout () {
      int       x         = 5 ;
      int       y         = 5 ;
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      size.width-=10 ;
      size.height-=30 ;

      ef_WORD.setBounds ( x, y, (size.width/2)-5, rowHeight ) ;
      pb_REPLACE.setBounds ( size.width-125, y, 120, 25 ) ;
      y+=30 ;

      st_SUGGESTIONS.setBounds ( x, y, (size.width/2)-5, rowHeight ) ;
      pb_IGNORE.setBounds ( size.width-125, y, 120, 25 ) ;
      lb_SUGGESTIONS.setBounds ( x, y+rowHeight, (size.width/2)-5, size.height-((y+rowHeight))-5 ) ;
      y+=30 ;

      pb_IGNOREALL.setBounds ( size.width-125, y, 120, 25 ) ;
      y+=30 ;

      pb_LEARN.setBounds ( size.width-125, y, 120, 25 ) ;
      pb_STOP.setBounds ( size.width-125, size.height-30, 120, 25 ) ;
   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when a value
    *  in the list has changed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( e.getSource() == lb_SUGGESTIONS ) {
         String selection = (String) lb_SUGGESTIONS.getSelectedValue() ;

         ef_WORD.setText ( selection ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to buttons being
    *  pressed
    *  
    *
    *  @param e The action event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() instanceof DButton ) {
         DButton button = (DButton ) e.getSource() ;

         if ( button == pb_STOP ) {
            spellToolkit.setStopped ( true ) ;
            errorInfo.buttonHit = PB_STOP ;
            hideWindow() ;
         }

         else if ( button == pb_REPLACE ) {
            errorInfo.setWord ( ef_WORD.getText() ) ;
            errorInfo.buttonHit = PB_REPLACE ;
            hideWindow() ;
         }

         else if ( ( button == pb_IGNORE ) ||
                   ( button == pb_IGNOREALL ) ) {
            spellToolkit.addIgnore ( ef_WORD.getText() ) ;
            errorInfo.buttonHit = PB_IGNORE ;
            hideWindow() ;
         }

         else if ( button == pb_LEARN ) {
            spellToolkit.addLearned ( ef_WORD.getText() ) ;
            errorInfo.buttonHit = PB_LEARN ;
            hideWindow() ;
         }
      }
   }






   /*------------------------------------------------------------*/
   /* hideWindow
    */
   /**
    *    This method will currently just
    *  hide our window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void hideWindow () {
      dispose() ;
   }

} /* SpellCheckDlg */

