package com.ibm.nzna.projects.qit.wordproc ;
/*-----------------------------------------------------------*/
/*   StampedGraphicDlg                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StampedGraphicDlg                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog holds a notebook of 'Stamped' graphics. These are 
 * graphics that are constant throughout documents. The web server
 * carries only 1 copy of each graphic. This dialog allows the user
 * to choose from a set of stamped graphics
 *
 *
 * @version 1.0, 11/16/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   StampedGraphicDlg                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class StampedGraphicDlg extends JDialog implements Runnable, ActionListener, AppConst {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   GUI                                                     */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private JTabbedPane pnl_NOTEBOOK   = null ;
    private DButton     pb_CANCEL      = null ;

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Vars                                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    StampedGraphicDlgListener listener = null ;





    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create, createControls and show
     *  our non-modal dialog.
     *  
     *
     *  @param parent The parent frame
     *  @param listener The StampedGraphicDlgListener to use
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public StampedGraphicDlg ( Frame parent, StampedGraphicDlgListener listener ) {
        super ( parent, Str.getStr ( STR_STAMPED_GRAPHIC ), false ) ;

        createControls() ;
        setListener ( listener ) ;

        setSize ( 300, 300 ) ;
        WinUtil.centerChildInParent ( this, parent ) ;

        setVisible ( true ) ;
        new Thread ( this ).start() ;
    }





    /*------------------------------------------------------------*/
    /* setListener
     */
    /**
     *    This method will set our StampedGraphicDlgListener. This
     *  method will be notified when the user has chosen a graphic
     *  to stamp.
     *  
     *  
     *
     *  @param listener The StampedGraphicDlgListener to use
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setListener ( StampedGraphicDlgListener listener ) {
        this.listener = listener ;
    }





    /*------------------------------------------------------------*/
    /* createControls
     */
    /**
     *    This method will createControls our window
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void createControls () {
        ButtonPanel pnl_BUTTONHOLD  = null ;
        Container   contentPane     = getContentPane() ;               
        String[]    groups          = StampedGraphicFactory.getStampedGroups() ;
        int         i               = 0 ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create our normal controls                              */
        /*                                                           */
        /*-----------------------------------------------------------*/
        pnl_NOTEBOOK   = new JTabbedPane() ;
        pb_CANCEL      = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
        pnl_BUTTONHOLD = new ButtonPanel() ;      


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust properties                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        pb_CANCEL.setMinimumSize ( new Dimension ( 75, 25 ) ) ;  


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add listeners                                           */
        /*                                                           */
        /*-----------------------------------------------------------*/
        pb_CANCEL.addActionListener ( this ) ;                                              

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add the panels to our notebook                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        while ( i < groups.length ) {
            addNotebookPage ( groups[i] ) ;
            i++ ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add the controls to our content pane                    */
        /*                                                           */
        /*-----------------------------------------------------------*/
        contentPane.setLayout ( new BorderLayout (5,5) ) ;
        pnl_BUTTONHOLD.setLayout ( new StraightLayoutMgr() ) ;
        pnl_BUTTONHOLD.add ( pb_CANCEL ) ;
        contentPane.add ( pnl_NOTEBOOK, BorderLayout.CENTER ) ;
        contentPane.add ( pnl_BUTTONHOLD, BorderLayout.SOUTH ) ;

        pnl_NOTEBOOK.revalidate() ;
    }





    /*------------------------------------------------------------*/
    /* actionPerformed
     */
    /**
     *    This method will be called when an ActionEvent
     *  is fired against us
     *  
     *
     *  @param e The event fired
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void actionPerformed ( ActionEvent e ) {

        if ( e.getSource() == pb_CANCEL )
            dispose() ;

        else if ( e.getSource() instanceof DButton ) {
            DButton button = (DButton) e.getSource() ;
            String  graphicName = button.getText() ;

            if ( listener != null ) {
                listener.stampGraphic ( graphicName ) ;
            }

        }
    }




    /*------------------------------------------------------------*/
    /* addNotebookPage
     */
    /**
     *    This method will add a notebook
     *  page for the stamped graphic group 
     *  passed in
     *
     *  @param group The name of the group
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void addNotebookPage ( String group ) {
        StampedGraphicPanel panel = new StampedGraphicPanel ( group, this ) ;

        pnl_NOTEBOOK.addTab ( group, null, panel ) ;
        pnl_NOTEBOOK.invalidate() ;
    }

    public void run () {
       try {
          Thread.currentThread().sleep ( 500 ) ;
          pnl_NOTEBOOK.revalidate() ;
       } catch ( Exception e ) {
       }
    }

} /* StampedGraphicDlg */
