package com.ibm.nzna.projects.qit.wordproc ;
/*-----------------------------------------------------------*/
/*   StampedGraphicFactory                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StampedGraphicFactory               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This factory holds a list of Stamped graphics for documents. It
 * also has static methods usefull in manipulating the lists and getting
 * the graphics
 *
 *
 * @version 1.0, 11/16/1999 
 * @author  David Lentz
 */
import javax.swing.* ;
import java.util.* ;
import java.io.File ;
import com.ibm.nzna.projects.qit.app.* ;



/*-----------------------------------------------------------*/
/*                                                           */
/*   StampedGraphicFactory                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class StampedGraphicFactory extends Object implements AppConst {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Vars                                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static final String[][] graphicList =
    {   { Str.getStr ( STR_DOCLINKS ),    "diskette.gif"},
        { Str.getStr ( STR_DOCLINKS ),    "doclink.gif"},
        { Str.getStr ( STR_DOCLINKS ),    "photolink.gif"},
        { Str.getStr ( STR_ESALES   ),    "buyibm.gif"},
        { Str.getStr ( STR_ESALES   ),    "buy_ww.gif"},
        { Str.getStr ( STR_ESALES   ),    "buy_us.gif"},
        { Str.getStr ( STR_WINDOWS2000 ), "win2000.gif"},
        { Str.getStr ( STR_WINDOWS2000 ), "biossmall.gif"},
        { Str.getStr ( STR_WINDOWS2000 ), "video2small.gif"},
        { Str.getStr ( STR_WINDOWS2000 ), "othersmall.gif"},
        { Str.getStr ( STR_WINDOWS2000 ), "utilitiessmall.gif"},
        { Str.getStr ( STR_WINDOWS2000 ), "installsmall.gif"}

    } ;






    /*------------------------------------------------------------*/
    /* getStampedGroups
     */
    /**
     *    This method will return the list of different
     *  groups of Stamped Graphics. These groups
     *  include 'eSales', 'Document Links' etc..
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String[]
     *//*---------------------------------------------------------*/
    public static final String[] getStampedGroups () {
        String[] list = null ;
        Vector   vec  = new Vector ( graphicList.length ) ;
        int      i    = 0 ;
        int      k    = 0 ;

        while ( k < graphicList.length ) {
            try {
                if ( !graphicList[k][0].equals ( graphicList[k-1][0] ) )
                    vec.addElement ( graphicList[k][0] ) ;
            } catch ( Exception e ) {
                vec.addElement ( graphicList[k][0] ) ;
            }
            k++ ;
        }

        vec.trimToSize() ;
        list = new String[vec.size()] ;
        vec.copyInto ( list ) ;

        return( list ) ;
    }





    /*------------------------------------------------------------*/
    /* getImageIconsFromGroup
     */
    /**
     *    This method will return a Vector of ImageIcons
     *  that belong to the group name passed in
     *  
     *
     *  @param group The name of the Stamped Graphic Group. For example 'eSales'
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static final Vector getImageIconsFromGroup ( String group ) {
        Vector     retVec       = new Vector ( 10, 10 ) ;
        int        i            = 0 ;
        ImageIcon  imageIcon    = null ;

        while ( i < graphicList.length ) {
            if ( graphicList[i][0].equals ( group ) ) {
                imageIcon = new ImageIcon ( "skins"+File.separator+graphicList[i][1] ) ;
                imageIcon.setDescription ( graphicList[i][1] ) ;
                retVec.addElement ( imageIcon ) ;
            }
            i++ ;
        }

        retVec.trimToSize() ;

        return( retVec ) ;
    }





    /*------------------------------------------------------------*/
    /* getNameFromImageIcon
     */
    /**
     *    This method will determine the name of a 
     *  graphic from the ImageIcon that represents it
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static final String getNameFromImageIcon ( ImageIcon imageIcon ) {
        return( imageIcon.getDescription () ) ;
    }





    /*------------------------------------------------------------*/
    /* containsImage
     */
    /**
     *    This method will determine if the image
     *  passed matches any of the stamped
     *  images
     *
     *  @param imageName The name of the image
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static final boolean containsImage ( String imageName ) {
        int     i     = 0 ;
        boolean found = false ;

        while ( ( !found ) && ( i < graphicList.length ) ) {
            found = ( graphicList[i][1].equals ( imageName ) ) ;
            i++ ;
        }

        return ( found ) ;

    }


    /**
     *    This method will copy all of the current stamped graphics
     *  to the temp. directory
     *  
     *
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static final Vector copyToTemp (  ) {
        Vector     retVec       = new Vector ( 10, 10 ) ;
        int        i            = 0 ;
        ImageIcon  imageIcon    = null ;
        String     homeDir      = PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR ) ;
        String     skinName     = PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME ) ;

        while ( i < graphicList.length ) {
            com.ibm.nzna.projects.qit.app.util.QuestUtil.copyFileToTempDir ( homeDir+File.separator+"skins"+File.separator+skinName+File.separator+graphicList[i][1] ) ;
            i++ ;
        }

        retVec.trimToSize() ;

        return( retVec ) ;
    }

} /* StampedGraphicFactory */
