package com.ibm.nzna.projects.qit.wordproc ;
/*-----------------------------------------------------------*/
/*   StampedGraphicPanel                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StampedGraphicPanel                 */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel shows the user a single 'Group' of stamped graphics 
 *
 *
 * @version 1.0, 11/17/99 
 * @author  David Lentz
 */
import javax.swing.* ;
import javax.swing.border.* ;
import java.util.* ;
import java.awt.* ;
import java.awt.event.* ;        
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   StampedGraphicPanel                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class StampedGraphicPanel extends JPanel implements ActionListener, AppConst, Runnable {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ActionListener listener  = null ;
   private String         groupName = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  our panel
    *  
    *
    *  @param group The group of graphics that we are representing
    *  @param listener The action listener to notify when a graphic is selected
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public StampedGraphicPanel ( String groupName, ActionListener listener ) {
      this.listener  = listener ;
      this.groupName = groupName ;

      new Thread ( this ).start() ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      JScrollPane scr_PANE       = null ;
      JPanel      pnl_BUTTONHOLD = null ;
      Vector      imageVec       = StampedGraphicFactory.getImageIconsFromGroup ( groupName ) ;
      int         i              = 0 ;              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI Controls                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_BUTTONHOLD = new JPanel () ;
      scr_PANE = new JScrollPane ( pnl_BUTTONHOLD ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      scr_PANE.setBackground ( Color.white ) ;
      pnl_BUTTONHOLD.setBackground ( Color.white ) ;
      scr_PANE.setBorder ( new BevelBorder ( BevelBorder.LOWERED ) ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the buttons to the panel                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_BUTTONHOLD.setLayout ( new FlowLayout ( FlowLayout.LEFT ) ) ;
      if ( imageVec != null ) {
         DButton     button    = null ;
         int         length    = imageVec.size() ;
         ImageIcon   imageIcon = null ;

         while ( i < length ) {
            imageIcon = (ImageIcon) imageVec.elementAt(i) ;
            button = new DButton ( imageIcon ) ;
            if ( ( imageIcon.getIconWidth() < 75 ) && ( imageIcon.getIconHeight() < 75 ) )
               button.setPreferredSize ( new Dimension ( 75, 75 ) ) ;
            else if ( imageIcon.getIconWidth() > 75 )
               button.setPreferredSize ( new Dimension ( imageIcon.getIconWidth()+5, 75 ) ) ;
            else if ( imageIcon.getIconHeight() > 75 ) 
               button.setPreferredSize ( new Dimension ( 75, imageIcon.getIconHeight()+25 ) ) ;

            button.setBorderPainted ( false ) ;
            button.setFocusPainted ( false ) ;
            button.setOpaque ( false ) ;
            button.setContentAreaFilled ( false ) ;
            button.setText ( ((ImageIcon)imageVec.elementAt(i)).getDescription() ) ;
            button.addActionListener ( this ) ;
            button.setVerticalTextPosition ( JButton.BOTTOM ) ;
            button.setHorizontalTextPosition ( JButton.CENTER ) ;
            button.setHorizontalAlignment ( JButton.TRAILING ) ;
            pnl_BUTTONHOLD.add ( button ) ;
            i++ ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our panel                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout ( 10, 10 ) ) ;
      add ( scr_PANE, BorderLayout.CENTER ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method simply passes the event
    *  down to our listener
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e) {
      if ( listener != null )
         listener.actionPerformed ( e ) ;
   }

} /* StampedGraphicPanel */
