package com.ibm.nzna.projects.qit.wordproc ;
/*-----------------------------------------------------------*/
/*   StyleTag                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StyleTag                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object represents one beginning tag for style.
 *
 *
 * @version 1.0, 8/21/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   StyleTag                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class StyleTag extends Object implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String  fontName      = "" ;
   private int     size          = 0 ;
   private String  color         = "" ;
   private String  styleName     = "" ;
   private int     ind           = 0 ;
   private int     charPos       = 0 ;
   private int     length        = 0 ;
   private boolean valid         = false ;



   public StyleTag () {
   } /* StyleTag */

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a StyleTag
    *  object from the text that is the name in the
    *  anchor tag. No charPos, or length data will be stored
    *
    *  @param text The text that is the name of the anchor tag
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public StyleTag ( String text ) {
      setTagInfo ( text ) ;
   } /* StyleTag */

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a StyleTag
    *  object from the text that is the name in the
    *  anchor tag. The charPos is stored.
    *
    *  @param text The text that is the name of the anchor tag
    *  @param charPos The character position of this tag in the HTML Code.
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public StyleTag ( String text, int charPos ) {
      this.charPos = charPos ;
      setTagInfo ( text ) ;
   } /* StyleTag */


   /*------------------------------------------------------------*/
   /* isValid
    */
   /**
    *    returns TRUE if this object is valid
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = valid, FALSE = not valid
    *//*---------------------------------------------------------*/
   public boolean isValid () {
      return ( valid ) ;
   } /* isValid */




   /*-----------------------------------------------------------*/
   /* getFontName
    */
   /**
    *    This method will return the font name
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return String Font Name
    */
   public String getFontName () {
      return ( fontName ) ;
   } /* getFontName */





   /*-----------------------------------------------------------*/
   /* getSize
    */
   /**
    *    This method will return the font size.
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return int size
    */
   public int getSize () {
      return ( size ) ;
   } /* getSize */



   /*-----------------------------------------------------------*/
   /* getColor
    */
   /**
    *    This method will return the font color
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return String The string representation of the color
    */
   public String getColor () {
      return ( color ) ;
   } /* getColor */


   /*------------------------------------------------------------*/
   /* setTagInfo
    */
   /**
    *    This method will take the text that is the
    *  name of a style anchor tag and extract
    *  all of the values from the tag, populating this record. The
    *  tag MUST be in this format:
    *
    *  UniqueID:STYLENAME:FONTNAME:SIZE:COLOR
    *
    *  @param text The text that is the name of the tag
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTagInfo ( String text ) {
      try {
         StringTokenizer strtok  = new StringTokenizer ( text, ":" ) ;
         int             index   = 0 ;
         String          tempStr = null ;

         while ( strtok.hasMoreTokens() ) {
            tempStr = strtok.nextToken() ;

            switch ( index ) {
               case 0 :
                  ind = new Integer ( tempStr ).intValue() ;
                  break ;

               case 1 :
                  styleName = tempStr ;
                  break ;

               case 2 :
                  fontName = tempStr ;
                  break ;

               case 3 :
                  size = new Integer ( tempStr ).intValue() ;
                  break ;

               case 4 :
                  color = tempStr ;
                  break ;
            }
            index++ ;
         } /* while */
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   } /* setTagInfo */





   /*------------------------------------------------------------*/
   /* getFontTag
    */
   /**
    *    This method will get the font tag that
    *  should represent this temporary style anchor
    *  in the html document.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The full font tag including font name, size and color
    *//*---------------------------------------------------------*/
   public String getFontTag () {
      String retStr = null ;

      try {
         retStr = "<font name=" ;

         if ( fontName.equals ( "Helvetica" ) )
            retStr = retStr+"\"Helvetica, Arial\">" ;
         else
            retStr = retStr+"\""+fontName+"\">";

         retStr = retStr+"<font size=\""+size+"\">" ;

         retStr = retStr+"<font color=\""+color+"\">" ;

         if ( styleName.equals ( Str.getStr ( STR_TABLE_TITLE ) ) )
            retStr = "<b>"+retStr ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return ( retStr ) ;
   } /* getFontTag */



   /*-----------------------------------------------------------*/
   /* getInd
    */
   /**
    *    This method will return the unique number
    *  associated to this style tag.
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return int The unique indicator for this tag
    */
   public int getInd () {
      return ( ind ) ;
   } /* getInd */



   /*-----------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will determine if the fields
    *  in another StyleTag matches this one. All fields
    *  are compared except the unique id.
    *             
    *  @param styleTag The style tag to compare this one to
    *  @param 
    *  @param 
    *
    *  @return TRUE = The fields match, FALSE = they do not
    */
   public boolean equals ( StyleTag styleTag ) {
      boolean ret = false;

      if ( styleTag != null ) {
         ret = ( ( size == styleTag.getSize() ) &&
                 ( color.equals ( styleTag.getColor() ) ) &&
                 ( fontName.equals ( styleTag.getFontName() ) ) ) ;
      } /* if */

      return ( ret ) ;
   } /* equals */


    /*-----------------------------------------------------------*/
    /* getEndTag
     */
    /**
     *    This method will return the end tag(s) that
     *  needs to follow this style tag.
     *  
     *             
     *  @param 
     *  @param 
     *  @param 
     *
     *  @return String end tag
     */
     public String getEndTag () {
        String retStr = "</font></font></font>" ;

        if ( styleName.equals ( Str.getStr ( STR_TABLE_TITLE ) ) )
           retStr = retStr+"</b>" ;

        return ( retStr ) ;
     } /* getEndTag */




      






} /* StyleTag */
