package com.ibm.nzna.projects.qit.wordproc ;
/*-----------------------------------------------------------*/
/*   TPMainPage                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TPMainPage                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This is the Table Property Main page for the Table Property Dialog.
 * This panel holds the number of rows and columns and will have the
 * border width and Border Check Box
 *
 * @version 1.0, 5/26/1999 
 * @author  David Lentz
 */
import javax.swing.* ;
import java.awt.* ;
import lotus.wp.ifx.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   TPMainPage                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class TPMainPage extends JPanel implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel     st_ROWS        = new JLabel ( Str.getStr ( STR_ROWS ) ) ;
   private JLabel     st_COLS        = new JLabel ( Str.getStr ( STR_COLS ) ) ;
   private JLabel     st_BORDERWIDTH = new JLabel ( Str.getStr ( STR_BORDER_WIDTH ) ) ;
   private JTextField ef_ROWS        = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 3 ), "", 0 ) ;
   private JTextField ef_COLS        = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 2 ), "", 0 ) ;
   private JTextField ef_BORDERWIDTH = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 1 ), "0", 0 ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Table  table    = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will construct our page
    *  
    *  
    *
    *  @param tableRec The TableRec to alter
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TPMainPage ( Table table ) {

      /* First, copy over our data  */
      this.table = table ;

      /* Add our controls  */
      setLayout ( null ) ;
      add ( st_ROWS ) ;
      add ( st_COLS ) ;
      add ( st_BORDERWIDTH ) ;
      add ( ef_ROWS ) ;
      add ( ef_COLS ) ;
      add ( ef_BORDERWIDTH ) ;

      /* Populate our controls if we have a valid table, otherwise
         this must be a new table creation  */
      if ( table != null ) {
         int borderWidth = table.getBorderWidth() ;

         ef_ROWS.setText ( ""+table.getRowCount() ) ;
         ef_COLS.setText ( ""+table.getColumnCount() ) ;
         ef_BORDERWIDTH.setText ( ""+borderWidth ) ;
      } else {
         ef_ROWS.setText ( "2" ) ;
         ef_COLS.setText ( "2" ) ;
         ef_BORDERWIDTH.setText ( "1" ) ;
      }

   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;
      int x         = 5 ;
      int y         = 5 ;
      int rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_ROWS.setBounds ( x, y, 100, rowHeight ) ;
      ef_ROWS.setBounds ( x+100, y, 40, rowHeight ) ;
      y+=rowHeight ;

      st_COLS.setBounds ( x, y, 100, rowHeight ) ;
      ef_COLS.setBounds ( x+100, y, 40, rowHeight ) ;
      y+=rowHeight ;

      st_BORDERWIDTH.setBounds ( x, y, 100, rowHeight ) ;
      ef_BORDERWIDTH.setBounds ( x+100, y, 30, rowHeight ) ;
   }




   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save our data
    *  and apply the changes to the saveTable passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void saveData ( Table saveTable ) {
      try {
         int rows = new Integer ( ef_ROWS.getText() ).intValue () ;
         int cols = new Integer ( ef_COLS.getText() ).intValue () ;
         int borderWidth = new Integer ( ef_BORDERWIDTH.getText() ).intValue() ;             

         if ( ( rows > 0 ) && ( cols > 0 ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, handle the rows                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rows < saveTable.getRowCount() ) {
               boolean rc = GUISystem.printBox ( STR_QUESTION, STR_ROW_COUNT_LESS ) ;

               /* If they answer 'Yes', delete the rows  */
               if ( rc ) {
                  int i = 0 ;
                  int length = saveTable.getRowCount()-rows ;

                  for ( i = 0; i < length; i++ )
                     saveTable.deleteRow ( saveTable.getRowCount()-1 ) ;
               } else
                  ef_ROWS.setText ( ""+saveTable.getRowCount() ) ;
            }

            /* Add the rows  */
            else {
               int i = 0 ;
               int length = rows-saveTable.getRowCount() ;

               for ( i = 0; i < length; i++ )
                  saveTable.insertRow ( saveTable.getRowCount() ) ;
            }



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, handle the columns                                 */              
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( cols < saveTable.getColumnCount() ) {
               boolean rc = GUISystem.printBox ( STR_QUESTION, STR_COLUMN_COUNT_LESS ) ;

               /* If they answer 'Yes', delete the columns  */
               if ( rc ) {
                  int i = 0 ;
                  int length = saveTable.getColumnCount()-cols ;
                  for ( i = 0; i < length; i++ )
                     saveTable.deleteColumn ( saveTable.getColumnCount()-1 ) ;
               } else
                  ef_COLS.setText ( ""+saveTable.getColumnCount() ) ;
            }

            /* Add the cols  */
            else {
               int i = 0 ;
               int length = cols-saveTable.getColumnCount() ;

               for ( i = 0; i < length; i++ )
                  saveTable.insertColumn ( saveTable.getColumnCount() ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, save the border properties                         */                      
            /*                                                           */
            /*-----------------------------------------------------------*/
            saveTable.setBorderWidth ( borderWidth ) ;
         } else
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_ROW_COL_WIDTH_BAD ) ) ;



      } catch ( NumberFormatException e ) {
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_ROW_COL_WIDTH_BAD ) ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* getColumns
    */
   /**
    *    This method will get the amount of columns
    *  that the user has entered in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getColumns () {
      return( new Integer ( ef_COLS.getText() ).intValue() ) ;
   }


   /*------------------------------------------------------------*/
   /* getRows
    */
   /**
    *    This method will get the amount of rows
    *  that the user has entered in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getRows () {
      return( new Integer ( ef_ROWS.getText() ).intValue() ) ;
   }
} /* TPMainPage */
