package com.ibm.nzna.projects.qit.wordproc ;
/*-----------------------------------------------------------*/
/*   TablePropertyDlg                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TablePropertyDlg                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This method will allow the user to select a table to do operations on  
 * it.
 *
 *
 * @version 1.0, 5/25/1999 
 * @author  David Lentz
 */
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import lotus.wp.ifx.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/*-----------------------------------------------------------*/
/*                                                           */
/*   TablePropertyDlg                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class TablePropertyDlg extends JDialog implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTabbedPane pnl_NOTEBOOK         = new JTabbedPane() ;
   private DButton     pb_OK                = new DButton ( Str.getStr ( STR_OK ) ) ;
   private DButton     pb_APPLY             = new DButton ( Str.getStr ( STR_APPLY ) ) ;
   private DButton     pb_CANCEL            = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
   private Container   contentPane          = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Table                  table    = null ;
   private lotus.wp.WordProcessor wp_PANE  = null ;
   private TPMainPage             mainPage = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create but not show the
    *  dialog. Please use the getResult method
    *  to show the dialog and get user information from the dialog
    *
    *  @param tableVec The vector containing all of the tables
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TablePropertyDlg ( Frame parent, lotus.wp.WordProcessor wp_PANE, Table table ) {
      super ( parent, Str.getStr ( STR_TABLE_PROPERTIES ), false ) ;

      /* Copy over our data  */
      this.table   = table ;
      this.wp_PANE = wp_PANE ;

      /* Add our controls */
      contentPane = getContentPane() ;
      contentPane.setLayout ( null ) ;
      contentPane.add ( pnl_NOTEBOOK ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_APPLY ) ;
      contentPane.add ( pb_CANCEL ) ;

      /* Add our pages */
      pnl_NOTEBOOK.addTab ( Str.getStr ( STR_GENERAL ), mainPage = new TPMainPage ( table ) ) ;

      /* Add us as an action listener */
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_APPLY.addActionListener ( this ) ;

      setSize ( 350, 250 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
      setResizable ( false ) ;
      setVisible ( true ) ;

   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the dialog and not
    *  return until the user has pressed either ok or
    *  cancel. It will return the TableRec that the user has selected
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TableRec getResult() { 
      setVisible ( true ) ;
      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = 18 ;

      super.doLayout () ;

      pnl_NOTEBOOK.setBounds ( x, y, size.width-(x*3), size.height-70 ) ;

      pb_OK.setBounds ( x, size.height-50, 70, 25 ) ;
      pb_APPLY.setBounds ( x+75, size.height-50, 75,25 ) ;
      pb_CANCEL.setBounds ( x+75+80, size.height-50, 80, 25 ) ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are listening for is fired
    *  
    *
    *  @param e The action event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_CANCEL ) {
         setVisible ( false ) ;
      } else if ( ( e.getSource() == pb_APPLY ) ||
                  ( e.getSource() == pb_OK ) ) {

         if ( table == null ) {
            String name = "Table("+System.currentTimeMillis()+")" ;

            if ( ( name != null ) &&
                 ( name.length() > 0 ) ) {
               Table curTable = wp_PANE.findTable ( name ) ;

               if ( curTable != null )
                  GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_TABLE_ALREADY_EXISTS ) ) ;
               else {
                  WPHTMLGenerator.insertTable ( wp_PANE, name, mainPage.getRows(), mainPage.getColumns() ) ;

                  table = wp_PANE.findTable ( name ) ;
               }
            } else
               GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_MUST_ENTER_VALUE ) ) ;
         }

         if ( table != null ) {
            mainPage.saveData ( table ) ;
         }

         if ( e.getSource() == pb_OK )
            setVisible ( false ) ;
      }
   }


} /* TablePropertyDlg */
