package com.ibm.nzna.projects.qit.wordproc ;
/*-----------------------------------------------------------*/
/*   WPHTMLGenerator                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WPHTMLGenerator                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This generator basically takes the Lotus eSuite Word Processor 
 * and generates HTML from what the word processor currently contains
 *
 *
 * @version 1.0, 6/22/1999 
 * @author  David Lentz
 */
import lotus.wp.WordProcessorBean;
import lotus.wp.* ;
import lotus.wp.ifx.* ;
import lotus.fc.* ;
import java.io.* ;
import java.util.* ;
import java.awt.Color ;
import com.ibm.nzna.shared.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   WPHTMLGenerator                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class WPHTMLGenerator extends Object implements Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static WordProcessor  wp                 = null ;
   static TextElement    curElement         = null ;
   static ParagraphStyle curParagraph       = null ;
   static int            inBold             = 0 ;
   static int            inItalic           = 0 ;
   static int            inFont             = 0 ;
   static String         curChar            = null ;
   static boolean        continueGenerating = true ;
   static String         curFontName        = "Helvetica, Arial" ;
   static int            curFontSize        = 12 ;
   static Color          curFontColor       = Color.black ;
   static int            tableCount         = 0 ;
   static int            graphicCount       = 0 ;
   static int            passes             = 0 ;
   static boolean        firstFontChange    = true ;
   static boolean        insideBulletList   = false ;
   static boolean        insideOrderedList  = false ;
   static boolean        processNextBreak   = true ;
   static String         curAlignment       = "left" ;
   static int            inPre              = 0 ;    

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Our listeners                                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static WPHTMLGeneratorListener statusListener = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create, initialize and start
    *  the generator code.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WPHTMLGenerator ( WordProcessor wp ) {
      this.wp = wp ;
      new Thread ( this ).start() ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will simply call our generateHTML
    *  method
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      if ( wp != null ) {
         generateHTML (wp, null ) ;
      } 
   }






   /*------------------------------------------------------------*/
   /* generateHTML
    */
   /**
    *    This method will start generating the
    *  HTML from the Word Processor
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String generateHTML ( WordProcessor wordproc, WPHTMLGeneratorListener listener ) {
      String        retStr             = null ;
      StringBuffer  htmlText           = new StringBuffer() ;
      int           i                  = 0 ;
      boolean       processText        = true ;                                  

      wp = wordproc ;
      wp.setRefreshEnabled ( false ) ;
      wp.setVisible ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, put an anchor tag around every table             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      applyAnchorsToTables() ;                   
      wp.setRefreshEnabled ( true ) ;
      wp.setRefreshEnabled ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our vars                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      curElement         = null ;
      curParagraph       = null ;
      inBold             = 0 ;
      inItalic           = 0 ;
      inFont             = 0 ;
      curChar            = null ;
      continueGenerating = true ;
      curFontName        = "Helvetica, Arial" ;
      curFontSize        = 12 ;
      tableCount         = 0 ;
      graphicCount       = 0 ;
      firstFontChange    = true ;
      insideBulletList   = false ;
      insideOrderedList  = false ;
      curAlignment       = "left" ;


      statusListener = listener ;
      notifyListener ( "Pass "+passes+" Starting" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put the headers in                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      htmlText.append ( "<font face=\"Helvetica, Arial\" size=3 color=\"black\">" ) ;

      /* Make the insertion point go to the beginning of the document */
      wp.gotoBeginningOfDocument ( false ) ;                                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the first item is not a character, insert a space    */
      /* because Lotus won't detect graphics in the first position */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( wp.getCharacter() == null ) || ( wp.getCharacter().length()==0  ) ) {
         wp.insertText ( " " ) ;
         wp.gotoBeginningOfDocument ( false ) ;                     
      } 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Loop until the end of document                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( continueGenerating ) {

         handleParagraphAlign ( htmlText ) ;
         if ( !handleGraphics ( htmlText, true ) ) {
            /* Get the next character */
            curChar    = wp.getCharacter() ;

            /* Bump the position down. This will allow us to get the */
            /* Correct selected text element. If the first element is a graphic,
               get it now */
            gotoNextChar ( "generateHTML, normal step" ) ;                            


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Generate HTML From the Word Processor                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            continueGenerating = generateElementHTMLText ( htmlText, true, true, false ) ;
         }
      }

      /* Ok, end the tags now */
      closeTags ( htmlText ) ;

      /* if we are still inside of a font change, end it now  */
      if ( inFont > 0 ) {
         if ( curFontName.equals ( "Courier" ) ) {
            htmlText.append ( "</pre>" ) ;
            inPre-- ;
         }
         htmlText.append ( "</font>" ) ;
      }

      /* End our header tag  */
      htmlText.append ( "</font>" ) ;

      saveFile ( "output.html", htmlText.toString() ) ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we missed tables, go back and do again!              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( ( missedTables() ) || ( missedGraphics() ) ) && ( passes < 2 ) ) {
         passes++ ;
         continueGenerating=true ;
         htmlText = new StringBuffer ( generateHTML(wp, listener ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remove our temp anchors                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      removeAnchorsFromTables() ;


      wp.setVisible ( true ) ;
      wp.setRefreshEnabled ( true ) ;

      /* Do some final cleanup */
      retStr = htmlText.toString() ;
      retStr = Text.replaceAllStrInStr ( retStr, "<br>\n<center>", "<center>" ) ;
      retStr = Text.replaceAllStrInStr ( retStr, "<br>\n</center>", "</center>" ) ;
      retStr = Text.replaceAllStrInStr ( retStr, "</center><br>", "</center>" ) ;
      retStr = Text.replaceAllStrInStr ( retStr, "</div><br>", "</div>" ) ;
      retStr = Text.replaceAllStrInStr ( retStr, "<center></center>", "" ) ;
      retStr = Text.replaceAllStrInStr ( retStr, "<div align=\"right\"></div>", "" ) ;
      return( retStr ) ;
   }



   /*------------------------------------------------------------*/
   /* generateElementHTMLText
    */
   /**
    *    This method will generate the HTML Element
    *  Text for the current element. It will also append
    *  the current character to the buffer passed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean generateElementHTMLText ( StringBuffer htmlText, boolean handleLinks, boolean handleTables, boolean handleAlignment ) {
      boolean processText = true ;
      boolean rc          = true ;

      notifyListener ( "Parsing normal text" ) ;

      curElement = wp.findSelectedTextElement() ;
      curParagraph = curElement.getParagraphStyle() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Handle Paragraph Alignments                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( handleAlignment )
         handleParagraphAlign ( htmlText ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Handle any Graphics, process the text if we didn't have */
      /* a graphic. The handleGraphics method would have done that */
      /*                                                           */
      /*-----------------------------------------------------------*/
      processText = !handleGraphics(htmlText, true) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Handle any special bookmarks                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( wp.findCurrentObject ( "bookmark" ) != null ) {
         Bookmark bookMark     = (Bookmark) wp.findCurrentObject ( "bookmark" ) ;
         String   bookMarkName = bookMark.getName() ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is our table bookmark, generate a table         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( bookMarkName.indexOf ( "TABLE-" ) >= 0 ) {
            if ( handleTables )
               processText = !generateTableHTMLText ( htmlText, wp.findTable ( bookMarkName.substring ( bookMarkName.indexOf("-")+1 ) ) ) ;
            if ( processText )
               gotoNextChar ( "After generating HTML Table Text, processText was TRUE" ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, treat this as a normal link                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( handleLinks )
            generateLinkHTMLText ( htmlText, handleAlignment ) ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, handle as normal text                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( processText ) {

         if ( ( curChar != null ) && ( curChar.length() > 0 ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   BOLD                                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            handleBold ( htmlText ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   ITALIC                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            handleItalic ( htmlText ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Handle Numbered(Ordered) lists and bulleted             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            handleLists ( htmlText ) ;            


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Handle Font Name Change                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            handleFontName ( htmlText ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Handle Font Size Changes                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            handleFontSize ( htmlText ) ;                            

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Handle Font Color changes                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            handleFontColor ( htmlText ) ;



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Append the character to the HTML file                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            htmlText.append ( curChar ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, the last character we read was nothing, check to    */
         /*   see if that was a line break, or the end of the doc.    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            TextElement tempElement = null ;

            /* Usually indicates no more characters by returning false */
            rc = gotoNextChar ( "generateHTML, Closing Paragraph" ) ;
            tempElement = wp.findSelectedTextElement() ;

            /* If we were able to get the next character, bump it back once
               for the loop and add in our <br> tag  */
            if ( rc ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Close our currently open tags                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               closeTags ( htmlText ) ;                                                         

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Handle a graphic if need be                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               handleGraphics ( htmlText, false ) ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If we are inside of a list, put the <LI> tags in,       */
               /* otherwise handle ending the alignment tags. If the        */
               /* current alignment is left, and new alignment is center,   */
               /* or the current alignment is left, we will put a break     */
               /* tag in.                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( insideOrderedList ) || ( insideBulletList ) ) {
                  if ( ( tempElement.getParagraphStyle().getName().startsWith ( "OLLI" ) ) ||
                       ( tempElement.getParagraphStyle().getName().startsWith ( "ULLI" ) ) )
                     htmlText.append ( "\n<LI>" ) ;
               } else {


                  if ( handleAlignment ) {
                     if ( !curAlignment.equals ( "left" ) ) {
                        if ( curAlignment.equals ( "center" ) )
                           htmlText.append ( "</center>\n" ) ;
                        else if ( curAlignment.equals ( "right" ) )
                           htmlText.append ( "</div>\n" ) ;
                        else
                           handleLineBreak ( htmlText ) ;
                        curAlignment = "left" ;
                     } else {
                        if ( ( tempElement.getAlignment().equals ( curAlignment ) ) ||
                             ( tempElement.getAlignment().equals ( "center" ) ) ) {
                           handleLineBreak ( htmlText ) ;
                        }
                     } 
                  } else
                     handleLineBreak ( htmlText ) ;
               }
               gotoPreviousChar ( "GenerateHTML - Closing Paragraph" ) ;
            }
         }
      }

      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* closeTags
    */
   /**
    *    This method will close all of the known open
    *  tags.
    *  
    *
    *  @param htmlText The StringBuffer containing the html 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void closeTags ( StringBuffer htmlText ) {                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First append the actual closing tags                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( inItalic > 0 )
         htmlText.insert ( htmlText.length(),  "</i>" ) ;
      if ( inBold > 0 )
         htmlText.insert ( htmlText.length(),  "</b>" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, set the counting vars back to 0                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      inBold = 0 ;
      inItalic = 0 ;
   }







   /*------------------------------------------------------------*/
   /* insertImage
    */
   /**
    *    This method will insert the current image
    *  into the HTML Text
    *  
    *
    *  @param htmlText The StringBuffer to put the tag in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void insertImage ( StringBuffer htmlText ) {
      WPImage image = (WPImage) wp.findCurrentObject ( "image" ) ;

      if ( image != null ) {

         graphicCount++ ;

         if ( image.getImageName().indexOf ( "doclink.gif" ) == -1 ) {
            String imageName = image.getImageName() ;

            try {
               if ( imageName.indexOf ( "/" ) >= 0 )
                  imageName = imageName.substring ( imageName.lastIndexOf ( "/" )+1,  imageName.length() ) ;
               if ( imageName.indexOf ( "\\" ) >= 0 )
                  imageName = imageName.substring ( imageName.lastIndexOf ( "\\" )+1, imageName.length() ) ;
            } catch ( Exception e ) {
               if ( imageName.indexOf ( "/" ) >= 0 )
                  imageName = imageName.substring ( imageName.lastIndexOf ( "/" ),  imageName.length() ) ;
               if ( imageName.indexOf ( "\\" ) >= 0 )
                  imageName = imageName.substring ( imageName.lastIndexOf ( "\\" ), imageName.length() ) ;
            }

            /* Append the first piece of the tag */
            htmlText.append ( "<img src=\"./" ) ;

            /* Append the filename of the graphic */
            htmlText.append ( imageName ) ;

            htmlText.append ( "\" WIDTH="+image.getWidth()+" HEIGHT="+image.getHeight()+" BORDER=0>" ) ;

         } else
            htmlText.append ( "<img src=\"###IMAGELOC###doclink.gif\">" ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* saveFile
    */
   /**
    *    This method will save a string to a file.
    *  
    *  
    *
    *  @param filename The filename of the file
    *  @param string The String to save to the file
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean saveFile ( String filename, String string ) {
      boolean rc = false ;

      try {
         File             outFile = new File ( filename ) ;
         FileOutputStream out     = new FileOutputStream ( outFile ) ;

         out.write ( string.getBytes() ) ;
         out.close() ;
         rc = true ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* insertTable
    */
   /**
    *    This method will insert a table into
    *  a WordProcessor
    *  
    *
    *  @param WordProcessorBean The word proc
    *  @param name The name of the table
    *  @param rows/cols
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void insertTable ( WordProcessor wp, String name, int rows, int cols ) {
      StringBuffer htmlText =  new StringBuffer ( new String ( "<TABLE NAME=\""+name+"\">" ) ) ;

      for ( int i = 0; i < rows; i++ ) {
         htmlText.append ( "<tr>" ) ;
         for ( int k = 0; k < cols; k++ ) {
            htmlText.append ( "<td><font face=\"Helvetica, Arial\", size=2></td>" ) ;
         }
         htmlText.append ( "</tr>" ) ;
      }

      htmlText.append ( "</TABLE>" ) ;
      saveFile ( "c:\\t.html", htmlText.toString() ) ;
      wp.insertDocument ( "file:/c:/t.html" ) ;
      deleteFile ( "c:\\t.html" ) ;

   }

   /*------------------------------------------------------------*/
   /* deleteFile
    */
   /**
    *    This method will delete the specified
    *  file
    *  
    *
    *  @param fileName The name of the file to delete
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void deleteFile ( String fileName ) {
      try {
         File file = new File ( fileName ) ;

         if ( file != null )
            file.delete() ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }





   /*------------------------------------------------------------*/
   /* generateTableHTMLText
    */
   /**
    *    This method will generate the HTML text
    *  from a table in the word processor
    *  
    *
    *  @param Table The Table object
    *  @param None
    *  @param None
    *
    *  @return TRUE if last item in the last cell is a link
    *//*---------------------------------------------------------*/
   private static boolean generateTableHTMLText ( StringBuffer htmlText, Table table ) {
      boolean rc = true ;

      notifyListener ( "Handling Table" ) ;

      if ( table != null ) {
         String       tableCellAlignment  = null ;
         int          cellCharCount       = 0 ;
         StringBuffer cellHTMLText        = null ;
         StringBuffer cellTagHTMLText     = null ;
         String       name                = table.getName() ;
         int          tableWidth          = table.getTableWidth() ;                
         int          rowCount            = table.getRowCount() ;
         int          colCount            = table.getColumnCount() ;
         TableCell    tableCell           = null ;
         int          row                 = 0 ;
         int          col                 = 0 ;
         boolean      cellContainedBreaks = false ;
         String       tableAlignment      = table.getAlignment() ;
         String       currentHTML         = stripLastBreak ( htmlText.toString() ) ;


         /* Strip out the last br tag before the table  */
         htmlText.setLength(0) ;
         htmlText.append ( currentHTML ) ;

         tableCount++ ;                               
         curAlignment = "left" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are in an ordered list, end it now                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( insideOrderedList ) {
            htmlText.append ( "</OL>" ) ;
            insideOrderedList = false ;
         }
         if ( insideBulletList ) {
            htmlText.append ( "</UL>" ) ;
            insideBulletList = false ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are in a different alignment than left, set it bak*/
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !curElement.getAlignment().equals ( "left" ) ) {
            if ( curElement.getAlignment().equals ( "right" ) )
               htmlText.append ( "</div>" ) ;
            else if ( curElement.getAlignment().equals ( "center" ) )
               htmlText.append ( "</center>" ) ;
            curAlignment="left"; 
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Put in our alignment                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( tableAlignment.equals ( "center" ) )
            htmlText.append ( "<center>" ) ;
         else if ( tableAlignment.equals ( "right" ) )
            htmlText.append ( "<div align=\"right\">" ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Generate the Table Header                               */
         /*   We will put in the width, border and alignment          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         wp.gotoObject ( "table", name ) ;
         htmlText.append( "\n<table name=\""+name+"\" " ) ;
         if ( tableWidth == 0 )
            htmlText.append ( "width=\"%100\" " ) ;
         else if ( tableWidth < 0 )
            htmlText.append ( "width=\"%"+Math.abs(tableWidth)+"\" " ) ;
         else
            htmlText.append ( "width=\""+tableWidth+"\" " ) ;

         htmlText.append ( "border="+table.getBorderWidth() ) ;

         htmlText.append ( ">\n" ) ;                                                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Put the <COL> width tags in for the Word Processor.     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         for ( col = 0; col < colCount; col++ ) {
            htmlText.append ( "<COL " ) ;
            convertWidth ( tableWidth, table.findTableCellByPosition ( row, col ).getColumnWidth(), htmlText ) ;
            htmlText.append ( "></COL>\n" ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through each cell                                  */
         /*   Generate the HTML text inside of each cell              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         for ( row = 0; row < rowCount; row++ ) {
            htmlText.append ( "<tr>\n" ) ;
            cellCharCount=0 ;
            for ( col = 0; col < colCount; col++ ) {
               boolean cont = true ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get the instance of the cell                            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               tableCell = table.findTableCellByPosition ( row, col ) ;

               if ( tableCell != null ) {

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Create the cell header                                  */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  cellTagHTMLText = new StringBuffer() ;
                  cellTagHTMLText.append ( "<td " ) ;

                  /* Put in the alignment  */
                  tableCellAlignment=getTableCellAlignment(tableCell);
                  cellTagHTMLText.append ( "align=\""+tableCellAlignment+"\" ") ;

                  /* Put in vertical alignment  */
                  cellTagHTMLText.append ( "valign=\"top\" " ) ;

                  /* Put in the width */
                  cellTagHTMLText.append ( "width=\""+tableCell.getColumnWidth()+"\">\n" ) ;               

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Put the text in the cell                                */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  cellHTMLText = new StringBuffer ( "" ) ;
                  wp.gotoObject ( "tablecell", tableCell.getName() ) ;
                  try {
                     while ( cont ) {

                        curChar    = wp.getCharacter() ;
                        if ( !handleGraphics ( htmlText, true ) )
                           gotoNextChar("generateTableHTMLText, normal step inside of cell" ) ;

                        if ( ((TableCell)wp.findCurrentObject ( "tablecell" )).getName().equals ( tableCell.getName() ) ) {

                           if ( wp.findCurrentObject ( "link" ) != null ) {
                              generateLinkHTMLText ( cellHTMLText, false ) ;
                              cont = ((TableCell)wp.findCurrentObject ( "tablecell" )).getName().equals ( tableCell.getName() ) ;

                              /* Return TRUE if this link ends in the last cell  */
                              rc = ( ( ( cont ) &&
                                       ( row == (rowCount-1) ) &&
                                       ( col == (colCount-1) ) ) ) ;
                           }

                           if ( cont ) {
                              cellCharCount++ ;
                              generateElementHTMLText ( cellHTMLText, true, false, false ) ;
                           }
                        } else
                           cont = false ;
                     }
                  } catch ( NullPointerException e ) {
                     // Will be thrown for the last cell
                  }

                  /* This has to be here because the Word Processor will not show the
                     first break if the alignment is not left unless we put a break in  */
                  if ( cellHTMLText.toString().indexOf ( "<br>" ) != -1 )
                     cellTagHTMLText.append ( "<WP_CELL_BR>" );

                  /* Put in the font tag  */
                  cellTagHTMLText.append ( "<font face=\""+curFontName+"\" size="+getFontSize()+">" ) ;

                  /* Append our cell tag, and cell text  */
                  htmlText.append ( cellTagHTMLText.toString() ) ;
                  htmlText.append ( cellHTMLText.toString().trim() ) ;

                  /* Close off any alignemnts  */
                  if ( curAlignment.equals ( "center" ) )
                     htmlText.append ( "</center>" ) ;
                  else if ( curAlignment.equals ( "right" ) )
                     htmlText.append ( "</div>" ) ;
                  curAlignment="left" ;

                  closeTags(htmlText) ;

                  /* Close off the cell tag  */
                  if ( cellCharCount > 0 )
                     htmlText.append ( "</font></td>\n" ) ;
                  else
                     htmlText.append ( "&nbsp;</font></td>\n" ) ;
               }

            }

            /* Close off the row tag  */
            htmlText.append ( "</tr>\n" ) ;
         }

         /* Close out the tags  */
         htmlText.append ( "</table>\n" ) ;           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   End in our alignment                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( tableAlignment.equals ( "center" ) )
            htmlText.append ( "</center>" ) ;
         else if ( tableAlignment.equals ( "right" ) )
            htmlText.append ( "</div>" ) ;

         /* Move the cursor out of the table  */
         try {
            wp.gotoObject ( "bookmark", "TABLE-"+name ) ;
            while ( ((Table)wp.findCurrentObject ( "table" )).getName().equals ( table.getName() ) )
               gotoNextChar ( "generateTableHTMLText, Advancing after table" ) ;
         } catch ( Exception e ) {
         }
      }

      return(rc) ;
   }


   /*------------------------------------------------------------*/
   /* generateLinkHTMLTest
    */
   /**
    *    This method will walk through a link, generating
    *  all text inside of it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void generateLinkHTMLText ( StringBuffer htmlText, boolean handleAlignment ) {
      Link     link = (Link) wp.findCurrentObject ( "link" ) ;
      boolean  cont = true ;

      notifyListener ( "Handling Link" ) ;

      htmlText.append ( "<a href=\"" ) ;
      htmlText.append ( link.getDestination() ) ;
      htmlText.append ( "\">" ) ;                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check to see if there is a graphic starting the link    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      gotoPreviousChar ( "GenerateLinkHTMLText, Checking for Image" ) ;
      handleGraphics ( htmlText, false ) ;

      if ( ( curChar != null ) && ( curChar.length() > 0 ) ) {
         htmlText.append ( curChar ) ;
         gotoNextChar ( "generateLinkHTMLText, Stepping after first character in Link" ) ;
      }

      try {
         while ( cont ) {

            /* If we are inside of the same link, process the text  */
            curChar    = wp.getCharacter() ;
            gotoNextChar( "generateLinkHTMLText, Stepping in link" ) ;

            if ( ( wp.findCurrentObject("link") != null ) &&
                 ( ((Link)wp.findCurrentObject ( "link" )).getDestination().equals ( link.getDestination() ) ) ) {
               generateElementHTMLText ( htmlText, false, true, handleAlignment ) ;
            } else if ( wp.findCurrentObject ( "link" ) == null )
               cont = false;

         }
      } catch ( Exception e ) {
         if ( !(e instanceof NullPointerException) )
            e.printStackTrace() ;
      }

      htmlText.append ( "</a>" ) ;

   }





   /*------------------------------------------------------------*/
   /* applyAnchorsToTables
    */
   /**
    *    This method will apply an anchor tag
    *  to every table if the table does not
    *  already have one!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void applyAnchorsToTables () {
      WPEnumeration wpEnum    = wp.enumerateTables() ;
      String        tableName = null ;

      notifyListener ( "Adding anchors to tables" ) ;

      while ( wpEnum.hasMoreNames() ) {
         tableName = wpEnum.nextName() ;

         /* If we can't go to the bookmark, it doesn't exist!  */
         try {
            if ( !wp.gotoObject ( "bookmark", "TABLE-"+tableName ) ) {
               if ( wp.gotoObject ( "table", tableName ) ) {
                  gotoPreviousChar ( "applyAnchorsToTables" ) ;
                  wp.gotoNextCharacter ( true ) ;
                  wp.insertBookmark ( "TABLE-"+tableName ) ;
               }
            }
         } catch ( Exception e ) {
         }
      }
   }


   /*------------------------------------------------------------*/
   /* removeAnchorsFromTables
    */
   /**
    *    This method will apply an anchor tag
    *  to every table if the table does not
    *  already have one!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void removeAnchorsFromTables () {
      WPEnumeration wpEnum    = wp.enumerateTables() ;
      String        tableName = null ;

      notifyListener ( "Removing anchors from tables" ) ;

      while ( wpEnum.hasMoreNames() ) {
         tableName = wpEnum.nextName() ;
         wp.deleteBookmark ( "TABLE-"+tableName ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getFontName
    */
   /**
    *    This method will return the correct HTML
    *  fontname for the current font in the curElement.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private static String getFontName () {
      String fontName = curElement.getFontName() ;

      if ( ( fontName.indexOf ( "ourie" ) > 0 ) ||
           ( fontName.indexOf ( "onospac" ) > 0 ) )
         curFontName = "Courier" ;
      else
         curFontName = "Helvetica, Arial" ;

      return( curFontName ) ;
   }




   /*------------------------------------------------------------*/
   /* getFontSize
    */
   /**
    *    This method will return the correct HTML
    *  font size for the curElement
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static int getFontSize () {
      int fontSize = curElement.getFontSize() ;

      curFontSize = fontSize ;
      switch ( fontSize ) {
         case 12 :
            return( 3 ) ;
         case 10 :
            return( 2 ) ;
         case 8  :
            return( 1 ) ;
      }

      return( 3 ) ;
   }





   /*------------------------------------------------------------*/
   /* missedTables
    */
   /**
    *    This method will return TRUE if the word processor
    *  missed tables after the generation of the HTML
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean missedTables () {
      WPEnumeration wpEnum    = wp.enumerateTables() ;
      int           wpCount   = 0 ;

      while ( wpEnum.hasMoreNames() ) {
         if ( wpEnum.nextName() != null )
            wpCount++ ;
      }

      return( tableCount != wpCount ) ;

   }
   /*------------------------------------------------------------*/
   /* missedGraphics
    */
   /**
    *    This method will return TRUE if the word processor
    *  missed graphics after the generation of the HTML
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean missedGraphics () {
      WPEnumeration wpEnum    = wp.enumerateImages() ;
      int           wpCount   = 0 ;

      while ( wpEnum.hasMoreNames() ) {
         if ( wpEnum.nextName() != null )
            wpCount++ ;
      }

      return( graphicCount != wpCount ) ;

   }




   /*------------------------------------------------------------*/
   /* stripLastBreak
    */
   /**
    *    This method will take out the last <br>
    *  tag in the string passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The modified string
    *//*---------------------------------------------------------*/
   private static String stripLastBreak ( String str ) {
      int    startIndex = str.lastIndexOf ( "<br>" ) ;
      int    stopIndex  = startIndex+4 ;

      if ( ( startIndex >=0 ) && ( stopIndex >= 0 ) ) {
         return( str.substring (0, startIndex )+str.substring ( stopIndex ) ) ;
      }
      return( str ) ;
   }





   /*------------------------------------------------------------*/
   /* convertWidth
    */
   /**
    *    This method will convert a pixel width
    *  of a cell to it's percentage
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void convertWidth ( int tableWidth, int cellWidth, StringBuffer htmlText ) {

      /* If the table width is less than 0, it is a percentage  */
      if ( tableWidth < 0 ) {
         tableWidth=tableWidth/100 ;
         tableWidth=tableWidth*624 ;
      }

      if ( tableWidth==0 )
         tableWidth=624;

      /* Get it's percentage  */
      cellWidth = Math.abs(cellWidth*100/(tableWidth));

      htmlText.append ( "WIDTH=\""+cellWidth+"%\"" ) ;

   }





   /*------------------------------------------------------------*/
   /* getTableCellAlignment
    */
   /**
    *    This method will return a string representation
    *  of what the alignment of a table cell should be. This
    *  alignment will be the first non-left alignment paragraph in the cell
    *
    *  @param tableCell theTable Cell
    *  @param 
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private static String getTableCellAlignment ( TableCell tableCell ) {
      String  rc   = "left" ;
      boolean cont = true ;

      wp.gotoObject ( "tablecell", tableCell.getName() ) ;

      while ( cont ) {
         if ( !wp.findSelectedTextElement().getAlignment().equals ( "left" ) ) {
            cont = false ;
            rc = wp.findSelectedTextElement().getAlignment() ;
         }
         gotoNextChar("getTableCellAlignment");
         if ( wp.findCurrentObject("tablecell") != tableCell )
            cont = false ;
      }

      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* gotoPreviousChar
    */
   /**
    *    Advances the Word Processor's cursor
    *  one to the left
    *  
    *
    *  @param method The name of the method calling
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean gotoPreviousChar ( String method ) {
      boolean rc = false ;

      try {
         rc = wp.gotoPreviousCharacter(false) ;
      } catch ( Exception e ) {
      }
      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* gotoNextChar
    */
   /**
    *    Advances the Word Processor's cursor
    *  one to the right
    *  
    *
    *  @param method The name of the method calling
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean gotoNextChar ( String method ) {
      boolean rc = false ;

      try {
         rc = wp.gotoNextCharacter(false) ;
      } catch ( Exception e ) {
      }

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* handleGraphics
    */
   /**
    *    This method will put an img tag into the
    *  html text if the cursor is positioned on 
    *  a graphic
    *
    *  @param htmlText The HTML text to append the tags to
    *  @param gotoNextChar TRUE = advance cursor after image
    *  @param None
    *
    *  @return TRUE if graphic was found
    *//*---------------------------------------------------------*/
   private static boolean handleGraphics ( StringBuffer htmlText, boolean gotoNextChar ) {
      boolean rc = false ;

      if ( wp.findCurrentObject ( "image" ) != null ) {
         if ( ( curChar != null ) && ( curChar.length() > 0 ) )
            htmlText.append ( curChar ) ;
         insertImage ( htmlText ) ;
         if ( gotoNextChar )
            gotoNextChar ( "GenerateElementHTMLText, Stepping after image" ) ;
         curChar="" ;
         rc = true ;
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* handleBold
    */
   /**
    *    This method will handle putting the bold
    *  tags in to make text appear bold, and to turn
    *  off bold
    *
    *  @param htmlText The HTML text to append the tags to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void handleBold ( StringBuffer htmlText ) {
      if ( curElement.getBold() ) {
         if ( inBold == 0 ) {
            htmlText.append (    "<b>" ) ;
            inBold++ ;
         }
      } else {
         if ( inBold > 0 ) {
            inBold-- ;
            if ( inBold == 0 )
               htmlText.append (    "</b>" ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* handleItalic
    */
   /**
    *    This method will either begin an italic
    *  piece of text, or it will end it.
    *  
    *
    *  @param htmlText The HTML text to append the tags to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void handleItalic ( StringBuffer htmlText ) {
      if ( curElement.getItalic() ) {
         if ( inItalic == 0 ) {
            htmlText.append (    "<i>" ) ;
            inItalic++ ;
         }
      } else {
         if ( inItalic > 0 ) {
            inItalic-- ;

            if ( inItalic == 0 )
               htmlText.append (    "</i>" ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* handleLists
    */
   /**
    *    This method will start the ordered and bulleted
    *  lists.
    *  
    *
    *  @param htmlText The HTML text to append the tags to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void handleLists ( StringBuffer htmlText ) {
      if ( curParagraph.getName().startsWith ( "OLLI" ) ) {
         if ( insideBulletList ) {
            insideBulletList = false ;
            htmlText.append ( "\n</UL>\n" ) ;
         }

         if ( !insideOrderedList ) {
            String curHTMLText = htmlText.toString() ;

            htmlText.setLength(0) ;
            htmlText.append ( stripLastBreak(curHTMLText) ) ;
            htmlText.append ( "\n<OL><LI>\n" ) ;
            insideOrderedList = true ;
         }
      } else if ( curParagraph.getName().startsWith ( "ULLI" ) ) {
         if ( insideOrderedList ) {
            insideOrderedList = false ;
            htmlText.append ( "\n</OL>\n" ) ;
         }

         if ( !insideBulletList ) {
            String curHTMLText = htmlText.toString() ;

            htmlText.setLength(0) ;
            htmlText.append ( stripLastBreak(curHTMLText) ) ;
            htmlText.append ( "\n<UL>\n<LI>" ) ;
            insideBulletList = true ;
         }
      } else {
         if ( insideOrderedList ) {
            insideOrderedList = false ;
            htmlText.append ( "\n</OL>\n" ) ;
         }
         if ( insideBulletList ) {
            insideBulletList = false ;
            htmlText.append ( "\n</UL>\n" ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* handleFontName
    */
   /**
    *    This method will put in the HTML Tags if
    *  a Font Name changes
    *  
    *
    *  @param htmlText The HTML text to append the tags to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void handleFontName ( StringBuffer htmlText ) {
      if ( ( curElement.getFontName().indexOf ( curFontName ) == -1 ) &&
           ( curFontName.indexOf ( curElement.getFontName() ) == -1 ) &&
           ( curElement.getFontName().indexOf ( "imes" ) == -1 ) &&
           ( curElement.getFontName().indexOf ( "erif" ) == -1 ) ) {
         inFont=1 ;

         if ( !firstFontChange ) {
            if ( curFontName.indexOf ( "rier" ) != -1 ) {
               htmlText.append ( "</pre>" ) ;
               inPre-- ;
            }
            htmlText.append ( "</font>" ) ;
         } else
            firstFontChange = false ;

         if ( getFontName().equals ( "Courier" ) ) {
            htmlText.append ( "<pre>" ) ;
            inPre++ ;
         }

         htmlText.append ( "<font face=\""+getFontName()+"\" size="+getFontSize()+" color=\""+getFontColor()+"\">" ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* handleFontSize
    */
   /**
    *    This method will put in font tags if the 
    *  size of a font has changed
    *  
    *
    *  @param htmlText The current buffer to append the tags to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void handleFontSize ( StringBuffer htmlText ) {
      if ( curElement.getFontSize() != curFontSize ) {
         inFont=1 ;
         if ( !firstFontChange )
            htmlText.append ( "</font><font size="+getFontSize()+" face=\""+getFontName()+"\" color=\""+getFontColor()+"\" >" ) ;
         else {
            htmlText.append ( "<font size="+getFontSize()+" face=\""+getFontName()+"\" color=\""+getFontColor()+"\">" ) ;
            firstFontChange=false;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* handleParagraphAlign
    */
   /**
    *    This method will handle the paragraph
    *  alignment changes by putting in div and center
    *  tags
    *
    *  @param htmlText The current buffer to append the tags to
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void handleParagraphAlign ( StringBuffer htmlText ) {
      if ( curElement == null )
         curElement = wp.findSelectedTextElement() ;

      if ( curElement != null ) {
         if ( !curElement.getAlignment().equals ( curAlignment ) ) {

            if ( curAlignment.equals ( "center" ) )
               htmlText.append ( "</center>" ) ;
            else if ( curAlignment.equals ( "right" ) )
               htmlText.append ( "</div>" ) ;

            curAlignment = curElement.getAlignment() ;
            if ( curAlignment.equals ( "center" ) )
               htmlText.append ( "<center>" ) ;
            else if ( curAlignment.equals ( "right" ) )
               htmlText.append ( "<div align=\"right\">" ) ;

         }
      }
   }





   /*------------------------------------------------------------*/
   /* handleFontColor
    */
   /**
    *    This method will handle changes in the
    *  font color
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void handleFontColor ( StringBuffer htmlText ) {
      if ( curElement.getTextColor() != curFontColor ) {
         curFontColor = curElement.getTextColor() ;
         inFont=1 ;
         if ( !firstFontChange )
            htmlText.append ( "</font><font color=\""+getFontColor()+"\" size="+getFontSize()+" face=\""+getFontName()+"\" >" ) ;
         else {
            htmlText.append ( "<font color=\""+getFontColor()+"\" size="+getFontSize()+" face=\""+getFontName()+"\">" ) ;
            firstFontChange=false;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* getFontColorText
    */
   /**
    *    This method will return the current Color
    *  as a string
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private static String getFontColor () {
      Color  color    = wp.findSelectedTextElement().getTextColor() ;
      String colorStr = "black" ;

      if ( color.equals ( Color.green ) )
         colorStr = "green" ;
      else if ( color.equals ( Color.gray ) )
         colorStr = "gray" ;
      else if ( ( color.getRed()   == 0 ) &&
                ( color.getBlue()  == 255 ) &&
                ( color.getGreen() == 0 ) )
         colorStr = "blue" ;
      else if ( ( color.getRed()   == 255 ) &&
                ( color.getBlue()  == 0 ) &&
                ( color.getGreen() == 0 ) )
         colorStr = "red" ;

      return( colorStr ) ;
   }




   /*------------------------------------------------------------*/
   /* handleLineBreak
    */
   /**
    *    This method will put a line break <br> tag
    *  into the text if the 'processNextBreak' flag is
    *  true and we are not in the <pre></pre> tags. If
    *  we are in <pre> mode, a <WP_PRE_BR> tag will be put int
    *  so that when text is put back into the word processor, text
    *  will be formatted correctly
    *
    *  @param htmlText The text to append to
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void handleLineBreak ( StringBuffer htmlText ) {
      if ( processNextBreak ) {
         if ( inPre == 0 ) {
            htmlText.append ( "<br>\n" ) ;
         } else
            htmlText.append ( "<WP_PRE_BR>\n" ) ;
      } else
         processNextBreak = true ;
   }




   /*------------------------------------------------------------*/
   /* notifyListener
    */
   /**
    *    THis method will notify our listener of
    *  a status change
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void notifyListener ( String status ) {
      if ( statusListener != null )
         statusListener.HTMLGeneratorStatus ( status ) ;
   }
} /* WPHTMLGenerator */

