package com.ibm.nzna.projects.qit.wordproc ;
/*-----------------------------------------------------------*/
/*   WordProcUtil                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WordProcUtil                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class holds static methods that the Word Processor will use 
 *
 *
 * @version 1.0, 5/24/1999 
 * @author  David Lentz
 */
import javax.swing.* ;
import javax.swing.text.* ;
import javax.swing.text.html.* ;
import java.awt.* ;
import java.io.* ;
import java.util.* ;
import java.sql.* ;
import lotus.wp.WordProcessorBean;
import lotus.wp.* ;
import lotus.wp.ifx.* ;
import lotus.fc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.util.CDate ;
import com.ibm.nzna.shared.db.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   WordProcUtil                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class WordProcUtil extends Object implements AppConst, DocConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final Insets taskBarButtonMargin    = new Insets ( 0, 0, 0, 0 ) ;
   private static final String metaGenerator          = "<META NAME=Generator CONTENT=QuestInput>" ;
   private static final String defaultFontStr         = "<font face=\"Helvetica, Arial\" size=2>" ;



   /*------------------------------------------------------------*/
   /* destroyWordProcessor
    */
   /**
    *    This method will destroy a word processor
    *  and hopefully the garbage collector will
    *  free up it's resources
    *
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void destroyWordProcessor ( WordProcessorBean wp_PANE ) {
      if ( wp_PANE != null ) {
         wp_PANE.stop() ;
         wp_PANE.destroy() ;
         System.gc() ;
      }
   }


   /*------------------------------------------------------------*/
   /* setPageLayout
    */
   /**
    *    This method will set our page layout for the
    *  WordProcessor document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setPageLayout ( lotus.wp.WordProcessor wp_PANE ) {
      wp_PANE.findSelectedTextElement().setFontName ( "Helvetica, Arial" ) ;
      wp_PANE.findSelectedTextElement().setFontSize ( 10 ) ;
      //wp_PANE.setScaleFonts ( true ) ;
      //wp_PANE.setFontScaleDivisor ( 3 ) ;
      //wp_PANE.setFontScaleMultiplier ( 4 ) ;
   }


   /*------------------------------------------------------------*/
   /* addKeyListeners
    */
   /**
    *    This method will take the container passed in
    *  and add key listeners to that container and
    *  all components inside of that container recurse.
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void addKeyListeners ( java.awt.event.KeyListener wp, java.awt.Container container ) {
      int       compCount = container.getComponentCount() ;
      int       i         = 0 ;
      Component comp      = null ;

      while ( i < compCount ) {
         comp = container.getComponent ( i++ ) ;
         if ( comp instanceof java.awt.Container )
            addKeyListeners ( wp, (java.awt.Container) comp ) ;

         comp.addKeyListener ( wp ) ;
      }

      /* Add a key listener to the container */
      container.addKeyListener(wp) ;

   } /* addKeyListeners */


   /*-----------------------------------------------------------*/
   /* setDocContent
    */
   /**
    *    This method will set the document's content
    *  
    *  
    *             
    *  @param docContent the content to set
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public static void setDocContent ( WordProcessorBean wp_PANE, DocumentDraft document, String docContent ) {

      MainWindow.getInstance().setStatus ( Str.getStr ( STR_LOADING_DOC ) ) ;

      try {
         if ( wp_PANE != null ) {

            if ( docContent == null )
               docContent = "" ;

            /* Prepare the HTML so that the Word Processor displays it
               correctly */
            docContent = prepareHTMLForEdit ( document, docContent ) ;


            /* Set the Word Processor's contents */
            MainWindow.getInstance().setStatus ( Str.getStr ( STR_SETTING_DOC_CONTENT ) ) ;
            wp_PANE.setDocumentContentsAsHTML ( docContent ) ;

         } /* if */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      MainWindow.getInstance().setStatus ( null ) ;
   } /* setDocContent */



   /*------------------------------------------------------------*/
   /* prepareHTMLForEdit
    */
   /**
    *    This method will take a plain HTML Document
    *  loaded from the Quest database, attach the
    *  head and body tags, including BGCOLOR for use in the WordProcessorControl
    *
    *  @param htmlString The HTML String to prepare
    *
    *  @return String the prepared HTML
    *//*---------------------------------------------------------*/
   protected static String prepareHTMLForEdit ( DocumentDraft docRec, String htmlString ) {
      String       retStr   = null ;
      StringBuffer bodyHTML = new StringBuffer() ;

      try {
         boolean previousDoc = false ;                                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, see if this word proc has edited this doc        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( htmlString != null ) {
            FileUtil.saveFile ( "input.html", htmlString ) ;

            if ( htmlString.indexOf ( metaGenerator ) != -1 )
               previousDoc = true ;
            else
               previousDoc = false ;
         } else
            previousDoc = false ;                  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Form the header to the HTML                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         bodyHTML.append ( "<HTML><HEAD>\n" ) ;
         bodyHTML.append ( "<META NAME=Generator CONTENT=Quest Input Tool></META>\n" );
         bodyHTML.append ( "<META NAME=Version CONTENT=1.0></META>\n" ) ;
         bodyHTML.append ( "<META HTTP-EQUIV=Content-Type CONTENT=text/html; charset=UTF-8></META>\n" ) ;                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Put in Lotus' Style Sheet                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         bodyHTML.append ( "<STYLE TYPE=text/css>\n" ) ;
         bodyHTML.append ( "<!--\n" ) ;
         bodyHTML.append ( "@page {\n" ) ;
         bodyHTML.append ( "     size: 8.5in 11in;\n" ) ;
         bodyHTML.append ( "     margin-top: 1in; \n" ) ;
         bodyHTML.append ( "     margin-bottom: 1in; \n" ) ;
         bodyHTML.append ( "     margin-left: 1in; \n" ) ;
         bodyHTML.append ( "     margin-right: 1in; \n" ) ;
         bodyHTML.append ( "    }\n" ) ;
         bodyHTML.append ( "P {\n" ) ;
         bodyHTML.append ( "     font-family: Helvetica;\n" ) ;
         bodyHTML.append ( "     font-size: 10pt;\n" ) ;
         bodyHTML.append ( "    }\n" ) ;
         bodyHTML.append ( "-->\n" ) ;
         bodyHTML.append ( "</STYLE>\n" ) ;

         /* End the header and begin the body */
         bodyHTML.append ( "</HEAD><BODY text=black BGCOLOR=white>" ) ;                                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Migrator put blank HREF tags in, remove them now        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( htmlString != null ) {
            htmlString = Text.replaceAllStrInStr ( htmlString, "<a href=\"\"></a>", "" ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Append the doc html                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( htmlString != null ) {
            bodyHTML.append ( defaultFontStr ) ;
            bodyHTML.append ( htmlString.trim() ) ;
            bodyHTML.append ( "</BODY></HTML>" ) ;
         } else {
            bodyHTML.append ( defaultFontStr ) ;
            bodyHTML.append ( "<br></BODY></HTML>" ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Fix bad html tags that lotus does not handle            */                                   
         /*                                                           */
         /*-----------------------------------------------------------*/
         retStr = fixBodyTagsForEdit ( docRec, bodyHTML.toString() ) ;
         retStr = Text.replaceAllStrInStr ( retStr, "<br>", "<p>" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Free up some mem                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         bodyHTML.setLength(0) ;

      } catch ( Exception e ) {
      }

      System.gc() ;

      MainWindow.getInstance().setStatus ( null ) ;

      return( retStr ) ;
   } /* prepareHTMLForEdit */






   /*------------------------------------------------------------*/
   /* createHTML
    */
   /**
    *    This method will create the HTML out
    *  of the word processor. The String returned
    *  will be formatted good enough for the Quest database
    *
    *  @param wordProc The WordProcessorBean
    *  @param docRec The docRec. Used for generating numbers
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String createHTML ( WordProcessorBean wordProc, DocumentDraft docRec ) {
      String  retStr = null ;

      try {
         String                    lotusHTML = null ;
         StringWriter              writer    = new StringWriter () ;
         String                    tempStr   = null ;

         /* Get the HTML For Lotus */
         MainWindow.getInstance().setStatus ( STR_WAITING_FOR_LOTUS_HTML_PARSE ) ;
         lotusHTML = wordProc.getDocumentContentsAsHTML() ;

         /* Strip out everything but the body */
         /* Get the body out of the Full HTML Lotus gives us */
         MainWindow.getInstance().setStatus ( Str.getStr ( STR_FIXING_HTML)+"." ) ;
         lotusHTML = getDocBody ( lotusHTML ) ;
         FileUtil.saveFile ( PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+"bad.html", lotusHTML ) ;

         /* Ok, now fix things ourselves */
         MainWindow.getInstance().setStatus ( STR_FIXING_HTML ) ;
         retStr = fixBodyTagsForSave ( lotusHTML ) ;

         /* If we went through our filtering, do a little more on the HTML */
         if ( retStr != null ) {

            /* Take our the lotus character preserve and put in the HTML one */
            retStr = Text.replaceAllStrInStr ( retStr, "&#160;", "&nbsp;" ) ;

            /* Assemble All of the graphic links */
            MainWindow.getInstance().setStatus ( Str.getStr ( STR_ASSEMBLING_GRAPHICS ) ) ;
            assembleGraphics ( wordProc, docRec ) ;

            /* Prepend our meta tag, so we know this has gone through */
            retStr = metaGenerator+retStr ;
         }
         FileUtil.saveFile ( PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+"output.html", retStr ) ;


         writer.close() ;
         writer    = null ;
         lotusHTML = null ;
         System.gc() ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      MainWindow.getInstance().setStatus ( null ) ;
      return( retStr ) ;
   }


   /*-----------------------------------------------------------*/
   /* fixBodyTagsForEdit
    */
   /**
    *    This method will fix the editing tags
    *  
    *             
    *  @param HTMLString The HTML String
    *  @param statusWin The StatusWin to set our text on
    *  @param 
    *
    *  @return 
    */
   protected static String fixBodyTagsForEdit ( DocumentDraft docRec, String HTMLString ) {
      String  retStr         = HTMLString ;
      try {
         String  separator      = File.separator ;
         boolean inList         = false ;
         boolean inPar          = false ;
         String  tag            = null ;
         String  tempGraphicDir = "FILE:/"+PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator ;



         if ( HTMLString != null ) {
            HTMLParse parse = new HTMLParse ( HTMLString ) ;
            int       i     = 0 ;

            MainWindow.getInstance().setStatus ( Str.getStr ( STR_FIXING )+" "+parse.tagCount()+" "+Str.getStr ( STR_HTML_TAGS ) ) ;

            while ( i < parse.tagCount() ) {
               tag = parse.getTag(i).toUpperCase() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   ul, UL, ol, OL                                          */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( tag.equals ( "<UL>" ) ) ||
                    ( tag.equals ( "<OL>" ) ) )
                  inList = true ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   /ul, /UL, /ol, /OL                                      */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( ( tag.equals ( "</UL>" ) ) ||
                         ( tag.equals ( "</OL>" ) ) )
                  inList = false ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   <br> Take out if in a list, other wise put in a <P>     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tag.equals ( "<BR>" ) ) {
                  if ( inList )
                     parse.removeTag(i) ;
                  //else
                  //   parse.replaceTag ( i, "<p>" ) ;
               }

               else if ( tag.startsWith ( "<UL" ) )
                  parse.replaceTag ( i, "<UL>" ) ;

               else if ( tag.startsWith ( "<LI" ) )
                  parse.replaceTag ( i, "<LI>" ) ;



               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   <sup>                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tag.equals ( "<SUP>" ) ) {
                  parse.replaceTag ( i, "<A NAME=\""+Str.getStr ( STR_SUPERSCRIPT )+getRandomName(docRec)+"\"><FONT SIZE=\"1\" STYLE=\"font-size: 8pt\">" ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   </sup>                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tag.equals ( "</SUP>" ) ) {
                  parse.replaceTag ( i, "</FONT></A>" ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   <sub>                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tag.equals ( "<SUB>" ) ) {
                  parse.replaceTag ( i, "<A NAME=\""+Str.getStr ( STR_SUBSCRIPT )+getRandomName(docRec)+"\"><FONT SIZE=\"1\" STYLE=\"font-size: 8pt\">" ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   <ITP> Internal Ending Paragraph                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tag.equals ( "<ITP>" ) ) {
                  parse.replaceTag ( i, "</P>" ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   </sub>                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tag.equals ( "</SUB>" ) ) {
                  parse.replaceTag ( i, "</font></a>" ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Image tags                                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tag.startsWith ( "<IMG SRC=" ) ) {
                  String startImg  = null ;
                  String imageFile = null ;
                  String end       = null ;
                  String tempTag   = null ;

                  /* Get the original case */
                  tag = parse.getTag(i) ;

                  /* Strip our the information we care about */
                  startImg  = "<IMG SRC=\""+tempGraphicDir ;
                  imageFile = tag.substring ( tag.indexOf ( "\"" ), tag.indexOf ( "\"", tag.indexOf ( "\"" )+1 ) ) ;
                  end       = tag.substring ( tag.indexOf ( "\"", tag.indexOf ( "\"" )+1 ), tag.length() ) ;

                  /* Strip out the temp directory info */
                  imageFile = imageFile.substring ( imageFile.lastIndexOf ( "/" )+1, imageFile.length() ) ;

                  /* if by chance they had another double quote in the string, strip it out */
                  imageFile = Text.replaceAllStrInStr ( imageFile, "\"", "" ) ;

                  /* Create the image tag */
                  tempTag = startImg+imageFile.substring ( imageFile.lastIndexOf ( separator )+1, imageFile.length() )+end ;
                  tempTag = Text.replaceAllStrInStr ( tempTag, "###IMAGELOC###", "" ) ;
                  tempTag = Text.replaceAllStrInStr ( tempTag, "###imageloc###", "" ) ;

                  /* Ok, if this is our doc link graphic, add an underscore in front of it */
                  //if ( tempTag.indexOf ( "doclink.gif" ) >= 0 )
                  //   tempTag = "_"+tempTag ;

                  /* Replace the tag */
                  parse.replaceTag ( i, tempTag ) ;

               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If no tag was hit, increment i                          */
               /*                                                           */
               /*-----------------------------------------------------------*/
               i++ ;


            }

            retStr = parse.getHTMLString() ;
            parse.free() ;
            parse = null ;
            System.gc() ;

         } /* if */
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }


      return( retStr ) ;
   }



   /*-----------------------------------------------------------*/
   /* assembleDocLinks
    */
   /**
    *    This method will go through all of the DocLinkRecs
    *  in the DocRec and delete the ones that
    *  do not exist in the document body. The user could have
    *  backspaced over a doc link, or renamed it's notes id.
    *             
    *  @param HTMLString the HTMLString representing the body
    *  @param 
    *  @param 
    *
    *  @return 
    */
   protected static void assembleDocLinks ( WordProcessorBean wp, DocumentDraft docRec, String HTMLString ) {

      MainWindow.getInstance().setStatus ( Str.getStr ( STR_ASSEMBLING_LINKS ) ) ;
      if ( docRec != null ) {
         Vector docLinksVec = docRec.getDocumentLinks() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have doc links already, check them now            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( docLinksVec != null ) {
            WPEnumeration linkList = wp.enumerateLinks() ;
            String        curLink  = null ;
            Link          linkRec  = null ;

            if ( ( linkList != null ) && ( linkList.hasMoreNames() ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Remove everything first                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               docLinksVec.removeAllElements() ;                  

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through and add each one                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( linkList.hasMoreNames() ) {
                  linkRec = wp.findLink ( linkList.nextName() ) ;

                  /* If we get a link record, find it */
                  if ( linkRec != null ) {
                     if ( !docHasLink ( docLinksVec, linkRec ) )
                        docLinksVec.addElement ( createDocLink ( docRec, linkRec) ) ;
                  }
               }

            } else
               docLinksVec.removeAllElements() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, just add each one                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            WPEnumeration linkList = wp.enumerateLinks() ;
            String        curLink  = null ;
            Link          linkRec  = null ;

            if ( ( linkList != null ) && ( linkList.hasMoreNames() ) ) {

               docLinksVec = new Vector ( 10, 10 ) ;
               while ( linkList.hasMoreNames() ) {
                  linkRec = wp.findLink ( linkList.nextName() ) ;

                  /* If we get a link record, find it */
                  if ( linkRec != null ) {
                     DocumentLink docLinkRec = createDocLink ( docRec, linkRec ) ;

                     if ( docLinkRec != null )
                        docLinksVec.addElement ( docLinkRec ) ;
                  }
               } /* while */

            }

         }

         docRec.setDocumentLinks ( docLinksVec ) ;
      } /* if docRec */
      MainWindow.getInstance().setStatus ( null ) ;
   } /* assembleDocLinks */

   private static DocumentLink createDocLink ( DocumentDraft document, Link linkRec ) {
      String       rtnName = "createDocLink" ;
      SQLMethod    sql    = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      DocumentLink retRec = null ;

      if ( sql != null ) {
         try {
            String    dest  = linkRec.getDestination() ;

            if ( ( dest != null ) && ( dest.indexOf ( "." ) > 0 ) ) {
               String    docId = dest.substring ( 0, dest.indexOf ( "." ) ) ;
               Statement stmt  = sql.createStatement() ;

               if ( ( docId.length()==11 ) && ( docId.indexOf ( "-" ) == 4 ) ) {
                  ResultSet rs    = stmt.executeQuery ( "SELECT DOCIND, TITLE, DOCCLASSIND, LNDOCID "+
                                                        "FROM TIGRIS.DOCUMENTS "+
                                                        "WHERE LNDOCID = '"+docId+"' FOR FETCH ONLY" ) ;

                  if ( rs.next() ) {
                     retRec = new DocumentLink ( document.getDocInd(), rs.getInt(1), rs.getString(2).trim() ) ;
                  } else
                     retRec = null ;

                  rs.close() ;
               }

            }
         } catch ( Exception e ) {
            sql.rollBack() ;
            LogSystem.log ( 1, e ) ;
         }
      }

      sql.close() ;

      return( retRec ) ;
   }




   /*------------------------------------------------------------*/
   /* docHasLink
    */
   /**
    *    This method will go through the vector
    *  passed and tries to find the linkRec
    *  inside of it
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean docHasLink ( Vector vec, Link linkRec ) {
      return ( false ) ;
      //String  dest  = linkRec.getDestination() ;
      //boolean rc    = false ;

      //if ( ( vec != null ) && ( vec.size() > 0 ) ) {

      //   /* Only process this if it has a period */
      //   if ( dest.indexOf ( "." ) >= 0 ) {
      //      String docId = dest.substring ( 0, dest.indexOf ( "." ) ) ;

      //      /* Now, only worry if this is a true doc link */
      //      if ( ( docId.length()==11 ) && ( docId.indexOf ( "-" ) == 4 ) ) {
      //         int i      = 0 ;
      //         int length = vec.size() ;

      //         while ( ( !rc ) && ( i < length ) ) {
      //            try {
      //               if ( ( vec.elementAt(i) != null ) && 
      //                    ((DocLinkRec)vec.elementAt(i)).notesId != null )
      //                  rc = ( ((DocLinkRec)vec.elementAt(i)).notesId.equals ( docId ) ) ;
      //            } catch ( ClassCastException ex ) {
      //            }
      //            i++ ;
      //         }
      //      } else
      //         rc = true ; // Return TRUE so we don't add a bad link to the doc

      //   } else
      //      rc = true ; // Return TRUE so we don't add a bad link to the doc
      //}

      //return( rc ) ;
   }

   /*-----------------------------------------------------------*/
   /* assembleGraphics
    */
   /**
    *    This method will go through all of the Graphics
    *  in the DocRec and delete the ones that
    *  do not exist in the document body. The user could have
    *  backspaced over a graphic.
    *             
    *  @param HTMLString the HTMLString representing the body
    *  @param 
    *  @param 
    *
    *  @return 
    */
   protected static void assembleGraphics ( WordProcessorBean wp_PANE, DocumentDraft document ) {
      WPEnumeration   imageList   = wp_PANE.enumerateImages() ;
      String          imageName   = null ;
      WPImage         wpImage     = null ;
      DocumentGraphic imageRec    = null ;
      Vector          graphicVec = null; //VOY (Vector) docRec.getFieldData(FIELD_GRAPHICS) ;

      /* First, mark each graphic for delete, we will assume that
         they removed every graphic, then we will go through each
         graphic in the list from the Word Processor and add each one,
         if we marked one for delete and it is still in the document,
         we will mark it for insert */
      if ( graphicVec != null ) {
         int i      = 0 ;
         int length = graphicVec.size() ;

         while ( i < length ) {
            ((DocumentGraphic)graphicVec.elementAt(i)).updateRecStatus ( DocumentGraphic.REC_NOCHANGE ) ;
            ((DocumentGraphic)graphicVec.elementAt(i)).updateRecStatus ( DocumentGraphic.REC_DELETE ) ;
            i++ ;
         }
      }

      /* Ok, enumerate through the list of images and each one
         we have still, mark for insert  */
      while ( imageList.hasMoreNames() ) {
         imageName = imageList.nextName() ;
         wpImage = wp_PANE.findImage ( imageName ) ;

         /* Ok, process the image. We will try to add it, if we can't because
            it is already there, we'll mark it for insert so that it is FTPed up  */
         if ( wpImage != null ) {
            String filename = null ;

            imageName = wpImage.getImageName() ;
            filename = imageName.substring ( imageName.lastIndexOf ( File.separator )+1 ) ;
            if ( notStampedGraphic ( filename ) ) {
               try {
                  if ( imageName.indexOf ( "/" ) >= 0 )
                     imageName = imageName.substring ( imageName.lastIndexOf ( "/" )+1,  imageName.length() ) ;
                  if ( imageName.indexOf ( "\\" ) >= 0 )
                     imageName = imageName.substring ( imageName.lastIndexOf ( "\\" )+1, imageName.length() ) ;
               } catch ( Exception e ) {
                  if ( imageName.indexOf ( "/" ) >= 0 )
                     imageName = imageName.substring ( imageName.lastIndexOf ( "/" ),  imageName.length() ) ;
                  if ( imageName.indexOf ( "\\" ) >= 0 )
                     imageName = imageName.substring ( imageName.lastIndexOf ( "\\" ), imageName.length() ) ;
               }

               /* Ok, add the graphic to the list of graphic associations */
               if ( !graphicVec.contains ( imageRec ) )
                  graphicVec.addElement ( imageRec ) ;
               else
                  imageRec.updateRecStatus  ( DocumentDraft.REC_INSERT ) ;

            }

         } /* if wpImage != null  */
      } /* while  */
   } /* assembleGraphics  */


   /*------------------------------------------------------------*/
   /* getDocBody
    */
   /**
    *    This method will strip out all of the leading
    *  crap out of a Document body and return
    *  only the meat html.
    *
    *  @param HTMLString The current doc body
    *  @param None
    *  @param None
    *
    *  @return altered HTMLString
    *//*---------------------------------------------------------*/
   public static String getDocBody ( String htmlString ) {
      int       bodyTagIndex  = 0 ;
      int       endBraceIndex = 0 ;
      int       startDocIndex = 0 ;
      int       stopDocIndex  = 0 ;
      String    retStr        = htmlString ;

      /* First, find where the <BODY> tag is */
      bodyTagIndex = retStr.indexOf ( "<BODY" ) ;

      if ( bodyTagIndex > 0 ) {
         endBraceIndex = retStr.indexOf ( ">", bodyTagIndex ) ;

         if ( endBraceIndex > 0 ) {
            startDocIndex = endBraceIndex+1 ;

            stopDocIndex = retStr.indexOf ( "</BODY" ) ;

            retStr = retStr.substring ( startDocIndex, stopDocIndex ).trim() ;

         } /* if */

         /* Set the base font */
         retStr = defaultFontStr+retStr+"</font>" ;
      } /* if */

      return( retStr ) ;
   } /* getDocBody */



   /*-----------------------------------------------------------*/
   /* fixBodyTagsForSave
    */
   /**
    *    This method will fix all of the body tags in the
    *  body of the HTML. Here is a list of exactly what
    *  has been changed:
    *             
    *  @param HTMLString The HTMLString to alter
    *  @param 
    *  @param 
    *
    *  @return The altered HTML String
    */
   public static String fixBodyTagsForSave ( String HTMLString ) {
      int       tagCount               = 0 ;
      int       i                      = 0 ;
      String    separator              = File.separator ;
      Vector    colWidthVec            = new Vector ( 10, 1 ) ;
      int       colWidthNum            = 0 ;
      HTMLParse htmlParse              = null ;
      HTMLTag   tagRec                 = null ;
      String    retStr                 = HTMLString ;
      String    tagStr                 = null ;
      String    tempTag                = null ;
      int       inTable                = 0 ;      
      int       inSup                  = 0 ;
      int       inSub                  = 0 ;
      int       inPre                  = 0 ;
      int       removeNextEndParagraph = 0 ;
      boolean   error                  = false ;
      boolean   inParagraphFormat      = false ;
      boolean   inTableCenter          = false ;
      boolean   inCenterFormat         = false ;


      try {

         /* Take out all of the direct new target win links */
         HTMLString = Text.replaceAllStrInStr ( HTMLString, "_(newwin)\"", "\" TARGET=\"new\"" ) ; 

         /* First, add a new line character to certain tags */
         htmlParse = new HTMLParse ( HTMLString ) ;

         while ( ( i < htmlParse.tagCount() ) && ( !error ) ) {

            tagStr = htmlParse.getTag(i) ;

            if ( tagStr != null ) {

               tagStr = tagStr.toUpperCase() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   STYLE/FONT - Fix so that they are correct font names    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( tagStr.indexOf ( "STYLE" ) >= 0 ) &&
                    ( tagStr.indexOf ( "FONT" ) >= 0 ) ) {
                  tempTag = fixStyleFontTag ( tagStr )  ;

                  if ( tempTag != null ) {
                     htmlParse.replaceTag ( i, tempTag ) ;

                     tempTag = null ;
                  }
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   WPCOMMENT - Remove                                      */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.indexOf ( "WPCOMMENT" ) >= 0 ) {
                  htmlParse.removeTag ( i ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   /P Tags - Change to line breaks                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.equals ( "</P>" ) ) {

                  try {
                     String nextTag = htmlParse.getTag(i+1).toUpperCase() ;
                     String prevTag = htmlParse.getTag(i-1).toUpperCase() ;

                     /* If we are supposed to remove the next end of paragraph, do so now */
                     if ( removeNextEndParagraph > 0 ) {
                        htmlParse.removeTag ( i ) ;
                        removeNextEndParagraph-- ;
                     }

                     /* Otherwise, process as normal */
                     else {

                        /* If we are in the table, take out the last /P before a end of cell */
                        if ( ( inTable > 0 ) && ( nextTag.indexOf ( "/TD" ) >= 0 ) )
                           htmlParse.removeTag(i) ;

                        /* Remove the ending P tag before Lists */
                        else if ( ( nextTag != null ) && 
                                  ( ( nextTag.indexOf ( "OL" ) >= 0 ) || ( nextTag.indexOf ( "UL" ) >= 0 ) ) )
                           htmlParse.removeTag ( i ) ;

                        /* Remove the ending P tag after a list */
                        else if ( ( prevTag != null ) && 
                                  ( ( prevTag.indexOf ( "OL" ) >= 0 ) || ( prevTag.indexOf ( "UL" ) >= 0 ) ) )
                           htmlParse.removeTag ( i ) ;

                        /* If all else fails, replace the </P> with a <BR>  */
                        else
                           htmlParse.replaceTag ( i, "<br>" ) ;
                     }

                  } catch ( Exception e ) {
                     htmlParse.replaceTag ( i, "<br>" ) ;
                  }

               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   P Tags - remove                                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.startsWith ( "<P" ) ) {

                  /* If this is not a paragraph format tag, remove it!  */
                  if ( tagStr.indexOf ( "ALIGN" ) == -1 )
                     htmlParse.removeTag ( i ) ;

                  /* Remove center tags from table cells */
                  else if ( ( tagStr.indexOf ( "CENTER" ) >= 0 ) && ( inTable > 0 ) )
                     htmlParse.removeTag ( i ) ;

                  /* If this is a word processor CLASS paragaph, remove it and it's ending </P> tag */
                  else if ( tagStr.indexOf ( "CLASS" ) >= 0 ) {
                     htmlParse.removeTag ( i ) ;
                     removeNextEndParagraph++ ;
                  }

               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Superscript                                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.indexOf ( Str.getStr ( STR_SUPERSCRIPT ) ) >= 0 ) {
                  tagStr = "<sup>" ;
                  htmlParse.replaceTag ( i, tagStr ) ;
                  htmlParse.removeTag ( i+1 ) ; // Remove the font size="1"
                  tagRec = htmlParse.getTagRec ( i ) ;
                  inSup=tagRec.getTagNum() ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Subscript                                               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.indexOf ( Str.getStr ( STR_SUBSCRIPT ) ) >= 0 ) {
                  tagStr = "<sub>" ;
                  htmlParse.replaceTag ( i, tagStr ) ;
                  htmlParse.removeTag ( i+1 ) ; // Remove the font size="1"
                  tagRec = htmlParse.getTagRec ( i ) ;
                  inSub=tagRec.getTagNum() ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   End Superscript/Subscript                               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( ( tagStr.equals ( "</FONT>" ) ) && 
                         ( ( inSub > 0 ) ||
                           ( inSup > 0 ) ) ) {
                  tempTag = htmlParse.getTag ( i+1 ) ;

                  if ( tempTag != null ) {
                     tempTag = tempTag.toUpperCase() ;

                     if ( tempTag.equals ( "</A>" ) ) {

                        if ( inSub > inSup ) {
                           tempTag = "</sub>" ;
                           inSub = 0 ;
                        } else {
                           tempTag = "</sup>" ;
                           inSup = 0 ;
                        }

                        htmlParse.removeTag ( i+1 ) ;
                        htmlParse.replaceTag ( i, tempTag ) ;
                     } /* if /A */
                  } /* if not null */

                  tempTag = null ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   IMG SRC                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.startsWith ( "<IMG SRC=" ) ) {
                  String startImg  = "<IMG SRC=\"" ;
                  String imageFile = null ;
                  String end       = null ;

                  /* Preserve the case for the graphic name */
                  tagStr = htmlParse.getTag(i);

                  imageFile = tagStr.substring ( tagStr.indexOf ( "\"" ), tagStr.indexOf ( "\"", tagStr.indexOf ( "\"" )+1 ) ) ;
                  end       = tagStr.substring ( tagStr.indexOf ( "\"", tagStr.indexOf ( "\"" )+1 ), tagStr.length() ) ;

                  /* Strip out the temp directory info */
                  imageFile = imageFile.substring ( imageFile.lastIndexOf ( File.separator )+1, imageFile.length() ) ;

                  /* if by chance they had another double quote in the string,
                     strip it out */
                  imageFile = Text.replaceAllStrInStr ( imageFile, File.separator, "" ) ;                                              

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   If this is a stamped graphic, put in the ###IMAGELOC### */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( !notStampedGraphic ( imageFile ) ) {
                     tempTag = "<img src=\"###IMAGELOC###"+imageFile+"\">" ;
                  }

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Otherwise, strip out our temp dir. information          */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  else {

                     /* Create the image tag */
                     tempTag = startImg+"./"+imageFile.substring ( imageFile.lastIndexOf ( separator )+1, imageFile.length() )+end ;
                  }

                  /* Replace the tag */
                  htmlParse.replaceTag ( i, tempTag ) ;
                  tempTag = null ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   TABLE                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.startsWith ( "<TABLE" ) ) {
                  inTable++ ;

                  /* Turn off all paragraph tags  */
                  inCenterFormat = false ;

                  if ( tagStr.indexOf ( "CENTER" ) == -1 ) {
                     if ( tagStr.indexOf ( "LEFT" ) >0 )
                        tagStr = Text.replaceInStr ( tagStr, "LEFT", "CENTER" ) ;
                     else if ( tagStr.indexOf ( "RIGHT" ) >0 )
                        tagStr = Text.replaceInStr ( tagStr, "RIGHT", "CENTER" ) ;
                  }

                  /* Take out the COLS="x" piece of the tag */
                  if ( tagStr.indexOf ( "COLS" ) >= 0 ) {
                     tagStr = tagStr.substring ( 0, tagStr.indexOf ( "COLS=\"" ) )+
                              tagStr.substring ( tagStr.indexOf ( "\"", tagStr.indexOf ( "COLS=\"" )+6 )+1, tagStr.length() ) ;
                     htmlParse.replaceTag ( i, tagStr ) ;
                  }

                  if ( inTable > 1 ) {
                     GUISystem.printBox ( STR_ERROR, STR_NO_NESTED_TABLES ) ;
                     error = true ;
                  }

               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   /TABLE                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.startsWith ( "</TABLE" ) ) {

                  inTable-- ;

                  /* If we are no longer in a table, clear out the
                     col widths tags */
                  if ( inTable == 0 ) {
                     colWidthVec.removeAllElements() ;
                     colWidthNum = 0 ;
                  } /* if we are not in a table anymore */

               } /* else */

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   COL                                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.startsWith ( "<COL" ) ) {
                  String colString = tagStr.substring ( 4, tagStr.length()-1) ;

                  colWidthVec.addElement ( colString ) ;

                  /* Do not remove the tag for now, the word processor needs it */
                  //htmlParse.removeTag ( i ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   </COL>                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               //else if ( tagStr.equals ( "</COL>" ) )
               //   htmlParse.removeTag ( i ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   <TD>                                                    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.startsWith ( "<TD" ) ) {
                  String     cellStr   = null ;
                  String     tempStr   = null ;
                  String     newHtml   = htmlParse.getHTMLString() ;
                  boolean    inCenter  = false ;

                  /* Only apply the width values to the top table cells */
                  if ( colWidthVec.size() > colWidthNum ) {
                     String     colString = (String) colWidthVec.elementAt(colWidthNum) ;

                     colString=convertWidth(colString) ;

                     colWidthNum++ ;
                     tempStr = "<td "+colString+" " ;

                     tempStr+=tagStr.substring ( 4, tagStr.length() ) ;
                     tagStr = tempStr ;
                  } else
                     tagStr = "<td valign=\"top\""+tagStr.substring ( 4, tagStr.length() ) ; 


                  tagRec    = htmlParse.getTagRec(i) ;

                  /* Grab all of the data inside of the cell */
                  cellStr = newHtml.substring ( tagRec.getCharPos() ) ;
                  if ( cellStr.indexOf ( "/TD" ) >= 0 )
                     cellStr = cellStr.substring ( 0, cellStr.indexOf ( "/TD" ) ) ;
                  else if ( cellStr.indexOf ( "/td" ) >= 0 )
                     cellStr = cellStr.substring ( 0, cellStr.indexOf ( "/td" ) ) ;
                  else
                     cellStr = null ;

                  /* Ok, Find the center tag if we have one */
                  if ( cellStr != null ) {
                     if ( ( cellStr.indexOf ( "CENTER" ) >= 0 ) ||
                          ( cellStr.indexOf ( "center" ) >= 0 ) )
                        htmlParse.replaceTag ( i, "<td VALIGN = \"TOP\" ALIGN=\"CENTER\" "+tagStr.substring ( 4, tagStr.length() ) ) ;
                     else if ( ( cellStr.indexOf ( "RIGHT" ) == -1 ) )
                        htmlParse.replaceTag ( i, "<td VALIGN = \"TOP\" ALIGN=\"LEFT\" "+tagStr.substring ( 4, tagStr.length() ) ) ;
                  }

               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   <LI> - Remove the Word Processor Class Crap             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.indexOf ( "<LI" ) >= 0 ) {
                  htmlParse.replaceTag ( i, "<LI>" ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   <A HREF - If the link has nothing between the beginning */
               /*             and ending tag, remove the link               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( tagStr.startsWith ( "<A HREF" ) ) {
                  int     k       = i+1 ;
                  String  tempStr = htmlParse.getTag(i) ;
                  boolean found   = ( tempStr.startsWith ( "</A" ) ) || ( tempStr.startsWith ( "</a" ) ) ;

                  /* First, if this tag has any 'amp;' tags in it, replace
                     them with the real '&'  */
                  tempStr = Text.replaceAllStrInStr ( tempStr, "&AMP;AMP;", "&" ) ;
                  tempStr = Text.replaceAllStrInStr ( tempStr, "&AMP;", "&" ) ;
                  tempStr = Text.replaceAllStrInStr ( tempStr, "&amp;amp;", "&" ) ;
                  tempStr = Text.replaceAllStrInStr ( tempStr, "&amp;", "&" ) ;
                  if ( tempStr.indexOf ( "WP_PART" ) >= 0 )
                     tempStr = takeOutWPPart(tempStr) ;
                  htmlParse.replaceTag ( i, tempStr ) ;

                  /* Find the ending a tag */
                  while ( ( !found ) && ( k < htmlParse.tagCount() ) ) {
                     k++ ;
                     if ( k < htmlParse.tagCount() ) {
                        tempStr = htmlParse.getTag(k).toUpperCase() ;
                        found   = tempStr.startsWith ( "</A" ) ;
                     }
                  }

                  if ( found ) {
                     int hrefLength    = tagStr.length() ;
                     int startIndex    = htmlParse.getTagRec(i).getCharPos() ;
                     int startEndIndex = htmlParse.getTagRec(k).getCharPos() ;

                     /* Remove the tag if there is nothing between the beginning
                        and ending tag. Also, remove the tag if it is WP_PART 'x'of'y' where
                        x is not 1  */
                     if ( ( (startIndex+hrefLength+2) == startEndIndex ) ||
                          ( ( tagStr.indexOf ( "WP_PART" ) >= 0 ) &&
                            ( tagStr.indexOf ( "1OF" ) == -1 ) ) ) {
                        htmlParse.removeTag ( i ) ;
                        htmlParse.removeTag ( k ) ;
                     }
                  }
               }


            } /* if */
            i++ ;
         } /* while */

         if ( !error ) {

            /* get the string back */
            retStr = htmlParse.getHTMLString() ;

            /* Strip out the underscores before the doc link graphics */
            retStr = Text.replaceAllStrInStr ( retStr, "_<IMG", "<IMG" ) ;

            /* These are the ending image tags that are not valid! */
            retStr = Text.replaceAllStrInStr ( retStr, "</BR>", "" ) ;
            retStr = Text.replaceAllStrInStr ( retStr, "</br>", "" ) ;
            retStr = Text.replaceAllStrInStr ( retStr, "</HR>", "" ) ;
            retStr = Text.replaceAllStrInStr ( retStr, "</hr>", "" ) ;
            retStr = Text.replaceAllStrInStr ( retStr, "</IMG>", "" ) ;
            retStr = Text.replaceAllStrInStr ( retStr, "</img>", "" ) ;
            retStr = Text.replaceAllStrInStr ( retStr, "<font>", "" ) ;
            retStr = Text.replaceAllStrInStr ( retStr, "<FONT>", "" ) ;

         } else
            retStr = null ;

         htmlParse.free() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( retStr ) ;

   } /* fixBodyTags */




   /*------------------------------------------------------------*/
   /* fixStyleFontTag
    */
   /**
    *    This method will take one of the font tags that
    *  has a STYLE parameter and fix it so that we
    *  do not use the style param.
    *
    *  @param tagStr The String representation of the Font tag to fix
    *  @param None
    *  @param None
    *
    *  @return String The fixed font string
    *//*---------------------------------------------------------*/
   private static String fixStyleFontTag ( String tagStr ) {
      String retStr = null ;
      int sIndex = tagStr.indexOf ( "STYLE" ) ;

      /* If this tag has a STYLE in it, continue */
      if ( sIndex >= 0 ) {
         sIndex = tagStr.indexOf ( "\"", sIndex ) ;

         /* if the tag also has the params to the style, continue */
         if ( sIndex >= 0 ) {
            int eIndex = tagStr.indexOf ( "\"", sIndex+1 ) ;

            /* Add 12 to the sIndex to bump our pointer to the
               actual point size of the style font-size tag */
            sIndex+=12 ;

            /* If there is a closing double quote, continue */
            if ( eIndex >= 0 ) {
               String paramStr = null ;
               int    fontSize = 12 ;

               /* Subtract 2 from the end index, to bump the pointer
                  back around the 'pt' */
               paramStr = tagStr.substring ( sIndex, eIndex-2 ) ;

               try {
                  fontSize = new Integer ( paramStr ).intValue() ;
               } catch ( Exception e ) {
               }

               switch ( fontSize ) {
                  case 10 :                     
                     retStr = "<font size=2>" ;
                     //retStr = "<font size=-1>" ;
                     break ;

                  case 12 :
                     retStr = "<font size=3>" ;
                     break ;

                  case 14 :
                     //retStr = "<font size=+1>" ;
                     retStr = "<font size=4>" ;
                     break ;
               }

            } /* if */
         } /* if */
      } /* if */

      return( retStr ) ;
   } /* private */




   /*-----------------------------------------------------------*/
   /* getRandomName
    */
   /**
    *    This method will return a random name for
    *  such Word Processor elements as an Image or List.
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   protected static String getRandomName ( DocumentDraft docRec ) {
      String retStr = new CDate ( CDate.DATE_STAMP ).today() ;

      retStr = Text.replaceAllStrInStr (retStr, ".", "" ) ;
      retStr = Text.replaceAllStrInStr (retStr, "-", "" ) ;

      if ( docRec != null ) {

         /* If this is a document, and it has no indicator yet, create
            one now */
         retStr = docRec.getDocInd()+"_"+retStr ;
      }

      return( retStr ) ;
   }


   /*------------------------------------------------------------*/
   /* convertWidth
    */
   /**
    *    This method will convert a width tag from
    *  a percentage with to a pixel width. The assumed
    *  width of the document is 624 pixels. This is the size of the Word Processor panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static String convertWidth ( String widthTag ) {
      String retTag = "" ;

      if ( ( widthTag != null ) && (widthTag.length() > 5 ) ) {
         int    index  = widthTag.indexOf ( "\"" )+1 ;
         String numStr = widthTag.substring ( index, 
                                              widthTag.indexOf ( "\"", index )-1 ) ;
         double width  = 0 ;

         width = new Double ( numStr ).doubleValue() ;

         /* Divide by 100 to move the decimal  */
         width = width/100 ;

         /* Multiply by our width  */
         width = width*624 ;

         /* Now, form the return tag  */
         retTag = "WIDTH=\""+new Double ( width ).intValue()+"\"" ;

      }

      return( retTag ) ;

   }

   private static String takeOutWPPart ( String tag ) {
      try {
         String tempStr    = tag ;
         int    startIndex = tag.indexOf ( "WP_PART" ) ;
         int    stopIndex  = tag.lastIndexOf ( "\"" ) ;

         tag = tag.substring (0, startIndex)+tempStr.substring ( stopIndex+1, tempStr.length() ) ;
      } catch ( Exception e ) {
      }
      return( tag ) ;
   }

   public static boolean notStampedGraphic ( String name ) {
      boolean rc = true ;

      if ( ( name.indexOf ( "doclink" ) >=0 ) ||
           ( StampedGraphicFactory.containsImage ( name ) ) )
         rc = false ;

      return( rc ) ;
   }

} /* WordProcUtil */
