package com.ibm.nzna.projects.qit.workspace ;        
/*-----------------------------------------------------------*/
/*   WorkspacePanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WorkspacePanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.gui.* ;   
import com.ibm.nzna.projects.qit.app.* ;  
import com.ibm.nzna.shared.util.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This panel is the main Quest workspace. It is responsible for
 * displaying all QuestApplets that the user has access to.
 *
 *
 * @version 1.0, 1/20/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WorkspacePanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class WorkspacePanel extends JPanel implements PropertyListener, QuestPanel, AppConst, AppletSystemListener, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Image           backgroundImage       = null ;
   private UserStatPanel   userStatPanel         = null ;
   private JScrollPane     deskScrollPane        = null ;
   private DesktopPanel    desktopPanel          = null ;
   private long            lastAppletRecTime     = 0 ;
   private AppletRec       lastAppletRec         = null ;
   private boolean         skinned               = false ;
   private boolean         firstAppletStarted    = false ;
   private static WorkspacePanel  instance        = null ;



   /*------------------------------------------------------------*/
   /* WorkspacePanel
    */
   /**
    *    This constructor will do nothing but read in our backgrounds. The
    *  initialize and start methods do the real work.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WorkspacePanel () {
      instance = this ;
   }


   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_WORKSPACE ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will simply return this
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* notification
    */
   /**
    *    This method will be called when we are
    *  to be notified of something
    *  
    *
    *  @param questPanel The panel that is notifying us
    *  @param msg The string message
    *  @param object1 The user object 1
    *  @param object2 The user object 2
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object notification ( Component questPanel, String msg, Object object1, Object object2 ) {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will simply return TRUE since
    *  we don't care if the app shuts down
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called once we
    *  are removed from our window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      AppletSystem.removeAppletSystemListener ( this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our
    *  panel. This method will simply read
    *  for all QuestApplets.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      userStatPanel  = new UserStatPanel ( ) ;    
      desktopPanel   = new DesktopPanel () ;
      deskScrollPane = new JScrollPane ( desktopPanel ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      deskScrollPane.setBackground ( Color.white ) ;
      deskScrollPane.setBackground ( Color.white ) ;
      skinned = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;

      if ( skinned )
         deskScrollPane.setBorder ( null ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      AppletSystem.addAppletSystemListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout () ) ;
      add ( deskScrollPane, BorderLayout.CENTER ) ;      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our applets                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshApplets() ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add ourselves as a listener for Properties              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;
      setOpaque ( true ) ;

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will add each QuestApplet
    *  to our window, since we are now
    *  showing and completly initialized
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start() {
      String startupApplet = PropertySystem.getString ( PropertySystem.PROPERTY_STARTUP_APPLET ) ;
      
      new WorkspaceWhatsNew ( this ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have a startup applet, run it now                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( startupApplet != null ) &&
           ( startupApplet.length() > 0 ) ) {
         AppletRec appletRec = AppletSystem.getAppletFromName ( startupApplet ) ;

         if ( appletRec != null )
            AppletSystem.createApplet ( appletRec ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return NULL, since 
    *  we don't have a default nav panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( userStatPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param g The graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      Dimension size = getSize() ;

      g.setColor ( Color.white ) ;
      g.fillRect ( 0, 0, size.width, size.height ) ;
      //super.paintChildren ( g ) ;

      //if ( PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) )
      //   WinUtil.wallPaper ( this, g, ImageSystem.getImage ( this, ImageSystem.THEME_CONTENT_BACKGROUND ) ) ;
   }



   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Cut As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Paste As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Copy As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called when the user
    *  wishes to refresh the QuestPanel. DO NOT EVER CALL revalidate
    *  in this method, the default operation for this button is to
    *  have the AppDefaultWin holding the QuestPanel to repaint everything
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
   }





   /*------------------------------------------------------------*/
   /* appletRemoved
    */
   /**
    *    This method will be called when an applet is
    *  removed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void appletRemoved () {
      refreshApplets() ;
   }




   /*------------------------------------------------------------*/
   /* appletAdded
    */
   /**
    *    This method will be called when an applet
    *  is added to the system
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void appletAdded () {
      refreshApplets() ;
   }





   /*------------------------------------------------------------*/
   /* refreshApplets
    */
   /**
    *    This method will refresh our applets
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshApplets () {
      Vector      appletVec = AppletSystem.getAppletList() ;
      int         i         = 0 ;
      DButton     button    = null ; 
      AppletRec   appletRec = null ;

      int f = 1 ;

      desktopPanel.removeAll () ;
      if ( appletVec != null ) {
         int size = appletVec.size() ;

         while ( i < size ) {
            QuestApplet questApplet = null ;

            appletRec   = (AppletRec) appletVec.elementAt(i) ;
            try {
               questApplet = appletRec.createQuestApplet() ;

               if ( UserSystem.hasAuthority ( questApplet.getAuthority() ) ) {
                  button = new DButton ( appletRec.getName(),
                                         appletRec.getIcon() ) ;

                  desktopPanel.add ( button ) ;
                  button.addActionListener ( this ) ;                    

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Adjust the buttons properties                           */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  button.setOpaque                 ( false ) ;
                  button.setFocusPainted           ( false ) ;
                  button.setBorder                 ( null ) ;
                  button.actAsLink                 ( true ) ;
                  button.setHighlightForeground    ( Color.blue ) ;
                  button.setVerticalTextPosition   ( SwingConstants.BOTTOM ) ;
                  button.setVerticalAlignment      ( SwingConstants.TOP ) ;
                  button.setHorizontalTextPosition ( SwingConstants.CENTER ) ;
               }
            } catch ( QuestAppletNotFoundException e ) {
               GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_CANNOT_FIND_QUEST_APPLET )+": "+appletRec.getClassName() ) ;
            }

            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an 
    *  action we are listening for has been fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() instanceof DButton ) {
         String    text      = ((DButton)e.getSource()).getText() ;
         AppletRec appletRec = AppletSystem.getAppletFromName ( text ) ;
         boolean   start     = true ;                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Prevent the user from double clicking                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( lastAppletRec != null ) && ( appletRec.equals ( lastAppletRec ) ) ) {
            long time = System.currentTimeMillis() ;

            if ( (time-lastAppletRecTime) < 500 )
               start = false ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Kick off the applet if we have one                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( appletRec != null ) && ( start ) ) {
            AppletSystem.createApplet ( appletRec ) ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Mark this if this is the first applet started           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !firstAppletStarted ) {
               String startupApplet = PropertySystem.getString ( PropertySystem.PROPERTY_STARTUP_APPLET ) ;                                           


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Only mark if the user does not have a startup applet    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( startupApplet == null ) || ( startupApplet.length() < 3 ) ) {
                  String firstAppletName = PropertySystem.getString ( PropertySystem.PROPERTY_FIRST_APPLET_RUN ) ;
                  int    count           = PropertySystem.getInt    ( PropertySystem.PROPERTY_FIRST_APPLET_RUN_COUNT ) ;                                    

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   If the applet they selected matches that last applet    */
                  /* that was run first, add one to our counter                */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( firstAppletName.equals ( appletRec.getName() ) ) {

                     count++ ;
                     PropertySystem.putInt ( PropertySystem.PROPERTY_FIRST_APPLET_RUN_COUNT, count ) ;


                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   If they have started this applet first for the last 5   */
                     /* times, ask them if they always want to start with this    */
                     /* applet.                                                   */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( ( count > 5 ) &&
                          ( PropertySystem.getBool ( PropertySystem.PROPERTY_ASK_FIRST_APPLET_RUN ) ) ) {

                        switch ( new FirstAppletRunDlg ( GUISystem.getParentDefWin ( this ),
                                                         appletRec.getName() ).getResult() ) {
                           
                           case FirstAppletRunDlg.YES :
                              PropertySystem.putString ( PropertySystem.PROPERTY_STARTUP_APPLET, appletRec.getName() ) ;
                              break ;

                           case FirstAppletRunDlg.NO :
                              PropertySystem.putInt ( PropertySystem.PROPERTY_FIRST_APPLET_RUN_COUNT, 0 ) ;
                              break ;

                           case FirstAppletRunDlg.NEVER :
                              PropertySystem.putBool ( PropertySystem.PROPERTY_ASK_FIRST_APPLET_RUN, false ) ;
                              break ;
                        }

                     }

                  } else  /* if the name matches */
                     PropertySystem.putString ( PropertySystem.PROPERTY_FIRST_APPLET_RUN, appletRec.getName () ) ;

               } /* if startupApplet */

               firstAppletStarted = true ;
            } /* if firstAppletStarted */

         } /* If AppletRec */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Log information for next event                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         lastAppletRec     = appletRec ;
         lastAppletRecTime = System.currentTimeMillis() ;

      }
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name of
    *  this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_WORKSPACE ) ) ;
   }



   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the
    *  user has selected our panel from a
    *  tabbed pane. If we are in MDI mode, this method is never called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {

   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel onto another. This method
    *  will never be called in the MDI mode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we have been listening for has been changed
    *  
    *
    *  @param int The property that was changed
    *  @param value The value that was changed
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE ) {
         refreshApplets() ;
      } else if ( property == PropertySystem.PROPERTY_SHOWSKIN ) {
         skinned = ((Boolean)value).booleanValue() ;
         if ( skinned )
            deskScrollPane.setBorder ( null ) ;
         else
            deskScrollPane.setBorder ( GUISystem.loweredBorder ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return the single instance
    *  of WorkSpacePanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static WorkspacePanel getInstance () {
      return( instance ) ;
   }

} /* WorkspacePanel */
