package com.ibm.nzna.projects.qit.workspace ;              
/*-----------------------------------------------------------*/
/*   WorkspaceWhatsNew                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WorkspaceWhatsNew                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
/**
 * This object manages all 'What's New' pop ups for the Workspace 
 *
 *
 * @version 1.0, 12/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WorkspaceWhatsNew                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class WorkspaceWhatsNew extends Object implements WhatsNewPopUpListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Pop Up Indexes                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int WHATSNEW_START      = 0 ;
   private static final int WHATSNEW_TOOLBAR    = 1 ;                                            
   private static final int WHATSNEW_STOP       = 2 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Locations -  These are the locations for each popup     */
   /* each Rectangle represents the corresponding Pop Up Index  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Point[] locations = {
      new Point ( 170, 100),        // WHATSNEW_START
      new Point ( 170,  100),        // WHATSNEW_TOOLBAR
      new Point (  170, 100)         // WHATSNEW_STOP
   } ;                                   


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Titles - These are the titles of all Pop Ups            */
   /* each String represents the corresponding Pop Up Index     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static int[] titles = {
      STR_WORKSPACE,                            // WHATSNEW_START
      STR_TOOLBAR,                              // WHATSNEW_TOOLBAR
      STR_ICONS
   }  ;                                   


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Content - This is the content of all Pop Ups            */
   /* each String represents the corresponding Pop Up Index     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static int[] content = {
      STR_WHATS_NEW_WORKSPACE_CONTENT,          // WHATSNEW_START
      STR_WHATS_NEW_WORKSPACE_TOOLBAR,          // WHATSNEW_TOOLBAR
      STR_WHATS_NEW_WORKSPACE_ICONS             // WHATSNEW_ICONS
   }   ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private WorkspacePanel panel         = null ;
   private int            whatsNewIndex = WHATSNEW_START ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This initial constructor starts the show
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WorkspaceWhatsNew ( WorkspacePanel panel ) {
      if ( PropertySystem.getBool ( PropertySystem.PROPERTY_SHOW_WHATS_NEW ) ) {
         this.panel = panel ;
         showWhatsNew ( WHATSNEW_START ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* showWhatsNew
    */
   /**
    *    This method will show the pop up with
    *  the specified index
    *
    *  @param index The WHATSNEW_XXX index to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showWhatsNew ( int index ) {
      WhatsNewPopUp popup = WhatsNewPopUp.getInstance() ;

      whatsNewIndex = index ;
      popup.removeWhatsNewPopUpListener ( this ) ;
      popup.addWhatsNewPopUpListener ( this ) ;

      popup.setRelativeParent ( GUISystem.getParentDefWin ( panel ) ) ;
      popup.setLocation  ( locations[whatsNewIndex] ) ;
      popup.setTitle   ( Str.getStr ( titles  [ whatsNewIndex ] ) ) ;
      popup.setContent ( Str.getStr ( content [ whatsNewIndex ] ) ) ;
      popup.setVisible ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* whatsNewPerformed
    */
   /**
    *    This method will be called when a 
    *  new whats new button is hit. We will process
    *  this by incrementing the wahtsNewIndex and showing it
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void whatsNewPerformed ( WhatsNewPopUp popup ) {
      whatsNewIndex++ ;

      if ( whatsNewIndex > WHATSNEW_STOP )
         whatsNewIndex = WHATSNEW_START ;

      showWhatsNew ( whatsNewIndex ) ;
   }
} /* WorkspaceWhatsNew */
