package com.ibm.nzna.projects.topic ;
/*-----------------------------------------------------------*/
/*   Topic                                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Topic                               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This class is a simple OPIC/M Test class. Or a Test of OPICm. It runs through
 * the OPIC/M eSupport Views and tries to load the following parts:<br><br>
 *
 * 10K3780<br>
 * 633307N<br>
 * 634431U<br>
 * 684910U<br>                             
 * 686778U<br>                             
 * 868212Y<br>  
 * 6849CTO<br>
 * 865455U<br>
 * 8364CXX<br>
 * 86566RY<br>
 * 2675CTO<br>
 *
 * New Products added
 * 266031Z<br>
 *
 *
 * @version 1.0, 3/16/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Topic                                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class Topic extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String OUTPUT_FILE = "opicm.out" ;
   private static final String CSV         = "&!&" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String output = "OPIC/M Test Data\nDate:\n"+new CDate (CDate.DATE_DATE ).today()+"\n\n" ;






   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    The program takes no command line
    *  params
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String[] args ) {
      new LogSystem ( 1, "Topic.out" ) ;
      new Topic() ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor takes no arguments, it
    *  simply kicks off the script
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Topic () {
      long startTime = System.currentTimeMillis() ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Erase the current output file                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         FileUtil.deleteFile ( OUTPUT_FILE ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Connect to OPIC/M                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( connectToOPICM() ) {
            LogSystem.log ( 1, "Connected to OPIC/M" ) ;                        


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, write out each view's script                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            System.out.println ( "Running ES_COUNTRY_SOL" ) ;
            writeES_COUNTRY_SOL () ; 
            //System.out.println ( "Running ES_SOL_NO_CSOL" ) ;
            //writeES_SOL_NO_CSOL () ;
            //System.out.println ( "Running ES_MTM_NO_SOL" ) ;
            //writeES_MTM_NO_SOL () ;
            //System.out.println ( "Running ES_CTO_REGION_ONLY" ) ;
            //writeES_CTO_REGION_ONLY () ;
            //System.out.println ( "Running ES_CTO_NO_SOL" ) ;
            //writeES_CTO_NO_SOL() ;

            writeCounts() ;

            System.out.println ( "Getting full ES_COUNTRY_CPSL" ) ;
            writeFullES_COUNTRY_CPSL () ;
            //System.out.println ( "Getting full ES_CTO_REGION_ONLY" ) ;
            //writeFullES_CTO_REGION_ONLY() ;
            System.out.println ( "Getting full ES_COUNTRY_SOL" ) ;
            writeFullES_COUNTRY_SOL() ;
            System.out.println ( "Getting full ES_COUNTRY_CTO" ) ;
            writeFullES_COUNTRY_CTO() ;
            //System.out.println ( "Getting full ES_MTM_NO_SOL" ) ;
            //writeFullES_MTM_NO_SOL() ;
            //System.out.println ( "Getting full ES_AAP_MTM" ) ;
            //writeFullES_AAP_MTM() ;
            //System.out.println ( "Getting full ES_CTO_NO_SOL" ) ;
            //writeFullES_CTO_NO_SOL() ;
            //System.out.println ( "Getting full ES_SOL_NO_CSOL" ) ;
            //writeFullES_SOL_NO_CSOL() ;
            //System.out.println ( "Getting Full ES_REGION_PCSL_NO_CT" ) ;
            //writeFullES_REGION_PCSL_NO_CT() ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, save the final output                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            FileUtil.saveFile ( OUTPUT_FILE, output ) ;




         } else
            LogSystem.log ( 1, "Cannot Connect to OPIC/M" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }


      System.out.println ( "Runtime: "+(System.currentTimeMillis()-startTime)+"(ms)" ) ;
      System.exit(0) ;
   }




   /*------------------------------------------------------------*/
   /* connectToOPICM
    */
   /**
    *    This method will create a database connection
    *  to OPIC/M
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean connectToOPICM () throws Exception {
      new DatabaseSystem ( true, LogSystem.getInstance(), false ) ;

      return( DatabaseSystem.createConnection ( DatabaseSystem.OPICM,
                                                "OPICMODS",
                                                "hyper.raleigh.ibm.com",
                                                "5703",
                                                "opicmes2",
                                                "j0hnny" ) ) ;
   }




   /*------------------------------------------------------------*/
   /* writeES_COUNTRY_SOL
    */
   /**
    *    This method will write out the values
    *  from the ES_COUNTRY_SOL table
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeES_COUNTRY_SOL () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeES_COUNTRY_SOL", 5 ) ;
      Statement stmt      = sqlMethod.createStatement () ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our header information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      output+="View: ES_COUNTRY_SOL (Solutions with Country associations)\n" ;
      output+="Geography"+CSV+"Country"+CSV+"Brand"+CSV+"Family"+CSV+"Series"+CSV+"Project"+CSV+"Offering Part Number"+CSV+"Country Part Number"+CSV+"Description"+CSV+"Announce Date"+CSV+"Preannounce Date"+CSV+"Availability Date"+CSV+"Withdrawn Date"+CSV+"Language (1=English)\n" ;

      if ( stmt != null ) {
         ResultSet rs = stmt.executeQuery ( "SELECT SUBGEOCODE, COUNTRYNAME, SUBGROUPNAME, FAMILYNAME, SERIESNAME, PROJECTNAME, OFFERINGPNUMB, PNUMB_CT, PNUMB_DESC_CT, ACT_ANN_DATE_CT, ACT_PREANN_CT, ACT_AVAIL_DATE_CT, ACT_WDRAWDATE_CT, NLSID "+
                                            "FROM OPICM.ES_COUNTRY_SOL "+
                                            "WHERE PNUMB_CT IN "+getPartNums()+"" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Walk the results                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {
            output+=trim ( rs.getString(1) )+CSV+
                    trim ( rs.getString(2) )+CSV+
                    trim ( rs.getString(3) )+CSV+
                    trim ( rs.getString(4) )+CSV+
                    trim ( rs.getString(5) )+CSV+
                    trim ( rs.getString(6) )+CSV+
                    trim ( rs.getString(7) )+CSV+
                    trim ( rs.getString(8) )+CSV+
                    trim ( rs.getString(9) )+CSV+
                    trim ( rs.getString(10) )+CSV+
                    trim ( rs.getString(11) )+CSV+
                    trim ( rs.getString(12) )+CSV+
                    trim ( rs.getString(13) )+CSV+
                    rs.getInt(14)+"\n" ;

         }
      } else
         LogSystem.log ( 1, "Could not create statement for writeES_COUNTRY_SOL" ) ;


      output+="\n\n\n" ;
      sqlMethod.close() ;
   }



   /*------------------------------------------------------------*/
   /* writeES_SOL_NO_CSOL
    */
   /**
    *    This method will write out the values
    *  from the ES_SOL_NO_CSOL table
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeES_SOL_NO_CSOL () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeES_SOL_NO_CSOL", 5 ) ;
      Statement stmt      = sqlMethod.createStatement () ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our header information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      output+="View: ES_SOL_NO_CSOL (Solutions with Country associations)\n" ;
      output+="Geography"+CSV+"Brand"+CSV+"Family"+CSV+"Series"+CSV+"Project"+CSV+"Offering Part Number"+CSV+"Geography Part Number"+CSV+"Machine Type/Model Geography"+CSV+"Geography Marketing Description"+CSV+"Language (1=English)\n" ;

      if ( stmt != null ) {
         ResultSet rs = stmt.executeQuery ( "SELECT SUBGEOCODE, SUBGROUPNAME, FAMILYNAME, SERIESNAME, PROJECTNAME, OFFERINGPNUMB, PNUMB_SG, MTMPNUMB_SG, MARKETDESC_SG, NLSID "+
                                            "FROM OPICM.ES_SOL_NO_CSOL "+
                                            "WHERE PNUMB_SG IN "+getPartNums()+"" ) ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Walk the results                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {
            output+=trim ( rs.getString(1) )+CSV+
                    trim ( rs.getString(2) )+CSV+
                    trim ( rs.getString(3) )+CSV+
                    trim ( rs.getString(4) )+CSV+
                    trim ( rs.getString(5) )+CSV+
                    trim ( rs.getString(6) )+CSV+
                    trim ( rs.getString(7) )+CSV+
                    trim ( rs.getString(8) )+CSV+
                    trim ( rs.getString(9) )+CSV+
                    rs.getInt(10)+"\n" ;

         }
      } else
         LogSystem.log ( 1, "Could not create statement for writeES_SOL_NO_CSOL" ) ;


      output+="\n\n\n" ;
      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* writeES_MTM_NO_SOL
    */
   /**
    *    This method will write out the values
    *  from the ES_MTM_NO_SOL table
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeES_MTM_NO_SOL () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeES_MTM_NO_SOL", 5 ) ;
      Statement stmt      = sqlMethod.createStatement () ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our header information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      output+="View: ES_MTM_NO_SOL (Solutions with Country associations)\n" ;
      output+="Brand"+CSV+"Family"+CSV+"Series"+CSV+"Project"+CSV+"Offering Part Number"+CSV+" Marketing Description"+CSV+"Language (1=English)\n" ;

      if ( stmt != null ) {
         ResultSet rs = stmt.executeQuery ( "SELECT SUBGROUPNAME, FAMILYNAME, SERIESNAME, PROJECTNAME, OFFERINGPNUMB, OFWWMI, NLSID "+
                                            "FROM OPICM.ES_MTM_NO_SOL "+
                                            "WHERE OFFERINGPNUMB in "+getPartNums()+"" ) ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Walk the results                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {
            output+=trim ( rs.getString(1) )+CSV+
                    trim ( rs.getString(2) )+CSV+
                    trim ( rs.getString(3) )+CSV+
                    trim ( rs.getString(4) )+CSV+
                    trim ( rs.getString(5) )+CSV+
                    trim ( rs.getString(6) )+CSV+
                    rs.getInt(7)+"\n" ;

         }
      } else
         LogSystem.log ( 1, "Could not create statement for write ES_MTM_NO_SOL" ) ;


      output+="\n\n\n" ;
      sqlMethod.close() ;
   }




   /*------------------------------------------------------------*/
   /* writeES_MTM_NO_SOL
    */
   /**
    *    This method will write out the values
    *  from the ES_MTM_NO_SOL table
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeES_CTO_REGION_ONLY () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "write ES_CTO_REGION_ONLY", 5 ) ;
      Statement stmt      = sqlMethod.createStatement () ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our header information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      output+="View: ES_CTO_REGION_ONLY (CTO Solutions with Geography associations only)\n" ;
      output+="Geography"+CSV+""+
              "Sub Geography"+CSV+""+
              "Brand"+CSV+""+
              "Family"+CSV+""+
              "Series"+CSV+""+
              "Project"+CSV+""+
              "Country Level Part Number"+CSV+""+
              "Marketing Description"+CSV+""+
              "Announce Date"+CSV+""+
              "Language (1=English)\n" ;

      if ( stmt != null ) {
         ResultSet rs = stmt.executeQuery ( "SELECT GEOCODE, SUBGEOCODE, SUBGROUPNAME, FAMILYNAME, SERIESNAME, PROJECTNAME, COFPNUMB, CTOSOLMARKETDESC, COFANNOUNCEACT, NLSID "+
                                            "FROM OPICM.ES_CTO_REGION_ONLY "+
                                            "WHERE CVSL_PNUMB in "+getPartNums()+"" ) ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Walk the results                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {
            output+=trim ( rs.getString(1) )+CSV+
                    trim ( rs.getString(2) )+CSV+
                    trim ( rs.getString(3) )+CSV+
                    trim ( rs.getString(4) )+CSV+
                    trim ( rs.getString(5) )+CSV+
                    trim ( rs.getString(6) )+CSV+
                    trim ( rs.getString(7) )+CSV+
                    trim ( rs.getString(8) )+CSV+
                    trim ( rs.getString(9) )+CSV+
                    rs.getInt(10)+"\n" ;

         }
      } else
         LogSystem.log ( 1, "Could not create statement for write ES_CTO_REGION_ONLY" ) ;


      output+="\n\n\n" ;
      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* trim
    */
   /**
    *    This method is a safe method for trimming
    *  and cleaning up data retrieved from DB/2. It
    *  will not throw an exception for a null being passed
    *
    *  @param string The string to clean up
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String trim ( String string ) {
      if ( string == null )
         return( "" ) ;
      else
         return( string.trim() ) ;
   }





   /*------------------------------------------------------------*/
   /* writeCounts
    */
   /**
    *    This method will write out all of the counts
    *  for all of the OPIC/M Views
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int getCounts ( String table ) throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeCounts", 5 ) ;
      ResultSet rs        = null ;
      int       count     = 0 ;

      try {
         System.out.println ( "Pulling Row Counts for Table: "+table ) ;

         rs = sqlMethod.createStatement().executeQuery ( "SELECT COUNT(key) FROM OPICM."+table ) ;
         rs.next() ;
         count = rs.getInt(1) ;
         output+=table+CSV+count+"\n" ;
         //rs.close() ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }


      sqlMethod.close() ;

      return( count ) ;
   }


   private void writeCounts () throws Exception {
      output+="\n\nView Counts\nView,Number of Rows\n" ;

      getCounts ( "ES_COUNTRY_CPSL" ) ;
      //getCounts ( "ES_CTO_REGION_ONLY" ) ;
      getCounts ( "ES_COUNTRY_SOL" ) ;
      getCounts ( "ES_COUNTRY_CTO" ) ;
      //getCounts ( "ES_MTM_NO_SOL" ) ;
      //getCounts ( "ES_AAP_MTM" ) ;
      //getCounts ( "ES_CTO_NO_SOL" ) ;
      //getCounts ( "ES_SOL_NO_CSOL" ) ;
      
      //getCounts ( "ES_LANGUAGE_PACK" ) ;
      //getCounts ( "ES_SBB_RELATORS" ) ;
      //getCounts ( "ES_REGION_PCSL_NO_CT" ) ;

   }






   /*------------------------------------------------------------*/
   /* writeFullES_COUNTRY_CPSL
    */
   /**
    *    This method will write out a complete view for the ES_COUNTRY_CPSL
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_COUNTRY_CPSL () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_COUNTRY_CPSL", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;
      int count = 0 ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_COUNTRY_CPSL WHERE COFPNUMB IN "+getPartNums()+" OR CV_SOLPNUMB IN "+getPartNums()+" OR CVSL_PNUMB IN "+getPartNums() ) ;
         ResultSetMetaData meta = rs.getMetaData() ;

         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_COUNTRY_CPSL\n" ;
         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            System.out.println ( "Read Record "+count++ ) ;
            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            //output+=rs.getInt(i+1)+"\n" ;
            output+="\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }




   /*------------------------------------------------------------*/
   /* writeFullES_CTO_REGION_ONLY
    */
   /**
    *    This method will write out a complete view for the ES_CTO_REGION_ONLY
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_CTO_REGION_ONLY () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_CTO_REGION_ONLY", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_CTO_REGION_ONLY WHERE "+
                                                   "COFPNUMB IN "+getPartNums()+" OR "+
                                                   "CVSL_PNUMB IN "+getPartNums()+"" ) ;

         ResultSetMetaData meta = rs.getMetaData() ;

         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_CTO_REGION_ONLY\n" ;
         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            output+=""+rs.getInt(i+1)+"\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }




   /*------------------------------------------------------------*/
   /* writeFullES_COUNTRY_SOL
   * DAL _ NEEDS FIXING
    */
   /**
    *    This method will write out a complete view for the ES_COUNTRY_SOL
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_COUNTRY_SOL () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_COUNTRY_SOL", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_COUNTRY_SOL WHERE "+
                                                   "PNUMB_CT IN "+getPartNums()+"" ) ;

         ResultSetMetaData meta = rs.getMetaData() ;
         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_COUNTRY_SOL\n" ;
         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            //output+=""+rs.getInt(i+1)+"\n" ;
            output+="\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }


   /*------------------------------------------------------------*/
   /* writeFullES_COUNTRY_CTO
    */
   /**
    *    This method will write out a complete view for the ES_COUNTRY_CTO
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_COUNTRY_CTO () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_COUNTRY_CTO", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_COUNTRY_CTO WHERE "+
                                                   "COFPNUMB IN "+getPartNums()+" OR "+
                                                   "CCOSOLPNUMB IN "+getPartNums()+"" ) ;

         ResultSetMetaData meta = rs.getMetaData() ;
         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_COUNTRY_CTO\n" ;

         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            //output+=""+rs.getInt(i+1)+"\n" ;
            output+="\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }



   /*------------------------------------------------------------*/
   /* writeFullES_MTM_NO_SOL
    */
   /**
    *    This method will write out a complete view for the ES_MTM_NO_SOL
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_MTM_NO_SOL () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_MTM_NO_SOL", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_MTM_NO_SOL WHERE "+
                                                   "OFWWMI IN "+getPartNums()+" OR "+
                                                   "OFFERINGPNUMB IN "+getPartNums()+"" ) ;

         ResultSetMetaData meta = rs.getMetaData() ;
         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_MTM_NO_SOL\n" ;

         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            output+=""+rs.getInt(i+1)+"\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }



   /*------------------------------------------------------------*/
   /* writeFullES_AAP_MTM
    */
   /**
    *    This method will write out a complete view for the ES_AAP_MTM
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_AAP_MTM () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_AAP_MTM", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_AAP_MTM WHERE "+
                                                   "AAPSOLPNUMB IN "+getPartNums()+" OR "+
                                                   "OFFERINGPNUMB IN "+getPartNums()+" OR "+
                                                   "OFWWMI IN "+getPartNums()+"" ) ;

         ResultSetMetaData meta = rs.getMetaData() ;
         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_AAP_MTM\n" ;

         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            output+=""+rs.getInt(i+1)+"\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }




   /*------------------------------------------------------------*/
   /* writeFullES_CTO_NO_SOL
    */
   /**
    *    This method will write out a complete view for the ES_CTO_NO_SOL
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_CTO_NO_SOL () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_CTO_NO_SOL", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_CTO_NO_SOL WHERE "+
                                                   "COFPNUMB IN "+getPartNums()+"  " ) ;

         ResultSetMetaData meta = rs.getMetaData() ;
         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_CTO_NO_SOL\n" ;

         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            output+=""+rs.getInt(i+1)+"\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }





   /*------------------------------------------------------------*/
   /* writeFullES_SOL_NO_CSOL
    */
   /**
    *    This method will write out a complete view for the ES_SOL_NO_CSOL
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_SOL_NO_CSOL () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_SOL_NO_CSOL", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_SOL_NO_CSOL WHERE "+
                                                   "MTMPNUMB_SG IN "+getPartNums()+" OR "+
                                                   "PNUMB_SG IN "+getPartNums()+" OR "+
                                                   "OFFERINGPNUMB IN "+getPartNums()+" OR "+
                                                   "OFWWMI IN "+getPartNums()+"" ) ;

         
         ResultSetMetaData meta = rs.getMetaData() ;
         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_SOL_NO_CSOL\n" ;

         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            output+=""+rs.getInt(i+1)+"\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }




   /*------------------------------------------------------------*/
   /* writeFullES_REGION_PCSL_NO_CT
    */
   /**
    *    This method will write out a complete view for the ES_REGION_PCSL_NO_CT
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeFullES_REGION_PCSL_NO_CT () throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.OPICM, "writeFullES_REGION_PCSL_NO_CT", 5 ) ;
      Statement stmt      = sqlMethod.createStatement() ;

      try {
         ResultSet rs        = stmt.executeQuery ( "SELECT * FROM OPICM.ES_REGION_PCSL_NO_CT WHERE "+
                                                   "COFPNUMB IN "+getPartNums()+" OR "+
                                                   "VARSOLPNUMB IN "+getPartNums()+" OR "+
                                                   "CVSL_PNUMB IN "+getPartNums()+"" ) ;

         
         ResultSetMetaData meta = rs.getMetaData() ;
         output+="\n\nFull Pull of all queried products against all machine/model fields for view ES_REGION_PCSL_NO_CT\n" ;

         for ( int i = 1; i < meta.getColumnCount(); i++ ) {
            output+=meta.getColumnLabel ( i ) ;
            if ( i < meta.getColumnCount()-1 )
                output+=CSV ;
         }
         output+="\n" ;

         while ( rs.next() ) {
            int i = 1 ;

            for ( i = 1; i < meta.getColumnCount()-1; i++ ) {
               if ( rs.getString(i) != null )
                  output+=rs.getString(i).trim() ;
               else
                  output+="---" ;
               output+=CSV ;
            }

            output+=""+rs.getInt(i+1)+"\n" ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         e.printStackTrace() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

   }


   /*------------------------------------------------------------*/
   /* getPartNums
    */
   /**
    *    This method will return all of the part numbers
    *  that we are querying. It return them as one
    *  string, separated by commas for use in the 'WHERE XXX IN' clause of SQL
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getPartNums( ) {
      return( "('MOB3CTO', MOB2CTO', 'OPT1CTO', 'OPT1CUS','NET1CUS','MOB1CUS','CDT1CUS','OPT01US','NET01US','MOB01US','CDT01US','NET1CTO','NET1CTO','MOB1CTO','CDT1CTO','266031Z', '2675CTO', '6287CCJ', '6578CTO', '266031U', '10K3780', '633307N','634431U','684910U','686778U','868212Y','6849CTO','865455U','8364CXX','86566RY')" ) ;
   }


} /* Topic */
