package com.ibm.nzna.samples.servlet ;  // test 2
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.servlet.* ;

   /*********************************************************
   * This is an example of writing a servlet extending the
   * ServletBase class.  You must provide two methods:
   * the getPropertyFileName method and the servletService methods.
   * When this servlet is loaded by the WebServer service it will
   * search for property files in this order.
   * Global.properties,
   * HelloWorldServlet.properties,
   * HelloWorldServlet.InstanceName.properties.
   * The InstanceName is the name or alias name by which your
   * servlet is invoked in the URL.  So if this servlet is loaded
   * by the WebServer as HelloEveryone, then the last property file
   * searched for will be HelloWorldServlet.HelloEveryone.properties.
   * To test your property values this sample servlet prints the values
   * in the HTML data so you can verify the behaviour of the values you
   * specify in your property files.
   * The property files must be located in the classpath of the VM.
   * NOTE: in WebSphere this is NOT the reloadable classpath.
   * The property files are searched first in this location,
   * com.ibm.nzna.properties.  If not found the property will be searched
   * for in the root of the classpath.
   * Note: if your servlet needs to display error messages you should use the
   * StandardError class in the com.ibm.nzna.shared.servlet package.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/14/100  12:58pm
   * @author Chad Meadows
   * @version
   *********************************************************/
public class HelloWorldServlet extends ServletBase {
   private final String propertyFilename = "HelloWorldServlet" ;
   /*********************************************************
   * Returns the propertyFile name which will be loaded in
   * the init method of the servlet.  The property file is 
   * located by searching the classpath of the vm in which
   * the servlet is running.  The filename should be returned
   * without any extension.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  1:37pm
   * @param
   * @return A String of the property file name without extension.
   *********************************************************/
   protected String getPropertyFileName() {
      return ( propertyFilename ) ;
   }
   /*********************************************************
   * All units of work for the servlet are performed in this
   * method.  This method is called by the service method of
   * this servlet.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  2:23pm
   * @param
   * @return
   *********************************************************/
   public void servletService ( HttpServletRequest req , HttpServletResponse resp )
                         throws ServletException, IOException, MalformedURLException {

      /* This is used to write our output to the WebServer */
      ServletOutputStream         servletOutput                    = resp.getOutputStream() ;

      /* Write data to the web */
      servletOutput.print( "<HTML>Hello World") ;

      /* Writing to a Log file */
      /* This output will appear in the stdout log of WebSphere */
      writeLog ( "Sample Output") ;

      /* Write property values to web view */
      //properties.list( servletOutput ) ;

      /* End the HTML data tag */
      servletOutput.println( "</HTML>") ;

      /* Close the output Stream */
      servletOutput.close() ;
   }
}
