package com.ibm.nzna.shared.batch ;     
/*-----------------------------------------------------------*/
/*   BatchListener                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BatchListener                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This interface will act as a communication mechanism for a batch 
 *
 *
 * @version 1.0, 3/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BatchListener                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public interface BatchListener {






   /*------------------------------------------------------------*/
   /* logError
    */
   /**
    *    This method should be called when an error
    *  occurs. This error should not be a severe error,
    *  but a simple error, for example, 'Cannot find product 2312-XCX', or 'Cannot find Question 1'
    *
    *  @param errorStr The error string
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void logError ( String errorStr ) ;




   /*------------------------------------------------------------*/
   /* logSevereError
    */
   /**
    *    This method should log program errors that
    *  prevent operation. For example, 'Cannot Connect To FTP Site'
    *  or 'FTP Site Disconnected'.
    *
    *  @param errorStr The Error message
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void logSevereError ( String errorStr ) ;





   /*------------------------------------------------------------*/
   /* logSevereError
    */
   /**
    *    This method should be called in a catch
    *  block of when an exception happens
    *  
    *
    *  @param e The exception that happened
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void logSevereError ( Exception e ) ;




   /*------------------------------------------------------------*/
   /* logProgress
    */
   /**
    *    This method should be called to notify
    *  progress.
    *  
    *
    *  @param outputStr The output for the progress
    *  @param totalOperations The total number of operations needed to complete the batch
    *  @param operation The current operation that is being done
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void logProgress ( String outputStr, int totalOperations, int operation ) ;





   /*------------------------------------------------------------*/
   /* batchStart
    */
   /**
    *    This method should be called when the
    *  batch starts
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void batchStart () ;





   /*------------------------------------------------------------*/
   /* batchStop
    */
   /**
    *    This method should be called when the
    *  batch has finished.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void batchStop () ;
} /* BatchListener */
