package com.ibm.nzna.shared.db ;     
/*-----------------------------------------------------------*/
/*   DatabaseDlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DatabaseDlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
/**
 * This dialog shows what connections/methods etc..  that the
 * DatabaseSystem is reporting.
 *
 *
 * @version 1.0, 2/4/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DatabaseDlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DatabaseDlg extends JDialog implements ActionListener, DatabaseSystemListener {              


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private, static members                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final Color       oddLinesColor = new Color ( 240, 240, 240 ) ;
   private static final FontUtil    fontUtil      = new FontUtil () ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   MultiList Titles                                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String[] databaseTitles = { "", "Name", "Server", "Port", "Busy", "Secondary"} ;
   private String[] methodTitles   = { "Method Name", "Run Seconds"} ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private MultiList cnr_DATABASES         = null ;
   private MultiList cnr_METHODS           = null ;
   private JLabel    st_DATABASES          = null ;
   private JLabel    stt_DATABASES         = null ;
   private JLabel    st_METHODS            = null ;
   private JLabel    stt_METHODS           = null ;
   private DButton   pb_OK                 = null ;
   private Database  selectedDatabase      = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize and show this
    *  dialog. It will return immediatly since this
    *  dialog is not modal
    *
    *  @param parent The parent Frame
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DatabaseDlg ( ) {
      super ( new Frame(), "Database Debug Window", false ) ;     
      Container contentPane = getContentPane() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_DATABASES = new MultiList ( fontUtil ) ;
      cnr_METHODS   = new MultiList ( fontUtil ) ;
      st_DATABASES  = new JLabel    ( "Active Databases:" ) ;
      stt_DATABASES = new JLabel    ( "0" ) ;
      st_METHODS    = new JLabel    ( "Active Methods:" ) ;
      stt_METHODS   = new JLabel    ( "0" ) ;                                  
      pb_OK         = new DButton   ( "Ok" ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      stt_DATABASES.setForeground     ( Color.blue ) ;
      stt_METHODS.setForeground       ( Color.blue ) ;    
      cnr_DATABASES.setColumnHeadings ( databaseTitles ) ;
      cnr_METHODS.setColumnHeadings   ( methodTitles ) ;
      cnr_DATABASES.setBackground     ( Color.white ) ;
      cnr_DATABASES.setOddLinesColor  ( oddLinesColor ) ;
      cnr_METHODS.setOddLinesColor    ( oddLinesColor ) ;
      cnr_DATABASES.setColumnWidth    ( 0, 20 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( cnr_DATABASES ) ;
      contentPane.add ( st_DATABASES ) ;
      contentPane.add ( stt_DATABASES ) ;
      contentPane.add ( st_METHODS ) ;
      contentPane.add ( stt_METHODS ) ;
      contentPane.add ( cnr_METHODS ) ;
      contentPane.add ( pb_OK ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      DatabaseSystem.addDatabaseSystemListener ( this ) ;
      pb_OK.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our size, position and show us!                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 400, 400 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Our Data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       y         = 0 ;
      int       x         = 0 ;
      int       rowHeight = 18 ;

      super.doLayout() ;

      st_DATABASES.setBounds ( x, y, 150, rowHeight ) ;
      stt_DATABASES.setBounds ( x+150, y, 50, rowHeight ) ;
      y+=rowHeight ;

      cnr_DATABASES.setBounds ( x, y, size.width-13, rowHeight*7 ) ;
      y+=(rowHeight*8) ;


      st_METHODS.setBounds ( x, y, 150, rowHeight ) ;
      stt_METHODS.setBounds ( x+150, y, 50, rowHeight ) ;
      y+=rowHeight ;

      cnr_METHODS.setBounds ( x, y, size.width-13, size.height-70-y ) ;

      pb_OK.setBounds ( x, size.height-55, 70, 25 ) ;
   }

   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  is perfermed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      DatabaseSystem.removeDatabaseSystemListener ( this ) ;
      dispose() ;
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh ALL data
    *  on the dialog
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      Vector vec = DatabaseSystem.getActiveDatabases() ;

      if ( vec != null ) {
         cnr_DATABASES.removeAll() ;
         cnr_DATABASES.add ( vec ) ;
         stt_DATABASES.setText ( ""+vec.size() ) ;
         cnr_METHODS.add ( DatabaseSystem.getActiveMethods() ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* methodAdded
    */
   /**
    *    This method will be called when another
    *  SQL has kicked off to run. We will add
    *  this to our list if we are displaying the right database
    *
    *  @param database The database that the method was added to
    *  @param method The method
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void methodAdded ( Database database, SQLMethod method ) {

      cnr_METHODS.add ( (MultiListRow)method ) ;
      try {
         stt_METHODS.setText ( ""+cnr_METHODS.getData().size() ) ;
      } catch ( Exception e ) {
         stt_METHODS.setText ( "0" ) ;
      }
      cnr_DATABASES.revalidate() ;
   }



   /*------------------------------------------------------------*/
   /* methodRemoved
    */
   /**
    *    This method will be called when another
    *  SQL has kicked off to run. We will add
    *  this to our list if we are displaying the right database
    *
    *  @param database The database that the method was removed to
    *  @param method The method
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void methodRemoved ( Database database, SQLMethod method ) {
      cnr_METHODS.remove ( method ) ;
      try {
         stt_METHODS.setText ( ""+cnr_METHODS.getData().size() ) ;
      } catch ( Exception e ) {
         stt_METHODS.setText ( "0" ) ;
      }

      cnr_DATABASES.revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* databaseAdded
    */
   /**
    *    This method will be called when a
    *  database is added
    *  
    *
    *  @param database The database added
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void databaseAdded ( Database database ) {
      cnr_DATABASES.add ( database ) ;
      stt_DATABASES.setText ( ""+DatabaseSystem.getActiveDatabases().size() ) ;
   }





   /*------------------------------------------------------------*/
   /* databaseRemoved
    */
   /**
    *    This method will be called when a database
    *  is removed from the list of active databases.
    *  We will remove the database from our list here
    *
    *  @param database The database that was removed
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void databaseRemoved ( Database database ) {
      cnr_DATABASES.remove ( database ) ;
      stt_DATABASES.setText ( ""+DatabaseSystem.getActiveDatabases().size() ) ;
   }






   /*------------------------------------------------------------*/
   /* dispose
    */
   /**
    *    This method is subclassed so that we close
    *  our parents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void dispose () {
      super.dispose() ;
      ((Frame)getParent()).dispose() ;
   }


} /* DatabaseDlg */
