package com.ibm.nzna.shared.db ;                             
/*-----------------------------------------------------------*/
/*   DatabaseStatusListener                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DatabaseStatusListener              */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This listener will be notified of changes in the status of the DatabaseSystem. 
 * These status changes can range from 'Connecting to Database' to 'Disconnecting'.
 *
 * @version 1.0, 3/20/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DatabaseStatusListener                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public interface DatabaseStatusListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Status Types                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * Status type sent when a connection is being attempted
    */
   public static final int DATABASE_STATUS_CONNECTING    = 1 ;

   /**
    * Status type sent when a connection has been completed
    */
   public static final int DATABASE_STATUS_CONNECTED     = 2 ;

   /**
    * Status type sent when disconnecting from a database
    */
   public static final int DATABASE_STATUS_DISCONNECTING = 3 ;

   /**
    * Status type sent when a disconnection is complete
    */
   public static final int DATABASE_STATUS_DISCONNECTED  = 4 ;

   /**
    * Status type sent when a database becomes 'busy', meaning it is running an SQL
    */
   public static final int DATABASE_STATUS_BUSY          = 5 ;

   /**
    * Status type sent when a database becomes 'free' again, meaning it is running NO SQL.
    */
   public static final int DATABASE_STATUS_FREE          = 6 ;


   /**
    * Status type sent when a connection fails because the database cannot be accessed, this
    * may be from a variety of reasons (Database name wrong, network problems etc..
    */
   public static final int DATABASE_STATUS_UNAVAILABLE   = 7 ;







   /*------------------------------------------------------------*/
   /* databaseStatusChange
    */
   /**
    *    This method will be called when the current
    *  status of a database has changed.
    *  
    *
    *  @param statusType The new type of status
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void databaseStatusChange ( int statusType, Database database ) ;

} /* DatabaseStatusListener */
