package com.ibm.nzna.shared.db ;                                   
/*-----------------------------------------------------------*/
/*   DatabaseSystemListener                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DatabaseSystemListener              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This listener is notified when events happen in the Database System. These 
 * events may include 
 *
 *
 * @version 1.0, 2/4/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DatabaseSystemListener                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public interface DatabaseSystemListener {





    /*------------------------------------------------------------*/
    /* databaseAdded
     */
    /**
     *    This method will be called when a database
     *  has been added to the list of available
     *  databases
     *
     *  @param database The database added
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public abstract void databaseAdded ( Database database ) ;




    /*------------------------------------------------------------*/
    /* databaseRemoved
     */
    /**
     *    This method will be called when a database
     *  is removed
     *  
     *
     *  @param database The database removed
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public abstract void databaseRemoved ( Database database ) ;





    /*------------------------------------------------------------*/
    /* methodAdded
     */
    /**
     *    This method will be called when a 
     *  method has been added
     *  
     *
     *  @param database The database that the method was added to
     *  @param method The SQLMethod object added
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public abstract void methodAdded ( Database database, SQLMethod method ) ;





    /*------------------------------------------------------------*/
    /* methodRemoved
     */
    /**
     *    This method will be called when a SQLMethod
     *  is removed
     *  
     *
     *  @param database The database that the method was removed from
     *  @param method The method that was removed
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public abstract void methodRemoved  ( Database database, SQLMethod method ) ;




} /* DatabaseSystemListener */
