package com.ibm.nzna.shared.db ;
/*-----------------------------------------------------------*/
/*   SQLParse                                                */
/*                                                           */
/*   Description:                                            */
/*      This class holds routines to help with sql in Java   */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM Corporation                  */
/*         Licensed Materials - Property of IBM              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   SQLParse                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class SQLParse extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected static final int    EOS        = 3 ;   // End of String
   protected static final int    EOR        = 4 ;   // End of Record
   protected static final int    EOB        = 5 ;   // End of Buffer
   protected static final int    BEGIN_DATA = 1 ;   // Beginning of the actual data
   protected static final String EOR_STR    = new Character ( (char) EOR ).toString() ;
   protected static final String EOS_STR    = new Character ( (char) EOS ).toString() ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Protected Variables                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int            curSet    = -1 ;
   private String[]       records   = null ;
   private String[]       stringSet = null ;
   private int            numOfRecs = 0 ;
   private int            numOfStr  = 0 ;
   private String         SQLBuffer = null ;
   private boolean        valid     = false ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constructors                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public SQLParse () {
   }

   public SQLParse ( String SQLBuffer ) {
      if ( ( SQLBuffer != null ) && ( SQLBuffer.length() > 1 ) )
         setSQLBuffer ( SQLBuffer ) ;
   }


   /*-----------------------------------------------------------*/
   /*   setSQLBuffer()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the current                     */
   /*                                                           */
   /*   Parameters:                                             */
   /*                                                           */
   /*                                                           */
   /*   Returns:                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setSQLBuffer ( String SQLBuffer ) {
      this.SQLBuffer = SQLBuffer ;
      trimBuffer() ;
      readBuffer () ;
      valid = true ;
   } /* setSQLBuffer */



   /*-----------------------------------------------------------*/
   /*   close()                                                 */
   /*                                                           */
   /*   Description:                                            */
   /*      This function will dispose of all buffers assoc.     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void close() {
      int recLength       = 0 ;
      int stringSetLength = 0 ;
      int i               = 0 ;

      SQLBuffer = null ;
      if ( records != null )
         recLength       = records.length ;
      if ( stringSet != null )
         stringSetLength = stringSet.length ;

      i=0 ;
      while ( i < recLength ) {
         records[i] = null ;
         i++ ;
      }

      i=0 ;
      while ( i < stringSetLength ) {
         stringSet[i] = null ;
         i++ ;
      }

      records   = null ;
      stringSet = null ;

   }

   /*-----------------------------------------------------------*/
   /*   readBuffer()                                            */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will read the header information of the  */
   /*   buffer                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private void readBuffer ( ) {
      int          bufLen  = 0 ;
      String       tempBuf = null ;
      int          recnum  = 0 ;
      int          i       = 0 ;

      if ( ( SQLBuffer != null ) && ( SQLBuffer.length() > 2) ) {
         StringTokenizer token      = null ;

         /* Get the number of strings in each record, and initialize our
            stringSet array */
         numOfStr = SQLBuffer.charAt(0) ;
         stringSet = new String[numOfStr] ;

         /* Chop off the first character */
         SQLBuffer = SQLBuffer.substring ( BEGIN_DATA, SQLBuffer.length()-BEGIN_DATA ) ;

         /* Initialize the tokenizer */
         token = new StringTokenizer ( SQLBuffer, EOR_STR ) ;              
         numOfRecs = token.countTokens() ;
         records = new String [ numOfRecs ] ;

         /* Tok the records and store them in our array */
         while ( token.hasMoreTokens() ) 
            records[i++] = token.nextToken() ;

      } /* SQLBuffer */

   } /* readHeader */

   /*-----------------------------------------------------------*/
   /*   readRecord()                                            */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will break the record at recNum into     */
   /*   an array of strings                                     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      recNum - The record number to break up               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private void readRecord ( int recNum ) {
      try {
         StringBuffer    tempBuf     = null ;
         int             i           = 0 ;
         int             length      = records[recNum].length() ;
         int             strnum      = 0 ;
         boolean         cont        = true ;
         int             start       = 0 ;
         int             end         = 0 ; 
         StringTokenizer token       = new StringTokenizer ( records[recNum], EOS_STR ) ;

         while ( token.hasMoreTokens() ) 
            stringSet[i++] = token.nextToken() ;

      } catch ( ArrayIndexOutOfBoundsException e ) {
      }
   } /* readRecord */

   /*-----------------------------------------------------------*/
   /*   next()                                                  */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will return TRUE and advance the parser  */
   /*   to the next result set if there are more results        */
   /*                                                           */
   /*   Returns:                                                */
   /*      FALSE if no more results are here                    */
   /*-----------------------------------------------------------*/
   public boolean next () {
      boolean ret = true ;

      if ( valid ) {
         curSet++ ;
         if ( curSet >= numOfRecs )
            ret = false ;
         else
            readRecord ( curSet ) ;
      } else
         ret = false ;
      return ( ret ) ;
   } /* next */

   /*-----------------------------------------------------------*/
   /*   reset()                                                 */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will reset the parser to the beginning   */
   /*   of the records                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void reset () {
      curSet = 0 ;
   }


   /*-----------------------------------------------------------*/
   /*   readString()                                            */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will return the String at the specified  */
   /*   index                                                   */
   /*                                                           */
   /*   Parameters:                                             */
   /*      index - The index of what String to read             */
   /*                                                           */
   /*   Returns:                                                */
   /*      The String at the index                              */
   /*-----------------------------------------------------------*/
   public String readString ( int index ) {

      if ( ( valid ) && ( index < numOfStr ) ) {
         if ( stringSet[index] != null )
            return ( stringSet[index].trim() ) ;
      }

      return ( "" ) ;

   } /* readString */

   public boolean readBoolean ( int index ) {
      boolean ret = false ;

      if ( ( valid ) && ( index < numOfStr ) ) {
         if ( ( stringSet[index] != null ) && ( stringSet[index].equals ( "Y" ) ) )
            ret = true ;
      }

      return ( ret ) ;
   } /* readBoolean */

   public short readShort ( int index ) {
      short  ret    = 0 ;

      if ( valid ) {   
         if ( index < numOfStr ) {
            try {
               ret = new Short ( stringSet[index] ).shortValue() ;
            } catch ( Exception e ) {}
         }
      } /* if valid */

      return ( ret ) ;
   } /* readString */

   public int readInt ( int index ) {
      int    ret    = 0 ;

      if ( valid ) {
         if ( index < numOfStr ) {
            try {
               ret = new Integer ( stringSet[index] ).intValue() ;
            } catch ( Exception e ) {}

         }
      }
      return ( ret ) ;
   } /* readInt */


   public double readDouble ( int index ) {
      double ret    = 0.0 ;

      if ( valid ) {
         if ( index < numOfStr ) {
            try {
               ret = new Double ( stringSet[index] ).doubleValue() ;
            } catch ( Exception e ) {}
         }
      }
      return ( ret ) ;
   } /* readDouble */



   /*-----------------------------------------------------------*/
   /*   trimBuffer                                              */         
   /*                                                           */
   /*   Description:                                            */
   /*      Trims all trailing spaces from the buffer            */                                                          
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void trimBuffer () {
      String       eobStr   = new Character((char)EOB).toString() ;
      StringBuffer buf      = new StringBuffer ( SQLBuffer ) ;             
      int          eobIndex = SQLBuffer.indexOf ( eobStr ) ;

      if ( eobIndex > 0 ) 
         SQLBuffer = SQLBuffer.substring ( 0, eobIndex ) ;
   }

   /*-----------------------------------------------------------*/
   /*   getRecordCount                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      Gets the number of records in the buffer             */                                                     
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public int getRecordCount() {
      if ( valid )
         return ( numOfRecs ) ;
      else
         return ( 0 ) ;
   }

   protected void finalize () {
      close() ;
   }

} /* sql */
