package com.ibm.nzna.shared.db ;
/*-----------------------------------------------------------*/
/*   SQLStatement                                            */
/*                                                           */
/*   Description:                                            */
/*      This file holds source code for the SQL Statements   */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM Corporation                  */
/*         Licensed Materials - Property of IBM              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.util.* ;
/**
 * The SQLStatement is a String that has special methods to create a 
 * string the programmer can pass to a JDBC database. Currentlt the only
 * supported SQL statements are SELECT, INSERT, UPDATE and DELETE Statements. The SELECT
 * Statement does not support joining tables yet.
 *
 *
 * @version 1.0, Nov 24, 1998
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   SQLStatement                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class SQLStatement extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int STMT_QUERY              = 0 ;
   public static final int STMT_INSERT             = 1 ;
   public static final int STMT_UPDATE             = 2 ;
   public static final int STMT_DELETE             = 3 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Protected Variables                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected int           stmtType          = 0 ;
   protected String[]      colNames          = null ;
   protected String[][]    whereColNames     = null ;         
   protected String        tableName         = "" ;
   protected Object[]      colNamesData      = null ;
   protected Object[]      whereColNamesData = null ;
   protected String        whereClause       = "" ;





   /*------------------------------------------------------------*/
   /* SQLStatement
    */
   /**
    *    This will create a blank SQL Statement 
    *  with no type and no columns
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SQLStatement () {
   } /* SQLStatement */





   /*------------------------------------------------------------*/
   /* SQLStatement
    */
   /**
    *    This will create a SQL Statement object with
    *  the specified table name
    *  
    *
    *  @param tableName The Table name including any qualifierd
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SQLStatement ( String tableName ) {
      this.tableName = tableName ;
   }


   /*------------------------------------------------------------*/
   /* SQLStatement
    */
   /**
    *    This will create a specific type of SQL Statement
    *  going against a certain table
    *  
    *
    *  @param stmtType The type of statement ( STMT_INSERT, STMT_UPDATE etc.. )
    *  @param tableName The Name of the table to go against
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SQLStatement ( int stmtType, String tableName ) {
      this.stmtType  = stmtType ;
      this.tableName = tableName ;
   } /* tabelName */


   /*------------------------------------------------------------*/
   /* SQLStatement
    */
   /**
    *    This will create a specific type of SQL Statement
    *  going against a certain table with certain columns
    *  
    *
    *  @param stmtType The type of statement ( STMT_INSERT, STMT_UPDATE etc.. )
    *  @param tableName The Name of the table to go against
    *  @param colNames An array of strings. Each string should represent a column name
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SQLStatement ( int stmtType, String tableName, String[] colNames ) {
      this.stmtType  = stmtType ;
      this.tableName = tableName ;
      setColumnNames ( colNames ) ;
   }


   /*------------------------------------------------------------*/
   /* SQLStatement
    */
   /**
    *    This will create a specific type of SQL Statement
    *  going against a certain table with certain columns and where clause columns
    *  
    *
    *  @param stmtType The type of statement ( STMT_INSERT, STMT_UPDATE etc.. )
    *  @param tableName The Name of the table to go against
    *  @param colNames An array of strings. Each string should represent a column name
    *  @param whereColNames A two dimensional array of strings. The first dimension should be the column name, the second should be the operator ( =, <>, LIKE etc.. )
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SQLStatement ( int stmtType, String tableName, String[] colNames, String[][] whereColNames ) {
      this.stmtType      = stmtType ;
      this.tableName     = tableName ;
      setColumnNames ( colNames ) ;
      setWhereColumnNames ( whereColNames ) ;
   }


   /*-----------------------------------------------------------*/
   /*   setType()                                               */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set what type of statement this is  */
   /*                                                           */
   /*   Parameters:                                             */
   /*      newType - The new type of the stmt. STMT_QUERY ...   */
   /*                                                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setType ( int newType ) {
      this.stmtType = newType ;
   } /* setType */


   /*-----------------------------------------------------------*/
   /*   findColIndex()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will return the index in the array       */
   /*   of the passed colName.                                  */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - The column name to look for                */
   /*      list    - The list to look in                        */
   /*                                                           */
   /*   Returns:                                                */
   /*      index of colName in list                             */
   /*-----------------------------------------------------------*/
   private int findColIndex ( String colName, String[] list ) {
      int     i     = 0 ;
      int     index = -1 ;
      boolean found = false ;

      while ( ( i < list.length ) && ( index == -1 ) ) {
         if ( list[i].compareTo ( colName ) == 0 ) {
            index = i ;
            found = true ;
         }
         i++ ;
      } /* while */

      return ( index ) ;

   } /* findColIndex */

   /*-----------------------------------------------------------*/
   /*   findColIndex()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will return the index in the array       */
   /*   of the passed colName.                                  */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - The column name to look for                */
   /*      list    - The list to look in                        */
   /*                                                           */
   /*   Returns:                                                */
   /*      index of colName in list                             */
   /*-----------------------------------------------------------*/
   private int findColIndex ( String colName, String[][] list ) {
      int     i     = 0 ;
      int     index = -1 ;
      boolean found = false ;

      while ( ( i < list.length ) && ( index == -1 ) ) {
         if ( list[i][0].compareTo ( colName ) == 0 ) {
            index = i ;
            found = true ;
         }
         i++ ;
      } /* while */

      return ( index ) ;

   } /* findColIndex */

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      strData - The string data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( String colName, String strData ) {
      int index = findColIndex ( colName, colNames ) ;

      if ( strData == null )
         strData = "NULL" ;

      colNamesData[index] = strData ;
   }

   public void setColumnDataAbsolute ( String colName, String strData ) {
      int index = findColIndex ( colName, colNames ) ;

      if ( strData == null )
         strData = "NULL" ;

      colNamesData[index] = "'"+strData ;
   }


   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      intData - The integer data                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( String colName, int intData ) {
      int index = findColIndex ( colName, colNames ) ;

      colNamesData[index] = new Integer ( intData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      dblData - The double data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( String colName, double dblData ) {
      int index = findColIndex ( colName, colNames ) ;

      colNamesData[index] = new Double ( dblData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName  - String of the column name                 */
   /*      boolData - The boolean data                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( String colName, boolean boolData ) {
      int index = findColIndex ( colName, colNames ) ;

      if ( boolData )
         colNamesData[index] = new String ( "Y" ) ;
      else
         colNamesData[index] = new String ( "N" ) ;

   } /* setColumnData */

   /*-----------------------------------------------------------*/
   /*   setColumnDate()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName  - String of the column name                 */
   /*      strDate  - The String date                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnDate ( String colName, String strDate ) {
      int index = findColIndex ( colName, colNames ) ;

      if ( strDate != null ) {
         if ( strDate.length() > 6 ) {
            strDate = strDate.replace ( '/', '-' ) ;
            colNamesData[index] = strDate ;
         }
      }

   } /* setColumnData */



   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      strData - The string data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( int index, String strData ) {

      if ( strData == null )
         strData = "NULL" ;

      colNamesData[index] = strData ;
   }

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      intData - The integer data                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( int index, int intData ) {

      colNamesData[index] = new Integer ( intData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      dblData - The double data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( int index, double dblData ) {

      colNamesData[index] = new Double ( dblData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName  - String of the column name                 */
   /*      boolData - The boolean data                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( int index, boolean boolData ) {

      if ( boolData )
         colNamesData[index] = new String ( "Y" ) ;
      else
         colNamesData[index] = new String ( "N" ) ;

   } /* setColumnData */

   /*-----------------------------------------------------------*/
   /*   setColumnDate()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName  - String of the column name                 */
   /*      strDate  - The String date                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnDate ( int index, String strDate ) {

      if ( strDate != null ) {
         if ( strDate.length() > 6 ) {
            strDate = strDate.replace ( '/', '-' ) ;
            colNamesData[index] = strDate ;
         }
      }

   } /* setColumnData */




   /*------------------------------------------------------------*/
   /* setWhereColumnNames
    */
   /**
    *    This method will set the names and operators
    *  to use in the where clause of the SQL Statement
    *  
    *
    *  @param whereColNames A two dimensional array of strings. The first dimension should 
    *                       be the column name, the second should be the operator ( =, <>, LIKE etc.. )
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWhereColumnNames ( String[][] whereColNames ) {
      this.whereColNames = whereColNames ;
      whereColNamesData = new Object [ whereColNames.length ] ;
   }


   /*-----------------------------------------------------------*/
   /*   setWhereClause ()                                       */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the where clause                */
   /*                                                           */
   /*   Parameters:                                             */
   /*      whereClause - The new where clause                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereClause ( String whereClause ) {
      this.whereClause = whereClause ;
   } /* setWhereClause */         

   /*-----------------------------------------------------------*/
   /*   setWhereData()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      strData - The string data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereData ( String colName, String strData ) {
      int index = findColIndex ( colName, whereColNames ) ;

      if ( index > -1 )
         whereColNamesData[index] = strData ;

   }

   /*-----------------------------------------------------------*/
   /*   setWhereData()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      intData - The integer data                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereData ( String colName, int intData ) {
      int index = findColIndex ( colName, whereColNames ) ;

      if ( index > -1 )
         whereColNamesData[index] = new Integer ( intData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setWhereData()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      dblData - The double data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereData ( String colName, double dblData ) {
      int index = findColIndex ( colName, whereColNames ) ;

      if ( index > -1 )
         whereColNamesData[index] = new Double ( dblData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setWhereData()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName  - String of the column name                 */
   /*      boolData - The boolean data                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereData ( String colName, boolean boolData ) {
      int index = findColIndex ( colName, whereColNames ) ;

      if ( index > -1 ) {
         if ( boolData )
            whereColNamesData[index] = new String ( "Y" ) ;
         else
            whereColNamesData[index] = new String ( "N" ) ;
      }
   } /* setWhereData */

   /*-----------------------------------------------------------*/
   /*   getData()                                               */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will look at what type of Object was     */
   /*   passed, and create a DB/2 string from the object        */
   /*                                                           */
   /*   Parameters:                                             */
   /*      obj - The object that was passed                     */
   /*                                                           */
   /*   Returns:                                                */
   /*      String for DB/2                                      */
   /*-----------------------------------------------------------*/
   private String getData ( Object obj ) {
      String retStr = "" ;

      if ( obj instanceof String ) {
         String tempStr = (String) obj ;

         /* Do not put quotes around a null,
            or question mark */
         if ( ( !tempStr.equals ( "CURRENT TIMESTAMP" ) ) &&
              ( !tempStr.equals ( "USER" ) ) &&
              ( !tempStr.equals ( "NULL" ) ) &&
              ( tempStr.indexOf ( "(SELECT" ) == -1 ) &&
              ( tempStr.indexOf ( "( SELECT" ) == -1 ) &&
              ( !tempStr.equals ( "?" ) ) )
            retStr = "'"+Text.replaceAllStrInStr ( tempStr, "'", "''" )+"'" ;
         else
            retStr = tempStr ;
      } else if ( obj instanceof Integer )
         retStr = ""+(Integer)obj ;
      else if ( obj instanceof Double )
         retStr = ""+(Double)obj ;
      else if ( obj == null )
         retStr = "NULL" ;

      retStr = retStr+" " ;

      return ( retStr ) ;

   } /* getData */

   /*-----------------------------------------------------------*/
   /*   getAllData()                                            */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will get a list of ALL column data, and  */
   /*   return it in the form of:"'this', 1, 'that', 2"         */
   /*                                                           */
   /*   Parameters:                                             */
   /*                                                           */
   /*                                                           */
   /*   Returns:                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String getAllData ( ) {
      StringBuffer retStr = new StringBuffer ( "" ) ;
      String       allStr = null ;
      int          i      = 0 ;

      while ( i < colNamesData.length ) {

         if ( colNames[i] != null )
            retStr.append ( getData ( colNamesData[i] )+", " ) ;

         i++ ;
      } /* while */

      /* Take off the last comma */
      allStr = retStr.toString() ;
      allStr = allStr.substring ( 0, allStr.length()-2 )+" " ;

      return ( allStr ) ;
   } /* getAllData */


   /*-----------------------------------------------------------*/
   /*   getColNames()                                           */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will return a string of column names     */
   /*   seperated by commas                                     */
   /*                                                           */
   /*   Parameters:                                             */
   /*                                                           */
   /*                                                           */
   /*   Returns:                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String getColNames () {
      String retStr = "" ;
      int    i      = 0 ;

      while ( i < colNames.length ) {
         if ( colNames[i] != null )
            retStr = retStr + colNames[i]+", " ;
         i++ ;
      } /* while */

      retStr = retStr.substring ( 0, retStr.length()-2 ) ;

      return ( retStr ) ;
   } /* getColNames */


   /*-----------------------------------------------------------*/
   /*   getWhereClause()                                        */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will create a where clause for this stmt */
   /*                                                           */
   /*   Returns:                                                */
   /*      The string representation of a where clause          */
   /*-----------------------------------------------------------*/
   private String createWhereClause () {
      StringBuffer retStr = new StringBuffer ( "" ) ;
      int          i      = 0 ;

      if ( whereColNames != null ) {
         retStr.append ( " WHERE " ) ;

         while ( i < whereColNames.length ) {

            if ( ( whereColNames[i][0] != null ) && ( whereColNames[i][1] != null ) ) {

               /* Append the equal sign and restriction value */
               retStr.append ( whereColNames[i][0] ) ;
               retStr.append ( " = " ) ;
               retStr.append ( getData ( whereColNamesData[i] ) ) ;

               if ( i != ( whereColNames.length-1 ) )
                  retStr.append ( whereColNames[i][1] ) ;

               retStr.append ( " " ) ;

            }
            i++ ;   

         } /* while */
      } /* if */
      else
         return ( whereClause );

      retStr.append ( " " ) ;
      return ( retStr.toString () ) ;

   } /* getWhereClause */




   /*-----------------------------------------------------------*/
   /*   createSetClause()                                       */
   /*                                                           */
   /*   Description:                                            */
   /*      THis method will create a set clause for use         */
   /*                                                           */
   /*   Returns:                                                */
   /*      String representation of the set clause              */
   /*-----------------------------------------------------------*/
   private String createSetClause () {
      StringBuffer retStr = new StringBuffer ( "" ) ;
      String       setStr = null ;
      int          i      = 0 ;

      if ( colNames != null ) {
         while ( i < colNames.length ) {

            if ( colNames[i] != null ) {
               retStr.append ( colNames[i] ) ;
               retStr.append ( "=" ) ;
               retStr.append ( getData ( colNamesData[i] )+", " ) ;
            } /* if */
            i++ ;
         } /* while */
      } /* if */

      /* Take the last comma out of the string */
      setStr = retStr.toString() ;
      setStr = setStr.substring ( 0, setStr.length()-2 ) ;

      return ( setStr ) ;
   } /* createSetClause */




   /*-----------------------------------------------------------*/
   /*   toString()                                              */
   /*                                                           */
   /*   Description:                                            */
   /*    This method will create the string and return it       */
   /*                                                           */
   /*   Returns:                                                */
   /*      The SQL Statement                                    */
   /*-----------------------------------------------------------*/
   public String toString () {
      String retStr = "" ;

      switch ( stmtType ) {

         case STMT_QUERY :
            retStr = "SELECT "+getColNames()+" FROM "+tableName+" "+createWhereClause()+" FOR FETCH ONLY" ;
            break ;

         case STMT_INSERT :
            retStr = "INSERT INTO "+tableName+" ( "+getColNames()+" ) values ( "+getAllData()+" ) " ;
            break ;

         case STMT_UPDATE :
            retStr = "UPDATE "+tableName+" SET "+createSetClause()+createWhereClause() ;
            break ;

         case STMT_DELETE :
            retStr = "DELETE FROM "+tableName+" "+createWhereClause() ;
            break ;


      } /* stmtType */

      return ( retStr ) ;
   } /* String */


   /*------------------------------------------------------------*/
   /* setColumnNames
    */
   /**
    *    This method will set the column name array to
    *  use when creating the SQL String
    *  
    *
    *  @param colNames An array of strings. Each string should represent a column name
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnNames ( String[] colNames ) {
      this.colNames = colNames ;
      colNamesData = new Object[colNames.length] ;
   }





   /*------------------------------------------------------------*/
   /* setColumnNames
    */
   /**
    *    This method will allocate enough room for the
    *  specified column names. It will not set any column
    *  names though. Please use setColumnName() to set the colunm name
    *
    *  @param columnCount The count of all columns
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnNames ( int columnCount ) {
      colNames = new String[columnCount] ;
      colNamesData = new Object[columnCount] ;
   }



   /*------------------------------------------------------------*/
   /* setColumnName
    */
   /**
    *    This method will set the name of a specific
    *  column
    *  
    *
    *  @param index The column index to set the name to
    *  @param columnName The name of the column
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnName ( int index, String columnName ) {
      colNames[index] = columnName ;
   }




   /*------------------------------------------------------------*/
   /* setWhereColumnNames
    */
   /**
    *    This method will allocate enough room for
    *  the specified amount of where clause column names. This
    *  will not set any column names. Please use setWhereColumnName() to set the column name and operator
    *
    *  @param columnCount The number of where clause column names
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWhereColumnNames ( int columnCount ) {
      whereColNames = new String[columnCount][2] ;
      whereColNamesData = new Object [ columnCount ] ;
   }




   /*------------------------------------------------------------*/
   /* setWhereColumnName
    */
   /**
    *    This method will set the column name and operator
    *  of a column specified in the where clause of
    *  the SQL Statement
    *
    *  @param index The Where Column index to set the name and operator to
    *  @param columnName The string representation of the column name
    *  @param op The string representation of the operator ( =, <>, IS NOT, LIKE etc.. )
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWhereColumnName ( int index, String columnName, String op ) {
      whereColNames[index][0] = columnName ;
      whereColNames[index][1] = op ;
   }




   /*-----------------------------------------------------------*/
   /*   setWhereData()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      strData - The string data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereData ( int index, String strData ) {

      if ( index > -1 )
         whereColNamesData[index] = strData ;

   }

   /*-----------------------------------------------------------*/
   /*   setWhereData()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      intData - The integer data                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereData ( int index, int intData ) {

      if ( index > -1 )
         whereColNamesData[index] = new Integer ( intData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setWhereData()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      dblData - The double data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereData ( int index, double dblData ) {

      if ( index > -1 )
         whereColNamesData[index] = new Double ( dblData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setWhereData()                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName  - String of the column name                 */
   /*      boolData - The boolean data                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setWhereData ( int index, boolean boolData ) {

      if ( index > -1 ) {
         if ( boolData )
            whereColNamesData[index] = new String ( "Y" ) ;
         else
            whereColNamesData[index] = new String ( "N" ) ;
      }
   } /* setWhereData */



   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      strData - The string data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( int index, String colName, String strData ) {

      colNames[index] = colName ;
      if ( ( strData == null ) || ( strData.length() == 0 ) )
         strData = "NULL" ;

      colNamesData[index] = strData ;
   }

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      intData - The integer data                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( int index, String colName,  int intData ) {

      colNames[index] = colName ;
      colNamesData[index] = new Integer ( intData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName - String of the column name                  */
   /*      dblData - The double data                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( int index, String colName,  double dblData ) {

      colNames[index] = colName ;
      colNamesData[index] = new Double ( dblData ) ;
   }

   /*-----------------------------------------------------------*/
   /*   setColumnData()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName  - String of the column name                 */
   /*      boolData - The boolean data                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnData ( int index, String colName,  boolean boolData ) {

      colNames[index] = colName ;
      if ( boolData )
         colNamesData[index] = new String ( "Y" ) ;
      else
         colNamesData[index] = new String ( "N" ) ;

   } /* setColumnData */

   /*-----------------------------------------------------------*/
   /*   setColumnDate()                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the column data of a column     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      colName  - String of the column name                 */
   /*      strDate  - The String date                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setColumnDate ( int index, String colName,  String strDate ) {

      colNames[index] = colName ;
      if ( strDate != null ) {
         if ( strDate.length() > 6 ) {
            strDate = strDate.replace ( '/', '-' ) ;
            colNamesData[index] = strDate ;
         }
      }

   } /* setColumnData */





   /*------------------------------------------------------------*/
   /* setTableName
    */
   /**
    *    This method will set the fully qualified table
    *  name to use.
    *  
    *
    *  @param tableName The fully qualified table name
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTableName ( String tableName ) {
      this.tableName = tableName ;
   }


    /*-----------------------------------------------------------*/
    /* getType
     */
    /**
     *    This method will return what type of statement
     *  will be formed from this SQLStatement
     *  
     *             
     *  @param 
     *  @param 
     *  @param 
     *
     *  @return STMT_DELETE, STMT_UPDATE or STMT_INSERT
     */
     public int getType () {
       return ( stmtType ) ;
     }






} /* SQLStatement */
