// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FTP/FileTransfer.java,v 1.1.1.1.4.2 2001/09/24 18:39:42 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
//

package com.ibm.nzna.shared.ftp.FTP ;

import java.io.File;
import java.io.IOException;

import com.ibm.nzna.shared.ftp.* ;

/**
 * A generic file transfer.
 *
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public abstract class FileTransfer extends Transfer {

    /** Local file **/
    protected File localFile;
    /** Name of remote file **/
    protected String remoteName;
    /** true if we are attempting restart **/
    protected boolean restart = false;

    /**
     * Create new FileTransfer.
     **/
    public FileTransfer(Ftp ftp, File localFile, String remoteName, String command) {
        super(ftp, command);
        this.localFile = localFile;
        this.remoteName = remoteName;
    }

    /**
     * Check if transfer is restartable.
     * @return true if restart can be attempted
     **/
    public boolean isRestartable() {
        // Restart only possible in case of error or abort
        return (state != null &&
                (state.getState() == TransferEvent.ERROR ||
                 state.getState() == TransferEvent.ABORTED));
    }

    /**
     * Restart failed transfer (if possible).
     **/
    public void restart() throws IOException, FtpError {
        if (isRestartable()) {
            restart = true;
            start();
        }
    }

    /**
     * Get file size from latest host reply.
     * @return file size or -1 if not possible
     **/
    protected int getHostFileSize() {
        int size = -1;
        try {
            String s = ftp.getReply().getMessage();
            // "opening ... data connection for foo.bar (3748 bytes). ..."
            int i2 = s.lastIndexOf(" bytes)");
            int i1 = i2 - 1;
            while (Character.isDigit(s.charAt(i1))) {
                --i1;
            }
            size = Integer.parseInt(s.substring(i1 + 1, i2));
        }
        catch (Exception e) {
            // ignored -- size will just be -1
        }
        return size;
    }

}
