// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FTP/ListTransfer.java,v 1.1.1.1.4.2 2001/09/24 18:39:42 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
// 

package com.ibm.nzna.shared.ftp.FTP ;

import java.io.*;
import java.util.Vector;

import com.ibm.nzna.shared.ftp.* ;

/**
 * A transfer for the LIST and NLST commands.
 *
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public class ListTransfer extends Transfer {

    /** List of file names returned from host **/
    String[] result;

    /**
     * Create new ListTransfer.
     * @param path directory and/or name of file(s) to list.
     *        May include wildcards if the host supports it.
     *        <code>null</code> means current directory
     * @param verbose if <code>true</code>, information (size, date, etc.)
     *        about each file is returned if <code>false</code>,
     *        only the file names are returned
     **/
    public ListTransfer(Ftp ftp, String path, boolean verbose) {
        super(ftp, ((verbose) ? "LIST" : "NLST") + ((path != null) ? (" " + path) : ""));
    }

    public String[] getList() throws FtpError, IOException {
        if (result == null) {
            start();
        }
        return result;
    }

    protected void before() throws FtpError, IOException {
        // Change transfer type to ascii if necessary
        if (ftp.getType() != Ftp.ASCII) {
            ftp.command("TYPE " + Ftp.ASCII);
        }
    }

    protected void doIt() throws IOException {
        BufferedReader in = new BufferedReader(new InputStreamReader(dataSocket.getInputStream()));
        Vector v = new Vector();
        String str = in.readLine();
        while (str != null) {
            v.addElement(str);
            str = in.readLine();
        } 
        in.close();
        // Create return value
        result = new String[v.size()];
        v.copyInto(result);
    }

    protected void after() throws FtpError, IOException {
        // Reset transfer type if necessary
        if (ftp.getType() != Ftp.ASCII) {
            ftp.command("TYPE " + ftp.getType());
        }
    }
    
}
