// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FTP/PasswordEditor.java,v 1.1.1.1.4.2 2001/09/24 18:39:42 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
// 

package com.ibm.nzna.shared.ftp.FTP ;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.TextField;
import java.awt.event.TextEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;

/**
 * A password editor.
 * This custom editor will allow password editing using the EchoChar
 * feature of TextField, so that the password will only be visible as
 * a string of stars (*).
 *
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public class PasswordEditor extends TextField implements PropertyEditor {
    
    PropertyChangeSupport support = new PropertyChangeSupport(this);

    // ---------------------- constructors ---------------------------
    
    public PasswordEditor() {
        super(20);
        setEchoChar('*');
        enableEvents(AWTEvent.TEXT_EVENT_MASK);
    }

    // ----------------- get/set property value ----------------------
    
    public void setValue(Object obj) {
        setText((String)obj);
    }
    
    public Object getValue() {
        return getText();
    }

    // ----------------- this is a custom editor ---------------------
    
    public boolean isPaintable() {
        return true;
    }
    
    public boolean supportsCustomEditor() {
        return true;
    }
    
    public Component getCustomEditor() {
        return this;
    }

    // ---------- get/set as text & getTags not supported ------------
    
    public void setAsText(String text) throws IllegalArgumentException {   
    }
    
    public String getAsText() {
        return null;
    }
    
    public String[] getTags() {
        return null;
    }

    // ------------- manage property change listeners -----------------
    
    public void addPropertyChangeListener(PropertyChangeListener lis) {
        support.addPropertyChangeListener(lis);
    }

    public void removePropertyChangeListener(PropertyChangeListener lis) {
        support.removePropertyChangeListener(lis);
    }

    protected void processTextEvent(TextEvent e) {
        super.processTextEvent(e);
	    if (e.getID() == TextEvent.TEXT_VALUE_CHANGED) {
            support.firePropertyChange("", null, null);
        }
    }

    // --------------- construct the initialization string ------------
    
    public String getJavaInitializationString() {
        return '"' + getText() + '"';
    }

    // ---------------- paint the password ----------------------------

    static final String stars = "********************";

    public void paintValue(Graphics gfx, Rectangle box) {
        String s = stars.substring(0, Math.min(stars.length(), getText().length()));
        gfx.drawRect(box.x, box.y, box.width, box.height);
        gfx.drawString(s, box.x, box.y + box.height/2);
    }

}
