// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FTP/StreamGet.java,v 1.1.1.1.4.2 2001/09/24 18:39:42 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
// 

package com.ibm.nzna.shared.ftp.FTP ;

import java.io.*;

/**
 * A 'get' in stream transfer mode.
 *
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public class StreamGet extends StreamTransfer {

    /**
     * Create new 'get' transfer.
     **/
    public StreamGet(Ftp ftp, File localFile, String remoteName) {
        super(ftp, localFile, remoteName, "RETR " + remoteName);
    }

    protected void before() throws IOException {
        if (!restart && localFile.exists()) {
            // Make sure the file is empty
            localFile.delete();
        }
        raf = new RandomAccessFile(localFile, "rw");
        if (restart) {
            raf.seek(total);
            ftp.command("REST " + total);
        }
    }

    protected void doIt() throws IOException {
        InputStream is = null;
        try {
            // Open I/O streams
            is = dataSocket.getInputStream();
            // Start transfer
            state = new TransferEvent(ftp, TransferEvent.GET, remoteName, getHostFileSize());
            if (restart) {
                state.setStartsize(total);
            }
            start = System.currentTimeMillis();
            lastEvent = start;
            ftp.fireTransferStartedEvent(state);
            try {
                if (ftp.getType() == Ftp.ASCII) {
                    asciiGet(is);
                }
                else {
                    binaryGet(is);
                }
            }
            catch (java.net.SocketException e) {
                if (!aborted) {
                    // a real error -- rethrow
                    throw e;
                }
            }
            // If we get here, all went well or transfer was aborted
            state.update(total, System.currentTimeMillis() - start);
            state.setState((aborted) ? TransferEvent.ABORTED : TransferEvent.COMPLETED);
            ftp.fireTransferEndedEvent(state);
        }
        finally {
            if (raf != null) raf.close();
            if (is != null) is.close();
        }
    }

    /**
     * Perform a binary get.
     * @param is stream to read from
     **/
    protected void binaryGet(InputStream is) throws IOException {
        byte[] buf = new byte[ftp.getBufferSize()];
        int len;
        while ((len = is.read(buf)) > -1) {
            raf.write(buf, 0, len);
            total += len;
            if (System.currentTimeMillis() > lastEvent + PROGRESS_INTERVAL) {
                lastEvent = System.currentTimeMillis();
                state.update(total, lastEvent - start);
                ftp.fireTransferProgressEvent(state);
            }
        }
    }

    /**
     * Perform an ASCII get.
     * @param is stream to read from
     **/
    protected void asciiGet(InputStream is) throws IOException {
        byte[] buf = new byte[ftp.getBufferSize()];
        int len;
        while ((len = is.read(buf)) > -1) {
            int first = 0;                         // index of first char on current line
            int lineLength = 0;                    // length of current line
            for (int i = 0; i < len; ++i) {
                switch (buf[i]) {
                  case CR:
                    // CR means end of line
                    if (lineLength > 0) {
                        raf.write(buf, first, lineLength);
                    }
                    raf.write(LINESEP);
                    lineLength = 0;
                    break;
                  case LF:
                    // Ignored
                    break;
                  default:
                    if (lineLength++ == 0) { 
                        // Neither CR nor LF but first character in new line
                        first = i;
                    }
                }
            }
            if (lineLength > 0) {
                // Write unterminated line
                raf.write(buf, first, lineLength);
            }
            // Update count
            total += len;
            if (System.currentTimeMillis() > lastEvent + PROGRESS_INTERVAL) {
                // Time for a new progress event
                lastEvent = System.currentTimeMillis();
                state.update(total, lastEvent - start);
                ftp.fireTransferProgressEvent(state);
            }
        }
    }

}
