// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FTP/StreamPut.java,v 1.1.1.1.4.2 2001/09/24 18:39:42 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
// 

package com.ibm.nzna.shared.ftp.FTP ;

import java.io.*;
import com.ibm.nzna.shared.ftp.* ;

/**
 * A 'put' in stream transfer mode.
 *
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public class StreamPut extends StreamTransfer {

    /**
     * Create new 'put' transfer.
     **/
    public StreamPut(Ftp ftp, File localFile, String remoteFile, boolean append) {
        super(ftp, localFile, remoteFile,
              ((append) ? "APPE " : "STOR ") + remoteFile);
    }

    /**
     * Create new 'put' transfer.
     **/
    public StreamPut(Ftp ftp, File localFile, boolean append) {
        this(ftp, localFile, localFile.getName(), append);
    }
    
    protected void before() throws FtpError, IOException {
        raf = new RandomAccessFile(localFile, "r");
        if (restart) {
            // Get size of remote file
            total = ftp.size(remoteName);
            raf.seek(total);
            ftp.command("REST " + total);
        }
    }

    protected void doIt() throws IOException {
        OutputStream os = null;
        try {
            // Open output stream
            os = dataSocket.getOutputStream();
            // Start transfer
            state = new TransferEvent(ftp, TransferEvent.PUT, localFile.getName(),
                                      localFile.length());
            if (restart) {
                state.setStartsize(total);
            }
            start = System.currentTimeMillis();
            lastEvent = start;
            ftp.fireTransferStartedEvent(state);
            try {
                if (ftp.getType() == Ftp.ASCII) {
                    asciiPut(os);
                }
                else {
                    binaryPut(os);
                }
            }
            catch (java.net.SocketException e) {
                if (!aborted) {
                    // a real error -- rethrow
                    throw e;
                }
            }
            // If we get here, all went well or transfer was aborted
            state.update(total, System.currentTimeMillis() - start);
            state.setState((aborted) ? TransferEvent.ABORTED : TransferEvent.COMPLETED);
            ftp.fireTransferEndedEvent(state);
            // Close the data connection to signal EOF
            if (!aborted) {
                dataSocket.close();
                dataSocket = null;
            }
        }
        finally {
            if (os != null) os.close();
            if (raf != null) raf.close();
        }
    }

    /**
     * Perform a binary put.
     * @param os stream to write to
     **/
    protected void binaryPut(OutputStream os) throws IOException {
        byte[] buf = new byte[ftp.getBufferSize()];
        int len;
        while ((len = raf.read(buf)) > -1) {
            os.write(buf, 0, len);
            total += len;
            if (System.currentTimeMillis() > lastEvent + PROGRESS_INTERVAL) {
                lastEvent = System.currentTimeMillis();
                state.update(total, lastEvent - start);
                ftp.fireTransferProgressEvent(state);
            }
        }
    }

    /**
     * Perform an ASCII put.
     * @param os stream to write to.
     **/
    protected void asciiPut(OutputStream os) throws IOException {
        BufferedOutputStream bos = null;
        try {
            bos = new BufferedOutputStream(os);
            int c = raf.read();
            while (c != -1) {
                switch (c) {
                  case CR:          // ignored in ASCII mode
                    break;
                  case LF:          // end of line
                    bos.write(CRLF);
                    total += 2;
                    if (System.currentTimeMillis() > lastEvent + PROGRESS_INTERVAL) {
                        lastEvent = System.currentTimeMillis();
                        state.update(total, lastEvent - start);
                        ftp.fireTransferProgressEvent(state);
                    }
                    break;
                  default:          // write byte to stream
                    bos.write(c);
                    ++total;
                }
                c = raf.read();
            }
        }
        finally {
            if (bos != null) bos.close();
        }
    }

}
