// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FTP/TransferEvent.java,v 1.1.1.1.4.2 2001/09/24 18:39:42 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
//

package com.ibm.nzna.shared.ftp.FTP ;

/**
 * A TransferEvent is sent from an FTP bean to its FtpListeners to
 * inform about the state of a file transfer.
 * @see Ftp
 * @see FtpListener
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public class TransferEvent extends java.util.EventObject {

    /** Indicates a "get" transfer **/
    public static final int GET = 1;
    /** Indicates a "put" transfer **/
    public static final int PUT = 2;


    /** Transfer is in progress **/
    public static final int ACTIVE = 1;
    /** Transfer completed successfully **/
    public static final int COMPLETED = 2;
    /** Transfer aborted by a TransferListener **/
    public static final int ABORTED = 3;
    /** Transfer aborted due to an error **/
    public static final int ERROR = 4;


    /** Name of the file being transferred **/
    protected String filename;

    /** Transfer type **/
    protected int type;
    
    /** Transfer state **/
    protected int state;

    /** Start size ( > 0 in case of restart) **/
    protected long startsize;

    /** Number of bytes transferred **/
    protected long byteCount;

    /** Total file size **/
    long filesize;
    
    /** The transfer age **/
    protected long duration;

    /**
     * Create new transfer event
     **/
    public TransferEvent(Object source, int type, String filename, long filesize) {
        super(source);
        this.type = type;
        this.filename = filename;
        this.filesize = filesize;
        state = ACTIVE;
        byteCount = 0;
        duration = 0;
        startsize = 0;
    }

    /**
     * Set start size
     **/
    protected void setStartsize(long size) {
        startsize = size;
    }
    

    /**
     * Name of file being transferred.
     **/
    public String getFilename() {
        return filename;
    }

    /**
     * Size of file being transferred.
     * @return file size or -1 if the file size is unknown
     **/
    public long getFilesize() {
        return filesize;
    }

    /**
     * Type of file transfer (GET or PUT).
     **/
    public int getType() {
        return type;
    }

    /**
     * Transfer state (ACTIVE, COMPLETED, ABORTED or ERROR).
     **/
    public int getState() {
        return state;
    }

    protected void setState(int newState) {
        state = newState;
    }

    /**
     * Number of bytes transferred so far.
     **/
    public long getByteCount() {
        return byteCount;
    }

    protected void setByteCount(long newByteCount) {
        byteCount = newByteCount;
    }

    /**
     * The number of milliseconds this transfer has spent so far.
     **/
    public long getDuration() {
        return duration;
    }

    protected void setDuration(long newDuration) {
        duration = newDuration;
    }

    /**
     * Get the average number of bytes transferred per second.
     **/
    public int getTransferRate() {
        return (duration != 0) ? (int) ((1000 * (byteCount - startsize)) / duration) : 0;
    }

    /**
     * Get the percentage completed
     * @return percentage of file transfer completed or -1 if file size is
     * unknown
     **/
    public int getPercent() {
        if (filesize > 0) {
            if (state == COMPLETED) {
                return 100;
            }
            else {
                return (int) (100*byteCount/filesize);
            }
        }
        return -1;
    }

    /**
     * Update this event.
     * @param byteCount number of bytes transferred so far
     * @param duration number of msecs since start
     **/
    protected void update(long byteCount, long duration) {
        this.byteCount = byteCount;
        this.duration = duration;
    }

    /**
     * Make printable representation.
     * @return information about transfer type, filename, filesize,
     * percent completed and transfer rate in a readable format
     **/
    public String toString() {
        switch (state) {
          case ACTIVE:
            return ((type == GET) ? "Getting " : "Putting ") + filename + " " +
                ((filesize >= 0) ? "" + getPercent() + "% of " + filesize + " bytes" : "") +
                " at " + getTransferRate() + " bytes/sec";
          case COMPLETED:
            return ((type == GET) ? "Did get " : "Did put ") + filename + " " +
                ((filesize >= 0) ? "" + filesize + " bytes" : "") +
                " at " + getTransferRate() + " bytes/sec";
          case ABORTED:
            return ((type == GET) ? "Get " : "Put ") + filename + " " +
                ((filesize >= 0) ? "" + filesize + " bytes" : "") +
                " aborted after " + byteCount + " bytes";
          case ERROR:
            return ((type == GET) ? "Get " : "Put ") + filename + " " +
                ((filesize >= 0) ? (filesize + " bytes") : "") +
                " failed after " + byteCount + " bytes";
        }
        return null;
    }

}


