// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FtpReply.java,v 1.1.1.1.4.2 2001/09/24 18:39:43 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
// 

package com.ibm.nzna.shared.ftp ;

/**
 * This interface encapsulates FTP host replies. The structure of replies
 * is defined in the FTP specification. A reply consists of a three-digit
 * <i>reply code</i> - of which the first digit indicates the <i>reply
 * type</i> - and a human-readable <i>message</i>.
 *
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public interface FtpReply {

    /**
     * Reply type signalling a successful initiation of a command
     **/
    int POSITIVE_PRELIMINARY = 1;
    
    /**
     * Reply type signalling successful command completion
     **/
    int POSITIVE_COMPLETION = 2;
    
    /**
     * Reply type signalling succesful completion of a subcommand
     **/
    int POSITIVE_INTERMEDIATE = 3;
    
    /**
     * Reply type signalling that the command was unsuccessful
     * but should be retried
     **/
    int TRANSIENT_NEGATIVE = 4;
    
    /**
     * Reply type signalling an unsuccessful command
     **/
    int PERMANENT_NEGATIVE = 5;	 

    /**
     * Get the human-readable reply of the last command
     **/
    String getMessage();

    /**
     * Get the three-digit reply code of the last command.
     **/
    int getCode();

    /**
     * Get the reply-code type.
     * The reply-code type is the first of the three digits in the FTP reply,
     * represented by the constants:
     * <ol>
     * <li> POSITIVE_PRELIMINARY
     * <li> POSITIVE_COMPLETION
     * <li> POSITIVE_INTERMEDIATE
     * <li> TRANSIENT_NEGATIVE
     * <li> PERMANENT_NEGATIVE
     * </ol>
     **/
    int getType();

}
