package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   AnalogClock                                             */
/*                                                           */
/*   Description:                                            */
/*      This file holds the source for the analog clock      */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM Corporation                  */
/*         Licensed Materials - Property of IBM              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.math.* ;
import java.awt.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The Analog clock control is a simple control that displays an analog clock.
 * The control can also display the date to the right of the clock.
 *
 *
 * @version 1.0, August 23, 1997
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   AnalogClock                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class AnalogClock extends JComponent implements Runnable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Fonts                                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Font boldFont       = new Font ( "SansSerif", Font.PLAIN, 12 ) ;
   private Font titleFont      = new Font ( "SansSerif", Font.BOLD, 16 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final double pi         = Math.PI;
   static final double _2pi       = 2 * pi;
   static final int    DATE_WIDTH = 170 ;
   static final Color  mediumGray  = new Color ( 100, 100, 100 ) ;
   static final Color  lighterGray = new Color ( 170, 170, 170 ) ;
   static final Color  darkBlue    = new Color ( 0, 0, 180 ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Protected Variables                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected int        second        = 0 ;
   protected int        minute        = 0 ;
   protected int        hour          = 0 ;
   protected int        year          = 0 ;
   protected int        month         = 0 ;
   protected int        day           = 0 ;
   protected int        width         = 0 ;
   protected int        height        = 0 ;
   protected int        width_org     = 0 ;
   protected int        height_org    = 0 ;
   protected Point      center        = new Point ( 0, 0 ) ;
   protected int        radius        = 80 ;
   protected int        inside_radius = 0 ;
   protected int        h_len         = 0 ;
   protected int        m_len         = 0 ;
   protected int        s_len         = 0 ;
   protected boolean    am            = false ;                                                             
   private   Image      clockFaceImage = null ;
   protected Image      bufImage      = null ;
   protected boolean    showDate      = false ;
   protected String     strDay        = "Wednesday" ;
   protected String     strMonth      = "February" ;
   protected Image      backImage     = null ;
   protected boolean    pause         = false ;
   protected Dimension  size          = new Dimension ( 0, 0 ) ;
   private   int        clockFaceImageX = 0 ;
   private   int        clockFaceImageY = 0 ;



   /*-----------------------------------------------------------*/
   /*                                                             
   /* Constructor 1                                                     
    */
   /**
    *    Creates the clock with the date and time 
    * specified to 9/20/74 12:15 am
    * 
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   public AnalogClock () {
      CDate  dd = new CDate ( CDate.DATE_STAMP ) ;
      String today = dd.today() ;

      hour    = dd.parse ( today, CDate.DATE_HOUR,   CDate.DATE_STAMP ) ;
      minute  = dd.parse ( today, CDate.DATE_MINUTE, CDate.DATE_STAMP ) ;
      second  = dd.parse ( today, CDate.DATE_SECOND, CDate.DATE_STAMP ) ;
      day     = dd.parse ( today, CDate.DATE_DAY,    CDate.DATE_STAMP ) ;
      month   = dd.parse ( today, CDate.DATE_MONTH,  CDate.DATE_STAMP ) ;
      year    = dd.parse ( today, CDate.DATE_YEAR,   CDate.DATE_STAMP ) ;

      fillStrDates() ;

      init() ;
   } /* AnalogClock */

   /*-----------------------------------------------------------*/
   /*                                                             
   /* Constructor 2                                                     
    */                                                             
   /**
    *    Creates an analog clock and sets the time
    * and date to the values passed in
    * 
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   public AnalogClock ( int hour, int minute, int second, int year, int month, int day ) {
      this.hour   = hour ;
      this.minute = minute ;
      this.second = second ;
      this.year   = year ;
      this.month  = month ;
      this.day    = day ;
      init() ;
   } /* AnalogClock */

   /*-----------------------------------------------------------*/
   /*   set                                                     */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will set the clock                       */                                           
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public void set ( int hour, int minute, int second, int year, int month, int day ) {
      this.hour   = hour ;
      this.minute = minute ;
      this.second = second ;
      this.year   = year ;
      this.month  = month ;
      this.day    = day ;
      createBufImage () ;
      paint ( getGraphics() ) ;
   }

   /*-----------------------------------------------------------*/
   /*   showDate                                                */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will either show or hide the date        */                                                          
   /*                                                           */
   /*   Parameters:                                             */
   /*      showDate TRUE = show Date, FALSE = Do not show       */                                      
   /*                                                           */
   /*   Returns:                                                */
   /*      `                                                    */
   /*-----------------------------------------------------------*/
   public void showDate ( boolean showDate ) {
      this.showDate = showDate ;
      createBufImage () ;
   }


   /*-----------------------------------------------------------*/
   /*                                                             
   /* init                                                     
    */                                                             
   /**
    *    Finishes initializing this control
    * 
    * 
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   private void init () {
      //ThreadTimer threadTimer = new ThreadTimer ( this ) ;
      //threadTimer.setTimer(1000) ;
      //threadTimer.setPriority(ThreadJob.PRIORITY_LAZY) ;
      //threadTimer.start() ;
      new Thread ( this ).start() ;
   }

   /*-----------------------------------------------------------*/
   /*                                                             
   /* run                                                     
    */                                                             
   /**
    *    The actual thread that will keep time
    * 
    * 
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   public void run() {
      revalidate() ;
      while ( true ) {
         updateSecond() ;

         try {
            if ( ( !pause ) && ( isVisible() ) ) {
               repaint() ;
            }
            Thread.currentThread().sleep ( 1000 ) ;
         } catch ( Exception e ) {
         }
      }
   } /* run */

   /*-----------------------------------------------------------*/
   /*   setBackground                                           */
   /*                                                           */
   /*   Description:                                            */
   /*      Sets the background to an image                      */                                            
   /*                                                           */
   /*   Parameters:                                             */
   /*      Image - The new Background Image                     */                        
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public void setBackground ( Image backImage ) {
      this.backImage = backImage ;
      if ( bufImage != null ) {
         drawImage ( bufImage.getGraphics() ) ;
         repaint() ;
      }
   }


   /*-----------------------------------------------------------*/
   /*                                                             
   /* setCoordinates                                                     
    */                                                             
   /**
    *    This method will set the coordinates for the
    * clock
    * 
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   void setCoordinates() {
      if ( showDate ) {
         size.width=100 ;
         size.height=70 ;
      }

      width_org = size.width;
      height_org = size.height;

      height = (int)(0.8 * height_org);
      width = width_org;
      if ( width < height )
         height = width;
      else
         width = height;

      center.x = width_org/2 ;
      center.y = height/2 ;
      radius = (width * 8) / 20;
      inside_radius = (radius * 8) / 10;

      h_len = (radius * 4) / 10;
      m_len = (radius * 7) / 10;
      s_len = (radius * 6) / 10;

   }




   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our control
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      try {
         // if ( bufImage == null ) {
         //   createBufImage() ;
         //   if ( bufImage != null )
         //       drawImage ( bufImage.getGraphics() ) ;
         //} /* if */

         // if ( bufImage != null )
         //   g.drawImage ( bufImage, 0, 0, this ) ;
         drawImage ( g ) ;
      } catch ( NullPointerException e ) {
      }
   }


   /*-----------------------------------------------------------*/
   /*   createBufImage                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      Creates the buffered image to double buffer with     */                                                             
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public void createBufImage () {
      Dimension size = getSize() ;

      if ( bufImage != null ) {
         bufImage.flush() ;
         bufImage = null ;
         System.gc () ;
      }

      try {
         bufImage = createImage ( size.width, size.height ) ;
      } catch ( Exception e ) {
      }
   }


   /*-----------------------------------------------------------*/
   /*   drawImage                                               */
   /*                                                           */
   /*   Description:                                            */
   /*      Draws to our buffered image                          */                                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   void drawImage ( Graphics g ) {
      int        x            = center.x;
      int        y            = center.y;
      double     hour_value   = hour + minute/60.0 + second/3600.0;
      double     minute_value = minute + second/60.0;
      double     second_value = second * _2pi / 60.0;


      setCoordinates() ;

      if ( showDate ) {
         x-=25 ;
         y-=5 ;
      }

      if ( clockFaceImage == null ) {
         g.setColor( Color.white ) ;
         g.drawOval( x-1 - inside_radius, y-1 - inside_radius,
                     inside_radius * 2, inside_radius * 2);

         g.setColor( lighterGray ) ;
         g.drawOval(x - inside_radius, y   - inside_radius,
                    inside_radius * 2, inside_radius * 2);
      } else
         g.drawImage ( clockFaceImage, clockFaceImageX, clockFaceImageY, this ) ;

      hour_value *= _2pi / 12.0;

      g.setColor ( getForeground () );
      g.drawLine (x, y,(int) (x + h_len * Math.sin(hour_value)),(int) (y - h_len * Math.cos(hour_value)));
      g.drawLine (x-1, y-1,(int) (x + h_len * Math.sin(hour_value)),(int) (y - h_len * Math.cos(hour_value)));
      g.drawLine (x+1, y+1,(int) (x + h_len * Math.sin(hour_value)),(int) (y - h_len * Math.cos(hour_value)));


      minute_value *= _2pi / 60.0;

      g.drawLine (x, y,(int) (x + m_len * Math.sin(minute_value)),(int) (y - m_len * Math.cos(minute_value)));

      g.setColor ( mediumGray ) ;
      g.drawLine (x, y, (int) (x + s_len * Math.sin(second_value)),(int) (y - s_len * Math.cos(second_value)));


      /* Ok, now if we are supposed to draw the Date, draw it now */
      if ( showDate ) {
         g.setColor ( darkBlue ) ;
         g.setFont ( titleFont ) ;
         g.drawString ( strDay, 45, 20 ) ;
         //g.setColor ( Color.black ) ;
         g.setFont ( boldFont ) ;
         g.drawString ( strMonth+" "+day+", "+year, 45, 33 ) ;
      }
   }


   /*-----------------------------------------------------------*/
   /*   updateSecond                                            */
   /*                                                           */
   /*   Description:                                            */
   /*      Updates the second                                   */                               
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public void updateSecond () {
      second++ ;
      if ( second == 60 ) {
         second = 0 ;
         updateMinute() ;
      }
   }

   /*-----------------------------------------------------------*/
   /*                                                             
   /* updateMinute                                                     
    */                                                             
   /**
    *    Takes the current minute and adds 1 to it
    * 
    * 
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   public void updateMinute () {
      minute++ ;
      if ( minute == 60 ) {
         minute = 0 ;
         updateHour() ;
      }

   }

   /*-----------------------------------------------------------*/
   /*                                                             
   /* updateHour                                                     
    */                                                             
   /**
    *    Updates the current hour by one
    * 
    * 
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   public void updateHour () {
      hour++ ;

      if ( ( hour == 12 ) && ( !am ) ) {
         updateDay() ;
         am   = true ;
      } else if ( ( hour == 12 ) && ( am ) ) {
         am = false ;
      } else if ( hour == 13 )
         hour = 1 ;
   } /* updateHour */

   /*-----------------------------------------------------------*/
   /*                                                             
   /* updateDay                                                     
    */                                                             
   /**
    *    Updates the current day by 1
    * 
    * 
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   public void updateDay () {
      String date = CDate.createDate ( year, month, day, 0, 0, 0, CDate.DATE_DATE ) ;

      date  = CDate.addDaysToDate ( date, 1, CDate.DATE_DATE ) ;
      day   = CDate.parse ( date, CDate.DATE_DAY,   CDate.DATE_DATE ) ;
      month = CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ) ;
      year  = CDate.parse ( date, CDate.DATE_YEAR,  CDate.DATE_DATE ) ;

      fillStrDates() ;
   } /* updateDay */

   /*-----------------------------------------------------------*/
   /*   finalize                                                */       
   /*                                                           */
   /*   Description:                                            */
   /*      Subclassed to free our images                        */                                              
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void finalize () {
      if ( bufImage != null ) {
         bufImage.flush() ;
         bufImage = null ;
      }
      try {
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }

   /*-----------------------------------------------------------*/
   /*   fillStringDates                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will fill the Month and Day Strings      */                                                            
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public void fillStrDates () {
      String date = new Date().toString() ;
      String t1   = date.substring ( 0, 4 ).trim() ;
      String t2   = date.substring ( 4, 7 ).trim() ;

      if ( t1.equals ( "Sun" ) )
         strDay = "Sunday" ;
      else if ( t1.equals ( "Mon" ) )
         strDay = "Monday" ;
      else if ( t1.equals ( "Tue" ) )
         strDay = "Tuesday" ;
      else if ( t1.equals ( "Wed" ) )
         strDay = "Wednesday" ;
      else if ( t1.equals ( "Thu" ) )
         strDay = "Thursday" ;
      else if ( t1.equals ( "Fri" ) )
         strDay = "Friday" ;
      else if ( t1.equals ( "Sat" ) )
         strDay = "Saturday" ;

      switch ( month ) {
         case 1 :
            strMonth = "January" ;
            break ;

         case 2 :
            strMonth = "February" ;
            break ;

         case 3 :
            strMonth = "March" ;
            break ;

         case 4 :
            strMonth = "April" ;
            break ;

         case 5 :
            strMonth = "May" ;
            break ;

         case 6 :
            strMonth = "June" ;
            break ;

         case 7 :
            strMonth = "July" ;
            break ;

         case 8 :
            strMonth = "August" ;
            break ;

         case 9 :
            strMonth = "September" ;
            break ;

         case 10 :
            strMonth = "October" ;
            break ;

         case 11 :
            strMonth = "November" ;
            break ;

         case 12 :
            strMonth = "December" ;
            break ;

      }

   }

   /*-----------------------------------------------------------*/
   /*                                                             
   /* pause                                                         
    */                                                             
   /**
    *    Pauses the clock. The clock still keeps time, but will not
    * update the screen.
    * 
    * @param None
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
   */
   /*-----------------------------------------------------------*/
   public void pause ( boolean pause ) {
      this.pause = pause ;
   }


   /*------------------------------------------------------------*/
   /* setBoldFont
    */
   /**
    *    The Bold font displays the day and
    *  year information
    *  
    *
    *  @param font The new font
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBoldFont ( Font font ) {
      this.boldFont = font ;
      repaint() ;
   }




   /*------------------------------------------------------------*/
   /* setTitleFont
    */
   /**
    *    This method will set the font that
    *  displays the large day name
    *  
    *
    *  @param font The new font
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitleFont ( Font font ) {
      this.titleFont = font ;
      repaint() ;
   }


   public void setClockFaceImage ( Image image ) {
      this.clockFaceImage = image ;
   }

   public void setClockFaceImageOffset ( int clockFaceImageX, int clockFaceImageY ) {
      this.clockFaceImageX = clockFaceImageX ;
      this.clockFaceImageY = clockFaceImageY ;
   }
} /* AnalogClock */
