package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   ButtonPanel                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ButtonPanel                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel will show 1 row of buttons. If the amount 
 * of button exceeds the maximum width, two scroll buttons
 * will appear, allowing the user to scroll through the buttons
 *
 * @version 1.0, 12/14/1998 
 * @author David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ButtonPanel                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ButtonPanel extends DPanel implements Runnable, MouseListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int BUTTONWIDTH = 15 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JPanel    buttonPanel             = null ;
   private PopButton pb_LEFT                 = null ;
   private PopButton pb_RIGHT                = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean           initialized        = false ;
   private int               panelX             = 0 ;
   private StraightLayoutMgr buttonPanelLayout  = null ;
   private int               buttonPanelWidth   = 0 ;
   private boolean           scroll             = false ;
   private boolean           scrollRight        = true ;
   private boolean           continueThread     = true ;
   private boolean           scrolling          = false ;
   private Component         leftComponent      = null ;
   private int               leftComponentIndex = 0 ;
   private Thread            scrollThread       = null ;
   private Dimension         preferredSize      = new Dimension ( 100, 25 ) ;
   private Image             backgroundImage    = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a regular default ButtonPanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ButtonPanel () {
      init() ;
   }




   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our button panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void init () {
      setLayout ( null ) ;          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      super.add ( buttonPanel = new JPanel () ) ;
      super.add ( pb_LEFT     = new PopButton ( "<" ) ) ; //CSystem.getImage ( this, "buttonpanel_left.gif" ) ) ) ;
      super.add ( pb_RIGHT    = new PopButton ( ">" ) ) ; //CSystem.getImage ( this, "buttonpanel_right.gif" ) ) ) ;

      pb_LEFT.addMouseListener(this) ;
      pb_RIGHT.addMouseListener(this) ;

      pb_LEFT.setBorderPainted ( false ) ;
      pb_RIGHT.setBorderPainted ( false ) ;
      pb_LEFT.setFocusPainted ( false ) ;
      pb_RIGHT.setFocusPainted ( false ) ;
      pb_LEFT.setOpaque ( false ) ;
      pb_RIGHT.setOpaque ( false ) ;
      buttonPanel.setOpaque ( false ) ;

      buttonPanel.setLayout ( buttonPanelLayout = new StraightLayoutMgr() ) ;

      scrollThread = new Thread ( this ) ;
      scrollThread.start() ;

      initialized = true ;

   }







   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will layout out all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      super.doLayout() ;

      if ( initialized ) {

         pb_LEFT.setBounds ( (width-BUTTONWIDTH)+3, 0, BUTTONWIDTH-3, height/2 ) ;
         pb_RIGHT.setBounds ( (width-BUTTONWIDTH)+3, height/2, BUTTONWIDTH-3, height/2 ) ;

         buttonPanelWidth = buttonPanelLayout.minimumLayoutSize(buttonPanel).width ;

         if ( (width-BUTTONWIDTH) > buttonPanelWidth )
            panelX=0 ;

         buttonPanel.setBounds ( 0-panelX, 0, panelX+(width-BUTTONWIDTH), height ) ;

         pb_LEFT.setVisible ( ( buttonPanelWidth > width ) ) ;
         pb_RIGHT.setVisible ( ( buttonPanelWidth > width ) ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* remove
    */
   /**
    *    This method is subclassed to remove directly
    *  from our buttonPanle
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void remove ( Component comp ) {
      buttonPanel.remove ( comp ) ;
      buttonPanelLayout.removeLayoutComponent ( comp ) ;
      buttonPanelWidth = buttonPanelLayout.minimumLayoutSize(buttonPanel).width ;
      buttonPanelLayout.layoutContainer ( this ) ;

      if ( (width-BUTTONWIDTH) > buttonPanelWidth )
         panelX=0 ;
      buttonPanel.setBounds ( 0-panelX, 0, panelX+(width-BUTTONWIDTH), height ) ;
      repaint() ;

   }

   /*------------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove all components
    *  from the button bar
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAll() {
      buttonPanel.removeAll() ;
      buttonPanelWidth = buttonPanelLayout.minimumLayoutSize(buttonPanel).width ;
   }



   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method is subclassed to add directly
    *  to our button panel, not us.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Component add ( Component comp ) {
      int preferredHeight = comp.getPreferredSize().height ;

      buttonPanel.add ( "", comp ) ;
      buttonPanelWidth = buttonPanelLayout.minimumLayoutSize(buttonPanel).width ;
      buttonPanelLayout.layoutContainer ( this ) ;
      comp.invalidate() ;
      comp.validate() ;

      if ( preferredHeight > preferredSize.height )
         preferredSize.height = preferredHeight ;

      return( comp ) ;
   }

   public Component add ( String str, Component comp ) {
      return( add ( comp ) ) ;
   }






   /*------------------------------------------------------------*/
   /* mouseMethods
    */
   /**
    *    These mouse methods are blank for now
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseClicked(MouseEvent e) {}                                       

   public void mouseEntered(MouseEvent e) {
      if ( scrolling ) {
         scrollRight = ( e.getSource() == pb_RIGHT ) ;
         scroll = true ;
         scrollThread.interrupt() ;
      }
   }

   public void mouseExited(MouseEvent e) {
      scroll = false ;
   }





   /*------------------------------------------------------------*/
   /* mousePressed
    */
   /**
    *    This method will respond to mouse pressed
    *  events. It will set the flags for the thread to scroll
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mousePressed(MouseEvent e) {
      scrollRight = ( e.getSource() == pb_RIGHT ) ;
      scroll = true ;
      scrollThread.interrupt() ;
      scrolling = true ;
   }





   /*------------------------------------------------------------*/
   /* mouseReleased
    */
   /**
    *    This method will be called when the user
    *  lets up on a mouse button. It will set the flags
    *  for the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseReleased(MouseEvent e) {
      scrolling = scroll = false ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will continually run until the
    *  component dies. It will process the scrolling
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      int scrollWidth = 0 ;
      int i           = 0 ;

      buttonPanelWidth = buttonPanelLayout.minimumLayoutSize(buttonPanel).width ;

      while ( continueThread ) {

         if ( scroll ) {

            if ( scrollRight )
               panelX+=15 ;
            else
               panelX-=15 ;

            if ( panelX <= 0 )
               panelX=0 ;

            else if ( ((0-panelX)+buttonPanelWidth) < (width-BUTTONWIDTH) ) {
               panelX-=15 ;
            } else {
               buttonPanel.setBounds ( 0-panelX, 0, panelX+(width-BUTTONWIDTH), height ) ;
            }

            sleep ( 25 ) ;
         }

         /* If we are not scrolling, sleep */
         else {

            sleep ( 10000 ) ;
         }
      }

   } /* run */





   /*------------------------------------------------------------*/
   /* sleep
    */
   /**
    *    This method is a convience method for
    *  sleeping. It will catch the Interupted exception
    *  
    *
    *  @param ms The amount of milliseconds to sleep
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void sleep ( int ms ) {
      try {
         Thread.sleep ( ms ) ;
      } catch ( Exception e ) {
      }

   }





   /*------------------------------------------------------------*/
   /* finalize
    */
   /**
    *    This method will override the default to
    *  stop our scroll thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finalize () {
      try {
         scrollThread.stop() ;
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred 
    *  size of the panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( preferredSize ) ;
   }
   public Dimension getMinimumSize () {
      return ( getPreferredSize( ) ) ;
   }
   public Dimension getMaximumSize () {
      return ( getPreferredSize( ) ) ;
   }


   public void setPreferredSize ( Dimension size ) {
      preferredSize = size ;
   }


   /*------------------------------------------------------------*/
   /* setBackground
    */
   /**
    *    This method will set the background to not
    *  a color, but a tiled image.
    *  
    *
    *  @param image The image to tile
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBackground ( Image image ) {
      this.backgroundImage = image ;
   }




   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param g The graphics context to paint with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      if ( backgroundImage == null )
         super.paintComponent(g) ;
      else {
         WinUtil.wallPaper ( this, g, backgroundImage ) ;
         paintChildren ( g );
      }

   }




   /*------------------------------------------------------------*/
   /* setArrowImage
    */
   /**
    *    This method will set the image used in the
    *  buttons that signify left and right
    *  
    *
    *  @param leftArrow
    *  @param rightArrow
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setArrowImage ( ImageIcon leftArrow, ImageIcon rightArrow ) {
      pb_LEFT.setText ( "" ) ;
      pb_LEFT.setIcon ( rightArrow ) ;
      pb_RIGHT.setText ( "" ) ;
      pb_RIGHT.setIcon ( leftArrow ) ;
   }
} /* ButtonPanel */

