package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   DefaultTreeComponent                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DefaultTreeComponent                */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;      
/**
 * This Renderer which do the actual drawing work for items in
 * the Document Maint. Filter
 *
 *
 * @version 1.0, 1/4/1999 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   DefaultTreeComponent                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class DefaultTreeComponent extends Component  {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Statics                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Color    selBackground = (Color)javax.swing.UIManager.getDefaults().get ( "Tree.selectionBackground" ) ;
   private static Color    selForeground = (Color)javax.swing.UIManager.getDefaults().get ( "Tree.selectionForeground" ) ;
   private static Font     font          = (Font)javax.swing.UIManager.getDefaults().get ( "Tree.font" ) ;




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected Object    data           = null ;
   protected boolean   selected       = false ;
   protected boolean   leaf           = false ;
   protected boolean   expanded       = false ;
   protected Dimension prefSize       = new Dimension ( 640, 21 ) ;
   private   Dimension noShowDim      = new Dimension ( 640, 0 ) ;
   private   boolean   noShow         = false ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a blank DefaultTreeComponent
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DefaultTreeComponent () {
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data in
    *  memory
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object data, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus ) {
      this.data     = data ;
      this.selected = selected ;
      this.expanded = expanded ;
      this.leaf     = leaf ;

      noShow = ( data == null ) ;

      if ( !noShow ) {
         if ( ( data.toString() == null ) ||
              ( data.toString().length() == 0 ) )
            noShow = true ;
      }
   }

   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method is subclassed because the height
    *  of our component depends on the data we
    *  are showing
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      if ( noShow )
         return ( noShowDim ) ;
      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* setPreferredWidth
    */
   /**
    *    This method will set the preferred width of
    *  all items in the list
    *  
    *
    *  @param width The preferred width
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPreferredWidth ( int width ) {
      prefSize.width = width ;
      noShowDim.width = width ;
   }


   /*------------------------------------------------------------*/
   /* paint
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param g The Graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paint ( Graphics g ) {
      String dataString = data.toString() ;

      if ( dataString != null ) {

         /* Ok, if we are selected, draw the selection now */
         if ( selected ) {
            g.setColor ( selBackground ) ;
            g.fillRect ( 0, 0, prefSize.width, prefSize.height ) ;
            g.setColor ( selForeground ) ;
         } else
            g.setColor ( Color.black ) ;

         /* Now, draw our text */
         g.setFont ( font ) ;
         g.drawString ( dataString, 3, 12 ) ;

      }
   }


} /* DefaultTreeComponent */
