package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   DetailColumnComponent                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DetailColumnComponent               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 Cogent Software                  */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;     
import javax.swing.border.* ;
/**
 * This Component appears as one column title in the MultiListTitlePanel 
 *
 *
 * @version 1.0, 2/21/1999 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DetailColumnComponent                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class DetailColumnComponent extends JComponent implements MouseListener, MouseMotionListener {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Vars                                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private Object      title          = null ;
    private BevelBorder bevelBorder    = new BevelBorder ( BevelBorder.RAISED ) ;
    private int         width          = 0 ;
    private int         height         = 0 ;
    private int         x              = 0 ;
    private int         y              = 0 ;
    private Dimension   prefSize       = new Dimension ( 100, 17 ) ;
    private boolean     cursorSizing   = false ;
    private boolean     mouseDown      = false ;
    private DetailList  detailList     = null ;
    private int         columnNum      = 0 ;
    private boolean     bevelDown      = false ;
    private Polygon     ascendingPoly  = null ;
    private Polygon     descendingPoly = null ;



    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create a DetailColumnComponent
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public DetailColumnComponent ( int columnNum, DetailList detailList, Object object ) {
        this.title     = object ;
        this.detailList = detailList ;
        this.columnNum = columnNum ;
        addMouseMotionListener ( this ) ;
        addMouseListener ( this ) ;
    }






    /*------------------------------------------------------------*/
    /* paintComponent
     */
    /**
     *    This method will paint our component
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void paintComponent ( Graphics g ) {
       Dimension size = getSize() ;

        size.width-- ;
        size.height-- ;

        super.paintComponent(g) ;                        

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Draw the sort triangle if need be                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( detailList.getSortColumn() == columnNum ) {
            Polygon poly = (detailList.getSortType() == DetailList.SORT_ASCENDING ) ? ascendingPoly : descendingPoly ;

            g.setColor ( Color.darkGray ) ;
            //g.fillPolygon ( poly ) ;

        }

         g.setColor ( bevelDown ? Color.darkGray : Color.white ) ;
         g.drawLine ( 0, 0, size.width, 0 ) ;
         g.drawLine ( 0, 0, 0, size.height ) ;

         g.setColor ( bevelDown ? Color.white : Color.darkGray ) ;
         g.drawLine ( size.width, 0, size.width, size.height ) ;
         g.drawLine ( 1, size.height, size.width, size.height ) ;

        g.setColor ( Color.black ) ;

        if ( title  instanceof String ) {
            if ( bevelDown )
                g.drawString ( (String)title , 5, 14 ) ;
            else
                g.drawString ( (String)title , 3, 12 ) ;
        }

        size.width++ ;
        size.height++ ;
    }






    /*------------------------------------------------------------*/
    /* setBounds
     */
    /**
     *    Subclassed to capture width and height
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setBounds ( int x, int y, int cx, int cy ) {
        int     xa[]  = new int[3] ;
        int     ya[]  = new int[3] ;
        int     px    = cx-10 ;

        this.width = cx ;
        this.height = cy ;
        this.x      = x ;
        this.y      = y ;
        super.setBounds ( x,y,cx,cy ) ;


        xa[0] = px ;    ya[0] = height-3 ;
        xa[1] = px+4 ;  ya[1] = 3 ;
        xa[2] = px+8 ;  ya[2] = height-3 ;
        ascendingPoly = new Polygon ( xa, ya, 3 ) ;

        xa[0] = px;    ya[0] = 3 ;
        xa[1] = px+8;  ya[1] = 3 ;
        xa[2] = px+4;  ya[2] = height-3 ;
        descendingPoly = new Polygon ( xa, ya, 3 ) ;

    }




    /*------------------------------------------------------------*/
    /* getPreferredSize () 
     */
    /**
     *    This method will return the preferred size
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Dimension getPreferredSize() {
        return( prefSize ) ;
    }

    public Dimension getMinimumSize() {
        return( prefSize ) ;
    }





    /*------------------------------------------------------------*/
    /* getX
     */
    /**
     *    This method will return our X
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public int getX() {
        return( x ) ;
    }

    /*------------------------------------------------------------*/
    /* getY
     */
    /**
     *    This method will return our Y
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public int getY() {
        return( y ) ;
    }


    /*------------------------------------------------------------*/
    /* getHeight
     */
    /**
     *    This method will return our height
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public int getHeight () {
        return( height ) ;
    }



    /*------------------------------------------------------------*/
    /* getWidth
     */
    /**
     *    This method will return our width
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public int getWidth () {
        return( prefSize.width ) ;
    }





    /*------------------------------------------------------------*/
    /* setPreferredWidth
     */
    /**
     *    This method will set our preferred width
     *  
     *  
     *
     *  @param width The width in pixels
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    protected void setWidth ( int width ) {
        prefSize.width = width ;
        invalidate() ;
    }

    /*------------------------------------------------------------*/
    /* setPreferredHeight
     */
    /**
     *    This method will set our preferred height
     *  
     *  
     *
     *  @param height The height in pixels
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    protected void setHeight ( int height ) {
        prefSize.height = height ;
    }






    /*------------------------------------------------------------*/
    /* mouseClicked
     */
    /**
     *    We will use this method to send
     *  a message to the DetailList to
     *  set it's order column to us.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void mouseClicked(MouseEvent e) {
        bevelDown = false ;
        detailList.switchSortType () ;
        detailList.setSortColumn ( columnNum ) ;
        repaintAll() ;
    }



    public void mouseEntered(MouseEvent e) {
        if ( !cursorSizing )
            setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
    } 

    public void mouseExited(MouseEvent e) {
        if ( !cursorSizing )
            setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
        bevelDown = false ;
        repaintAll() ;
    } 


    public void mouseDragged(MouseEvent e) {
        //if ( e.getX() > 20 )
        if ( cursorSizing )
            detailList.showDivider ( x+e.getX() ) ;
        bevelDown = false ;
    } 




    /*------------------------------------------------------------*/
    /* mousePressed
     */
    /**
     *    Subclassed to mark that a button is now
     *  down
     *  
     *
     *  @param e The mouse event
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void mousePressed(MouseEvent e) {
        mouseDown = true ;
        if ( !cursorSizing )
            bevelDown = true ;
        repaintAll() ;
    } 





    /*------------------------------------------------------------*/
    /* mouseReleased
     */
    /**
     *    Subclassed to capture when the mouse is
     *  released, if we were dragging, a resize
     *  will occur
     *
     *  @param e The mouse event
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void mouseReleased(MouseEvent e) {
        setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
        if ( ( cursorSizing ) && ( e.getX() > 18 ) ) {
            detailList.setColumnWidth ( columnNum, e.getX() ) ;
        }

        detailList.hideDivider() ;
        mouseDown = false ;
        cursorSizing = false ;
        bevelDown = false ;
        setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
        repaintAll() ;
    } 

    public void mouseMoved(MouseEvent e) {
        if ( !mouseDown ) {
            if ( e.getX() > (prefSize.width-4) ) {
                setCursor( Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR)) ;
                cursorSizing = true ;
            } else {
                setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
                cursorSizing = false ;
            }
        }
    }

    private void repaintAll () {
        //((JComponent)getParent()).revalidate() ;
        repaint() ;
    }
} /* DetailColumnComponent */
