package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   DetailColumnHeader                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DetailColumnHeader                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import javax.swing.* ;      
/**
 * This panel holds the column headings for the DetailList control 
 *
 *
 * @version 1.0, 2/21/1999 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DetailColumnHeader                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class DetailColumnHeader extends JPanel {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DetailList detailList = null ;
   private Vector     columns    = new Vector ( 10, 1 ) ;
   private boolean    showing    = true ;
   private int        padding    = 0 ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our Panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DetailColumnHeader ( DetailList detailList ) {
      this.detailList = detailList ;

      setLayout ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* addColumn
    */
   /**
    *    This method will add a column to our
    *  title
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addColumn ( Object column ) {
      DetailColumnComponent newComp = null ;

      add ( newComp = new DetailColumnComponent ( columns.size(), detailList, column ) ) ;
      columns.addElement ( newComp ) ;

      //setFont ( new Font ( "SansSerif", Font.BOLD, 12 ) ) ;
   }






   /*------------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove all of our
    *  components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAll() {
      int i       = getComponentCount() ;
      int length  = 0 ;

      while ( i < length )
         remove ( getComponent(i++) ) ;

      columns.removeAllElements() ;
   }





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our preferredSize
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize() {
      int x      = 0 ;
      int i      = 0 ;
      int length = columns.size() ;

      while ( i < length ) {
         x+=((Component)columns.elementAt(i)).getPreferredSize().width ;
         i++ ;
      }

      x=1280;
      if ( showing )
         return( new Dimension ( x, 18 ) ) ;
      else
         return( new Dimension ( x, 0 ) ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will do our layout
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int i = 0 ;
      int length = 0 ;

      super.doLayout() ;

      if ( columns != null ) {
         Component comp  = null ;
         Dimension size  = getSize() ;
         Dimension cSize = null ;
         int       x     = padding ;

         length = columns.size() ;

         while ( i < length ) {
            comp = (Component) columns.elementAt(i) ;
            if ( comp != null ) {
               cSize = comp.getPreferredSize() ;
               if ( i == (length-1) )
                  comp.setBounds ( x, 1, size.width-x-2, size.height-2 ) ;
               else
                  comp.setBounds ( x, 1, cSize.width, size.height-2 ) ;
               x+=cSize.width ;
            }
            i++ ;
         }
      }

   }




   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   //public void paintComponent ( Graphics g ) {
   //   Dimension size = getSize() ;

   //   size.width-- ;
   //   size.height-- ;
   //   super.paintComponent(g) ;
   //   g.setColor ( Color.white ) ;
   //   g.drawLine ( 0, 0, size.width, 0 ) ;
   //   g.drawLine ( 0, 0, 0, size.height ) ;

   //   g.setColor ( Color.black ) ;
   //   g.drawLine ( size.width, 0, size.width, size.height ) ;
   //   g.drawLine ( 1, size.height, size.width, size.height ) ;
   //}


   public void setPadding ( int width ) {
      this.padding  = width ;
   }





   /*------------------------------------------------------------*/
   /* getColumnX
    */
   /**
    *    This method will return a column's x coordinate
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnX ( int column ) {
      int x = 0 ;

      if ( column < columns.size() ) {
         x = ((DetailColumnComponent)columns.elementAt(column)).getX() ;
      }

      return( x ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnY
    */
   /**
    *    This method will return a column's y coordinate
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnY ( int column ) {
      int y = 0 ;

      if ( column < columns.size() ) {
         y = ((DetailColumnComponent)columns.elementAt(column)).getY() ;
      }

      return( y ) ;
   }

   /*------------------------------------------------------------*/
   /* getColumnWidth
    */
   /**
    *    This method will return a column's width coordinate
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnWidth ( int column ) {
      int width = 0 ;

      if ( column < columns.size() ) 
         width = ((DetailColumnComponent)columns.elementAt(column)).getWidth() ;

      return( width ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnHeight
    */
   /**
    *    This method will return a column's height coordinate
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnHeight ( int column ) {
      int height = 0 ;

      if ( column < columns.size() ) {
         height = ((DetailColumnComponent)columns.elementAt(column)).getHeight() ;
      }

      return( height ) ;
   }




   /*------------------------------------------------------------*/
   /* setColumnWidth
    */
   /**
    *    This method will set the width of a column
    *  
    *  
    *
    *  @param columnNum The column number to set
    *  @param width The width in pixels
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setColumnWidth ( int columnNum, int width ) {
      try {
         ((DetailColumnComponent)columns.elementAt(columnNum)).setWidth ( width ) ;
      } catch ( Exception e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    Returns the number of columns being
    *  displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnCount() {
      return( columns.size() ) ;
   }





   /*------------------------------------------------------------*/
   /* setShowing
    */
   /**
    *    This method will either hide or show
    *  this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setShowing ( boolean show ) {
      showing = show ;
      invalidate() ;
   }





   /*------------------------------------------------------------*/
   /* setColumnHeadingBorder
    */
   /**
    *    This method will set the border type around
    *  each column heading
    *  
    *
    *  @param border The new Border type
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnHeadingBorder ( javax.swing.border.Border border ) {
      if ( ( columns != null ) && ( columns.size() > 0 ) ) {
         int i       = 0 ;
         int length  = columns.size() ;

         while ( i < length ) {
            ((JComponent)columns.elementAt(i)).setBorder ( border ) ;
            i++ ;
         }

      }
   }

} /* DetailColumnHeader */
