package com.ibm.nzna.shared.gui ;   
/*-----------------------------------------------------------*/
/*   DetailJTree                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DetailJTree                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import javax.swing.tree.* ;
/**
 * The DetailJTree is simply a JPanel which holds a JScrollPane
 * and a JTree. Any data adding/removing or property changes on the JTree should
 * be handled through the getJTree() method. This panel's job
 * is to basically put a titled column header above the JTree<br><br>
 *
 * <b>Please Note</b> If the Model of the JTree contained in this control changes
 * for any reason, you must call the 'reset' method on this control.
 *
 *
 * @version 1.0, 12/8/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DetailJTree                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DetailJTree extends JPanel implements Runnable, MouseListener, TreeModelListener, DetailList {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Thread Names                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_RESORT       = "RESORT" ;
   private static final String THREAD_FIRE_ACTION  = "FIRE" ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JScrollPane            scrollPane            = null ;
   private JTree                  tree                  = null ;
   private DetailColumnHeader     titlePanel            = null ;
   private int                    columnCount           = 0 ;
   private Vector                 headings              = null ;
   private int                    sortColumn            = 0 ;
   private Panel                  pnl_SPLIT             = null ;
   private int                    sortType              = 0 ;
   private boolean                wordWrap              = false ;
   private Vector                 actionListeners       = new Vector ( 1 ) ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DetailJTree () {
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor created a DetailJTree
    *  from the JTree that is passed in
    *  
    *
    *  @param tree The tree to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DetailJTree ( JTree tree ) {
      this.tree = tree ;
      tree.getModel().addTreeModelListener ( this ) ;
      initialize() ;
   }



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize all parts of our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI and add it                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( tree == null )
         tree       = new JTree () ;
      scrollPane = new JScrollPane ( tree ) ;  
      scrollPane.setBorder (null) ;
      pnl_SPLIT = new Panel () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( scrollPane ) ;
      add ( pnl_SPLIT ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_SPLIT.setVisible ( false ) ;
      pnl_SPLIT.setBackground ( Color.black ) ;

      tree.addMouseListener ( this ) ;

   }


   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;
      scrollPane.setBounds ( 0, 0, size.width, size.height ) ;
   }



   /*------------------------------------------------------------*/
   /* getJTree
    */
   /**
    *    This method will return the instance of the JTree
    *  that this control is using. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JTree
    *//*---------------------------------------------------------*/
   public JTree getJTree () {
      return( tree ) ;
   }





   /*------------------------------------------------------------*/
   /* setJTree
    */
   /**
    *    This method will set the JTree that this
    *  component is using
    *  
    *
    *  @param tree The new JTree to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setJTree ( JTree tree ) {
      remove ( scrollPane ) ;
      this.tree = tree ;
      scrollPane = new JScrollPane ( tree ) ;
      add ( scrollPane ) ;
      revalidate() ;

      tree.getModel().addTreeModelListener ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* setColumnHeadings
    */
   /**
    *    This method will set the objects that are to
    *  be used for the heading of each column.
    *  
    *
    *  @param columnHeadings A Vector containing objects to use for the headings
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnHeadings ( Vector columnHeadings ) {
      int i      = 0 ;
      int length = 0 ;

      titlePanel = new DetailColumnHeader (this) ;
      titlePanel.setPadding ( 35 ) ;

      if ( columnHeadings == null ) {
         titlePanel.removeAll() ;
      } else {
         scrollPane.setColumnHeaderView ( titlePanel ) ;

         length = columnHeadings.size() ;
         headings = columnHeadings;

         while ( i < length ) {
            titlePanel.addColumn ( columnHeadings.elementAt(i) ) ;
            i++ ;
         }

         columnCount = length ;
      }


      if ( tree.getCellRenderer() instanceof DetailTreeCellRenderer ) {
         DetailTreeCellRenderer renderer = (DetailTreeCellRenderer) tree.getCellRenderer() ;

         renderer.setColumnCount ( columnCount ) ;
      }



   }

   /**
     *  This method will set the padding (in pixels) before the first column
     **/
   public void setTitlePadding ( int width ) {
      titlePanel.setPadding ( width ) ;
   }

   /*------------------------------------------------------------*/
   /* setColumnHeadings
    */
   /**
    *    This method will set the objects that are to
    *  be used for the heading of each column.
    *  
    *
    *  @param columnHeadings A Vector containing objects to use for the headings
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnHeadings ( Object[] columnHeadings ) {
      if ( columnHeadings != null ) {
         int      i = 0 ;
         Vector   vec = new Vector ( columnHeadings.length ) ;

         while ( i < columnHeadings.length )
            vec.addElement ( columnHeadings[i++] ) ;

         setColumnHeadings ( vec ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setColumnHeadingBorder
    */
   /**
    *    This method will set the border that appears
    *  around the heading of each column
    *  
    *
    *  @param Border The new Border
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnHeadingBorder ( javax.swing.border.Border border ) {
      titlePanel.setColumnHeadingBorder ( border ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnHeadings
    */
   /**
    *    This method will set the column headings
    *  used for each column
    *  
    *
    *  @param columnHeadings An array of objects to use for the headings
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getColumnHeadings () {
      return( headings ) ;
   }







   /*------------------------------------------------------------*/
   /* setSort
    */
   /**
    *    This method will set what type of
    *  sort, SORT_ASCENDING or SORT_DESCENDING
    *  
    *
    *  @param sortType The type of sort
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSort ( int sortType ) {
      if ( ( sortType == SORT_ASCENDING ) || ( sortType == SORT_DESCENDING ) )
         this.sortType = sortType ;
   }




   /*------------------------------------------------------------*/
   /* switchSortType
    */
   /**
    *    This method will switch the sort type
    *  from ascending to descending, or 
    *  from descending to ascending. PLEASE NOTE, this method will
    *  not automatically reorder the list. A subsequent call to setSortColumn
    *  will re-sort the columns.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void switchSortType () {
      if ( sortType == SORT_ASCENDING )
         sortType = SORT_DESCENDING ;
      else
         sortType = SORT_ASCENDING ;
   }

   public int getSortType () {
      return( sortType ) ;
   }



   /*------------------------------------------------------------*/
   /* getSortColumn
    */
   /**
    *    This method will return what column
    *  is being sorted
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getSortColumn () {
      return( sortColumn ) ;
   }


   /*------------------------------------------------------------*/
   /* setSortColumn
    */
   /**
    *    This method will set what column
    *  we are currently sorting on. This sort
    *  will apply to any additional data added
    *
    *  @param columnNum The column Number
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSortColumn ( int columnNum ) {
      this.sortColumn = columnNum ;
      //titlePanel.repaint() ;

      resort() ;
   }




   public void showDivider ( int x ) {
      pnl_SPLIT.setVisible ( true ) ;
      pnl_SPLIT.setBounds ( x, 0, 2, getSize().height ) ;
   }
   public void hideDivider () {
      pnl_SPLIT.setVisible ( false ) ;
   }




   /*------------------------------------------------------------*/
   /* setColumnWidth
    */
   /**
    *    This method will set a column's width in
    *  pixels.
    *  
    *
    *  @param columnNumber The column number to adjust
    *  @param width The width in pixels
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnWidth ( int columnNum, int width ) {
      titlePanel.setColumnWidth ( columnNum, width ) ;                                    

      if ( tree.getCellRenderer() instanceof DetailTreeCellRenderer ) {
         DetailTreeCellRenderer renderer = (DetailTreeCellRenderer) tree.getCellRenderer() ;

         renderer.setPreferredWidth ( titlePanel.getPreferredSize().width ) ;
         renderer.setColumnWidth ( columnNum, width ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, if we have any ExtendedMultiListRows in,           */
      /* we must tell them the new column width                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshExtendedColumnWidths ( ) ;


      refreshColumns() ;
   }





   /*------------------------------------------------------------*/
   /* refreshExtendedColumnWidths
    */
   /**
    *    This method will notify each ExtendedMultiListRow
    *  in our data set that the column number specified
    *  is now at the pixel width specificed
    *
    *  @param column The column
    *  @param pixelWidth The width in pixels
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshExtendedColumnWidths ( int columnNum, int width ) {
      Vector listModel = getData() ;
      int    i    = 0 ;
      int    size = listModel.size() ;

      while ( i < size ) {
         if ( listModel.elementAt(i) instanceof ExtendedMultiListRow ) {
            ((ExtendedMultiListRow)listModel.elementAt(i)).columnSized ( columnNum, width ) ;
         }
         i++ ;
      }
   }





   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return all of the data that
    *  has been added to the JTree
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getData () {
      DefaultTreeModel model = (DefaultTreeModel)tree.getModel() ;
      TreeNode         node  = (TreeNode)model.getRoot() ;
      Vector           data  = new Vector ( 10, 100 ) ;

      getDataFromNode ( node, data ) ;

      return( data ) ;

   }



   private void getDataFromNode ( TreeNode node, Vector data ) {
      int childCount=node.getChildCount() ;

      if ( ( node != null ) && ( childCount >0 ) ) {
         for ( int i = 0; i < childCount; i++ ) {
            data.addElement ( node.getChildAt ( i ) ) ;
            try {
               getDataFromNode ( node.getChildAt ( i ), data ) ;
            } catch ( Exception e ) {
            }
         }
      }
   }



   /*------------------------------------------------------------*/
   /* refreshExtendedColumnWidths
    */
   /**
    *    This method will notify each ExtendedMultiListRow
    *  in our data set to refresh it's column width
    *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshExtendedColumnWidths ( ) {
      try {
         Vector listModel = getData() ;
         int    i         = 0 ;
         int    size      = listModel.size() ;
         int    headIndex = 0 ;
         int    headSize  = headings.size() ;

         while ( i < headSize ) {
            if ( tree.getCellRenderer() instanceof DetailTreeCellRenderer )
               ((DetailTreeCellRenderer)tree.getCellRenderer()).setColumnWidth ( i, getColumnWidth(i) ) ;
            i++ ;
         }

         i=0 ;
         while ( i < size ) {

            if ( listModel.elementAt(i) instanceof MutableTreeWrapper ) {
               Object object = ((MutableTreeWrapper)listModel.elementAt(i)).getDataObject() ;

               if ( object instanceof ExtendedMultiListRow ) {
                  headIndex = 0 ;

                  while ( headIndex < headSize ) {
                     ((ExtendedMultiListRow)object).columnSized ( headIndex, getColumnWidth ( headIndex ) ) ;
                     headIndex++ ;
                  }
               }
            }
            i++ ;
         }
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }





   /*------------------------------------------------------------*/
   /* refreshColumns
    */
   /**
    *    This method will refresh and reposition all
    *  of the columns in the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshColumns() {
      invalidate() ;
      scrollPane.invalidate() ;
      scrollPane.revalidate() ;
      revalidate() ;
      scrollPane.repaint() ;
   }





   /*------------------------------------------------------------*/
   /* setWordWrap
    */
   /**
    *    If the JTree contained in the DetailJTree
    *  has a Renderer of DetailTreeCellRenderer, then this method
    *  will turn on or off the word wrapping capabilities
    *  
    *
    *  @param wordWrap TRUE = on, FALSE = off
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWordWrap ( boolean wordWrap ) {
      if ( tree.getCellRenderer() instanceof DetailTreeCellRenderer ) {
         this.wordWrap = wordWrap ;
         ((DetailTreeCellRenderer)tree.getCellRenderer()).setWordWrap ( wordWrap ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getColumnWidth
    */
   /**
    *    This method will return the width in pixels
    *  of a column
    *  
    *
    *  @param columnNum The column number to return the width for
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnWidth ( int column ) {
      return( titlePanel.getColumnWidth ( column ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    This method will return the number of
    *  columns being returned
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getColumnCount () {
      if ( titlePanel != null )
         return( titlePanel.getColumnCount() ) ;
      return( 0 ) ;
   }

   public void treeNodesChanged ( TreeModelEvent e ) {
   }

   public void treeNodesInserted ( TreeModelEvent e ) {
      refreshExtendedColumnWidths () ;
   }

   public void treeStructureChanged ( TreeModelEvent e ) {
      refreshExtendedColumnWidths () ;
   }

   public void treeNodesRemoved ( TreeModelEvent e ) {
   }

   public void reset () {
      tree.getModel().removeTreeModelListener (this ) ;
      tree.getModel().addTreeModelListener ( this ) ;
      ((DetailTreeCellRenderer)tree.getCellRenderer()).setWordWrap ( wordWrap ) ;

      if ( !wordWrap )
         tree.setRowHeight ( 18 ) ;
      else
         tree.setRowHeight ( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* resort
    */
   /**
    *    This method will re-sort all displayed items
    *  in the tree to the currently selected sorted
    *  column. Please keep in mind that all items must
    *  implements MultiListSort to be sorted
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void resort () {
      new Thread ( this, THREAD_RESORT ).start() ;
   }


   public void resortThread () {
      try {
         DefaultTreeModel model = (DefaultTreeModel)tree.getModel() ;                                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, kick off the recursive sort by starting with     */
         /* the root                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         resort ( model, (DefaultMutableTreeNode)model.getRoot() ) ;

         //tree.setModel ( model ) ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

   }





   /*------------------------------------------------------------*/
   /* resort
    */
   /**
    *    This method will recursivly sort all children
    *  of the specified object passed in.
    *  
    *
    *  @param model The DefaultTreeModel to start sorting with
    *  @param object The object to start sorting with
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void resort ( DefaultTreeModel model, DefaultMutableTreeNode parent ) throws Exception {
      int     childCount = model.getChildCount ( parent ) ;
      int     i          = 0 ;
      boolean sorted     = false ;

      while ( i < childCount ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this child has children, we must go into that child, */
         /* otherwise re sort the parent we were passed in            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( model.getChildCount ( model.getChild ( parent, i ) ) > 0 ) {
            resort ( model, (DefaultMutableTreeNode)model.getChild ( parent, i ) ) ;
            sorted = false ;
         } else if ( !sorted ) {
            resortChildren ( model, parent ) ;
            sorted = true ;
         }
         i++ ;
      }
   } /* resort */




   /*------------------------------------------------------------*/
   /* resortChildren
    */
   /**
    *    This method will resort all of the children
    *  in the given parent and model
    *  
    *
    *  @param model The DefaultTreeModel that the parent belongs to
    *  @param parent The parent object
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void resortChildren ( DefaultTreeModel model, DefaultMutableTreeNode parent ) {
      int    childCount   = parent.getChildCount() ;
      int    i            = 0 ;
      Vector children     = new Vector ( childCount ) ;
      int[]  childIndices = new int [ childCount ] ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Copy the children to a vector                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < childCount ) {
         if ( ((DefaultMutableTreeNode)model.getChild (parent, i )).getChildCount() == 0 )
            children.addElement ( model.getChild ( parent, i++ ) ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, remove all the children from the parent            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      i=1 ;
      while ( i < childCount )
         ((MutableTreeNode)children.elementAt(i++)).removeFromParent() ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now sort the children                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      new MultiListSorter ( children, sortColumn, sortType ) ;                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, reinsert the children                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      i=0; 
      while ( i < childCount ) {
         parent.add ( (MutableTreeNode)children.elementAt(i) ) ;
         childIndices[i]=i ;
         i++ ;
      }

      model.nodesChanged ( (TreeNode)parent, childIndices ) ; // reload ( parent ) ;

   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run a variety of operations in
    *  a thread. The operation performed depends
    *  on the name of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_RESORT                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_RESORT ) )
         resortThread() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   FireAction                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( operation.equals ( THREAD_FIRE_ACTION ) ) {
         ActionEvent e = new ActionEvent ( this, 0, "" ) ;

         int i    = 0 ;
         int size = actionListeners.size() ;

         while ( i < size ) {
            ((ActionListener)actionListeners.elementAt(i)).actionPerformed ( e ) ;
            i++ ;
         }
      }

   }


   public void addActionListener ( ActionListener e ) {
      actionListeners.addElement ( e) ;
   }

   public void removeActionListener ( ActionEvent e ) {
      actionListeners.removeElement ( e ) ;
   }


   public void mouseClicked ( MouseEvent e ) {
      if ( e.getClickCount() > 1 )
         new Thread ( this, THREAD_FIRE_ACTION ).start() ;
   }

   public void mouseEntered ( MouseEvent e ) {
   }

   public void mousePressed ( MouseEvent e ) {
   }

   public void mouseReleased ( MouseEvent e ) {
   }

   public void mouseExited ( MouseEvent e ) {
   }

   /**
     *  This method will return an instance of the JScroll Pane we are using
     **/
   public JScrollPane getJScrollPane () {
      return( scrollPane ) ;
   }
} /* DetailJTree */
