package com.ibm.nzna.shared.gui ;  
/*-----------------------------------------------------------*/
/*   DetailList                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DetailList                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This interface defines all methods needed to be implemented by any list of Detailed items 
 *
 *
 * @version 1.0, 12/8/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DetailList                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public interface DetailList {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Sort Types                                              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int SORT_ASCENDING       = 1 ;
   public static final int SORT_DESCENDING      = 2 ;




   /*------------------------------------------------------------*/
   /* getSortColumn
    */
   /**
    *    This method must return the current column
    *  that is being sorted
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The index of the column, 0 is the left most column
    *//*---------------------------------------------------------*/
   public abstract int getSortColumn () ;




   /*------------------------------------------------------------*/
   /* getSortType
    */
   /**
    *    This method must return the sort type of
    *  the sorted column
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int Sort Type
    *//*---------------------------------------------------------*/
   public abstract int getSortType () ;





   /*------------------------------------------------------------*/
   /* switchSortType
    */
   /**
    *    This method must switch the type of sort (Ascending
    *  or Descending) on the current sorted column
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void switchSortType () ;





   /*------------------------------------------------------------*/
   /* setSortColumn
    */
   /**
    *    This method must change the current column
    *  that is being sorted to the column passed
    *  
    *
    *  @param sortColumn The column that now needs to be sorted
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void setSortColumn ( int column ) ;





   /*------------------------------------------------------------*/
   /* showDivider
    */
   /**
    *    This method must show a divider component
    *  at the given X position
    *  
    *
    *  @param x The X position in pixels to show the divider
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void showDivider ( int x ) ;






   /*------------------------------------------------------------*/
   /* setColumnWidth
    */
   /**
    *    This method must set the width of the
    *  column passed to the width passed
    *  
    *
    *  @param column The column number. 0 is the left most column
    *  @param width The width in pixels of the column
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void setColumnWidth ( int column, int width ) ;




   /*------------------------------------------------------------*/
   /* hideDivider
    */
   /**
    *    This method must hide the divided that shows
    *  when the 'showDivider' method is called
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void hideDivider () ;





   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    This method will return the number of
    *  columns being displayed in the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public abstract int getColumnCount() ;




   /*------------------------------------------------------------*/
   /* getColumnWidth
    */
   /**
    *    This method will return the width of
    *  a column being displayed
    *  
    *
    *  @param column The column to return the width of
    *  @param None
    *  @param None
    *
    *  @return int The width in pixels
    *//*---------------------------------------------------------*/
   public abstract int getColumnWidth ( int column ) ;
} /* DetailList */
