package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   DetailTreeCellRenderer                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DetailTreeCellRenderer              */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The DetailTreeCellRenderer is the backbone to creating a Detailed
 * view JTree. This means that a Tree view will not only have an order
 * of presence, but the rows themselves can contain columns.
 *
 * If an object added to a JList also implements the MultiListRow interface,
 * each column for that row will be drawn at the width intervels (see setColumnWidth). For any
 * other object, the toString method will be called to represent the data.
 *
 * @version 1.0, 3/7/2000
 * @author  David Lentz
 */
import javax.swing.tree.* ;
import javax.swing.JTree ;
import java.awt.Component ;
import com.ibm.nzna.shared.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   DetailTreeCellRenderer                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class DetailTreeCellRenderer extends Object implements TreeCellRenderer {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DetailTreeCellComponent filterComp   = null ;
   private int                     columnCount  = 0 ;
   private int[]                   columnWidths = null ;
   private boolean                 wordWrap     = false ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create the DefaultTreeRenderer
    *  
    *  
    *
    *  @param columnCount The number of columns that will be displayed
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DetailTreeCellRenderer ( FontUtil fontUtil, int columnCount ) {
      filterComp = new DetailTreeCellComponent ( this, columnCount, fontUtil ) ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize the Column Width array                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( columnCount > 0 )
         columnWidths = new int[columnCount] ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the default column widths to 100                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      for ( int i = 0; i < columnCount; i++ )
         setColumnWidth ( i, 100 ) ;
   }





   /*------------------------------------------------------------*/
   /* setColumnWidth
    */
   /**
    *    This method will set the width in pixels
    *  of the column passed in. Columns are 0 based.
    *  
    *
    *  @param column The column to set the width for
    *  @param width The width in pixels
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnWidth ( int column, int width ) {
      columnWidths[column] = width ;
      filterComp.setColumnWidth ( column, width ) ;
   }

   public int getColumnWidth ( int column ) {
      return ( columnWidths[column] ) ;
   }

   public void setColumnCount ( int count ) {
      columnWidths = new int[count] ;
      filterComp.setColumnCount ( count ) ;
   }



   /*------------------------------------------------------------*/
   /* getTreeCellRendererComponent
    */
   /**
    *    This method will return our ccRendererComponent
    *  instead of the normal JLabel Component
    *  
    *
    *  @param The normal getTreeCellRenderer params
    *
    *  @return Component
    *//*---------------------------------------------------------*/
   public Component getTreeCellRendererComponent ( JTree jTree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus ) {
      filterComp.setWordWrap ( getWordWrap() ) ;
      filterComp.refreshData ( value, selected, expanded, leaf, row, hasFocus ) ;
      return( (Component) filterComp ) ;
   } /* getTreeCellRendererComponent */






   /*------------------------------------------------------------*/
   /* setWordWrap
    */
   /**
    *    This method will set word wrapping on or off. If 
    *  set to on any string displayed will wrap to show
    *  the full string, instead of having the "..."
    *
    *  @param wordWarp TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWordWrap ( boolean wordWrap ) {
      this.wordWrap = wordWrap ;
   }


   /*------------------------------------------------------------*/
   /* setPreferredWidth
    */
   /**
    *    This method will set the preferred width of
    *  all items in the list. Please be advised that YOU must
    *  call JTree.setRowHeight(0) to have rows with different heights!
    *  
    *
    *  @param width The preferred width
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPreferredWidth ( int width ) {
      filterComp.setPreferredWidth ( width ) ;
   }




   /*------------------------------------------------------------*/
   /* setFont
    */
   /**
    *    This method will set the font that we are displaying
    *  the data in
    *  
    *
    *  @param font The font to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFont ( java.awt.Font font ) {
      filterComp.setFont ( font ) ;
   }




   /*------------------------------------------------------------*/
   /* getWordWrap
    */
   /**
    *    This method will return the wordWrap property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public boolean getWordWrap ( ) {
      return ( wordWrap ) ;
   }

} /* DefaultTreeRenderer */
