package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   DropButton                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DropButton                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This button acts like a normal JButton except for that it
 * has a drop down segment on the right side of the button. Pushing
 * the drop down segment will result in a popup menu directly below
 * the button.
 *
 *
 * @version 1.0, 11/2/1999 
 */
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   DropButton                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public final class DropButton extends JPanel implements Runnable, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_FIREACTIONEVENT = "FAE" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector     listenerVec = null ;
   private JButton    mainButton  = null ;
   private JButton    dropButton  = null ;
   private JPopupMenu popupMenu   = null ;
   private Dimension  prefSize    = null ;




   /*------------------------------------------------------------*/
   /* Constructor(0)
    */
   /**
    *    This method will create an initially blank
    *  DropButton
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DropButton() {
      init() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor(1)
    */
   /**
    *    This method will create a Button
    *  with the text specified
    *  
    *
   *  @param text The text to display
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DropButton ( String text ) {
      init() ;
      setText ( text ) ;
   }




   /*------------------------------------------------------------*/
   /* Constructor(2)
    */
   /**
    *    This constructor will create a DropButton
    *  with the specified ImageIcon
    *  
    *
    *  @param imageIcon The Image to display on the face of the button
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DropButton ( ImageIcon image ) {
      init() ;
      setIcon ( image ) ;
   }





   /*------------------------------------------------------------*/
   /* Constructor(3)
    */
   /**
    *    This constructor will create a DropButton
    *  with the text and image specified
    *  
    *
    *  @param text The text to display
    *  @param image The image to display
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DropButton ( String text, ImageIcon image ) {
      init() ;
      setText ( text ) ;
      setIcon ( image ) ;
   }





   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void init () {          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mainButton = new JButton ( ) ;
      dropButton = new JButton ( "" ) ;
      popupMenu  = new JPopupMenu () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      dropButton.setVisible ( false ) ;
      dropButton.setFocusPainted ( false ) ;
      dropButton.addActionListener ( this ) ;
      mainButton.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout(0,0) ) ;
      add ( mainButton, BorderLayout.CENTER ) ;
      add ( dropButton, BorderLayout.EAST ) ;

   }



   /*------------------------------------------------------------*/
   /* setBorderPainted
    */
   /**
    *    This method will set the 'BorderPainted'
    *  property of the JButton
    *  
    *
    *  @param TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBorderPainted ( boolean borderPainted ) {
      mainButton.setBorderPainted ( borderPainted ) ;
      dropButton.setBorderPainted ( borderPainted ) ;
   }





   /*------------------------------------------------------------*/
   /* setHorizontalAlignment
    */
   /**
    *    Sets the horizontal alignement property in
    *  the JButton
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setHorizontalAlignment ( int alignment ) {
      mainButton.setHorizontalAlignment ( alignment ) ;
   }




   /*------------------------------------------------------------*/
   /* setMargin
    */
   /**
    *    This method will set the margin in the
    *  JButton being displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMargin ( Insets insets ) {
      mainButton.setMargin ( insets ) ;
      dropButton.setMargin ( insets ) ;
   }





   /*------------------------------------------------------------*/
   /* setMneumonic
    */
   /**
    *    This method will set the Mneumonic property
    *  of the JButton
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMnemonic ( int mnemonic ) {
      mainButton.setMnemonic ( mnemonic ) ;
   }


   /*------------------------------------------------------------*/
   /* setMneumonic
    */
   /**
    *    This method will set the Mneumonic property
    *  of the JButton
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMnemonic ( char mnemonic ) {
      mainButton.setMnemonic ( mnemonic ) ;
   }




   /*------------------------------------------------------------*/
   /* setModel
    */
   /**
    *    Set the model that this button represents.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   void setModel(ButtonModel newModel) {
      mainButton.setModel(newModel) ;
   }




   /*------------------------------------------------------------*/
   /* setPressedIcon
    */
   /**
    *    Sets the pressed icon for the button.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPressedIcon(Icon pressedIcon) {
      mainButton.setPressedIcon ( pressedIcon ) ;
   }




   /*------------------------------------------------------------*/
   /* setRolloverEnabled
    */
   /**
    *    Sets whether rollover effects should be enabled.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRolloverEnabled(boolean b) {
      mainButton.setRolloverEnabled ( b ) ;
   }




   /*------------------------------------------------------------*/
   /* setRolloverIcon
    */
   /**
    *    Sets the rollover icon for the button.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRolloverIcon(Icon rolloverIcon) {
      mainButton.setRolloverIcon ( rolloverIcon ) ;
   }




   /*------------------------------------------------------------*/
   /* setRolloverSelectedIcon
    */
   /**
    *    Sets the rollover selected icon for the button.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRolloverSelectedIcon(Icon rolloverSelectedIcon) {
      mainButton.setRolloverSelectedIcon ( rolloverSelectedIcon ) ;
   }




   /*------------------------------------------------------------*/
   /* setSelected
    */
   /**
    *  Sets the state of the button.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelected(boolean b) {
      mainButton.setSelected ( b ) ;
   }




   /*------------------------------------------------------------*/
   /* setSelectedIcon
    */
   /**
    *    Sets the selected icon for the button.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelectedIcon(Icon selectedIcon) {
      mainButton.setSelectedIcon ( selectedIcon ) ;
   }




   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    Sets the button's text.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText(String text) {
      mainButton.setText ( text ) ;
   }





   /*------------------------------------------------------------*/
   /* setActionCommand
    */
   /**
    *    Sets the action command for this button.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setActionCommand(String actionCommand) {
      mainButton.setActionCommand ( actionCommand ) ;
   }




   /*------------------------------------------------------------*/
   /* setContentAreaFilled
    */
   /**
    *    Sets whether the button should paint the content area or leave it transparent.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setContentAreaFilled(boolean b) {
      mainButton.setContentAreaFilled ( b ) ;
      dropButton.setContentAreaFilled ( b ) ;
   }




   /*------------------------------------------------------------*/
   /* setDisabledIcon
    */
   /**
    *    Sets the disabled icon for the button.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDisabledIcon(Icon disabledIcon) {
      mainButton.setDisabledIcon ( disabledIcon ) ;
   }





   /*------------------------------------------------------------*/
   /* setDisabledSelectedIcon
    */
   /**
    *    Sets the disabled selection icon for the button.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDisabledSelectedIcon(Icon disabledSelectedIcon)  {
      mainButton.setDisabledSelectedIcon ( disabledSelectedIcon ) ;
   }




   /*------------------------------------------------------------*/
   /* setFocusPainted
    */
   /**
    *    Sets whether focus should be painted.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocusPainted(boolean b) {
      mainButton.setFocusPainted ( b ) ;
   }




   /*------------------------------------------------------------*/
   /* setHorizontalTextPosition
    */
   /**
    *    Sets the horizontal position of the text relative to the icon.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setHorizontalTextPosition(int textPosition) {
      mainButton.setHorizontalTextPosition ( textPosition ) ;
   }




   /*------------------------------------------------------------*/
   /* setIcon
    */
   /**
    *    Sets the button's default icon.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIcon(Icon defaultIcon) {
      mainButton.setIcon ( defaultIcon ) ;
   }

   /*------------------------------------------------------------*/
   /* setIcon
    */
   /**
    *    Sets the button's default icon.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIcon(ImageIcon defaultIcon) {
      mainButton.setIcon ( defaultIcon ) ;
   }




   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener to this
    *  button
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      if ( listenerVec == null )
         listenerVec = new Vector ( 1, 1 ) ;

      listenerVec.addElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* fireActionEvent
    */
   /**
    *    This method will fire the action event from
    *  this button
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void fireActionEvent ( ActionEvent e ) {
      int i       = 0 ;


      if ( listenerVec != null ) {
         int length = listenerVec.size() ;

         while ( i < length ) {

            ((ActionListener)listenerVec.elementAt(i)).actionPerformed ( e ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform any operations
    *  in a thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName() ;             


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_FIREACTIONEVENT                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( name.equals ( THREAD_FIREACTIONEVENT ) )
         fireActionEvent ( new ActionEvent ( this, 
                                             ActionEvent.ACTION_FIRST, 
                                             mainButton.getActionCommand() ) ) ;

   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when either of
    *  our buttons are called
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == mainButton )
         new Thread ( this, THREAD_FIREACTIONEVENT ).start() ;
      else if ( e.getSource() == dropButton ) {
         Point pt = getLocation() ;

         popupMenu.setLightWeightPopupEnabled ( true ) ;
         popupMenu.setInvoker ( this ) ;

         SwingUtilities.convertPointToScreen ( pt, this ) ;
         pt.y+=getSize().height ;

         popupMenu.setLocation ( pt ) ;
         popupMenu.setVisible ( true ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* setDropButtonPosition
    */
   /**
    *    This method will set the position where the
    *  drop button is located. The default location is BorderLayout.EAST 
    *  
    *
    *  @param position - Valid positions are the BorderLayout positions. CENTER is invalid
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDropButtonPosition ( String position ) {
      if ( !position.equals ( BorderLayout.CENTER ) ) {
         remove ( dropButton ) ;
         add ( dropButton, position ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* addJMenu
    */
   /**
    *    This method will add a menu component
    *  to the drop down
    *  
    *
    *  @param jMenu The JMenu to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addJMenu ( JMenu jMenu ) {
      popupMenu.add ( jMenu ) ;
      dropButton.setVisible ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* addJMenuItem
    */
   /**
    *    This method will add a Menu Item to the
    *  Drop down menu
    *  
    *
    *  @param menuItem The JMenuItem to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addJMenuItem ( JMenuItem menuItem ) {
      popupMenu.add ( menuItem ) ;
      dropButton.setVisible ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* addSeparator
    */
   /**
    *    This method will add a separator to the drop
    *  down menu
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addSeparator ( ) {
      popupMenu.addSeparator() ;
      dropButton.setVisible ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* remove
    */
   /**
    *    This method will remove the component
    *  passed in
    *  
    *
    *  @param comp The component to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void remove ( Component comp ) {
      popupMenu.remove ( comp ) ;
      dropButton.setVisible ( popupMenu.getComponentCount() > 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* remove
    */
   /**
    *    This method will remove the component
    *  at the index specified
    *  
    *
    *  @param index The index to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void remove ( int index ) {
      popupMenu.remove ( index ) ;
      dropButton.setVisible ( popupMenu.getComponentCount() > 0 ) ;
   }

   /**
    * Returns the margin between the button's border and
    * the label.
    * @see #setMargin
    */
   public Insets getMargin() {
      return mainButton.getMargin() ;
   }

   /**
    * Returns the default icon.
    * @see #setIcon
    */
   public Icon getIcon() {
      return mainButton.getIcon() ;
   }

   /**
 * Returns the pressed icon for the button.
 * @see #setPressedIcon
 */
   public Icon getPressedIcon() {
      return mainButton.getPressedIcon() ;
   }

   /**
    * Returns the selected icon for the button.
    * @see #setSelectedIcon
    */
   public Icon getSelectedIcon() {
      return mainButton.getSelectedIcon() ;
   }

   /**
    * Returns the rollover icon for the button.
    * @see #setRolloverIcon
    */
   public Icon getRolloverIcon() {
      return mainButton.getRolloverIcon() ;
   }


   /**
    * Returns the rollover seletion icon for the button.
    * @see #setRolloverSelectedIcon
    */
   public Icon getRolloverSelectedIcon() {
      return mainButton.getRolloverSelectedIcon() ;
   }

   /**
    * Returns the icon used by the button when it's disabled.
    * If not no disabled icon has been set, the button constructs
    * one from the default icon. 
    * PENDING(jeff): the disabled icon really should be created 
    * (if necesary) by the L&F.
    * @see #getPressedIcon
    * @see #setDisabledIcon
    */
   public Icon getDisabledIcon() {
      return( mainButton.getDisabledIcon() ) ;
   }

   /**
    * Returns the icon used by the button when it's disabled and selected.
    * If not no disabled selection icon has been set, the button constructs
    * one from the selection icon. 
    * PENDING(jeff): the disabled selection icon really should be created 
    * (if necesary) by the L&F.
    * @see #getPressedIcon
    * @see #setDisabledIcon
    */
   public Icon getDisabledSelectedIcon() {
      return( mainButton.getDisabledSelectedIcon() ) ;
   }

   /**
    * Returns the vertical alignment of the text and icon.
    * Valid keys: CENTER (the default), TOP, BOTTOM
    */
   public int getVerticalAlignment() {
      return mainButton.getVerticalAlignment() ;
   }

   /**
    * Returns the horizontal alignment of the icon and text.
    * Valid keys: CENTER (the default), LEFT, RIGHT
    */
   public int getHorizontalAlignment() {
      return mainButton.getHorizontalAlignment() ;
   }

   /**
    * Returns the vertical position of the text relative to the icon
    * Valid keys: CENTER (the default), TOP, BOTTOM
    */
   public int getVerticalTextPosition() {
      return mainButton.getVerticalTextPosition() ;
   }


   /**
    * Sets the horizontal position of the text relative to the icon.
    * Valid keys: RIGHT (the default), LEFT, CENTER
    */
   public int getHorizontalTextPosition() {
      return mainButton.getHorizontalTextPosition() ;
   }

   /**
    * Returns whether focus should be painted.
    * @see #setFocusPainted
    */
   public boolean isFocusPainted() {
      return mainButton.isFocusPainted() ;
   }

   /**
    * Returns whether the border should be painted.
    * @see #setBorderPainted
    */
   public boolean isBorderPainted() {
      return mainButton.isBorderPainted() ;
   }

   /**
    * Checks whether the "content area" of the button should be filled.
    * @see #setFocusPainted
    */
   public boolean isContentAreaFilled() {
      return mainButton.isContentAreaFilled() ;
   }


   /**
    * Checks whether rollover effects are enabled.
    * @see #setFocusPainted
    */
   public boolean isRolloverEnabled() {
      return mainButton.isRolloverEnabled() ;
   }

   /**
    * Get the keyboard mnemonic from the the current model 
    */
   public int getMnemonic() {
      return mainButton.getMnemonic() ;
   }

   /**
    * Get the model that this button represents.
    * @see #setModel
    */
   public ButtonModel getModel() {
      return mainButton.getModel() ;
   }





   /*------------------------------------------------------------*/
   /* setVerticalTextPosition
    */
   /**
    *    Sets the Vertical text position. Please see AbstractButton
    *  
    *  
    *
    *  @param int
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setVerticalTextPosition ( int pos ) {
      mainButton.setVerticalTextPosition ( pos ) ;
   }

   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred size
    *  for this button
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      if ( prefSize == null ) {
         Dimension prefSize = mainButton.getMinimumSize() ;
         Dimension dropSize = dropButton.getMinimumSize() ;

         if ( prefSize.height < dropSize.height )
            prefSize.height = dropSize.height ;

         prefSize.width+=dropSize.width ;

         return prefSize  ;
      }

      return( prefSize ) ;
   }


   public Dimension getMinimumSize() {
      return( getPreferredSize() ) ;
   }

   public Dimension getMaximumSize() {
      return( getPreferredSize() ) ;
   }

   public void setPreferredSize( Dimension size ) {
      this.prefSize = size ;
   }




   /*------------------------------------------------------------*/
   /* setDropIcon
    */
   /**
    *    This method will set the icon that is used in the
    *  drop button. Without calling this method the
    *  drop button will not show
    *
    *  @param imageIcon The ImageIcon to use as the drop button
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDropIcon ( ImageIcon imageIcon ) {
      dropButton.setIcon ( imageIcon ) ;
   }

} /* DropButton */
