package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   ErrorDlg                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ErrorDlg                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog will show an exception. The constructor will not return until the window closes 
 *
 *
 * @version 1.0, 12/13/1998 
 */
import javax.swing.* ;
import java.awt.event.* ;
import java.awt.* ;
import java.io.* ;
import java.util.* ;
import java.awt.datatransfer.* ;
import com.ibm.nzna.shared.util.* ;



/*-----------------------------------------------------------*/
/*                                                           */
/*   ErrorDlg                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class ErrorDlg extends JDialog implements ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea   mle_ERROR         = null ;
   private JTextArea   mle_OUTPUT        = null ;
   private JScrollPane scr_OUTPUT        = null ;
   private DButton     pb_OK             = null ;
   private DButton     pb_COPYTOCLIP     = null ;
   private DButton     pb_SHOWDETAILS    = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Exception e           = null ;
   private String    methodName  = "Unknown" ;
   private String    className   = "Unknown" ;
   private Frame     parentFrame = null ;
   private boolean   showDetails = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a default ErrorDlg showing
    *  only the exception and stacktrace
    *  
    *
    *  @param e The Exception to report
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ErrorDlg ( Exception e ) {
      super ( new Frame(), "Error", true ) ;
      this.e = e ;
      init() ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create an ErrorDlg reporting
    *  method, class, exception and stack trace
    *  
    *
    *  @param className The class where the exception was thrown
    *  @param methodName The method where the exception was thrown
    *  @param e The Exception to report
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ErrorDlg ( String className, String methodName, Exception e ) {
      super ( new Frame(), "Error", true ) ;
      this.e  = e ;
      this.methodName = methodName ;
      this.className  = className ;
      init() ;
   }





   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize and show our window.
    *  It will not return until the user closes the window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void init () {
      parentFrame = (Frame)getParent() ;

      /* Create the controls */
      createControls() ;

      /* Refresh our data */
      refreshData () ;

      /* Set our size and position */
      setSize ( 300, 150 ) ;
      centerWindow ( ) ;
      setResizable ( false ) ;

      /* Show ourselves */
      setVisible ( true ) ;

      /* Close the frame we started */
      parentFrame.dispose() ;
      parentFrame = null ;
   }



   /*------------------------------------------------------------*/
   /* centerWindow
    */
   /**
    *    This method will center our window
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void centerWindow () {
      int x = 0 ;
      int y = 0 ;
      Dimension winSize = getSize () ;
      Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize () ;

      x = (scrSize.width/2)-(winSize.width/2) ;
      y = (scrSize.height/2)-(winSize.height/2) ;

      setLocation ( x, y ) ;
   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when action
    *  events need to be processed
    *  
    *
    *  @param event The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_OK )
         dispose() ;
      else if ( e.getSource() == pb_SHOWDETAILS ) {
         if ( !showDetails )
            setSize ( 375, 300 ) ;
         else
            setSize ( 300, 150 ) ;
         invalidate() ;
         centerWindow() ;
         showDetails = !showDetails ;
         doLayout() ;
         setVisible ( true ) ; //Don't ask me why, but the refresh won't work if this isn't called!
      }

      else if ( e.getSource() == pb_COPYTOCLIP ) {
         Clipboard       cp      = Toolkit.getDefaultToolkit().getSystemClipboard() ;

         if ( cp != null ) {
            StringSelection strSel = new StringSelection ( mle_OUTPUT.getText() ) ;

            cp.setContents ( strSel, strSel ) ;
            pb_COPYTOCLIP.setText ( "Details Copied" ) ;
         } else
            pb_COPYTOCLIP.setText ( "Could not copy details!" ) ;

      }
   }






   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createControls() {

      /* Set our layout to null */
      getContentPane().setLayout ( null ) ;

      /* Create our controls and add them */
      getContentPane().add ( mle_ERROR      = new JTextArea ( "An error has occured in the last action you took. Press 'Ok' to continue." ) ) ;
      getContentPane().add ( pb_SHOWDETAILS = new DButton ( "Show Details" ) ) ;
      getContentPane().add ( pb_OK          = new DButton ( "Ok" ) ) ;
      getContentPane().add ( pb_COPYTOCLIP  = new DButton ( "Copy Details to Clipboard" ) ) ;

      mle_OUTPUT    = new JTextArea ( "" ) ;
      getContentPane().add ( scr_OUTPUT = new JScrollPane ( mle_OUTPUT ) ) ;

      /* Set some properties */
      WinUtil.setJTextAreaAsLabel ( mle_ERROR ) ;
      mle_ERROR.setFont ( new Font ( "Helv", Font.PLAIN, 12 ) ) ;
      mle_ERROR.setForeground ( Color.black ) ;
      mle_OUTPUT.setFont ( new Font ( "Courier", Font.PLAIN, 12 ) ) ;
      mle_OUTPUT.setEditable ( false ) ;
      pb_SHOWDETAILS.setMargin ( new Insets ( 0, 0, 0, 0 ) ) ;

      pb_OK.addActionListener ( this ) ;
      pb_COPYTOCLIP.addActionListener ( this ) ;
      pb_SHOWDETAILS.addActionListener ( this ) ;

   }




   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh the static text fields
    *  and output fields with out data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      StringWriter    sw      = new StringWriter () ;
      PrintWriter     pw      = new PrintWriter ( sw ) ;
      StringTokenizer strtok  = null ;
      String          tempStr = "" ;

      /* Get the stackTrace */
      e.printStackTrace ( pw ) ;
      mle_OUTPUT.setText ( "Class: "+className+
                           "\nMethod: "+methodName+
                           "\n\n Exception:\n"+
                           "---------------------------------\n"+
                           e.toString()+
                           "\n\nStack:\n"+
                           "---------------------------------\n"+
                           sw.toString() ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will be called when we need
    *  to position and size our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       width     = size.width-(x*2) ;
      int       rowHeight = 21 ;

      super.doLayout() ;

      mle_ERROR.setBounds ( x, y, size.width-(x*2), rowHeight*3 ) ;
      y+=rowHeight*3 ;

      pb_SHOWDETAILS.setBounds ( x, y, 100, 18 ) ;
      y+=20 ;

      if ( showDetails ) {
         scr_OUTPUT.setVisible ( true ) ;
         scr_OUTPUT.setBounds ( x, y, width-x, size.height-y-60 ) ;
      }
      else
         scr_OUTPUT.setVisible ( false ) ;

      pb_OK.setBounds ( x, size.height-55, 70, 25 ) ;
      pb_COPYTOCLIP.setBounds ( x+75, size.height-55, 200, 25 ) ;
   }   


} /* ErrorDlg */
