package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   InfoBoxDlg                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: InfoBoxDlg                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This Dialog will show information to the user in a small popup
 * window. The information passed can be HTML but if it is DO NOT
 * include the <HTML><BODY> tags.
 *
 *
 * @version 1.0, 12/11/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   InfoBoxDlg                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class InfoBoxDlg extends JDialog implements KeyListener, ActionListener, WindowListener {

   /*----------------*/
   /*                */            
   /*  Static Fields */
   /*                */            
   /*----------------*/
   public static final int OK_HIT       = 0 ;
   public static final int CANCEL_HIT   = 1 ;
   public static final int YES_HIT      = 2 ;
   public static final int NO_HIT       = 3 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * This is the JLabel which holds our text
     **/
   private JLabel st_OUTPUT      = new JLabel ( "" ) ;

   /**
     * This is the Icon
     **/
   private JLabel st_ICON  = new JLabel ( ) ;

   /**
     * This is the Ok Button
     **/
   private JButton pb_OK = new JButton ( "Ok" ) ;


   /**
     * This is the Cancel Button
     **/
   private JButton pb_CANCEL = new JButton ( "Cancel" ) ;

   /**
     * This is the Yes Button
     **/
   private JButton pb_YES = new JButton ( "Yes" ) ;

   /**
     * This is the No Button
     **/
   private JButton pb_NO = new JButton ( "No" ) ;

   /**
     * This is the 'Never Show This Again Check Box
     **/
   private DCheckBox ck_NEVER_SHOW_AGAIN = new DCheckBox ( "Never show this message again" ) ;


   /**
     * This property holds what button was last pressed
     **/
   private int buttonPressed = -1 ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a default InfoBox
    *  with a description
    *  
    *
    *  @param parent The parent frame of this dialog
    *  @param title The Title of this InfoBox
    *  @param description The Description to show
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxDlg ( Frame parent, String title, String description ) {
      super ( parent, title, true ) ;
      initialize() ;
      setDescription ( description ) ;
   }


   public InfoBoxDlg ( Dialog parent, String title, String description ) {
      super ( parent, title, true ) ;
      initialize() ;
      setDescription ( description ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a default InfoBox
    *  with a description and an image
    *  
    *
    *  @param parent The parent frame of this dialog
    *  @param title The Title of this InfoBox
    *  @param description The Description to show
    *  @param image The image to show
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxDlg ( Frame parent, String title, String description, ImageIcon image ) {
      super ( parent, title, true ) ;
      initialize() ;
      setDescription ( description ) ;
      setImageIcon ( image ) ;
   }
   public InfoBoxDlg ( Dialog parent, String title, String description, ImageIcon image ) {
      super ( parent, title, true ) ;
      initialize() ;
      setDescription ( description ) ;
      setImageIcon ( image ) ;
   }



   /*------------------------------------------------------------*/
   /* wait
    */
   /**
    *    This method will wait until the user acts on the window
    *
    *  @param dispose TRUE = dispose the dialog once the user hits a button,
    *                 FALSE = Just hide the dialog
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void getResult ( boolean dispose ) {
      setSize ( 350, 190 ) ;
      ck_NEVER_SHOW_AGAIN.setSelected ( false ) ;
      setVisible ( true ) ;
      if ( dispose )
         dispose() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create and add all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {
      Container contentPane = getContentPane() ;

      contentPane.setLayout ( null ) ;
      contentPane.add ( st_ICON ) ;
      contentPane.add ( st_OUTPUT ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_CANCEL ) ;
      contentPane.add ( pb_YES ) ;
      contentPane.add ( pb_NO ) ;
      contentPane.add ( ck_NEVER_SHOW_AGAIN ) ;

      /*--------------------*/
      /*                    */        
      /*  Adjust properties */
      /*                    */        
      /*--------------------*/
      st_ICON.setVerticalAlignment ( SwingConstants.TOP ) ;
      st_OUTPUT.setVerticalAlignment ( SwingConstants.TOP ) ;

      //ck_NEVER_SHOW_AGAIN.setVisible ( false ) ;
      //pb_OK.setVisible ( false ) ;
      //pb_CANCEL.setVisible ( false ) ;
      //pb_YES.setVisible ( false ) ;
      //pb_NO.setVisible ( false ) ;


      /*----------------*/
      /*                */            
      /*  Add listeners */
      /*                */            
      /*----------------*/
      addWindowListener ( this ) ;
      addKeyListener ( this ) ;
      pb_OK.addKeyListener ( this ) ;
      pb_CANCEL.addKeyListener ( this ) ;
      pb_NO.addKeyListener ( this ) ;
      pb_YES.addKeyListener ( this ) ;
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_NO.addActionListener ( this ) ;
      pb_YES.addActionListener ( this ) ;

      setResizable ( false ) ;
   }





   /*------------------------------------------------------------*/
   /* setDescription
    */
   /**
    *    This method will set the description that
    *  will show
    *  
    *
    *  @param description The new description
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescription ( String description ) {
      st_OUTPUT.setText ( "<html><body><font size=2 face=\"Arial, Helvetica\">"+description+"</body></html>" ) ;
   }




   /*------------------------------------------------------------*/
   /* setImageIcon
    */
   /**
    *    This method will update the image that is
    *  being displayed. 
    *
    *  @param image The image to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setImageIcon ( ImageIcon image ) {
      st_ICON.setIcon ( image ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method is subclassed so that the window
    *  auto-sizes to encompass the entire InfoBoxPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size       = getSize() ;
      Dimension outputSize = null ;
      int       imageWidth = 0 ;

      try {
         imageWidth = st_ICON.getIcon().getIconWidth() ;
      } catch ( Exception e ) {
      }

      super.doLayout() ;

      st_OUTPUT.setSize ( size.width-20-imageWidth, 25 ) ;
      outputSize = st_OUTPUT.getMinimumSize() ;
      outputSize.height+=10 ;

      /*--------------------------------------------------------------*/
      /*                                                              */
      /*  If we are not large enough to show all of the text, set our */
      /* size and center the window, then call doLayout again         */
      /*                                                              */
      /*--------------------------------------------------------------*/
      if ( size.height < outputSize.height ) {
         setSize ( size.width, 
                   outputSize.height+
                   getInsets().top+
                   getInsets().bottom+
                   115 ) ;
         validate() ; // This will force another doLayout called
      }


      /*-----------------------------------*/
      /*                                   */
      /*  Otherwise, size as normally done */
      /*                                   */
      /*-----------------------------------*/
      else {
         int x = 5 ;
         int y = size.height-55 ;

         st_ICON.setBounds ( 5, 5, imageWidth, size.height-85 ) ;

         if ( ck_NEVER_SHOW_AGAIN.isVisible() ) {
            st_OUTPUT.setBounds ( 10+imageWidth, 5, size.width-imageWidth-20, size.height-85 ) ;
            ck_NEVER_SHOW_AGAIN.setBounds ( 5, size.height-80, size.width-10, 20 ) ;
         } else {
            st_OUTPUT.setBounds ( 10+imageWidth, 5, size.width-imageWidth-20, size.height-65 ) ;
         }


         if ( pb_OK.isVisible() ) {
            pb_OK.setBounds ( x, y, 75, 25 ) ;
            x+=80 ;
         }

         if ( pb_CANCEL.isVisible() ) {
            pb_CANCEL.setBounds ( x, y, 80, 25 ) ;
            x+=85 ;
         }


         if ( pb_YES.isVisible() ) {
            pb_YES.setBounds ( x, y, 75, 25 ) ;
            x+=80 ;
         }


         if ( pb_NO.isVisible() ) {
            pb_NO.setBounds ( x, y, 75, 25 ) ;
            x+=80 ;
         }

      }



      /*-----------------------*/
      /*                       */     
      /*  Ok, center ourselves */
      /*                       */     
      /*-----------------------*/
      if ( getParent().isVisible() ) {
         Component parent     = getParent() ;
         Dimension parentSize = parent.getSize() ;


         /*--------------------------------------------------------*/
         /*                                                        */
         /*  If the parent is smaller in width or height than this */
         /* window then we cannot center inside of it so we will   */
         /* just default to centering on the screen                */
         /*                                                        */
         /*--------------------------------------------------------*/
         if ( ( size.width > parentSize.width ) ||
              ( size.height > parentSize.height ) ) {
            WinUtil.centerWindow ( this ) ;
         } else {
            WinUtil.centerChildInParent ( this, parent ) ;
         }
      } else {
         WinUtil.centerWindow ( this ) ;
      }

   }


   /*------------------------------------------------------------*/
   /* setYes
    */
   /**
    *    This method will set the text that is displayed
    *  in the yes button
    *  
    *
    *  @param yesStr The Yes Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setYes ( String yesStr ) {
      pb_YES.setText ( yesStr ) ;
   }


   /*------------------------------------------------------------*/
   /* setCancel
    */
   /**
    *    This method will set the text that is displayed
    *  in the cancel button
    *  
    *
    *  @param cancelStr The Cancel Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCancel ( String cancelStr ) {
      pb_CANCEL.setText ( cancelStr ) ;
   }


   /**
     * This method will set the text that is displayed in the Ok button
     *
     * @param okStr The text to show
     **/
   public void setOk ( String okStr ) {
      pb_OK.setText ( okStr ) ;
   }

   /**
     * This method will set the text that is displayed in the No button
     *
     * @param noStr The text to show
     **/
   public void setNo ( String noStr ) {
      pb_NO.setText ( noStr ) ;
   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  we have decided to listen to is fired
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == pb_OK )
         buttonPressed = OK_HIT ;

      else if ( e.getSource() == pb_CANCEL )
         buttonPressed = CANCEL_HIT ;

      else if ( e.getSource() == pb_YES )
         buttonPressed = YES_HIT ;

      else if ( e.getSource() == pb_NO )
         buttonPressed = NO_HIT ;

      setVisible ( false ) ;
   }



   /**
     * This method will return the ID of the button that was last hit. This ID is
     * either YES_HIT, NO_HIT, CANCEL_HIT or OK_HIT
     **/
   public int getLastButtonHit () {
      return( buttonPressed ) ;
   }


   /**
     * This method will return the current isSelected() of the CHeck Box for the
     * 'Never Show This Again'.
     *
     * @return TRUE = The user selected it, FALSE = the user did not select it
     **/
   public boolean getNeverShowAgain () {
      return( ck_NEVER_SHOW_AGAIN.isSelected() ) ;
   }


   /**
     * This method will Show/Hide the OK button
     *
     * @param show TRUE = show the button, FALSE = hide the button
     **/
   public void showOk ( boolean show ) {
      pb_OK.setVisible ( show ) ;
   }

   /**
     * This method will Show/Hide the CANCEL button
     *
     * @param show TRUE = show the button, FALSE = hide the button
     **/
   public void showCancel ( boolean show ) {
      pb_CANCEL.setVisible ( show ) ;
   }



   /**
     * This method will Show/Hide the YES button
     *
     * @param show TRUE = show the button, FALSE = hide the button
     **/
   public void showYes ( boolean show ) {
      pb_YES.setVisible ( show ) ;
   }



   /**
     * This method will Show/Hide the NO button
     *
     * @param show TRUE = show the button, FALSE = hide the button
     **/
   public void showNo ( boolean show ) {
      pb_NO.setVisible ( show ) ;
   }



   /**
     * This method will Show/Hide the 'Never Show Again' Check Box
     *
     * @param show TRUE = show the button, FALSE = hide the button
     **/
   public void showNeverShowAgain ( boolean show ) {
      ck_NEVER_SHOW_AGAIN.setVisible ( show ) ;
   }


   public void windowClosing ( WindowEvent e ) {
   }
   public void windowClosed  ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e ) {
   }
   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowActivated ( WindowEvent e ) {
      setInitialFocus() ;
   }
   public void windowDeactivated ( WindowEvent e ) {
   }
   public void windowOpened ( WindowEvent e ) {
      setInitialFocus() ;
   }

   private void setInitialFocus() {
      if ( pb_OK.isVisible () )
         pb_OK.requestFocus() ;
   }


   public void keyPressed ( KeyEvent e ) {
      if ( e.getKeyText ( e.getKeyCode() ).equals ( "Enter" ) ) {
         if ( pb_OK.hasFocus() )
            buttonPressed = OK_HIT ;
         else if ( pb_YES.hasFocus() )
            buttonPressed = YES_HIT ;
         else if ( pb_CANCEL.hasFocus() )
            buttonPressed = CANCEL_HIT ;
         else if ( pb_NO.hasFocus() )
            buttonPressed = NO_HIT ;
         setVisible ( false ) ;
      } else if ( e.getKeyText ( e.getKeyCode() ).equals ( "Escape" ) ) {
         buttonPressed = CANCEL_HIT ;
         setVisible ( false ) ;
      }
   }
   public void keyReleased ( KeyEvent e ) {
   }

   public void keyTyped ( KeyEvent e ) {
   }
} /* InfoBoxDlg */

