package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   InfoBoxLayoutMgr                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: InfoBoxLayoutMgr                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.JLabel ;
import javax.swing.JButton ;
import javax.swing.JTextArea ;
import java.awt.* ;   

/**
 * This layout manager is used exclusivly for the InfoBoxPanel 
 *
 *
 * @version 1.0, 12/11/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   InfoBoxLayoutMgr                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class InfoBoxLayoutMgr extends Object implements LayoutManager {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Statics                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int MINHEIGHT = 155 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JButton   pb_OK         = null ;
   private JButton   pb_CANCEL     = null ;
   private JLabel    st_DESCRIPT   = null ;
   private JLabel    st_IMAGE      = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Dimension minimumSize = new Dimension ( 350, MINHEIGHT ) ; 


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our layout manager
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxLayoutMgr () {
   }




   /*------------------------------------------------------------*/
   /* addLayoutComponent
    */
   /**
    *    This method is subclassed to add components
    *  to our InfoBoxPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addLayoutComponent ( String name, Component comp ) throws NullPointerException {

      if ( name.equals ( "PB_OK" ) )
         pb_OK = (JButton) comp ;
      else if ( name.equals ( "PB_CANCEL" ) )
         pb_CANCEL = (JButton) comp ;
      else if ( name.equals ( "ST_DESCRIPT" ) )
         st_DESCRIPT = (JLabel) comp ;
      else if ( name.equals ( "ST_IMAGE" ) )
         st_IMAGE = (JLabel) comp ;
      else
         throw ( new NullPointerException() ) ;
   }




   /*------------------------------------------------------------*/
   /* removeLayoutComponent
    */
   /**
    *    This method will only be called when we
    *  need to remove an item
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeLayoutComponent ( Component comp ) {

      if ( comp == pb_OK )
         pb_OK = null ;
      if ( comp == st_DESCRIPT )
         st_DESCRIPT = null ;
      if ( comp == st_IMAGE )
         st_IMAGE = null ;
      if ( comp == pb_CANCEL )
         pb_CANCEL = null ;
   }


   /*------------------------------------------------------------*/
   /* layoutContainer
    */
   /**
    *    Sizes and positions our controls
    *  
    *  
    *
    *  @param parent The parent container
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void layoutContainer ( Container parent ) {
      Dimension size           = parent.getSize() ;
      int       top            = 10 ;
      int       descriptHeight = 21; //DALSystem.getEntryFieldHeight() ;

      /* Synchronize this in case someone takes
         off a component while sizing. This won't let em'! */
      //synchronized ( parent.getTreeLock() ) {

         if ( st_IMAGE != null )
            st_IMAGE.setBounds ( 5, 5, 75, size.height-30 ) ;

         if ( st_DESCRIPT != null ) {
            descriptHeight = st_DESCRIPT.getPreferredSize().height ;

            st_DESCRIPT.setBounds ( 75, top, size.width-100, descriptHeight ) ;
         }

         if ( ( pb_OK != null ) && ( pb_CANCEL != null ) && ( pb_CANCEL.isVisible() ) ) {
            pb_OK.setBounds     ( size.width-155, size.height-30, 70, 25 ) ;
            pb_CANCEL.setBounds ( size.width-80, size.height-30, 75, 25 ) ;
         } else if ( pb_OK != null )
            pb_OK.setBounds ( size.width-75, size.height-30, 70, 25 ) ;
      //} /* sync */

   }






   /*------------------------------------------------------------*/
   /* minimumLayoutSize
    */
   /**
    *    This method will return the minimum size
    *  that we can be.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension minimumLayoutSize ( Container parent ) {
      if ( st_DESCRIPT != null ) {

         if ( st_DESCRIPT != null ) {
            int minHeight = st_DESCRIPT.getPreferredSize().height+100 ;

            if ( minHeight < MINHEIGHT )
               minimumSize.height = MINHEIGHT ;
            else
               minimumSize.height = minHeight ;

         } else
            minimumSize.height = 150 ;
      }
      return( minimumSize ) ;
   }




   /*------------------------------------------------------------*/
   /* preferredLayoutSize
    */
   /**
    *    This method will return the preferred layout
    *  size. We will simply return minimumLayoutSize.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension preferredLayoutSize ( Container parent ) {
      return( minimumLayoutSize ( parent ) ) ;
   }




   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return a description of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( "InfoBoxLayoutMgr - David Lentz" ) ;
   }


} /* InfoBoxLayoutMgr */
