package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   JListDlg                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: JListDlg                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This simple window displays a vector of information in a JList. 
 * It will return the item(s) selected currently.
 *
 * @version 1.0, 3/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   JListDlg                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class JListDlg extends JDialog implements MouseListener, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private, Static Members                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] defaultTitle = {""};


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Members                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton      pb_OK       = null ;
   private DButton      pb_CANCEL   = null ;
   private JList        list    = null ;
   private JScrollPane  scrollPane  = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector    selVec      = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create, and initialize the window.
    *  It will return immediatly. To show the window please
    *  call getResult.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JListDlg ( Frame parent, String title, Vector dataList ) {
      super ( parent, title, true ) ;     
      initialize ( dataList ) ;
   }

   public JListDlg ( Dialog parent, String title, Vector dataList ) {
      super ( parent, title, true ) ;
      initialize ( dataList ) ;
   }

   public JListDlg ( JDialog parent, String title, Vector dataList ) {
      super ( parent, title, true ) ;
      initialize ( dataList ) ;
   }

   /**
     * This method will initialize our window and populate the list
     * with the items in the dataList vector
     */
   private void initialize ( Vector dataList ) {
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK       = new DButton ( "Ok" ) ;
      pb_CANCEL   = new DButton ( "Cancel" ) ;
      list    = new JList() ;
      scrollPane  = new JScrollPane ( list ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;  
      list.addMouseListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      Container contentPane = getContentPane() ;

      contentPane.setLayout ( null ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_CANCEL ) ;
      contentPane.add ( scrollPane ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.setPreferredSize ( new Dimension ( 25, 70 ) ) ;
      pb_CANCEL.setPreferredSize ( new Dimension ( 25, 80 ) ) ;
      pb_OK.setMinimumSize ( new Dimension ( 25, 70 ) ) ;
      pb_CANCEL.setMinimumSize ( new Dimension ( 25, 80 ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   RefreshData                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      list.setListData ( dataList ) ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our size and position                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 300, 300 ) ;
      WinUtil.centerWindow ( this ) ;
   }






   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the window and will
    *  wait until the user selects an item.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getResult () {
      setVisible ( true ) ;

      return( selVec ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to button's being hit
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_OK )
         okHit () ;
      super.dispose() ;
   }

   private void okHit () {
      Object[] values = list.getSelectedValues() ;

      if ( values != null ) {
         selVec = new Vector (  values.length ) ;
         for ( int i = 0; i < values.length; i++ )
            selVec.addElement ( values[i] ) ;
      }
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* setOkText
    */
   /**
    *    This method will set the text on the Ok Button
    *  
    *  
    *
    *  @param text The new text
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOkText ( String text ) {
      pb_OK.setText ( text ) ;
   }





   /*------------------------------------------------------------*/
   /* setCancelText
    */
   /**
    *    This method will set the text on the cancel
    *  button
    *  
    *
    *  @param text The text to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCancelText ( String text ) {
      pb_CANCEL.setText ( text ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will layout and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;
      scrollPane.setBounds ( 5, 5, size.width-15, size.height-80 ) ;
      pb_OK.setBounds ( 0, size.height-60, 70, 25 ) ;
      pb_CANCEL.setBounds ( 75, size.height-60, 80, 25 ) ;
   }


   public void mouseExited ( MouseEvent e ) {
   }
   public void mouseEntered ( MouseEvent e ) {
   }
   public void mouseReleased ( MouseEvent e ) {
   }
   public void mousePressed ( MouseEvent e ) {
   }
   public void mouseClicked ( MouseEvent e ) {
      if ( e.getClickCount() > 1 )
         okHit() ;
   }


   /**
     * This method will set the selection mode, please see the JList.setSelectionMode for more information
     **/
   public void setSelectionMode ( int mode ) {
      list.setSelectionMode ( mode ) ;
   }
} /* JListDlg */
