package com.ibm.nzna.shared.gui ;           
/*-----------------------------------------------------------*/
/*   JTextAreaCounter                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: JTextAreaCounter                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import javax.swing.text.* ;
import java.awt.* ;
import java.awt.event.* ;
/**
 * This is a simple counter which shows how much data is 
 * currently in the field and how much data the field allows.
 *
 * @version 1.0, 9/20/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   JTextAreaCounter                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class JTextAreaCounter extends DPanel implements KeyListener {

   /**
     *  This is the max number of chars allowed to be edited
     **/
   private int maxChars = 0 ;

   /**
     *  This is the Text Component which we are monitoring
     **/
   private JTextComponent textComponent = null ;

   /**
     *  This is the JLabel which shows the '0'
     **/
   private JLabel st_ZERO = new JLabel ( "0" ) ;

   /**
     *  This is the JLabel which shows the max
     **/
   private JLabel st_MAX = new JLabel ( ""+maxChars ) ;

   /**
     *  This is the progress bar
     **/
   private JProgressBar progressBar = new JProgressBar() ;



   /**
     *  This constructor takes a JTextComponent. If this
     * JTextComponent has a MaskDocument applied to it then
     * the max characters is derived from the MaskDocument.
     **/
   public JTextAreaCounter ( JTextComponent textComponent ) {
      this.textComponent = textComponent ;

      if ( textComponent.getDocument() instanceof MaskDocument ) {
         MaskDocument mask = (MaskDocument)textComponent.getDocument() ;

         maxChars = mask.getMaxTextLength() ;
      }

      initialize() ;
   }

   /**
     *  This constructor takes a JTextComponent and assigns
     * the max
     **/
   public JTextAreaCounter ( JTextComponent textComponent, int maxChars ) {
      this.textComponent = textComponent ;
      this.maxChars      = maxChars ;
      initialize() ;
   }



   /**
     *  This method will initialize our control
     **/
   private void initialize() {
      Font smallFont = new Font ( "Helv", Font.PLAIN, 10 ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_ZERO.setFont ( smallFont ) ;
      st_MAX.setFont  ( smallFont ) ;
      setOpaque ( false ) ;
      st_ZERO.setOpaque ( false ) ;
      st_MAX.setOpaque ( false ) ;
      progressBar.setOpaque ( false ) ;
      setBorder ( new javax.swing.border.LineBorder ( Color.black, 1 ) ) ;
      textComponent.addKeyListener ( this ) ;
      progressBar.setMaximum ( maxChars ) ;
      st_MAX.setText ( ""+maxChars ) ;
      progressBar.setValue ( textComponent.getText().length() ) ;  
      progressBar.setBorder ( null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( st_ZERO,     BorderLayout.WEST ) ;
      add ( progressBar, BorderLayout.CENTER ) ;
      add ( st_MAX,      BorderLayout.EAST ) ;
   }


   public void keyPressed ( KeyEvent e ) {
      keyTyped ( e ) ;
   }

   public void keyReleased ( KeyEvent e ) {
      keyTyped ( e ) ;
   }

   public void keyTyped ( KeyEvent e ) {
      int length = textComponent.getText().length() ;

      progressBar.setValue ( length ) ;
      st_ZERO.setText ( ""+length ) ;
      st_MAX.setText ( ""+maxChars ) ;
   }
} /* JTextAreaCounter */
