package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   ListLayoutMgr                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ListLayoutMgr                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;  
import java.util.* ;
/**
 * This Layout manager takes a list of component and orders
 * them one by one on top of each other to form a list.
 *
 *
 * @version 1.0, 12/14/1998 
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ListLayoutMgr                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class ListLayoutMgr extends Object implements LayoutManager {

   /**
     *  This is the padding (in pixels) between components.
     **/
   private int padding = 0 ;

   /**
     *  This method will add a component to the layout
     **/
   public void addLayoutComponent ( String name, Component comp ) {
   }

   /**
     *  This method will remove a component from the layout
     **/
   public void removeLayoutComponent ( Component comp ) {
   }


   /**
     *  This method will do the actual layout work
     **/
   public void layoutContainer ( Container parent ) {

      /* Synchronize this in case someone takes
         off a component while sizing. This won't let em'! */
      synchronized ( parent.getTreeLock() ) {
         int         i          = 0 ;
         Component[] compArray  = parent.getComponents() ;
         Dimension   size       = parent.getSize() ;
         Dimension   prefSize   = null ;
         Component   comp       = null ;
         int         y          = 0 ;

         while ( i < compArray.length ) {
            comp = (Component) compArray[i];

            if ( ( comp != null ) && ( comp.isVisible() ) ) {
               prefSize = comp.getMinimumSize() ;
               comp.setBounds ( 0, y, size.width, prefSize.height ) ;
               y+=prefSize.height+padding ;
            }
            i++ ;
         }

      } /* sync */

   }


   public Dimension minimumLayoutSize ( Container parent ) {
      Dimension   minimumSize = new Dimension ( parent.getSize().width, 0 ) ;
      int         i           = 0 ;
      Component[] compArray   = parent.getComponents() ;
      Dimension   size        = parent.getSize() ;
      Dimension   prefSize    = null ;
      Component   comp        = null ;

      while ( i < compArray.length ) {
         comp = (Component) compArray[i];

         if ( ( comp != null ) && ( comp.isVisible() ) ) {
            prefSize = comp.getMinimumSize() ;
            minimumSize.height+=(prefSize.height+padding) ;
         }
         i++ ;
      }

      return( minimumSize ) ;
   }




   /*------------------------------------------------------------*/
   /* preferredLayoutSize
    */
   /**
    *    This method will return the preferred layout
    *  size. We will simply return minimumLayoutSize.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension preferredLayoutSize ( Container parent ) {
      return( minimumLayoutSize ( parent ) ) ;
   }




   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return a description of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( "ListLayoutMgr - David Lentz" ) ;
   }






   /**
     *  This method will set the padding between components
     **/
   public void setPadding ( int padding ) {
      this.padding = padding ;
   }
} /* ListLayoutMgr */
