package com.ibm.nzna.shared.gui ;
/*-----------------------------------------------------------*/
/*   MultiListRowRenderer                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MultiListRowRenderer                */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This renderer implements ListCellRenderer to provide multiple columns 
 *
 *
 * @version 1.0, 5/5/1999 
 * @author David Lentz
 */
import java.awt.* ;
import java.awt.image.* ;
import javax.swing.* ;       
import com.ibm.nzna.shared.util.* ;
import java.awt.geom.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   MultiListRowRenderer                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class MultiListRowRenderer extends Component implements ListCellRenderer {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Color    selBackground = (Color)javax.swing.UIManager.getDefaults().get ( "List.selectionBackground" ) ;
   private static Color    selForeground = (Color)javax.swing.UIManager.getDefaults().get ( "List.selectionForeground" ) ;
   private static Font     selFont       = null ;
   private static Image    focusImage    = null ;
   private static FontUtil fontUtil      = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private MultiListRow          dataRow         = null ;
   private boolean               selected        = false ;
   private boolean               hasFocus        = false ;
   private int                   defaultHeight   = 18 ;
   private Dimension             prefSize        = new Dimension ( 300, defaultHeight ) ;
   private int                   columnNum       = 1 ;
   private MultiList             multiList       = null ;
   private ExtendedMultiListRow  xMultiListRow   = null ;
   private Rectangle             newClip         = new Rectangle ( 0, 0, 0, 0 ) ;
   private int[][]               colDim          = new int[columnNum][2] ;
   private boolean               paintBackground = true ;
   private Image                 backImage       = null ;
   private boolean               wordWrap        = false ;
   private boolean               useScreenClip   = true ;




   /*------------------------------------------------------------*/
   /* Constructor(1)
    */
   /**
    *    This constructor will create a default
    *  renderer
    *  
    *
    *  @param multiList The MultiList control that this renderer will draw in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MultiListRowRenderer ( MultiList multiList, FontUtil fontUtil ) {
      this.multiList = multiList ;
      this.fontUtil  = fontUtil ;
   }


   /*------------------------------------------------------------*/
   /* Constructor(2)
    */
   /**
    *    This constructor will create a default
    *  renderer
    *  
    *
    *  @param xMultiList The ExtendedMultiList control that this renderer will draw in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MultiListRowRenderer ( ExtendedMultiListRow xMultiListRow, FontUtil fontUtil ) {
      this.xMultiListRow = xMultiListRow ;
      this.fontUtil      = fontUtil ;
   }



   /*------------------------------------------------------------*/
   /* getListCellRendererComponent
    */
   /**
    *    Simply returns an instance of this
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Component getListCellRendererComponent ( JList list, Object value, int index, boolean isSelected, boolean cellHasFocus ) {

      if ( value instanceof ExtendedMultiListRow ) {
         dataRow = null ;
         xMultiListRow = (ExtendedMultiListRow) value ;
      } else if ( value instanceof MultiListRow ) {
         dataRow = (MultiListRow) value ;
         xMultiListRow = null ;
      }

      if ( ((index/2)*2) != index )
         setBackground ( multiList.getOddLinesColor() ) ;
      else
         setBackground ( multiList.getBackground() ) ;

      this.selected = isSelected ;
      this.hasFocus = cellHasFocus ;

      return(this);
   }






   /*------------------------------------------------------------*/
   /* setWordWrap
    */
   /**
    *    This method will turn word wrapping on or
    *  off
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setWordWrap ( boolean wordWrap ) {
      this.wordWrap = wordWrap ;
   }


   /*------------------------------------------------------------*/
   /* paint
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param g The graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paint ( Graphics g ) {
      Shape       shapeClip = g.getClip() ;
      Rectangle   clip      = null ;

      if ( prefSize.height == -1 )
         getPreferredSize() ;

      if ( shapeClip == null )
         clip = new Rectangle ( 0, 0, prefSize.width, prefSize.height ) ;
      else
         clip = (Rectangle) shapeClip ;
      newClip.x      = clip.x ;
      newClip.y      = 0 ;
      newClip.width  = prefSize.width ; //colDim[columnNum-1][0] ;
      newClip.height = clip.height ;

      /* First, paint our background  */
      if ( !selected ) {
         g.setColor ( getBackground() ) ;
         if ( backImage != null )
            WinUtil.wallPaper ( newClip, this, g, backImage ) ;
         else
            g.fillRect ( clip.x, clip.y, clip.width, clip.height ) ;

      } else {
         if ( selFont == null )
            selFont = g.getFont() ;
         g.setFont ( selFont ) ;
         g.setColor ( selBackground ) ;
         g.fillRect ( clip.x, clip.y, clip.width, clip.height ) ;
      }

      if ( ( dataRow != null ) || ( xMultiListRow != null ) ) {

         Object object    = null ;
         int    i         = 0 ;
         int    pointX    = 0 ;
         int    clipWidth = clip.width-clip.x ;

         if ( !selected )
            g.setColor ( getForeground() ) ;
         else {
            g.setColor ( selForeground ) ;

         }
         if ( columnNum > 0 ) {
            while ( i < columnNum ) {
               if ( ( colDim[i][1] == 0 ) || ( colDim[i][0] == 0 ) ) {
                  colDim[i][0] = multiList.getColumnX(i) ;
                  colDim[i][1] = multiList.getColumnWidth(i) ;
               }

               if ( clip.x < (colDim[i][0]+colDim[i][1]) ) {

                  if ( clip.width > (multiList.getColumnX(i)+colDim[i][1] ) )
                     g.setClip ( clip.x, clip.y, multiList.getColumnX(i)+colDim[i][1], clip.height ) ;

                  if ( xMultiListRow != null ) {
                     paintObject ( g, xMultiListRow, xMultiListRow.getColumnData(i), colDim[i][0], 10, colDim[i][1], prefSize.height, clip, i ) ;

                  } else {
                     paintObject ( g, null,dataRow.getColumnData(i), colDim[i][0], 10, colDim[i][1], prefSize.height, clip, i ) ;

                  }
                  g.setClip ( clip ) ;          
               }
               i++ ;
            }
         } else {
            if ( xMultiListRow != null )
               paintObject ( g, xMultiListRow, xMultiListRow.getColumnData(0), 0, 10, 1800, prefSize.height, clip, i ) ;
            else
               paintObject ( g, null, dataRow.getColumnData(0), 0, 10, 1800, prefSize.height, clip, i ) ;
         }

         if ( hasFocus ) {
            if ( wordWrap ) {
               if ( focusImage != null )
                  focusImage.flush() ;
               createFocusImage() ;
            } else {
               if ( focusImage == null )
                  createFocusImage() ;
            }

            if ( focusImage != null )
               g.drawImage ( focusImage, 0, 0, this ) ;

         }
      }
   }
   /*------------------------------------------------------------*/
   /* print
    */
   /**
    *    This method will print our component
    *  
    *  
    *
    *  @param g The graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void print ( Graphics g ) {
      Shape       shapeClip = new Rectangle ( 1024, 768 ) ;
      Rectangle   clip      = new Rectangle ( 1024, 768 ) ;

      if ( prefSize.height == -1 )
         getPreferredSize() ;

      if ( shapeClip == null )
         clip = new Rectangle ( 0, 0, prefSize.width, prefSize.height ) ;
      else
         clip = new Rectangle ( 1024, 768 ) ;
      
      newClip.x      = clip.x ;
      newClip.y      = 0 ;
      newClip.width  = prefSize.width ; //colDim[columnNum-1][0] ;
      newClip.height = clip.height ;

      /* First, paint our background  */
      if ( !selected ) {
         g.setColor ( Color.white ) ;
         if ( backImage != null )
            WinUtil.wallPaper ( newClip, this, g, backImage ) ;
         else
            g.fillRect ( clip.x, clip.y, clip.width, clip.height ) ;

      } else {
         if ( selFont == null )
            selFont = g.getFont() ;
         g.setFont ( selFont ) ;
         g.setColor ( Color.white ) ;
         g.fillRect ( clip.x, clip.y, clip.width, clip.height ) ;
      }

      if ( ( dataRow != null ) || ( xMultiListRow != null ) ) {

         Object object    = null ;
         int    i         = 0 ;
         int    pointX    = 0 ;
         int    clipWidth = clip.width-clip.x ;

         if ( !selected )
            g.setColor ( getForeground() ) ;
         else {
            g.setColor ( selForeground ) ;

         }
         if ( columnNum > 0 ) {
            while ( i < columnNum ) {
               if ( ( colDim[i][1] == 0 ) || ( colDim[i][0] == 0 ) ) {
                  colDim[i][0] = multiList.getColumnX(i) ;
                  colDim[i][1] = multiList.getColumnWidth(i) ;
               }

               if ( clip.x < (colDim[i][0]+colDim[i][1]) ) {

                  if ( clip.width > (multiList.getColumnX(i)+colDim[i][1] ) )
                     g.setClip ( clip.x, clip.y, multiList.getColumnX(i)+colDim[i][1], clip.height ) ;

                  if ( xMultiListRow != null ) {
                     printObject ( g, xMultiListRow, xMultiListRow.getColumnData(i), colDim[i][0], 10, colDim[i][1], prefSize.height, clip, i ) ;

                  } else {
                     printObject ( g, null,dataRow.getColumnData(i), colDim[i][0], 10, colDim[i][1], prefSize.height, clip, i ) ;

                  }
                  g.setClip ( clip ) ;          
               }
               i++ ;
            }
         } else {
            if ( xMultiListRow != null )
               printObject ( g, xMultiListRow, xMultiListRow.getColumnData(0), 0, 10, 1800, prefSize.height, clip, i ) ;
            else
               printObject ( g, null, dataRow.getColumnData(0), 0, 10, 1800, prefSize.height, clip, i ) ;
         }

         if ( hasFocus ) {
            if ( wordWrap ) {
               if ( focusImage != null )
                  focusImage.flush() ;
               createFocusImage() ;
            } else {
               if ( focusImage == null )
                  createFocusImage() ;
            }

            if ( focusImage != null )
               g.drawImage ( focusImage, 0, 0, this ) ;

         }
      }
   }
   
   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred size
    *  of this row.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {

      prefSize.height = defaultHeight ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to word wrap, we must define our height here  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( wordWrap ) {
         if ( dataRow != null )
            prefSize.height = getPreferredHeightFromMultiListRow ( (MultiListRow) dataRow ) ;
         else if ( xMultiListRow != null )
            prefSize.height = getPreferredHeightFromMultiListRow ( (MultiListRow) xMultiListRow ) ;
      }

      return( prefSize ) ;
   }


   /*------------------------------------------------------------*/
   /* setRowHeight
    */
   /**
    *    This method will set the height that this
    *  row will take up
    *  
    *
    *  @param height The new height
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRowHeight ( int height ) {
      prefSize.height = height ;
      if ( focusImage != null )
         focusImage.flush() ;
      focusImage = null ;
   }


   /*------------------------------------------------------------*/
   /* setRowWidth
    */
   /**
    *    This method will set the width that this
    *  row will take up
    *  
    *
    *  @param width The new width
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRowWidth ( int width ) {
      prefSize.width = width ;
      if ( focusImage != null )
         focusImage.flush() ;
      focusImage = null ;
   }





   /*------------------------------------------------------------*/
   /* setColumnNum
    */
   /**
    *    This method will set the number of
    *  columns that this item will try to display
    *  
    *
    *  @param columnNum The new number of columns
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnNum ( int columnNum ) {
      int i = 0 ;

      this.columnNum = columnNum ;

      colDim = new int[columnNum+1][2] ;
   }




   /*------------------------------------------------------------*/
   /* paintObject
    */
   /**
    *    This method will paint any object that the
    *  MultiListRow.getColumnData will return
    *  
    *
    *  @param g The Graphics Context
    *  @param object The object to paint
    *  @param x The x position to draw at
    *  @param y The y position to draw at
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void paintObject ( Graphics g, ExtendedMultiListRow xMultiListRow, Object object, int x, int y, int width, int height, Rectangle clipRect, int columnNum ) {

      if ( object != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw a String                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof String ) {
            if ( fontUtil.getFont() != g.getFont() )
               fontUtil.setFont ( g.getFont() ) ;

            if ( ( !selected ) && ( xMultiListRow != null ) )
               xMultiListRow.adjustColumnGraphics ( columnNum, g ) ;

            fontUtil.drawCharsEtc ( g, 
                                    ((String)object).toCharArray(), 
                                    ((String)object).length(), 
                                    x, 
                                    ( (wordWrap) && ( prefSize.height != defaultHeight) ) ? y+4 : fontUtil.getYCentered(height), 
                                    new Rectangle ( 1024, 768 ), 
                                    width ) ;

         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw an Image                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof Image )
            g.drawImage ( (Image)object, x, 0, this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw an ImageIcon                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof ImageIcon )
            g.drawImage ( ((ImageIcon)object).getImage(), x, 0, this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw an array of Strings                                */
         /*                                                           */
         /*-----------------------------------------------------------*/

         else if ( object instanceof String[] ) {

            String[]    stringArray      = (String[]) object ;
            int         i                = 0 ;
            int         tempStringLength = 0 ;
            int         tempY            = y+4;
            Shape       clip             = g.getClip();

            
            if ( xMultiListRow != null )
               xMultiListRow.adjustColumnGraphics ( columnNum, g ) ;

            for ( i = 0; i < stringArray.length; i++ ) {
               tempStringLength = stringArray[i].length() ;

               if ( tempStringLength > 0 ) {
                  /* Set up the font util for our font  */
                  if ( fontUtil.getFont() != g.getFont() )
                     fontUtil.setFont ( g.getFont() ) ;

                  fontUtil.drawCharsEtc ( g, 
                                          stringArray[i].toCharArray(), 
                                          tempStringLength, 
                                          x, 
                                          tempY, 
                                          (Rectangle)clip, 
                                          width ) ;
                  tempY+=defaultHeight ;

               }
            } 
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw Everything Else                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else
            fontUtil.drawCharsEtc ( g, 
                                    object.toString().toCharArray(), 
                                    object.toString().length(), 
                                    x, 
                                    fontUtil.getYCentered(height), 
                                    new Rectangle ( 1024, 768 ), 
                                    width ) ;
      }
   }
   /*------------------------------------------------------------*/
   /* printObject
    */
   /**
    *    This method will paint any object that the
    *  MultiListRow.getColumnData will return
    *  
    *
    *  @param g The Graphics Context
    *  @param object The object to paint
    *  @param x The x position to draw at
    *  @param y The y position to draw at
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void printObject ( Graphics g, ExtendedMultiListRow xMultiListRow, Object object, int x, int y, int width, int height, Rectangle clipRect, int columnNum ) {

      if ( object != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw a String                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof String ) {
            if ( fontUtil.getFont() != g.getFont() )
               fontUtil.setFont ( g.getFont() ) ;

            if ( xMultiListRow != null )
               xMultiListRow.adjustColumnGraphics ( columnNum, g ) ;

            fontUtil.drawCharsEtc ( g, 
                                    ((String)object).toCharArray(), 
                                    ((String)object).length(), 
                                    x, 
                                    ( (wordWrap) && ( prefSize.height != defaultHeight) ) ? y+4 : fontUtil.getYCentered(height), 
                                    new Rectangle ( 1024, 768 ), 
                                    width ) ;

         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw an Image                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof Image )
            g.drawImage ( (Image)object, x, 0, this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw an ImageIcon                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof ImageIcon )
            g.drawImage ( ((ImageIcon)object).getImage(), x, 0, this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw an array of Strings                                */
         /*                                                           */
         /*-----------------------------------------------------------*/

         else if ( object instanceof String[] ) {

            String[]    stringArray      = (String[]) object ;
            int         i                = 0 ;
            int         tempStringLength = 0 ;
            int         tempY            = y+4;
            Shape       clip             = new Rectangle ( 1024, 768 ) ;

           
            if ( xMultiListRow != null )
               xMultiListRow.adjustColumnGraphics ( columnNum, g ) ;

            for ( i = 0; i < stringArray.length; i++ ) {
               tempStringLength = stringArray[i].length() ;

               if ( tempStringLength > 0 ) {
                  /* Set up the font util for our font  */
                  if ( fontUtil.getFont() != g.getFont() )
                     fontUtil.setFont ( g.getFont() ) ;

                  fontUtil.drawCharsEtc ( g, 
                                          stringArray[i].toCharArray(), 
                                          tempStringLength, 
                                          x, 
                                          tempY, 
                                          new Rectangle ( 1024, 768 ), 
                                          width ) ;
                  tempY+=defaultHeight ;

               }
            } 
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw Everything Else                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else
            fontUtil.drawCharsEtc ( g, 
                                    object.toString().toCharArray(), 
                                    object.toString().length(), 
                                    x, 
                                    fontUtil.getYCentered(height), 
                                    new Rectangle ( 1024, 768 ), 
                                    width ) ;
      }
   }

   

   /*------------------------------------------------------------*/
   /* setSelectedBackground
    */
   /**
    *    Sets the rows selected background color
    *  
    *  
    *
    *  @param selectedBackground The new Background Selected Color
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelectedBackground ( Color selectedBackground ) {
      this.selBackground = selectedBackground ;
   }




   /*------------------------------------------------------------*/
   /* setSelectedForeground
    */
   /**
    *    Sets the rows selected foreground color
    *  
    *  
    *
    *  @param selectedForeground The new Foreground Selected Color
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelectedForeground ( Color selectedForeground ) {
      this.selForeground = selectedForeground ;
   }




   /*------------------------------------------------------------*/
   /* setSelectedFont
    */
   /**
    *    This method will set our selected font
    *  
    *  
    *
    *  @param selectedFont The font to display if selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelectedFont ( Font selectedFont ) {
      this.selFont = selectedFont ;
   }




   /*------------------------------------------------------------*/
   /* createFocusImage
    */
   /**
    *    This method will re-create the focus image
    *  of dotted yellow lines around the row
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createFocusImage ( ) {
      if ( prefSize.height >= 0 ) {

         if ( wordWrap )
            prefSize.height = getPreferredHeightFromMultiListRow ( ( xMultiListRow != null ) ? xMultiListRow : dataRow ) ;

         int               i         = 0 ;
         MemoryImageSource imgSource = null ;
         int[]             imgBytes  = new int [ prefSize.width*prefSize.height ] ;
         int               black     = 0xff000000 ;
         int               yellow    = 0xffffff9c ;
         int               color     = yellow ;

         /* Draw the horizontal lines */
         while ( i < prefSize.width ) {
            imgBytes[i]=color ;
            imgBytes[i+(prefSize.width*(prefSize.height-1))]=color ;
            i+=2 ;
         }

         i=0 ;
         while ( i < (prefSize.height*(prefSize.width-1)) ) {
            imgBytes[i]=color ;
            imgBytes[i+prefSize.width-1]=color ;
            i+=prefSize.width*2 ;
         }

         imgSource = new MemoryImageSource ( prefSize.width, prefSize.height, imgBytes, 0, prefSize.width ) ;

         focusImage = this.createImage ( imgSource ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* setBackground
    */
   /**
    *    This method will set the background of the
    *  row to the specified image
    *  
    *
    *  @param image The image to tile
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBackground ( Image image ) {
      this.backImage = image ;
      paintBackground = false ;
   }




   /*------------------------------------------------------------*/
   /* getPreferredHeightFromMultiListRow
    */
   /**
    *    Thie method is used to get the preferred
    *  height needed to display a MultiListRow Fully
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The height in pixels
    *//*---------------------------------------------------------*/
   protected int getPreferredHeightFromMultiListRow ( MultiListRow row ) {
      int i         = 0 ;
      int retHeight = 0 ;

      if ( colDim.length > 0 ) {
         int    columnNum = colDim.length ;
         Object data      = null ;

         while ( i < columnNum ) {

            data = row.getColumnData(i) ;

            if ( data instanceof String[] ) {
               if ( retHeight < ((String[])data).length*defaultHeight )
                  retHeight = ((String[])data).length*defaultHeight ;
            }

            else if ( data instanceof Image ) {
               if ( retHeight < ((Image)data).getHeight ( this ) )
                  retHeight = ((Image)data).getHeight ( this ) ;
            }

            else {
               if ( retHeight < defaultHeight )
                  retHeight = defaultHeight ;
            }

            i++ ;
         } /* while */
      } /* if */

      return( retHeight ) ;
   } /* getPreferredHeightFromMultiListRow */


   public void setUseScreenClip ( boolean useScreenClip ) {
      this.useScreenClip = useScreenClip ;
   }

} /* MultiListRowRenderer */
