package com.ibm.nzna.shared.gui ;    
/*-----------------------------------------------------------*/
/*   ViewTreeNode                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewTreeNode                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.tree.* ;
/**
 * The View Tree Node is specifically used for displaying incremental
 * data in a tree view. This is data that is not loaded until the tree
 * item is expanded. This node contains a 'Name' that is displayed in the
 * tree, and a method called 'toSQL' that should be subclassed to return
 * the SQL needed to refresh the items under this Node. If this Node is a
 * parent node with no children, the toSQL method should return NULL.
 *
 * @version 1.0, 3/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewTreeNode                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewTreeNode extends MutableTreeWrapper {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This is the name which is draw on the JTree
     **/
   private String name     = null ;

   /**
     *  This is the SQL that can be used to figure out what needs to be refreshed
     **/
   private String sql      = null ;

   /**
     *  This is the level. The level is dependant on how you use it. IT can be used
     * to figure out what drill down level something is etc..
     **/
   private String viewLevel    = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the object but no
    *  initialization will occur
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ViewTreeNode () {
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default and only constructor
    *  for the ViewTreeNode
    *  
    *
    *  @param name The name of the view
    *  @param sql The SQL that is needed to refresh this view. NULL if this view has no data to refresh
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ViewTreeNode ( String name, String sql ) {
      setName ( name ) ;
      setSQL ( sql ) ;
   }





   /*------------------------------------------------------------*/
   /* toSQL
    */
   /**
    *    This method will return the SQL needed to
    *  refresh this View's data. If this method returns
    *  NULL, than this node should not be refreshed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String toSQL () {
      return( sql ) ;
   }




   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return the 'name' of the
    *  View
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String toString () {
      return( name ) ;
   }





   /*------------------------------------------------------------*/
   /* setName
    */
   /**
    *    This method will set the name of this view
    *  
    *  
    *
    *  @param name The name to display to the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setName ( String name ) {
      this.name = name ;
   }





   /*------------------------------------------------------------*/
   /* setSQL
    */
   /**
    *    This method will set the SQL needed to
    *  refresh this view
    *  
    *
    *  @param sql The SQL
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSQL ( String sql ) {
      this.sql = sql ;
   }


   public String getViewLevel () {
      return ( viewLevel ) ;
   }

   public void setViewLevel ( String viewLevel ) {
      this.viewLevel = viewLevel ;
   }
} /* ViewTreeNode */
