package com.ibm.nzna.shared.gui.form;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      RadioFormComponent
/*   Creation Date: APRIL 19, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This component will take a Vector of objects and create a JRadioButton
 * for each element in the Vector. The text for each radio button is retrieved
 * from each Object's toString() method. Only 1 selection is possible since
 * the JRadioButtons will be grouped together.
 *
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* FormComponent
/*
/*----------------------------------------------------*/
public class RadioFormComponent extends FormComponent implements ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * A reference to the original list of elements
     **/
   private Vector elements   = null ;

   /**
     * This panel holds all of the JRadioButtons
     **/
   private ListPanel radioPanel = null ;

   /**
     * The currently selected Object
     **/
   private Object currentSelectedItem = null ;

   /**
     *  This is the list of ActionListners
     **/
   private Vector actionListeners = new Vector () ;

   /**
     *  This hash table holds the association of the Radio Button
     * and the Object
     **/
   private Hashtable radioHash = null ;

   /**
     *  This hash table holds the associations of the Objects to the
     * Radio Buttons
     **/
   private Hashtable objectHash = null ;

   /**
     *  This is the RadioButtonGroup that makes sure only 1 button
     * is selected at a time
     **/
   private ButtonGroup group = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     * This Constructor creates the RadioFormComponent and all JRadioButtons
     *
     **/
   public RadioFormComponent ( String title, Vector elements ) {
      super ( title ) ;

      setList ( elements ) ;
   }

   /**
     * This method will erase all current JRadioButtons and will add a
     * new JRadioButton for each objcet in the Vector
     **/
   public void setList ( Vector elements ) {
      JRadioButton radioButton = null ;
      int i = 0 ;

      removeAllButtons() ;
      this.elements = elements ;
      radioHash = new Hashtable () ;
      objectHash = new Hashtable () ;
      group = new ButtonGroup () ;

      if ( elements != null ) {
         int size = elements.size() ;

         while ( i < size ) {
            radioButton = new JRadioButton ( elements.elementAt(i).toString() ) ;
            radioButton.setOpaque ( false ) ;
            radioButton.addActionListener ( this ) ;
            group.add ( radioButton ) ;
            radioHash.put ( elements.elementAt(i), radioButton ) ;
            objectHash.put ( radioButton, elements.elementAt(i) ) ;
            radioPanel.add ( radioButton ) ;
            i++ ;
         }
      }
   }


   /**
     * This method will remove all JRadioButtons from this panel
     **/
   private void removeAllButtons () {
      if ( radioPanel != null )
         remove ( radioPanel ) ;

      radioPanel = new ListPanel () ;
      radioPanel.setOpaque ( false ) ;
      setFormComponent ( radioPanel ) ;
   }


   /**
     * This method will return the Object that is selected in this
     * list of JRadioButtons. NULL if nothing is selected
     **/
   public Object getSelectedItem () {
      return( currentSelectedItem ) ;
   }


   /**
     * This method will set what item is currently selected.
     **/
   public void setSelectedItem ( Object object ) {
      JRadioButton button = (JRadioButton)radioHash.get ( object ) ;

      if ( button != null ) {
         currentSelectedItem = object ;
         button.setSelected ( true ) ;
      }
   }


   /**
     *  This method will be called whenever a radio button is
     * selected.
     **/
   public void actionPerformed ( ActionEvent e  ){
      JRadioButton button = (JRadioButton)e.getSource() ;

      currentSelectedItem = objectHash.get ( button ) ;

      if ( actionListeners.size() > 0 ) {
         int i     = 0 ;
         int size  = actionListeners.size() ;
         ActionEvent newEvent = new ActionEvent ( this, 0, "" ) ;

         while ( i < size ) {
            ((ActionListener)actionListeners.elementAt(i)).actionPerformed ( newEvent ) ;
            i++ ;
         }
      }
   }


   /**
     *  This method will add an action listener that will be notified
     * if any radio buttons are selected
     **/
   public void addActionListener ( ActionListener listener ) {
      actionListeners.addElement ( listener ) ;
   }

   public void removeActionListener ( ActionListener listener ) {
      actionListeners.removeElement ( listener ) ;
   }

   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;
      Enumeration enum = radioHash.elements() ;

      while ( enum.hasMoreElements() ) {
         ((JComponent)enum.nextElement()).setEnabled ( enable );
      }

   }
}
/* Class1 */
