package com.ibm.nzna.shared.gui.wizard ;
/*-----------------------------------------------------------*/
/*   InfoStep                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: InfoStep                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This is the InfoStep that can be used to simply display text
 * to the user. This step automatically returns TRUE in the saveInfo method.
 *
 *
 * @version 1.0, 12/21/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   InfoStep                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class InfoStep extends WizardStep {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel   st_MESSAGE  = null ;
   private int      id          = 0 ;
   private String   title       = "" ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our InfoStep with only a message
    * string.
    *  
    *  
    *
    *  @param message The Message to display to the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoStep ( String message ) {
      initialize() ;
      setMessage ( message ) ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our InfoStep with the
    *  message and title given
    *  
    *
    *  @param message The message to display to the user
    *  @param title The title to show in the wizard
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoStep ( String message, String title ) {
      initialize() ;
      setMessage ( message ) ;
      setTitle ( title ) ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the InfoStep with
    *  a message to show and an id to return from
    *  the getId method
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoStep ( int id, String message ) {
      initialize() ;
      setMessage ( message ) ;
      this.id = id ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize the InfoStep
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      setLayout ( new CardLayout() ) ;
      add ( st_MESSAGE = new JLabel (""), "0" ) ;
      st_MESSAGE.setVerticalAlignment ( SwingConstants.TOP ) ;
   }

   /*------------------------------------------------------------*/
   /* setMessage
    */
   /**
    *    This method will set the message that is
    *  displayed. To call this method you should use the
    *  setWelcomeMessage() method in the Wizard class.
    *  
    *
    *  @param message The new Message to show.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMessage ( String message ) {
      st_MESSAGE.setText ( message ) ;
   }  

   public boolean saveInfo () {
      return( true ) ;
   }

   public void close () {}


   /*------------------------------------------------------------*/
   /* getId
    */
   /**
    *    This method will return the id of this InfoStep.
    *  A 0 will return if the Id has not yet been set
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getId () {
      return( id ) ;
   }




   /*------------------------------------------------------------*/
   /* setId
    */
   /**
    *    This method will set the Id of this InfoStep
    *  
    *  
    *
    *  @param Id The integer id for this InfoStep
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setId ( int id ) {
      this.id = id ;
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( title ) ;
   }




   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set the title that is  displayed
    *  in the wizard for this step
    *  
    *
    *  @param title The title to set to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitle ( String title ) {
      this.title = title ;
   }

} /* InfoStep */
