package com.ibm.nzna.shared.gui.wizard ;
/*-----------------------------------------------------------*/
/*   WizardEvent                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WizardEvent                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.EventObject ;
/**
 * The WizardEvent is used to communicate through listeners. For
 * any event sent to the WizardListener this will be the only
 * parameter. The source of this event will be a Wizard.
 *
 *
 * @version 1.0, 12/22/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WizardEvent                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class WizardEvent extends EventObject {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private WizardStep wizardStep        = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a new WizardEvent with 
    *  the initial values passed
    *  
    *
    *  @param wizardStep The WizardStep that is showing
    *  @param wizard The Wizard that this WizardEvent is for
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WizardEvent ( WizardStep wizardStep, Wizard wizard ) {
      super ( wizard ) ;
      this.wizardStep = wizardStep ;
   }






   /*------------------------------------------------------------*/
   /* getStepId
    */
   /**
    *    This method will return the 0 based step index
    *  of the WizardStep that is showing
    *  
    *
    *
    *  @return int The Id of the currently showing WizardStep
    *//*---------------------------------------------------------*/
   public int getStepId() {
      return( wizardStep.getId() ) ;
   }





   /*------------------------------------------------------------*/
   /* getWizardStep
    */
   /**
    *    This method will return an instance of the
    *  WizardStep that is currently showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WizardStep getWizardStep () {
      return( wizardStep ) ;
   }





   /*------------------------------------------------------------*/
   /* getWizard
    */
   /**
    *    This method will return the Wizard from which
    *  this event originated
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The instance of the Wizard
    *//*---------------------------------------------------------*/
   public Wizard getWizard () {
      return( (Wizard)getSource() ) ;
   }




} /* WizardEvent */
