package com.ibm.nzna.shared.io ;
import java.util.* ;
import java.io.* ;

   /*********************************************************
   * This class will fully load the data from an input stream
   * into a byte array buffer.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/5/100  2:09pm
   * @author 
   * @version
   *********************************************************/
public class OutputStreamCapture extends OutputStream {
   private byte byteBuffer[] = null ;
   private int  position = 0 ;
   /*********************************************************
   * Constructs a new OutputStreamCapture with the initial size
   * specified.
   * 
   *
   * Author:  
   * Created: 7/20/100  5:44pm
   * @param
   * @return
   *********************************************************/
   public OutputStreamCapture ( int bufferSize ) {
      byteBuffer = new byte[ bufferSize ] ;
   }

   /*********************************************************
   * Get the byte array from the capture.
   * 
   *
   * Author:  
   * Created: 7/20/100  5:50pm
   * @param
   * @return
   *********************************************************/
   public byte[] getBytes() {
      /* Trim the return buffer to the size of the data */
      byte returnBuffer[] = new byte[ position ] ;
      System.arraycopy( byteBuffer, 0, returnBuffer, 0, position ) ;

      return ( returnBuffer ) ;  
   }

   /**
    * Writes the specified byte to this output stream. 
    * <p>
    * Subclasses of <code>OutputStream</code> must provide an 
    * implementation for this method. 
    *
    * @param      b   the <code>byte</code>.
    * @exception  IOException  if an I/O error occurs.
    * @since      JDK1.0
    */
   public void write(int b) throws IOException {
      /* ensure the capacity of the buffer */
      if ( position >= byteBuffer.length ) {
         byte newBuffer[] = new byte[ byteBuffer.length * 2 ] ;
         System.arraycopy( byteBuffer, 0, newBuffer, 0, byteBuffer.length ) ;
         byteBuffer = newBuffer ;
      }
      byteBuffer[ position++ ] = (byte) b ;
   }

   /**
    * Writes <code>b.length</code> bytes from the specified byte array 
    * to this output stream. 
    * <p>
    * The <code>write</code> method of <code>OutputStream</code> calls 
    * the <code>write</code> method of three arguments with the three 
    * arguments <code>b</code>, <code>0</code>, and 
    * <code>b.length</code>. 
    *
    * @param      b   the data.
    * @exception  IOException  if an I/O error occurs.
    * @see        java.io.OutputStream#write(byte[], int, int)
    * @since      JDK1.0
    */
   public void write(byte b[]) throws IOException {
      write(b, 0, b.length);
   }

   /**
    * Writes <code>len</code> bytes from the specified byte array 
    * starting at offset <code>off</code> to this output stream. 
    * <p>
    * The <code>write</code> method of <code>OutputStream</code> calls 
    * the write method of one argument on each of the bytes to be 
    * written out. Subclasses are encouraged to override this method and 
    * provide a more efficient implementation. 
    *
    * @param      b     the data.
    * @param      off   the start offset in the data.
    * @param      len   the number of bytes to write.
    * @exception  IOException  if an I/O error occurs.
    * @since      JDK1.0
    */
   public void write(byte b[], int off, int len) throws IOException {
      for (int i = 0 ; i < len ; i++) {
         write(b[off + i]);
      }
   }
}
