package com.ibm.nzna.shared.mail ;
/*-----------------------------------------------------------*/
/*   Mail                                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Mail                                */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.*;
import java.net.InetAddress;
import java.util.Properties;
import java.util.Date;
import javax.mail.*;
import javax.mail.internet.*;
/**
 *  This class sends simple text only mail using an SMTP server.<BR><BR>
 *
 *
 * @version 1.0, 4/3/2000 
 * @author  Souvik Dutta
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Mail                                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class Mail {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Message msg = null;
   private static String mailer = "Mailer 1.0";





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates a Mail class initialized
    *  to the params given
    *  
    *
    *  @param from The From field (who this eMail is from)
    *  @param mailHost The hostname of the SMTP server
    *  @param None
    *
    *  @return Nothing
    *
    * @exception MessagingException thrown if unable to instantiate MimeMessage
    * or set the From field.
    *
    *//*---------------------------------------------------------*/
   public Mail(String from, String mailhost) throws MessagingException
   {
      // assume we're using SMTP host "mailhost"
      Properties props = System.getProperties();
      if ( mailhost != null )
         props.put("mail.smtp.host", mailhost);

      // Get a Session object
      Session session = Session.getDefaultInstance(props, null);

      // construct the message
      msg = new MimeMessage(session);
      if ( from!=null )
         msg.setFrom(new InternetAddress(from));
      else
         msg.setFrom();
      msg.setHeader("X-Mailer", mailer);
   }


   /**
    * This method sets the Recipients(To field) address for the mail object.<p>
    *
    * @param to String value of the recipients email address. Can be a list of
    * email addresses separated by comma ",".
    *
    * @exception MessagingException thrown if unable to set the to address
    *
    * @return Nothing
    */
   public void setRecipients(String to) throws MessagingException
   {
      msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(to, false));
   }


   /**
    * This method sets the Copy-To(CC field) address for the mail object.<p>
    *
    * @param cc String value of the recipients email address. Can be a list of
    * email addresses separated by comma ",".
    *
    * @exception MessagingException thrown if unable to set the cc address
    *
    * @return Nothing
    */
   public void setCC(String cc) throws MessagingException
   {
      msg.setRecipients(Message.RecipientType.CC, InternetAddress.parse(cc, false));
   }


   /**
    * This method sets the Blind-Copy-To(BCC field) address for the mail object.<p>
    *
    * @param bcc String value of the recipients email address whose email addresses
    * are hidden in the recipients list. Can be a list of email addresses separated
    * by comma ",".
    *
    * @exception MessagingException thrown if unable to set the bcc address
    *
    * @return Nothing
    */
   public void setBCC(String bcc) throws MessagingException
   {
      msg.setRecipients(Message.RecipientType.BCC, InternetAddress.parse(bcc, false));
   }


   /**
    * This method sets the Subject field of the mail object.<p>
    *
    * @param subject String value, subject line of the email message.
    *
    * @exception MessagingException thrown if unable to set the subject field
    *
    * @return Nothing
    */
   public void setSubject(String subject) throws MessagingException
   {
      msg.setSubject(subject);
   }


   /**
    * This method sets the Message Text of the mail object.<p>
    *
    * @param subject String value, body of the email message.
    *
    * @exception MessagingException thrown if unable to set the body of message
    *
    * @return Nothing
    */
   public void setText(String text) throws MessagingException
   {
      msg.setText(text);
   }


   /**
    * This method sends the email.<p>
    * This method also sets the sent time and date before sending the mail.
    *
    *
    * @exception MessagingException thrown if unable to send a message.
    *
    * @return Nothing
    */
   public void send() throws MessagingException
   {
      msg.setSentDate(new Date());
      Transport.send(msg);
   }

} // class Mail
