package com.ibm.nzna.shared.mq ;

import java.util.* ;

import com.ibm.mq.* ;

import com.ibm.nzna.shared.util.* ;

   /*********************************************************

   * This class manages and initializes a Message Queue interface

   * through the use of the FireWalkerMQI class.

   * 

   *

   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.

   * Created: 4/18/100  2:08pm

   * @author 

   * @version

   *********************************************************/

public class FireWalkerMGR implements SharedDataManager, TimerListener {

   /** The MQ connection properties **/

   /** The properties file should contain these standard  **/

   /** Properies.                                         **/

   /** mq.mgr.name                                        **/

   /** mq.mgr.openOptions                                 **/

   /** mq.connection.host                                 **/

   /** mq.connection.channel                              **/

   /** mq.connection.port                                 **/

   /** mq.connection.idle.timeout                         **/

   /** mq.queue.input.name                                **/

   /** mq.queue.input.group                               **/

   /** mq.queue.input.timeout                             **/

   /** mq.queue.input.expire                              **/

   /** mq.queue.output.name                               **/

   /** mq.queue.output.group                              **/

   /** mq.queue.output.timeout                            **/

   /** mq.queue.output.expire                             **/

   /** mq.auth.id                                         **/

   /** mq.auth.pw                                         **/

   Properties     properties               = null ;

   /** The interface to mq series input queue **/

   FireWalkerMQI  inFireWalkerMQI          = null ; 

   /** The interface to mq series output queue **/

   FireWalkerMQI  outFireWalkerMQI         = null ; 

   /** State of the MQ session input **/

   boolean        connectedInput           = false ;

   /** State of the MQ session output **/

   boolean        connectedOutput          = false ;

   /** The idle timeout value for the mq connection in seconds **/

   int            idleTimeout              = 30 ;

   /** Flag to determine if idle during last clock tick duration **/

   boolean        activeSinceLastClockTick = true ;

   /** Set true when MQ is waiting for work */

   boolean        active                   = false ;

   /** The connection timer used to auto timeout the connection **/

   Timer          connectionTimer          = null ;

   /* Logging module */

   LogModule      log                      = null ;





   /*********************************************************

   * Initializes queue values based on a set of property files.

   * These files are derived from the main class name of this

   * application.  Property file is Basename.mq.properties

   * 

   *

   * Author:  

   * Created: 4/18/100  2:11pm

   * @param

   * @return

   *********************************************************/

   public void init( Properties properties ) {

      /* set the properties for this instance */

      this.properties = properties ;

      /* Get the logging module */

      log = LogManager.getLogModule ( properties ) ;



      /* Get all the property values */

      String qMgrName            = properties.getProperty( "mq.mgr.name" ) ;

      String qMgrOptions         = properties.getProperty( "mq.mgr.openOptions" ) ;

      String host                = properties.getProperty( "mq.connection.host" ) ;

      String channel             = properties.getProperty( "mq.connection.channel" ) ;

      String sPort               = properties.getProperty( "mq.connection.port" ) ;

      int    port                = Integer.parseInt( sPort ) ;

      String sInputTimeout       = properties.getProperty( "mq.queue.input.timeout" , "1000") ;

      int    inputTimeout        = Integer.parseInt( sInputTimeout ) ;

      String sInputExpire        = properties.getProperty( "mq.queue.input.expire" , "999999") ;

      int    inputExpire         = Integer.parseInt( sInputExpire ) ;

      String inputQName          = properties.getProperty( "mq.queue.input.name" ) ;

      String inputCorrelationID  = properties.getProperty( "mq.queue.input.correlation" ) ;

      String sOutputTimeout      = properties.getProperty( "mq.queue.output.timeout" ,"1000") ;

      int    outputTimeout       = Integer.parseInt( sOutputTimeout ) ;

      String sOutputExpire       = properties.getProperty( "mq.queue.output.expire" , "999999") ;

      int    outputExpire        = Integer.parseInt( sOutputExpire ) ;

      String outputQName         = properties.getProperty( "mq.queue.output.name" ) ;

      String outputCorrelationID = properties.getProperty( "mq.queue.output.correlation" ) ;

      String user                = properties.getProperty( "mq.auth.user" ) ;

      String password            = properties.getProperty( "mq.auth.pw" ) ;

      String sIdleTimeout        = properties.getProperty( "mq.connection.idle.timeout", "0" ) ;

      idleTimeout                = Integer.parseInt( sIdleTimeout ) ;



      /* Show the startup values for the MQ manager */

      log.writeLog("FireWalkerMGR Initializing", 10 ) ;

      log.writeLog("mq.mgr.name                 " + qMgrName, 10) ;

      //log.writeLog("mq.mgr.openOptions          " + qMgrOptions, 10) ;               

      log.writeLog("mq.connection.host          " + host, 10) ;               

      log.writeLog("mq.connection.channel       " + channel, 10) ;            

      log.writeLog("mq.connection.port          " + sPort, 10) ;               

      log.writeLog("mq.queue.input.timeout      " + sInputTimeout, 10) ;   

      log.writeLog("mq.queue.input.expire       " + sInputExpire, 10) ;  

      log.writeLog("mq.queue.input.name         " + inputQName, 10) ;              

      log.writeLog("mq.queue.input.correlation  " + inputCorrelationID, 10) ;       

      log.writeLog("mq.queue.output.timeout     " + sOutputTimeout, 10) ;   

      log.writeLog("mq.queue.output.expire      " + sOutputExpire, 10) ; 

      log.writeLog("mq.queue.output.name        " + outputQName, 10) ;             

      log.writeLog("mq.queue.output.correlation " + outputCorrelationID, 10) ;      

      log.writeLog("mq.auth.user                " + user, 10) ;                     

      log.writeLog("mq.connection.idle.timeout  " + sIdleTimeout, 10) ;  



      /* Convert qMgrOptions into an integer value for the MQI */

      /* FINISH CODE HERE */

      int openOptions =   MQC.MQOO_OUTPUT |            

                          MQC.MQOO_INPUT_AS_Q_DEF |      

                          MQC.MQOO_SET_IDENTITY_CONTEXT; 





      /* If there is no user property no authorization required */

      if ( user == null ) {

         /* Initialize the firewalker mqseries interface object */

         inFireWalkerMQI = new FireWalkerMQI( openOptions,

                                            qMgrName,

                                            inputQName,

                                            inputCorrelationID,

                                            host,

                                            channel,

                                            port,

                                            inputTimeout,

                                            inputExpire ) ;

         /* Initialize the firewalker mqseries interface object */

         outFireWalkerMQI = new FireWalkerMQI( openOptions,

                                            qMgrName,

                                            outputQName,

                                            outputCorrelationID,

                                            host,

                                            channel,

                                            port,

                                            outputTimeout,

                                            outputExpire ) ;

      }

      /* Authorization is required for the MQ Server */

      else {

         inFireWalkerMQI = new FireWalkerMQI( openOptions,

                                            qMgrName,

                                            inputQName,

                                            inputCorrelationID,

                                            host,

                                            channel,

                                            port,

                                            inputTimeout,

                                            inputExpire,

                                            user,

                                            password ) ;

         outFireWalkerMQI = new FireWalkerMQI( openOptions,

                                            qMgrName,

                                            outputQName,

                                            outputCorrelationID,

                                            host,

                                            channel,

                                            port,

                                            outputTimeout,

                                            outputExpire,

                                            user,

                                            password ) ;

      }

      /* Create and start the connection timer */

      if ( idleTimeout > 0 ) {

         connectionTimer = new Timer( idleTimeout * 1000 , this ) ;

         connectionTimer.start() ;

      }

      /* Turn off MQSeries internal logging */

      MQException.log = null ;

   }

   /*********************************************************

   * Makes this data available to other applications who use 

   * this interface.

   * 

   *

   * Author:  

   * Created: 4/18/100  2:00pm

   * @param key Identifier for this data to be referenced.

   * @param object The data to share.

   * @return

   *********************************************************/

   public void put ( String key, Object object ) {

      /* Set activity flag */

      activeSinceLastClockTick = true ;



      if ( !connectedOutput ) {

         openOutput() ;

      }

      /* Places the object into the queue with the key id */

      try {

         outFireWalkerMQI.putObject ( object, key ) ;

      }

      catch ( Exception e ) {

         /* Force a reconnect after error */

         connectedOutput = false ;

         e.printStackTrace() ;

      }

   }

   /*********************************************************

   * Makes this data availabe to other applications who use

   * this interface.  

   * 

   *

   * Author:  

   * Created: 4/18/100  2:02pm

   * @param object Data to be shared with other applications.

   * @return A unique key to access the data.

   *********************************************************/

   public String put ( Object object ) {

      /* Set activity flag */

      activeSinceLastClockTick = true ;



      String key = null ;

      if ( !connectedOutput ) {

         openOutput() ;

      }

      try {

         key = outFireWalkerMQI.putObject ( object ) ;

      }

      catch ( Exception e ) {

         /* Force a reconnect after error */

         connectedOutput = false ;

         e.printStackTrace() ;

      }

      return ( key );

   }

   /*********************************************************

   * Gets an object from the shared data manager based on a

   * key value.  This method blocks until there is an object

   * available.

   * 

   *

   * Author:  

   * Created: 4/18/100  2:05pm

   * @param key unique identifier of the requested object.

   * @return An object matching the key reference.

   *********************************************************/

   public Object get ( String key ) {

      /* Set activity flag */

      activeSinceLastClockTick = true ;

      active = true ;



      Object object = null ;

      if ( !connectedInput ) {

         openInput() ;

      }

      try {

         object = inFireWalkerMQI.getObject( key ) ;

      }

      catch ( MQException e ) {

         if (e.reasonCode == 2033) {

            log.writeLog( "FireWalkerMGR:  timed out waiting for message", 10 ) ;

         }

         else {

            e.printStackTrace() ;

         }

      }

      catch ( Exception e ) {

         e.printStackTrace() ;

      }

      finally {

         /* Force a reconnect after error */

         close() ;

      }

      active = false ;

      return ( object );

   }

   /*********************************************************

   * Gets any object available for this application.  This

   * method blocks until there is an object availabe.

   * 

   *

   * Author:  

   * Created: 4/18/100  7:22pm

   * @param

   * @return A object and id inside of SharedDataObject

   *********************************************************/

   public SharedDataObject get () {

      SharedDataObject sharedDataObject = null ;

      /* Set activity flag */

      activeSinceLastClockTick = true ;

      active = true ;



      if ( !connectedInput ) {

         openInput() ;

      }

      Object[] objects = null ;

      /* waits for any object to enter the queue */

      try {

         objects = inFireWalkerMQI.getObject() ;

      }

      catch ( MQException e ) {

         if (e.reasonCode == 2033) {

            log.writeLog( "FireWalkerMGR:  timed out waiting for message", 10 ) ;

         }

         else {

            e.printStackTrace() ;

         }

      }

      catch ( Exception e ) {

         e.printStackTrace() ;

      }

      finally {

         /* Force a reconnect after error */

         close() ;

      }

      /* objects is an array which contains the */

      /* returned object and the identifier used*/

      /* to place the object into the queue     */

      if ( objects != null ) {

         sharedDataObject = new SharedDataObject( (String) objects[1], objects[0] ) ;

      }

      active = false ;

      return ( sharedDataObject ) ;

   }

   /*********************************************************

   * Opens the queue connections.

   *

   * Author:  

   * Created: 4/18/100  7:22pm

   * @param

   * @return 

   *********************************************************/

   private void openInput() {

      /* open the input queue */

      try {

         log.writeLog ( "FireWalkerMGR:  opening input connection", 10) ;

         inFireWalkerMQI.open() ;

      }

      catch ( Exception e ) {

         e.printStackTrace() ;

      }

      connectedInput  = true ;

   }

   /*********************************************************

   * Opens the queue connections.

   *

   * Author:  

   * Created: 4/18/100  7:22pm

   * @param

   * @return 

   *********************************************************/

   private void openOutput() {

      /* open the output queue */

      try {

         log.writeLog ( "FireWalkerMGR:  opening output connection", 10) ;

         outFireWalkerMQI.open() ;

      }

      catch ( Exception e ) {

         e.printStackTrace() ;

      }

      connectedOutput = true ;

   }

   /*********************************************************

   * Closes the queue connections for mqseries.

   *

   * Author:  

   * Created: 4/18/100  7:22pm

   * @param

   * @return 

   *********************************************************/

   private void close() {

      try {

         /* Close the input queue */

         if ( connectedInput ) {

            connectedInput  = false ;

            inFireWalkerMQI.close() ;

            log.writeLog( "FireWalkerMGR: idle input connection closed.", 10) ;

         }

         /* Close the output queue */

         if ( connectedOutput ) {

            connectedOutput = false ;

            outFireWalkerMQI.close() ;

            log.writeLog( "FireWalkerMGR: idle output connection closed.", 10) ;

         }

      }

      catch ( Exception e ) {

         e.printStackTrace() ;

      }

   }

   /*********************************************************

   * This is called by the timer.

   * 

   *

   * Author:  

   * Created: 6/9/100  9:01pm

   * @param

   * @return

   *********************************************************/

   public void clockTick ( Timer timer ) {

      if (( !activeSinceLastClockTick ) && ( !active )) {

         /* Timeout the connection for being idle */

         close() ;

      }

      /* Set activity flag */

      activeSinceLastClockTick = false ;

   }

}

   





