package com.ibm.nzna.shared.mq;



import com.ibm.mq.*;

import java.io.* ;

import java.lang.*;

import java.util.* ;

   /*********************************************************

   * This class creates a standard interface for MQSeries message

   * queues.  The constructor initializes defaut values.  The open

   * and close methods will create or destroy a connection to the

   * queue manager.

   * 

   *

   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.

   * Created: 4/10/100  5:27pm

   * @author Chad Meadows, original Ariva

   * @version 1.1

   *********************************************************/



public class FireWalkerMQI extends Object

{

   private String               queueManagerName;   //  = "FWServletQM.queue.manager";

   private String               queueName;          //  = "LOCAL.TEST.QUEUE";

   private int                  openOptions;        //  = MQC.MQOO_OUTPUT | 

                                                    //  MQC.MQOO_INPUT_AS_Q_DEF |

                                                    //  MQC.MQOO_SET_IDENTITY_CONTEXT;



   private String               messageCorrelationID = null ;

   private MQQueueManager       queueManager;

   private MQQueue              messageQueue;

   private MQGetMessageOptions  getMessageOptions  =  new MQGetMessageOptions();

   private MQPutMessageOptions  putMessageOptions  =  new MQPutMessageOptions();



   private MQMessage            receivedMsg        =  new MQMessage();    // To store retrived message from the queue.

   private MQMessage            sentMsg            =  new MQMessage();    // To store a message to be posted in queue.



   /* variables for the MQEnvironment Object */

   private String               hostname;

   private String               channel;

   private int                  port; 

   private String               userID;

   private String               password;



   /*********************************************************

   * Constructor for the Message queue interface.

   * 

   *

   * Author:  

   * Created: 4/10/100  5:36pm

   * @param openOptions      Queue open options

   * @param queueManagerName The name of the Queue Manager

   * @param queueName        The name of the Queue

   * @param host             The name of the host of the Queue Manager

   * @param channel          The channel name 

   * @param port             The port number for the connection

   * @param waitInterval     Time to wait for a message to enter the queue

   * @param expireTime       Limit of time a message can wait in a queue

   * @param user             The user for logging into the queue manager

   * @param password         The password for loggin into the queue manager

   * @return

   *********************************************************/

   public FireWalkerMQI(   int      openOptions,

                           String   queueManagerName,

                           String   queueName,

                           String   messageCorrelationID,

                           String   host,

                           String   channel,

                           int      port,

                           int      waitInterval ,

                           int      expireTime,

                           String   user,

                           String   password       ) 

   {

      this.openOptions           = openOptions;

      this.queueManagerName      = queueManagerName;

      this.queueName             = queueName;

      this.messageCorrelationID  = messageCorrelationID;

      this.hostname              = host;    

      this.channel               = channel; 

      this.port                  = port;    

      this.userID                = user;    

      this.password              = password;



      

      /* Set the default message put options */

      setMessagePutOptions(  MQC.MQPMO_SET_IDENTITY_CONTEXT |  // allows the application to set

                             MQC.MQPMO_NO_SYNCPOINT  );        // msg is put before comit



      /* Set the default message get options */

      setMessageGetOptions(  MQC.MQGMO_WAIT |

                             MQC.MQGMO_CONVERT );



      /* Set the timeout value for sending a message on the queue */

      setMessageSentTimeout ( expireTime ) ;

      /* Set the timeout value for waiting on a message to arive */

      setMessageGetTimeout ( waitInterval ) ;

   }

   /*********************************************************

   * Constructor for the Message queue interface.

   * 

   *

   * Author:  

   * Created: 4/10/100  5:36pm

   * @param openOptions      Queue open options

   * @param queueManagerName The name of the Queue Manager

   * @param queueName        The name of the Queue

   * @param host             The name of the host of the Queue Manager

   * @param channel          The channel name 

   * @param port             The port number for the connection

   * @param waitInterval     Time to wait for a message to enter the queue

   * @param expireTime       Limit of time a message can wait in a queue

   * @return

   *********************************************************/

   public FireWalkerMQI(   int      openOptions,

                           String   queueManagerName,

                           String   queueName,

                           String   messageCorrelationID,

                           String   host,

                           String   channel,

                           int      port,

                           int      waitInterval ,

                           int      expireTime ) 

   {

      /* Call the constructor */

      this( openOptions,

            queueManagerName,

            queueName,

            messageCorrelationID,

            host,

            channel,

            port,

            waitInterval,

            expireTime,

            "",

            "") ;

   }



   /*********************************************************

   * This opens the Queue Manager connection with the specified

   * options for the queue manager.

   * 

   *

   * Author:  

   * Created: 4/11/100  5:09pm

   * @param openOptions the Options for the queue manager.

   * @return

   *********************************************************/

   public void open( int openOptions ) throws MQException, IOException {

      /* This is a static Object which holds the values for constructing */

      /* a MQSeries queue manager object.  These values must be set      */

      /* prior to creating the Queue Manager.                            */

      MQEnvironment.hostname = hostname;  //========= "chaos.raleigh.ibm.com";  

      MQEnvironment.channel  = channel;   //========= "MYMQI.CHANNEL";

      MQEnvironment.port     = port;      //========= 1414;

      MQEnvironment.userID   = userID;    

      MQEnvironment.password = password;



      // Creates a connection to the queue manager specified by queueManagerName.

      queueManager = new MQQueueManager(queueManagerName);



      // opens a specified queue with the specified options.

      messageQueue = queueManager.accessQueue(queueName, openOptions, null, null, null);

   }



   /*********************************************************

   * Opens the Queue Manager with the default options.

   * 

   *

   * Author:  

   * Created: 4/11/100  5:20pm

   * @param

   * @return

   *********************************************************/

   public void open() throws MQException, IOException {

      open ( this.openOptions ) ;

   }



   /*********************************************************

   * Get the current message queue.

   * 

   *

   * Author:  

   * Created: 8/23/100  5:41pm

   * @param

   * @return

   *********************************************************/

   public MQQueue getQueue() {

      return ( messageQueue ) ;

   }





   /*********************************************************

   * Get the received message object.  This does not create a 

   * copy of the object.  This object is reused.  DO NOT SET

   * ANY DATA IN THE OBJECT.

   * 

   *

   * Author:  

   * Created: 8/23/100  7:10pm

   * @param

   * @return

   *********************************************************/

   public MQMessage getMessage() throws MQException {

      /* Match any object in the queue */        

      getMessageOptions.matchOptions = MQC.MQMO_NONE; 



      /* Wait for a message to arrive on the queue */

      messageQueue.get(receivedMsg, getMessageOptions);

      return ( receivedMsg ) ;

   }



   /*********************************************************

   * Sets the wait duration for a get method to wait for a

   * message to enter the queue.

   * 

   *

   * Author:  

   * Created: 4/11/100  5:22pm

   * @param waitInterval Time in seconds to wait for the message.

   * @return

   *********************************************************/

   public void setMessageGetTimeout ( int waitInterval ) {

      /* How long to wait for a message to appear in a queue.       */

      /* The value passed is seconds.  We convert to milli seconds. */

      /* Do not convert if set to the unlimited constant.           */

      if ( waitInterval != MQC.MQWI_UNLIMITED ) {

         waitInterval*=1000 ;

      }

      /* set the wait value */

      getMessageOptions.waitInterval = waitInterval ;

   }

   /*********************************************************

   * Set the timeout value of a message being sent.  This will

   * cause the message to expire after a duration on the message

   * queue.

   * 

   *

   * Author:  

   * Created: 4/11/100  5:28pm

   * @param expireTime the time in seconds for the message to expire.

   * @return

   *********************************************************/

   public void setMessageSentTimeout ( int expireTime ) {

      // How long should a message wait on the queue before the queue mangaer

      // discards the message

      // The value passed is in seconds.  We must convert to 10ths of seconds.

      sentMsg.expiry = expireTime * 10 ;

   }

   /*********************************************************

   * Sets the message options for messages placed onto the queue.

   * 

   *

   * Author:  

   * Created: 4/11/100  5:42pm

   * @param options The MQ options for placing messages on queues.

   * @return

   *********************************************************/

   public void setMessagePutOptions ( int options ) {

      putMessageOptions.options =  options ;

   }

   /*********************************************************

   * Sets the messages options for getting messages from the queue.

   * 

   *

   * Author:  

   * Created: 4/11/100  5:44pm

   * @param options The MQ options for getting messages from queues.

   * @return

   *********************************************************/

   public void setMessageGetOptions ( int options ) {

      getMessageOptions.options = options ;

   }

   /*********************************************************

   * Places a message on the queue identified by a message id.

   * 

   *

   * Author:  

   * Created: 4/11/100  2:08pm

   * @param data  String data as the message.

   * @param MsgId The message identifier.

   * @return

   *********************************************************/

   public void putMsg( String data, String MsgId ) throws MQException, IOException

   {

      sentMsg.clearMessage();

      sentMsg.writeUTF(data);

      sentMsg.messageId = MsgId.getBytes();

      messageQueue.put(sentMsg, putMessageOptions);

   }

   /*********************************************************

   * Gets a message from the message queue by its identifier.

   * 

   *

   * Author:  

   * Created: 4/11/100  2:10pm

   * @param MsgID The message identifier to get.

   * @return a string representing the message.

   *********************************************************/

   public String getMsg(String MsgId) throws MQException, IOException

   {

      getMessageOptions.matchOptions = MQC.MQMO_MATCH_MSG_ID; // specifies get to match for messageID.

      receivedMsg.messageId = MsgId.getBytes();

      try

      {

         messageQueue.get(receivedMsg, getMessageOptions);

      }

      catch (MQException exception)

      {

         return ("End");

      }

      return(receivedMsg.readUTF());

   }



   /*********************************************************

   * Places and object into a message with a message identifier.

   * 

   *

   * Author:  

   * Created: 4/11/100  2:11pm

   * @param data The object to be placed in the message.

   * @param MsgId The identifier of this message.

   * @return

   *********************************************************/

   public void putObject(Object data, String MsgId) throws MQException, IOException

   {

      sentMsg.clearMessage();

      sentMsg.writeObject(data);

      sentMsg.messageId = MsgId.getBytes();

      

      /* If a group id is set, set it in the message */

      if ( messageCorrelationID != null ) {

         sentMsg.correlationId = messageCorrelationID.getBytes() ;

      }



      messageQueue.put(sentMsg, putMessageOptions);

   }



   /*********************************************************

   * Places an object in the queue with no identifier.  The MQManager

   * will assign an unique identfier to the message when the message

   * is placed in the queue.

   * 

   *

   * Author:  

   * Created: 4/11/100  2:13pm

   * @param data The object to place in the message.

   * @return The unique identifier of the message on the queue.

   *********************************************************/

   public String putObject(Object data) throws MQException, IOException

   {

      /* The message object is reused for sending messages    */

      /* We must clear the object before we send each message */

      sentMsg.clearMessage();

      /* Place the object to send inside of the message */

      sentMsg.writeObject(data);

      /* We set the message id to MQC.MQMI_NONE.  This will cause  */

      /* The queue manager to assign an unique identifier when the */

      /* Message is placed in the queue.                           */

      sentMsg.messageId = MQC.MQMI_NONE ;

      /* If a group id is set, set it in the message */

      if ( messageCorrelationID != null ) {

         sentMsg.groupId = messageCorrelationID.getBytes() ;

         sentMsg.correlationId = messageCorrelationID.getBytes() ;

      }

      /* Place the message in the queue */

      messageQueue.put(sentMsg, putMessageOptions);

      /* We return the identifier which was stored by the queue    */

      /* manager when the message was placed in the queue.         */

      return ( new String (sentMsg.messageId) ) ;

   }

   /*********************************************************

   * Gets the object from the message queue by its identifier.

   * This method blocks until the object is read or the timeout

   * value is reached.

   * 

   *

   * Author:  

   * Created: 4/11/100  2:46pm

   * @param MsgId the message identifier of the message to get.

   * @return

   *********************************************************/

   public Object getObject(String MsgId) throws MQException, IOException, ClassNotFoundException

   {

      /* Attribute for getting an object by its identifier */

      getMessageOptions.matchOptions = MQC.MQMO_MATCH_MSG_ID; 

      /* The message id is in an byte array */

      receivedMsg.messageId = MsgId.getBytes();



      /* If a group id is set, set it in the message */

      if ( messageCorrelationID != null ) {

         receivedMsg.correlationId = messageCorrelationID.getBytes() ;

         /* Match on the group id */

         getMessageOptions.matchOptions |= MQC.MQMO_MATCH_CORREL_ID ;

      }



      /* Wait for a message to arrive on the queue */

      messageQueue.get(receivedMsg, getMessageOptions);



      return(receivedMsg.readObject());

   }

   /*********************************************************

   * Gets an object from the queue.  This method will get any

   * object in the queue.  This method returns the Object in an

   * object array.  The first element is the object received in the

   * message.  The second element is the message identifier id.

   * 

   *

   * Author:  

   * Created: 4/12/100  2:23pm

   * @param

   * @return Object array with first element being the object.

   *         Second element is the Object Identfier.

   *********************************************************/

   public Object[] getObject() throws MQException, IOException, ClassNotFoundException

   {

      /* Match any object in the queue */        

      getMessageOptions.matchOptions = MQC.MQMO_NONE; 



      /* If a group id is set, set it in the message */

      if ( messageCorrelationID != null ) {

         receivedMsg.correlationId = messageCorrelationID.getBytes() ;

         /* Match on the group id */

         getMessageOptions.matchOptions = MQC.MQMO_MATCH_CORREL_ID; 

      }



      /* Wait for a message to arrive on the queue */

      messageQueue.get(receivedMsg, getMessageOptions);



      /* Get the message ID */

      byte msgID[] = receivedMsg.messageId ;

      String id = new String ( msgID ) ;



      /* Create the return object array */

      Object[] objectArray = new Object[2] ;



      /* Set the values for the object array */

      /* To return back to the caller */

      objectArray[0] = receivedMsg.readObject();

      objectArray[1] = id ;



      return ( objectArray ) ;

   }



   /*********************************************************

   * Closes the queue and the queue manager connection. 

   *

   * Author:  

   * Created: 4/11/100  2:48pm

   * @param

   * @return

   *********************************************************/

   public void close() throws MQException, IOException

   {

      messageQueue.close();

      queueManager.disconnect();

   }

}



