package com.ibm.nzna.shared.mq ;

import com.ibm.nzna.shared.mq.* ;

import com.ibm.nzna.shared.util.* ;

import java.util.* ;

import com.ibm.mq.MQException ;

   /*********************************************************

   * This is a server daemon class for receiving data request

   * from other applications using a shared data manager 

   *

   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.

   * Created: 5/9/100  2:44pm

   * @author 

   * @version

   *********************************************************/

public abstract class SharedDataServer {

   SharedDataManager        sharedDataManager = null ;

   public Properties        properties        = new Properties() ;

   public void init ( Properties properties ) {



      try {

         /* Get the class name for the shared data manager */

         String value = (String )properties.get( "sharedDataManager.properties") ;



         if ( value != null ) {

            /* Load the properties file for the shared data manager */

            Properties sharedDataManagerProperties = new Properties() ;

            if ( !StandardPropertyLoader.load ( sharedDataManagerProperties, value ) ) {

               System.out.println( "SharedDataServer:  unable to load="+ value) ;

               System.exit(-1) ;

            }



            /* Get the class name of the shared data manager */

            value = (String) properties.get( "sharedDataManager.classname" ) ;



            if ( value != null ) {

               /* Load the class */

               Class sharedDataManagerClass = Class.forName( value ) ;

               /* Create an instance of the class */

               sharedDataManager = (SharedDataManager) sharedDataManagerClass.newInstance() ;

               /* Load the properties file for the shared data manager */

               /* Initialize the shared Data Manager */

               sharedDataManager.init( sharedDataManagerProperties ) ;

            }

            else {

               System.out.println( "Property not defined:  sharedDataManager.classname") ;

               System.exit( -1 ) ;

            }

         }

         else {

            System.out.println( "Property not defined:  sharedDataManager.properties") ;

            System.exit( -1 ) ;

         }

      }

      catch ( Exception e ) {

         e.printStackTrace() ;

         System.exit( -1 ) ;

      }



   }

   /*********************************************************

   * Returns the property file name for this shared data server

   * 

   *

   * Author:  

   * Created: 5/31/100  5:32pm

   * @param

   * @return property file name

   *********************************************************/

   abstract public String getPropertyFileName() ;



   /*********************************************************

   * Continuously monitors the data channel for incoming messages

   * and dispatches them to a new handler thread for each message.

   * 

   *

   * Author:  

   * Created: 5/31/100  5:33pm

   * @param

   * @return

   *********************************************************/

   public void monitorDataChannel() {

      while ( true ) {

         try {

            /* Wait for any message delivered to our queue */

            SharedDataObject sharedDataObject = sharedDataManager.get () ;

            if ( sharedDataObject != null ) {

               /* Check for a control message */

               if ( sharedDataObject.object instanceof StopSharedDataServer ) {

                  System.out.println( "SharedDataServer:  shutdown received") ;

                  /* Stop monitor */

                  break ;

               }

               /* Create a new thread to process the message */

               ProcessMessageThread processMessageThread = new ProcessMessageThread( sharedDataObject ) ;

               processMessageThread.start() ;

            }

         }

         catch ( Exception e ) {

            e.printStackTrace() ;

         }

      }

   }

   

   /*********************************************************

   * This method is called by the messages dispatcher thread

   * to process the incoming message.

   * 

   *

   * Author:  

   * Created: 5/31/100  5:34pm

   * @param

   * @return 

   *********************************************************/

   abstract public SharedDataObject processSharedDataRequest( SharedDataObject sharedDataObject ) ;



      /*********************************************************

      * This thread calls the handler to handle the incoming 

      * messages.

      * 

      *

      * Legal: Copyright (c) 2000 IBM. All Rights Reserved.

      * Created: 5/31/100  5:35pm

      * @author 

      * @version

      *********************************************************/

   

   class ProcessMessageThread extends Thread {

      SharedDataObject sharedDataObject = null ;

      ProcessMessageThread ( SharedDataObject sharedDataObject) {

         this.sharedDataObject = sharedDataObject ;

      }

      public void run() {

         long time = System.currentTimeMillis() ;

         try {

            /* Process the data in the message        */

            SharedDataObject processedResult = processSharedDataRequest( sharedDataObject ) ;

         }

         catch ( Exception e ) {

            e.printStackTrace() ;

            /* Create a valid result to send back to indicate */

            /* Status of failer                               */

            Hashtable resultHash = new Hashtable() ;

            resultHash.put("error.code", "2001") ;

            resultHash.put("error.msg", e ) ;

            /* Unable to process request, return null        */

            sharedDataObject.object = resultHash ;

         }

         try {

            /* Place the result back into the channel */

            /* Return a message to the client application    */

            /* Using the same id we received for the message */

            /* The message will be returned in a different   */

            /* queue than which it was received.             */

            sharedDataManager.put ( sharedDataObject.id, sharedDataObject.object ) ;

            System.out.println ("SharedDataServer:  message sent: "+ sharedDataObject.id ) ;

         }

         catch ( Exception e ) {

            e.printStackTrace() ;

         }



         System.out.println ("SharedDataServer: processSharedDataRequest: "+((float)( System.currentTimeMillis() - time ) / 1000)+ " seconds") ;   

      }

   }

}



