package com.ibm.nzna.shared.net ;
import java.util.* ;
import java.net.* ;
import java.io.* ;

   /*********************************************************
   * <b><font color="maroon">Description:</b></font>
   * This class will post data and read data from a url
   *
   * <p><b><font color="olive">Legal:</b></font> Copyright (c) 1998 IBM. All Rights Reserved.
   * <p><b><font color="teal">Created:</b></font> 8/10/1999  12:11pm
   * @author Chad Meadows
   * @version
   *********************************************************/
public class HTTPSocket {
   URL            url         =  null ;
   int            timeout     =  0 ;
   Socket         urlSocket   =  null ;
   InetAddress    inetAddress =  null ;
   int            port        =  80 ;
   Hashtable      headers     =  new Hashtable() ;

   public HTTPSocket ( URL url ) {
      this.url = url ;

      /* Default headers */
      headers.put ("Accept", "image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/msword, */*") ;
      headers.put ("Accept-Language", "en-us" ) ;
      headers.put ("Connection", "Keep-Alive" ) ;
      headers.put ("User-Agent", "Mozilla/4.0 (compatible; MSIE 4.01; Windows NT)" ) ;
      headers.put ("Accept-Encoding", "gzip, deflate" ) ;
   }
   public void connect () throws IOException {
      /* get the internet address of the host name */
      inetAddress = InetAddress.getByName( url.getHost() ) ;
      /* get the port number we should connect */
      port        = url.getPort() ;
      if (port == -1) {
         port = 80 ;
      }
      System.out.println( "HTTPSocket:  "+inetAddress.getHostAddress() ) ;
      /* make the socket connection */
      urlSocket   = new Socket( inetAddress, port ) ;
      /* set the timeout value */
      urlSocket.setSoTimeout( timeout ) ;
      /* add host header for this connection */
      headers.put ("Host", inetAddress.getHostName() );
   }

   public String post ( String[][] namesValues ) throws IOException {
      OutputStream         out      = null ;
      InputStream          in       = null ;
      DataOutputStream     dataOut  = null ;
      BufferedInputStream  bufIn    = null ;

      /* connect */
      connect() ;

      /* get the output stream */
      out      = urlSocket.getOutputStream() ;
      dataOut  = new DataOutputStream ( out) ;

      /* Write out the request headers */
      dataOut.writeBytes( "POST "+ url.getFile()+" HTTP/1.1\n" ) ;
      
      /* Write out user set headers */
      writeHeaders( dataOut ) ;

      /* End headers */
      dataOut.writeBytes("\n") ;

      /* Write out post data */
      String postString = "" ;
      for ( int i = 0 ; i < namesValues.length; i++) {
         postString += namesValues[i][0] + "=" + URLEncoder.encode( namesValues[i][1] ) ;
         if ( i< namesValues.length ) {
            postString += "&" ;
         }
      }
      dataOut.writeBytes( postString ) ;
      dataOut.close() ;

      /* Collect the response */
      in = urlSocket.getInputStream() ;
      String output = "" ;
      byte bytes[] = new byte[1024] ;
      bufIn = new BufferedInputStream( in, 1024 ) ;
      int bytesRead = 0 ;
      while ( (bytesRead = bufIn.read( bytes, 0, 1024)) != -1) {
         output += new String ( bytes, 0, bytesRead ) ;
      }
      bufIn.close() ;
      urlSocket.close() ;
      /* return response */
      return ( output ) ;
   }

   public String get () throws IOException  {
      OutputStream         out      = null ;
      InputStream          in       = null ;
      DataOutputStream     dataOut  = null ;
      BufferedInputStream  bufIn    = null ;

      /* connect */
      connect() ;

      /* get the output stream */
      out      = urlSocket.getOutputStream() ;
      in       = urlSocket.getInputStream() ;
      dataOut  = new DataOutputStream ( out) ;

      /* Write out the request headers */
      dataOut.writeBytes( "GET "+ url.getFile()+" HTTP/1.1\n" ) ;
      
      /* Write out user set headers */
      writeHeaders( dataOut ) ;

      /* End headers */
      dataOut.writeBytes("\n") ;
      dataOut.flush() ;

      /* Collect the response */
      String output = "" ;
      byte bytes[] = new byte[1024] ;
      int bytesRead = 0 ;
      bufIn = new BufferedInputStream( in, 1024 ) ;
      while ( (bytesRead = bufIn.read( bytes, 0, 1024)) != -1) {
         output += new String ( bytes, 0, bytesRead ) ;
      }
      dataOut.close() ;
      bufIn.close() ;
      urlSocket.close() ;
      /* return response */
      return ( output ) ;
   }

   void writeHeaders ( DataOutputStream dataOut ) throws IOException {
      Enumeration e = headers.keys() ;
      while (e.hasMoreElements() ) {
         String name = (String ) e.nextElement() ;
         dataOut.writeBytes( name + ": "+ headers.get( name )+"\n") ;
      }
   }

   public void setTimeout ( int timeout ) throws SocketException {
      this.timeout = timeout ;
      if ( urlSocket != null ) {
         /* set the timeout value */
         urlSocket.setSoTimeout( timeout ) ;
      }
   }
}

