package com.ibm.nzna.shared.servlet ;   
import java.io.Serializable ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.util.* ;
import java.io.* ;
   /*********************************************************
   * Contains session state information about the web user.
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/19/100  1:03pm
   * @author Chad Meadows
   * @version
   *********************************************************/
public class HttpRequest implements Serializable {
   Hashtable requestData                         = new Hashtable() ;
   Hashtable requestHeaders                      = new Hashtable() ;
   transient Hashtable responseHeaders           = new Hashtable() ;
   String    queryString                         = null ;
   transient HttpServletResponse servletResponse = null ;
   transient HttpServletRequest  servletRequest  = null ;
   /*********************************************************
   * Default Constructor
   * 
   *
   * Author:  
   * Created: 4/19/100  5:04pm
   * @param
   * @return
   *********************************************************/
   public HttpRequest ( HttpServletRequest request ) {
      setServletRequest( request ) ;
   }
   /*********************************************************
   * Gets the query string from the request
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @param
   * @return The id value of this session.
   *********************************************************/
   public String getQueryString () {
      return ( queryString ) ;
   }
   /*********************************************************
   * Sets the query string
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @param queryString the queryString value as received in 
   * the request.
   *********************************************************/
   public void setQueryString ( String queryString ) {
      this.queryString = queryString ;
      if ( queryString != null ) {
         /* Store the query string parameters in the parameter hash table */
         Hashtable queryStringHash = HttpUtils.parseQueryString( queryString ) ;
         Enumeration queryStringEnum = queryStringHash.keys() ;
         while ( queryStringEnum.hasMoreElements() ) {
            String key   = (String) queryStringEnum.nextElement() ;
            Object value = queryStringHash.get( key ) ;
            if ( value instanceof String ) {
               setParameterValue ( key, (String) value ) ;
            }
            else if ( value instanceof String[] ) {
               setParameterValues ( key, (String[]) value ) ;
            }
         }
      }
   }
   /*********************************************************
   * Gets the path info for this request
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @param
   * @return The path info for this request.
   *********************************************************/
   public String getPathInfo () {
      return ( servletRequest.getPathInfo() ) ;
   }
   /*********************************************************
   * Returns the number of parameters in the request.
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @return the number of parameters in the request.
   *********************************************************/
   public int getNumParameters () {
      return ( requestData.size() ) ;
   }
   /*********************************************************
   * Returns the names of the parameters in the request.
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @return names of the parameters in the request.
   *********************************************************/
   public Enumeration getParameterNames () {
      return ( requestData.keys() ) ;
   }

   /*********************************************************
   * Returns the values of the parameter in the request.
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @return values of the parameter in the request.
   *********************************************************/
   public String[] getParameterValues ( String key ) {
      return ( (String[]) requestData.get( key ) ) ;
   }

   /*********************************************************
   * Sets a value in the parameter list.
   * THIS FEATURE NEEDS TO BE MOVED TO SERVLET UTIL
   *
   * Author:  
   * Created: 8/29/100  7:52pm
   * @param
   * @return
   *********************************************************/
   public void setParameterValues ( String key, String[] values ) {
      requestData.put( key, values ) ;
   }

   /*********************************************************
   * Sets a value in the parameter list.
   * THIS FEATURE NEEDS TO BE MOVED TO SERVLET UTIL
   *
   * Author:  
   * Created: 8/29/100  7:52pm
   * @param
   * @return
   *********************************************************/
   public void setParameterValue ( String key, String value ) {
      String[] valueArray = new String[1] ;
      valueArray[1] = value ;
      requestData.put( key, valueArray ) ;
   }

   /*********************************************************
   * Removes a parameter from the parameter list.
   * THIS FEATURE NEEDS TO BE MOVED TO SERVLET UTIL
   *
   * Author:  
   * Created: 8/30/100  6:55pm
   * @param
   * @return
   *********************************************************/
   public void removeParameter ( String key ) {
      requestData.remove( key ) ;
   }

   /*********************************************************
   * Returns the value of the parameter in the request.
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @return values of the parameter in the request.
   *********************************************************/
   public String getParameter ( String key ) {
      String parameterValue = null ;
      String[] values = (String[])requestData.get( key ) ;
      /* This assumes there is only one value for this key */
      if ( values != null ) {
         parameterValue = values[0] ;
      }
      return ( parameterValue ) ;
   }

   /*********************************************************
   * Create a query string using the current values set in the
   * parameter list.
   * THIS FEATURE NEEDS TO BE MOVED TO SERVLET UTIL
   *
   * Author:  
   * Created: 8/29/100  8:56pm
   * @param
   * @return
   *********************************************************/
   public String createQueryString() {
      return ( ServletUtil.createQueryString( requestData )) ;
   }

   /*********************************************************
   * Sets the name value pairs received in the request.  These
   * values were received by post or query string data.
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @param
   * @return 
   *********************************************************/
   void setRequestHeaders( Hashtable headers ) { 
      /* Store the values in the request hashtable */
      this.requestHeaders = headers ;
   }
   /*********************************************************
   * Gets the headers from the request.
   * 
   *
   * Author:  
   * Created: 6/8/100  10:06pm
   * @param
   * @return
   *********************************************************/
   public Hashtable getRequestHeaders () {
      return ( requestHeaders ) ;
   }
   /*********************************************************
   * Gets the headers for the response
   * 
   *
   * Author:  
   * Created: 6/8/100  10:06pm
   * @param
   * @return
   *********************************************************/
   public Hashtable getResponseHeaders () {
      return ( responseHeaders ) ;
   }
   /*********************************************************
   * Gets the headers from the request.
   * 
   *
   * Author:  
   * Created: 6/8/100  10:06pm
   * @param
   * @return
   *********************************************************/
   public HttpServletRequest getServletRequest() {
      return ( servletRequest ) ;
   }
   /*********************************************************
   * Sets an header for the http request
   * 
   *
   * Author:  
   * Created: 6/8/100  9:59pm
   * @param
   * @return
   *********************************************************/
   void setRequestHeader ( String header, String value ) {
      /* Store the header in the hashtable */
      requestHeaders.put( header, value ) ;
   }
   /*********************************************************
   * Sets an header for the http response
   * 
   *
   * Author:  
   * Created: 6/8/100  9:59pm
   * @param
   * @return
   *********************************************************/
   void setResponseHeader ( String header, String value ) {
      /* Store the header in the hashtable */
      responseHeaders.put( header, value ) ;
   }

   /*********************************************************
   * Sets the name value pairs received in the request.  These
   * values were received by post or query string data.
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @param
   * @return 
   *********************************************************/
   void setRequestValues( String name, String values[] ) { 
      /* Store the values in the request hashtable */
      requestData.put ( name, values ) ;
   }
   /*********************************************************
   * Set the request values based on the ServletRequest
   * 
   *
   * Author:  
   * Created: 5/23/100  8:48pm
   * @param
   * @return
   *********************************************************/
   void setServletRequest( HttpServletRequest request ) {
      this.servletRequest = request ;
      /* Store data from the servlet request */
      Enumeration headerNames = request.getHeaderNames() ;
      while ( headerNames.hasMoreElements() ) {
         String name = (String) headerNames.nextElement() ;
         /* Get the header value */
         String headerValue = request.getHeader( name ) ;
         /* Store the headers */
         requestHeaders.put( name, headerValue ) ;
      }
      /* Set Request data, name value pairs */
      Enumeration parameterNames = request.getParameterNames() ;
      while ( parameterNames.hasMoreElements() ) {
         /* get the key name */
         String name = ( String )parameterNames.nextElement() ;
         /* Get the values of the key */
         String values[] = request.getParameterValues( name ) ;
         /* Set the parameter values in the session state */
         setRequestValues( name , values ) ;
      }

      /* Set the query string value from the servlet request */
      setQueryString ( request.getQueryString() );
   }
   /*********************************************************
   * Sets the httpServlet Response object.
   * 
   *
   * Author:  
   * Created: 6/8/100  11:39pm
   * @param
   * @return
   *********************************************************/
   void setServletResponse ( HttpServletResponse response ) {
      this.servletResponse = response ;
   }
   /*********************************************************
   * Sets a redirect for the response 
   * 
   *
   * Author:  
   * Created: 6/8/100  11:42pm
   * @param
   * @return
   *********************************************************/
   public void sendRedirect( String url ) throws IOException {
      servletResponse.sendRedirect( url ) ;
   }
}

