

package com.ibm.nzna.shared.servlet ;

import javax.servlet.* ;

import javax.servlet.http.* ;

import java.io.* ;

import java.net.* ;

import java.util.* ;

import com.ibm.nzna.shared.servlet.template.* ;

import com.ibm.nzna.shared.util.* ;

   /*********************************************************

   * The PageLayoutManager is an extremely versatile servlet

   * capable of providing many types of services through a

   * plugin type of architecture.  The design of the PageLayoutManager

   * strongly promotes code use through its plugin type of contstruction.

   * This servlet renders output to the web base on templates.  The 

   * templates used are controlled by a template module which determines

   * which templates are loaded and used at what times.  The templates

   * have meta tags which are bound to classes in a bindings file.  This

   * allows each tag to represent a class to be used to fill in the template.

   *

   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.

   * Created: 4/14/100  12:58pm

   * @author 

   * @version

   *********************************************************/

public class PageLayoutManager extends ServletBase {

   private final  String                  propertyFilename        = "PageLayoutManager" ;

   private        TemplateModule          templateModule          = null ;

   /*********************************************************

   * This initialization is called only on the first call to

   * the service method.  This allows for some initialization

   * which can not occur in the standard init because of 

   * some information which is only available on the first

   * call to the service method.

   * 

   *

   * Author:  Chad Meadows 

   * Created: 3/28/100  7:56pm

   * @param

   * @return

   *********************************************************/

   public boolean serviceInitialize () {

      super.serviceInitialize () ;

      try {

         /* Initialize the template module class */

         String classname = properties.getProperty( TemplateModule.classnameProperty ) ;

         if ( classname != null ) {

            /* Load the template module class */

            Class templateModuleClass = Class.forName( classname ) ;

            /* Create the instance of the class */

            templateModule = (TemplateModule) templateModuleClass.newInstance() ;

            /* Is a property file set for the template module */

            String value = properties.getProperty( TemplateModule.propertyFilename ) ;

            if ( value != null) {

               Properties templateProperties = new Properties() ;

               if (StandardPropertyLoader.load ( templateProperties , value )) {

                  /* properties loaded, Initialize the template module */

                  templateModule.init( templateProperties , templateLoader ) ;

               }

               else {

                  /* Unable to load property file */

                  writeLog("Unable to load: "+value ) ;

               }

            }

            else {

               /* Initialize the template module using our properties */

               templateModule.init( properties, templateLoader ) ;

            }

         }

      }

      catch ( Exception e ) {

         e.printStackTrace() ;

         return ( false );

      }

      return (true ) ;

   }



   /*********************************************************

   * Returns the propertyFile name which will be loaded in

   * the init method of the servlet.  The property file is 

   * located by searching the classpath of the vm in which

   * the servlet is running.  The filename should be returned

   * without any extension.

   * 

   *

   * Author:  Chad Meadows

   * Created: 3/29/100  1:37pm

   * @param

   * @return A String of the property file name without extension.

   *********************************************************/

   protected String getPropertyFileName() {

      return ( propertyFilename ) ;

   }

   /*********************************************************

   * All units of work for the servlet are performed in this

   * method.  This method is called by the service method of

   * this servlet.

   * 

   *

   * Author:  Chad Meadows

   * Created: 3/29/100  2:23pm

   * @param

   * @return

   *********************************************************/

   public void servletService ( HttpServletRequest req , HttpServletResponse resp )

                         throws ServletException, IOException, MalformedURLException {





      /* This is used to write our output to the WebServer */

      ServletOutputStream servletOutput = resp.getOutputStream() ;



      /* Load the template to display to the web user */

      writeLog("Loading template", 5) ;

      String output = templateModule.getTemplate( getSessionState() ) ;



      /* Fill the template with data */

      writeLog("Filling template", 5) ;

      output = templateModule.fillTemplate( getSessionState(), output ) ;



      writeLog("Sending output", 5) ;



      /* Write the output to the output stream */

      servletOutput.print( output ) ;



      /* Close the output Stream */

      servletOutput.close() ;

   }

}

