package com.ibm.nzna.shared.servlet ;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.nzna.shared.util.RegularExpression ;
   /*********************************************************
   * RedirectServlet uses the Refresh Header to redirect the
   * client to a new url while display an interim message.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 
   * @author 
   * @version
   *********************************************************/

public class RedirectServlet extends ServletBase {
   private final  String                  propertyFilename        = "RedirectServlet" ;
   private final  String                  propertyKeyUrl          = "servlet.redirect.url" ;
   private final  String                  propertyKeyMsg          = "servlet.redirect.msg" ;
   private final  String                  propertyKeyTemplate     = "servlet.redirect.template" ;
   private final  String                  defaultTemplate         = "standard/pleasewait.html" ;
   private final  String                  paramKeyUrl             = "redirect.url" ;
   private final  String                  paramKeyMsg             = "redirect.msg" ;

   /*********************************************************
   * Returns the propertyFile name which will be loaded in
   * the init method of the servlet.  The property file is 
   * located by searching the classpath of the vm in which
   * the servlet is running.  The filename should be returned
   * without any extension.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  1:37pm
   * @param
   * @return A String of the property file name without extension.
   *********************************************************/
   protected String getPropertyFileName() {
      return ( propertyFilename ) ;
   }
   /*********************************************************
   * All units of work for the servlet are performed in this
   * method.  This method is called by the service method of
   * the ServletBase servlet.
   * 
   *
   * Author:  
   * Created: 
   * @param
   * @return
   *********   ************************************************/

   public void servletService(HttpServletRequest req, HttpServletResponse res)
   throws ServletException, IOException {
      String value ;
      String redirectURL = null;
      String redirectMsg = null;

      /* Get the current session state for this servlet */
      SessionState sessionState = getSessionState() ;

      /* Get the current request object from the session state */
      HttpRequest httpRequest = sessionState.getRequest() ;

      /* Get the template file name */
      String templateFile = (String) properties.getProperty( propertyKeyTemplate , defaultTemplate ) ;

      
      /* Assing value from query string if not defined as a property */
      value = httpRequest.getParameter( paramKeyUrl );
      /* Was there a parameter in the query string */
      if ( value != null) {
         redirectURL = value;
      }

      /* Get the url which we should forward the request  */
      /* If defined in the property file this will always */
      /* forward to the url defined in the property file  */
      /* if not passed in as a parameter.                 */
      if ( redirectURL == null ) {
         redirectURL = (String) properties.getProperty( propertyKeyUrl ) ;
      }

      /* Get the text to display to the user */
      value = httpRequest.getParameter( paramKeyMsg );
      if ( value != null ) {
         /* do not forward our own parameters */
         httpRequest.removeParameter( paramKeyMsg ) ;
         redirectMsg = value;
      }

      /* Get the url which we should forward the request  */
      /* If defined in the property file this will always */
      /* forward to the url defined in the property file  */
      /* if not passed in as a parameter                  */
      if ( redirectMsg == null ) {
         redirectMsg = (String) properties.getProperty( propertyKeyMsg ) ;
      }

      if ( redirectURL != null ) {
         writeLog("Redirect to :" + redirectURL, 5 ) ;
         /* Add slash to end of url */
         if ( !redirectURL.endsWith("/") ) {
            redirectURL += "/" ;
         }

         /* do not forward our own parameters */
         httpRequest.removeParameter( paramKeyUrl ) ;

         String queryString = httpRequest.createQueryString() ;
         String newLocation = null ;

         if (( queryString != null) && (queryString.length() > 0 )) {
            newLocation = redirectURL + "?" + queryString ;
         }
         else {
            newLocation = redirectURL ; 
         }

         /* Forces the browser to reload the URL listed as newLocation */
         /* The browser will first display our content before refreshing */
         res.setHeader("Refresh", "1; URL=" + newLocation);
      }


      /* if no value set, then set to empty string */
      if ( redirectMsg == null ) {
         redirectMsg = "" ;
      }


      res.setContentType("text/html");
      PrintWriter out = res.getWriter();


      /* Load the template for the refresh */
      String template = templateLoader.getTemplate( templateFile ) ;

      /* Was the template loaded */
      if ( template != null ) {
         /* Fill in the template */
         template = RegularExpression.substitute( "##USERMSG##", redirectMsg, template ) ;
      }
      else {
         /* If unable to get a template display the standard error msg */
         template = StandardError.get( "IBM", "Service Temporarily Unavailable " + new Date() , getPropertyFileName() + 
                                              ":" + propertyInstanceName + " state -2 <!-- \n unable to load '"+ templateFile +"' -->" ) ;
      }

      /* Send the output back to the user */
      out.println( template );
   }

}
