package com.ibm.nzna.shared.servlet ;   
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import java.math.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.mq.* ;
import com.ibm.nzna.shared.servlet.template.* ;
import com.ibm.nzna.shared.servlet.filter.* ;
import com.ibm.nzna.shared.lang.* ;
   /*********************************************************
   * The servlet base class is developed to maintain a consistency
   * between all servlets.  This class logs all executions of the
   * service method by the servlet engine.  The logs show the 
   * duration of the execution of this servlet.  All servlets
   * must extend this class.
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 3/28/100  7:35pm
   * @author Chad Meadows
   * @version 1.0
   *********************************************************/
public abstract class ServletBase extends HttpServlet implements LogModule{
   /* WARNING:  All class variables have concurrent access        */
   /* Multiple session may execute the service simultaneously     */
   /* If you need to track information per session you must       */
   /* track it agains the currently executing thread.             */
   /* Otherwise the class variable should apply to all sessions.  */

   /* Set true if the init method executed successfully */
   private          boolean            initialized              = false ;
   /* Set true if the serviceInitialize() method executed successfully */
   private          boolean            serviceInitialized       = false ;
   /* Set true if the instance or alias name property file */
   /* for this servlet was found and loaded                */
   private          boolean            instancePropertyLoaded   = false ;
   /* The name of this servlet instance used to load property files */
   public           String             propertyInstanceName     = "" ;
   /* The properties loaded for this servlet */
   public           Properties         properties               = null ;
   /* When set true, log extra info for debugging */
   public           boolean            debug                    = false ;
   /* The level of logging detail 1-10 */                            
   public           int                logLevel                 = 1 ;
   /* Enables or disables logging to the log files */
   public           boolean            loggingEnabled           = false ;
   /* Enables logging of the request data */
                    boolean            logRequestEnabled        = false ;
   
   /* This sessionId is the number of requests this servlet */
   /* has received since the servlet was initialized        */
   /* This value is not used for the session tracking       */
          int                          sessionId                = 0 ;
   /* The version of this servlet */
          final String                 version                  = "v1.00" ;
   /* This name is used in the log file for this servlet */
          String                       servletName              = null ;
   /* This is used to identify this servlet instance from others */
   /* by storing the thread id which invoked the service method  */
   /* This is then used by the logging methods for unique id     */
          Hashtable                    serviceId                = new Hashtable() ;
          //SessionManager               sessionManager           = null ;
   /* The session tracker is available if defined in the property file  */
   /* The session tracker manages session states with a session manager */
          SessionTracker               sessionTracker           = null ;
   /* The session state stores all of the session data for each */
   /* request and maintains information needed between each     */
   /* web transaction.                                          */
   private Hashtable                   sessionStateTable        = new Hashtable();
   /* This is the key name of the session id.  This key name is */
   /* stored in the URL to maintain its state between requests  */
          String                       sessionTrackingKey       = null ;
   /* This is the key name of the sequence number.  This key is */
   /* stored in the URL.  Its value is the number of requests   */
   /* which have been issued by the web user for this session   */
          String                       sessionSequenceKey       = null ;
   /* The sequence number for the current session */
          int                          sessionTrackingSequence  = 0 ;
   /* Store the session id's in a cookie vs the URL */
          boolean                      sessionCookieEnabled     = false ;
   /* The filter manager allows the use of chained filters to   */
   /* manipulate the output of any servlet.  Filters are loaded */
   /* from the property file.                                   */
          ContentFilterManager        contentFilterManager    = null ;
   /* The template loader, is used to load templates for all    */
   /* presentations to the user.  All HTML rendered to the user */
   /* should use the template loader for rendering templates    */
          TemplateLoader               templateLoader           = null ;
   /* Enables administrative commands to be processed by the    */
   /* servlet.  This allows displaying property information     */
          boolean                      adminEnabled             = true ;
   /* The subdirectory name of the location of the servlet      */
   /* property files.                                           */
          String                       propertyFilePrefix       = null ;
   /* Get the file separator for the system.                    */
          String                       systemFileSeparator      = System.getProperty( "file.separator" ) ;
   /* Used for tracking execution times */
          Hashtable                    beginTime                = new Hashtable() ;
   /*********************************************************
   * During the servlet initialization we load the servlet properties.
   * The servlet properties are loaded utilizing the resource loader
   * which will load the properties from the classpath.
   * Your property file must be in the classpath and should be a 
   * fully qualified name so it does not conflict with other property
   * files in the same classpath.  All that subclass this method must
   * call super.init( config )  as the first
   * line of the method.
   * Example:  ServletBase.properties
   * 
   *
   * Author:  Chad Meadows 
   * Created: 3/28/100  7:56pm
   * @param
   * @return
   *********************************************************/
   public void init ( ServletConfig config) throws ServletException {
      super.init( config ) ;
      try {
         String value = null ;
         properties = new Properties() ;


         /* We must first load the global properties file */
         try {
            properties = StandardPropertyLoader.getProperties() ;
            /* Add ourselves as the logging module */
            properties.put( LogModule.propertyKey , this ) ;
         } catch ( IOException e ) {
             System.out.println( "ServletBase:  Unable to init.  Missing Property File: "+ StandardPropertyLoader.globalPropertyFile +", "+ new Date()) ;
             return ;
         }

         /* Gets the subdirectory name which contains the servlet property files */
         value = properties.getProperty( "servlet.property.file.prefix", "servlet" ) ;
         propertyFilePrefix = value ;

         /* The property loader will load the global properties by default, but */
         /* in this case we needed a value from the global properties before    */
         /* loading the servlet properties to determine the path of the servlet */
         /* property files.                                                     */

         String propertyFileName = propertyFilePrefix + systemFileSeparator + getPropertyFileName() ;
         /* Check to see if the property file was loaded */
         if ((StandardPropertyLoader.update( properties, propertyFileName )) != true ) {
             System.out.println( "ServletBase:  Unable to init.  Missing Property File: "+ propertyFileName +", "+ new Date()) ;
             return ;
         }
         
         /* Set standard properties for this servlet */
         /* Set the debug state */
         value = properties.getProperty( "debug", "false" ) ;
         if (value.equalsIgnoreCase("true")) {
            this.debug = true ;            
         }

         /* Set standard properties for this servlet */
         /* Set the debug state */
         value = properties.getProperty( "log.level", "1" ) ;
         logLevel = Integer.parseInt( value ) ;


         /* Enables or disables the logging to the logfile */
         value = properties.getProperty( "servlet.logging.enabled", "true" ) ;
         if (value.equalsIgnoreCase("true")) {
            this.loggingEnabled = true ;
         }
         


         /* Set the servlet name for logging */
         value = properties.getProperty( "servlet.name", "ServletBase" ) ;
         this.servletName = value ;
            
         /* Set the session id cookie name */
         /* This property should be in the */
         /* Global properties file.        */
         /* If cookie session tracking is  */
         /* not enabled this is the name   */
         /* of the URL parameter to get the*/
         /* session id from in the URL     */
         value = properties.getProperty( "servlet.session.tracking.key" ) ;
         this.sessionTrackingKey = value ;

         /* If this property is enabled session tracking will use */
         /* a cookie value to store the session id.  This property*/
         /* is disabled by default.                                */
         value = properties.getProperty( "servlet.session.cookie.enabled", "false" ) ;
         if (value.equalsIgnoreCase("true")) {
            this.sessionCookieEnabled = true ;            
         }

         /* Determine if admin capability should be enabled for this servlet */
         value = properties.getProperty( "servlet.admin.enabled", "true" ) ;
         if (value.equalsIgnoreCase("true")) {
            this.adminEnabled = true ;            
         }

         /* Load the template loader for the servlet */
         value = properties.getProperty( "servlet.template.loader.classname") ;
         if ( value != null) {
            /* Create a class object for the class file */
            Class templateLoaderClass = Class.forName( value ) ;
            /* Create an instance of the class */
            templateLoader = (TemplateLoader) templateLoaderClass.newInstance() ;
            writeLog("Loaded class: "+value ) ;
            /* Initialize the template loader */
            templateLoader.init( properties ) ;
         }

         /* Install session Manager if available */
         /* This property should be in the global*/
         /* properties file as all servlets must */
         /* use the same session manager to work */
         value = properties.getProperty( "servlet.sessionManager.classname" ) ;
         /* Load the session manager class */
         if ( value != null ) {
            String classname = value ;
            /* Check to see if the session manager is active for this servlet */
            value = properties.getProperty( "servlet.sessionManager.active", "false" ) ;
            /* Start the session manager if set active */
               if ( !value.equalsIgnoreCase( "false"))  {
               SessionManager sessionManager      = null ;
               Class          sessionManagerClass = Class.forName( classname ) ;
               /* Create an instance of the class */
               sessionManager = (SessionManager) sessionManagerClass.newInstance() ;
               writeLog("Loaded class: "+classname ) ;
               /* Get the property file for the session Manager */
               value = properties.getProperty( "servlet.sessionManager.properties" ) ;
               if ( value != null) {
                  Properties sessionManagerProperties = (Properties) properties.clone() ;
                  /* Load the properties */
                  if (!StandardPropertyLoader.update( sessionManagerProperties, value )) {
                     writeLog("Unable to load="+value) ;
                  }
                  /* Initialize the session Manager */
                  sessionManager.init( sessionManagerProperties ) ;
                  /* Create the session Tracker which will */
                  /* control the session states using the  */
                  /* loaded session manager.               */
                  sessionTracker = new SessionTracker( sessionManager, sessionTrackingKey, sessionCookieEnabled ) ;
                  /* Log init */
                  writeLog("Session Manager Activated: "+ classname ) ;
               }
               else {
                  /* Unable to load property file */
                  writeLog("Missing property definition = servlet.sessionManager.properties") ;
               }
            }
         }

         /* Log that we have been loaded */
         writeLog("Servlet Initialized") ;
         /* Set initialized flag */
         initialized = true ;
      }
      catch ( Exception e ) {
         e.printStackTrace() ;

         return ;
      }

   }
   /*********************************************************
   * This is the service initializer which is called only once.
   * This initializer serves the purposes of providing information
   * which is not available at the time the servlet init is called.
   * To use this feature simply overide this method in your subclass.
   * For proper operation you should call the super.serviceInitialize()
   * as your first line of this method.
   *
   * Author:  
   * Created: 4/25/100  7:11pm
   * @param
   * @return
   *********************************************************/
   protected boolean serviceInitialize () {
      String value = null ;
      /* Initialize the contentFilterManager */
      contentFilterManager   = new ContentFilterManager ( properties ) ;

      /* Check to see if any proxies are enabled */
      /* and the type ( socks or proxy )         */
      String proxyMode = properties.getProperty("proxyMode", "none") ;

      /* Enable socks in the VM if the socks property is set */
      value = properties.getProperty("socksProxyHost") ;
      if (( value != null ) && ( proxyMode.equalsIgnoreCase("socks"))) {
         /* Get the system properties */
         Properties systemProperties = System.getProperties() ;
         /* Set the socks values */
         systemProperties.put( "socksProxyHost", value ) ;
         /* Set the port value */
         value = properties.getProperty("socksProxyPort", "1080" ) ;
         systemProperties.put( "socksProxyPort", value ) ;
         /* Set the system properties */
         System.setProperties( systemProperties ) ;
      }

      /* Enable proxy server in the VM if proxy property is set */
      value = properties.getProperty("proxyHost") ;
      if (( value != null ) && ( proxyMode.equalsIgnoreCase("proxy"))) {
         /* Get the system properties */
         Properties systemProperties = System.getProperties() ;
         /* Set the socks values */
         systemProperties.put( "proxyHost", value ) ;
         /* Set the port value */
         value = properties.getProperty("proxyPort", "80" ) ;
         systemProperties.put( "proxyPort", value ) ;
         /* Proxy must be enabled with this property */
         systemProperties.put( "proxySet", "true" ) ;
         /* Set the system properties */
         System.setProperties( systemProperties ) ;
      }

      /* Set the serviceInitialized flag */
      serviceInitialized = true ;

      /* return state of success */
      return ( true ) ;
   }

   /*********************************************************
   * Returns the propertyFile name which will be loaded in
   * the init method of the servlet.  The property file is 
   * located by searching the classpath of the vm in which
   * the servlet is running.  The filename should be returned
   * without any extension.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  1:37pm
   * @param
   * @return A String of the property file name without extension.
   *********************************************************/
   abstract protected String getPropertyFileName() ;

   /*********************************************************
   * This servlet service method logs all servlet activity.
   * This method is called by the servlet engine.  This method
   * in turn calls your servletService method where you perform
   * your unit of work.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  1:42pm
   * @param 
   * @return void
   *********************************************************/
   public void service ( HttpServletRequest req , HttpServletResponse resp )
                         throws ServletException, IOException, MalformedURLException {
       int                 sessionId              = this.sessionId++ ;
       SessionState        sessionState           = null ;
       long                time                   = System.currentTimeMillis() ;

       try {
          String adminCommand = null ;
          /* If admin is enabled, first check for an admin command in the query string */
          if ( adminEnabled ) {
             /* A command has been issued in the query string */
             if ( (adminCommand = req.getParameter("admin.command")) != null ) {
                /* process the command */
                boolean continueService = ServletBaseAdmin.admin( this, req, resp , adminCommand) ;
                /* Should we continue processing the request */
                if ( !continueService ) {
                   return ;
                }
             }
          }
          /* Check to see if we were properly initialized */
          if ( initialized ) {

              /* Store unique service identifier */
              /* We must have an identifier to determine which invocation of */
              /* this servlet which is being invoked for logging purposes    */
              serviceId.put( Thread.currentThread() , new String( Integer.toString(sessionId) ) ) ;


              /* We must get the instance specified properties for       */
              /* This servlet.  These properties are located             */
              /* In the ServletName.InstanceName.properties file.        */
              /* The values in this property file will overide any value */
              /* set in the Global or SerlvetName properties files       */
              /* We will attempt to load this only on our first service  */
              /* invocation.                                             */
              /* If debug always check file for changes                  */
              if ( !instancePropertyLoaded ) {
                  String servletPath = req.getServletPath() ;
                  propertyInstanceName = servletPath.substring(servletPath.lastIndexOf("/") + 1, servletPath.length()) ;
                  /* update the values from the instance property file */
                  StandardPropertyLoader.update( properties, propertyFilePrefix + systemFileSeparator + 
                                                 propertyInstanceName + systemFileSeparator + getPropertyFileName() ) ;
                  /* Set the instancePropertyLoaded flag */
                  instancePropertyLoaded = true ;
              }

              /* Check to see if this servlet has been initialized   */
              /* For the service Invocation.  This is initialization */
              /* is called only once for the first service call      */
              if ( !serviceInitialized ) {
                 serviceInitialized = serviceInitialize() ;
                 /* was serviceInitialized */
                 if ( !serviceInitialized ) {
                    writeLog("Unabled to initialize service") ;
                 }
              }

              /* If this service has been initialized properly  */
              /* continue with the other services in the method */
              if ( serviceInitialized ) {
                 /* Are we using a SessionManager */
                 if ( sessionTracker != null ) {
                    /* get the current session state */
                    sessionState = sessionTracker.getSessionState( req, resp ) ;
                    /* Create our own response object so that */
                    /* we can apply session tracking id's to  */
                    /* any url's in the response stream       */
                    //resp = new ServletBaseResponse( resp, contentFilterManager ) ;
                 }
                 else {
                    /* FINISH CODE, need meaningful session state here */
                    /* Create a session state object for each transaction */
                    sessionState = new SessionState( Integer.toString(sessionId), req) ;
                 }

                 /* Put the session state in the sessionStateTable */
                 sessionStateTable.put( Thread.currentThread() , sessionState ) ;
                 /* Track the start time of this servlet for logging */
                 beginTime.put( sessionState.getId(), new Long(  time) );                 

                 /* Log the request data */
                 if ( logRequestEnabled ) {
                    logRequest( req ) ;
                 }

                 /* Install the filter manager */
                 resp = new ServletBaseResponse( properties, resp, contentFilterManager ) ;

                 /* Log the entry into the service method */
                 /* This log will appear in the servlet engine log file */
                 writeLog ("SERVICE BEGIN:  "+ new Date()) ;

                 /* Execute the service unit of work for this servlet */
                 servletService ( req, resp ) ;

                 /* Set any headers which were set in the sessionState */
                 Hashtable headerList = sessionState.getRequest().getResponseHeaders() ;
                 Enumeration headersEnum = headerList.keys() ;
                 while ( headersEnum.hasMoreElements()) {
                    String key = (String) headersEnum.nextElement() ;
                    /* Set each header in the servlet response object */
                    writeLog("Response Header: " + key + " = " + (String)headerList.get( key ), 10 ) ;
                    resp.setHeader( key, (String)headerList.get( key ) ) ;
                 }

                 
                 /* Write out what is in the buffers */
                 ((ServletBaseResponse)resp).writeOutput( sessionState ) ;

                 /* Log the termination of the service method */
                 writeLog ("SERVICE END: "+((float)( System.currentTimeMillis() - time ) / 1000)+ " seconds") ;   

                 /* If we are using a SessionManager */
                 /* Store the current session object */
                 if ( sessionTracker != null ) {
                    /* store the current session state */
                    sessionTracker.putSessionState ( sessionState ) ;
                 }
              }

              /* Remove the stored session time */
              beginTime.remove( sessionState.getId() );                 
              /* Remove the stored service identifier */
              serviceId.remove( Thread.currentThread() ) ;
              /* Remove the session state object */
              sessionStateTable.remove( Thread.currentThread() ) ;
          }
          else {
              writeLog( "service()  Error, not initialized") ;
              String htmlOut = "("+ getPropertyFileName() + ":"+ propertyInstanceName + ") Unable to process request service not Initialized ("+sessionId+") "+
                               new Date()+"<!-- \n service not initialized -->" ;
              try {
                 writeLog( "Exception caught in service method:  Check jvm_stderr.log") ;
                 htmlOut = StandardError.get( "IBM", "Service Temporarily Unavailable " + new Date() , getPropertyFileName() + 
                                              ":" + propertyInstanceName + " state -2 <!-- \n Service not initialized -->" ) ;
              }
              catch ( Exception ex ) {
                 ex.printStackTrace() ;
              }
              /* This is used to write our output to the WebServer */
              ServletOutputStream servletOutput = resp.getOutputStream() ;
              servletOutput.print( htmlOut ) ;
          }
       }
       catch ( Exception e ) {
          e.printStackTrace() ;
          String htmlOut = "("+ getPropertyFileName() + ":"+ propertyInstanceName + ") Unable to process request ("+sessionId+") "+
                           new Date()+"<!-- \n"+ StandardException.getStackTrace( e ) + " -->" ;
          try {
             writeLog( "Exception caught in service method:  Check jvm_stderr.log") ;
             htmlOut = StandardError.get( "IBM", "Service Temporarily Unavailable " + new Date() , getPropertyFileName() + 
                                          ":" + propertyInstanceName + " state -1 <!-- \n" + StandardException.getStackTrace( e ) + " -->" ) ;
          }
          catch ( Exception ex ) {
             ex.printStackTrace() ;
          }
          /* This is used to write our output to the WebServer */
          ServletOutputStream servletOutput = resp.getOutputStream() ;
          servletOutput.print( htmlOut ) ;
          /* The response object may have been substituted for our ServletBaseResponse */
          if ( resp instanceof ServletBaseResponse) {
             /* Write out what is in the buffers */
             ((ServletBaseResponse)resp).writeOutput( sessionState ) ;
          }
       }
   } /* service */ 

   /*********************************************************
   * All units of work for the servlet are performed in this
   * method.  This method is called by the service method of
   * this servlet.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  2:23pm
   * @param
   * @return
   *********************************************************/
   abstract public void servletService ( HttpServletRequest req , HttpServletResponse resp )
                         throws ServletException, IOException, MalformedURLException ;



   /*********************************************************
   * This gets the current session state for this servlet.
   * 
   *
   * Author:  
   * Created: 7/20/100  4:47pm
   * @param
   * @return
   *********************************************************/
   public SessionState getSessionState() {
      SessionState sessionState = (SessionState) sessionStateTable.get( Thread.currentThread() ) ;
      return ( sessionState ) ;
   }
   /*********************************************************
   * Log activity from this servlet.  Servlets log activity in
   * the WebSphere log file.  We need to capture some additional
   * information to identify our servlet in the log.  The debugLevel
   * property must be equal to or higher than the level value for
   * the message to be logged.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  2:39pm
   * @param info The message to log.
   * @param level The detail level of this message.
   * @return
   *********************************************************/
   public void writeLog ( String info , int level) {
       if ( loggingEnabled ) {
          if (( level > -1 ) && ( level <= logLevel) ) {
             writeLog( info ) ;
          }
          else {
             /* FINISH CODE */
             /* Detail logging of negative number range */
          }
       }
  }
   /*********************************************************
   * Log activity from this servlet.  Servlets log activity in
   * the WebSphere log file.  We need to capture some additional
   * information to identify our servlet in the log.  This
   * is always logged.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  2:39pm
   * @param info The message to log.
   * @return
   *********************************************************/
   public void writeLog ( String info ) {
       long time = 0 ;
       SessionState sessionState = getSessionState() ;
       if ( loggingEnabled ) {
          String sessionId = (String) serviceId.get( Thread.currentThread() ) ;
          /* If session not found this may be a child thread */
          /* We can not determine the correct sessionId      */
          if ( sessionId == null ) {
              sessionId = "unknown" ;
          }

          /* The session state will be null until the servlet */
          /* executes the service method for the first time   */
          if ( sessionState != null ) {
             /* Get the time the session started */
             Long beginTimeInteger = (Long) beginTime.get( sessionState.getId() ) ;
             if ( beginTimeInteger != null ) {
                time = (long) beginTimeInteger.longValue() ;
             }
             /* What is the time difference */
             time = ((long)( System.currentTimeMillis() - time )) ;
          }

          /* Write to the websphere log file */
          /* If we have a session state object log the session id */
          if ( sessionState != null ) {
             System.out.println (  getPropertyFileName() + ":" + propertyInstanceName + "("+version+", session:"+sessionId+"-"+sessionState.getId()+", "+ time + " ms ):  "+ info) ; 
          }
          else 
             System.out.println (  getPropertyFileName() + ":" + propertyInstanceName + "("+version+", session:"+sessionId+", "+ time + " ms ):  "+ info) ; 
       }
   }
   /*********************************************************
   * Writes out the request data to the log file for debugging.
   * 
   *
   * Author:  
   * Created: 9/15/100  7:42pm
   * @param
   * @return
   *********************************************************/
   public void logRequest( HttpServletRequest req ) {
      StringBuffer requestInfo = new StringBuffer() ;
      Enumeration enum = req.getParameterNames () ;
      requestInfo.append( "Logging Request Information:\nParameter Names: \n" ) ;
      while (enum.hasMoreElements()) {
         String parameterName    = (String) enum.nextElement() ;
         String parameterValue   = (String) req.getParameter( parameterName ) ;
         requestInfo.append( parameterName + "=" + parameterValue + "\n" ) ;
      }

      requestInfo.append( "\nHeader Names: \n" ) ;
      enum = req.getHeaderNames() ;
      while (enum.hasMoreElements()) {
         String parameterName    = (String) enum.nextElement() ;
         String parameterValue   = (String) req.getHeader( parameterName ) ;
         requestInfo.append( parameterName + "=" + parameterValue + "\n" ) ;
      }
      requestInfo.append( "query string=" + req.getQueryString() +"\n" ) ;
      writeLog( requestInfo.toString()) ;
   }
}
