package com.ibm.nzna.shared.servlet ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.servlet.filter.* ;
/*********************************************************
* 
* 
*
* Legal: Copyright (c) 2000 IBM. All Rights Reserved.
* Created: 4/24/100  5:32pm
* @author 
* @version
*********************************************************/
public class ServletBaseResponse implements HttpServletResponse {
   private HttpServletResponse         httpServletResponse        = null ;
   private ContentFilterManager       contentFilterManager      = null ;
   private ServletOutputStreamFilter   servletOutputStreamFilter  = null ;
   private PrintWriterFilter           printWriterFilter          = null ;
   private Properties                  properties                 = null ;
   /* Logging module */
   LogModule                           log                        = null ;
   /*********************************************************
   * 
   * 
   *
   * Author:  
   * Created: 4/24/100  5:32pm
   * @param
   * @return
   *********************************************************/
   public ServletBaseResponse ( Properties properties, HttpServletResponse httpServletResponse, ContentFilterManager contentFilterManager ) {
      this.httpServletResponse   = httpServletResponse ;
      this.contentFilterManager = contentFilterManager ;
      this.properties            = properties ;
      /* Get the logging module */
      log = LogManager.getLogModule ( properties ) ;

   }

   public void addCookie(Cookie cookie ) {
      httpServletResponse.addCookie( cookie )  ;
   }
   public boolean containsHeader(String header ) {
      return( httpServletResponse.containsHeader( header ) ) ;
   }
   public String encodeURL(String url ) {
      return( httpServletResponse.encodeURL( url ) ) ;
   }
   public String encodeRedirectURL(String url) {
      return( httpServletResponse.encodeRedirectURL( url ) ) ;
   }
   public String encodeUrl(String url) {
      return( httpServletResponse.encodeUrl( url ) ) ;
   }
   public String encodeRedirectUrl(String url) {
      return( httpServletResponse.encodeRedirectUrl( url ) ) ;
   }
   public String getCharacterEncoding() {
      return( httpServletResponse.getCharacterEncoding() ) ;
   }
   public void sendError(int code , String desc) throws IOException {
      httpServletResponse.sendError( code, desc ) ;
   }
   public void sendError(int code) throws IOException {
      httpServletResponse.sendError( code ) ;
   }
   public void sendRedirect(String redirect) throws IOException {
      httpServletResponse.sendRedirect( redirect ) ;
   }
   public void setDateHeader(String header , long value) {
      httpServletResponse.setDateHeader( header, value ) ;
   }
   public void setHeader(String header , String value ) {
      httpServletResponse.setHeader( header, value ) ;
   }
   public void setIntHeader(String header, int value) {
      httpServletResponse.setIntHeader( header, value ) ;
   }
   public void setStatus(int status ) {
      httpServletResponse.setStatus( status ) ;
   }
   public void setStatus(int status, String desc )  {
      httpServletResponse.setStatus( status, desc ) ;
   }
   public ServletOutputStream getOutputStream() throws IOException {
      /* Create a ServletOutputStreamFilter */
      servletOutputStreamFilter = new ServletOutputStreamFilter( httpServletResponse.getOutputStream()) ;
      return( servletOutputStreamFilter ) ;
   }
   public PrintWriter getWriter() throws IOException {
      /* Creat a PrintWriterFilter */
      printWriterFilter = new PrintWriterFilter ( httpServletResponse.getWriter()) ;
      return( printWriterFilter ) ;
   }
   public void setContentLength(int length ) {
      httpServletResponse.setContentLength( length ) ;
   }
   public void setContentType(String type ) {
      httpServletResponse.setContentType( type ) ;
   }
   void writeOutput ( SessionState sessionState ) {
      String output = "" ;
      byte[] bufferBytes = null ;
      char[] bufferChars = null ;

      if ( servletOutputStreamFilter != null ) {
         bufferBytes = servletOutputStreamFilter.getBuffer() ;
         output      = new String( bufferBytes ) ;
         log.writeLog("ServletBaseResponse: content length: " + output.length(), 11 ) ;
         output      = contentFilterManager.filter( output, sessionState ) ;
         output.trim() ;
         /* Get the real print writer to write the output */
         try {
            log.writeLog("ServletBaseResponse: content length: " + output.length(), 11 ) ;
            log.writeLog("ServletBaseResponse: content buffer: \n"+ output + 
                         "\n---   End ServletBaseResponse ---\n\n\n" , 11 ) ;
            ServletOutputStream outputStream = servletOutputStreamFilter.getOutputStream() ;
            setContentLength ( output.length() ) ;
            outputStream.println( output ) ;
            outputStream.close() ;
         }
         catch ( Exception e ) {
            e.printStackTrace() ;
         }
      }
      else if ( printWriterFilter != null ) {
         bufferChars = printWriterFilter.getBuffer() ;
         output      = new String( bufferChars ) ;
         log.writeLog("ServletBaseResponse: content length: " + output.length(), 11 ) ;
         output      = contentFilterManager.filter( output, sessionState ) ;
         output.trim() ;
         /* Get the real print writer to write the output */
         try {
            log.writeLog("ServletBaseResponse: content length: " + output.length(), 11 ) ;
            log.writeLog("ServletBaseResponse: content buffer: \n"+ output + 
                         "\n---   End ServletBaseResponse ---\n\n\n" , 11 ) ;
            PrintWriter writer = printWriterFilter.getWriter() ;
            setContentLength( output.length() ) ;
            writer.println( output ) ;
            writer.close() ;
         }
         catch ( Exception e ) {
            e.printStackTrace() ;
         }
      }
      //System.out.println("ServletBaseResponse: writeOutput: "+ output) ;
   }
   /*********************************************************
   * 
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/24/100  5:32pm
   * @author 
   * @version
   *********************************************************/
   class ServletOutputStreamFilter extends ServletOutputStream {
      private ServletOutputStream servletOutputStream = null ;
      private int nextByte = 0;
      private final int defaultBufferSize = 8192;
      /* The buffer we write into */
      private byte buffer[] = new byte[defaultBufferSize];

      public ServletOutputStreamFilter( ServletOutputStream servletOutputStream ) {
         this.servletOutputStream = servletOutputStream ;
      }
      /** Flush the stream. */
      public void flush() throws IOException {
         //servletOutputStream.flush() ;
      }
      /** Close the stream. */
      public void close() throws IOException {
         //servletOutputStream.close() ;
      }
      /** Write a portion of an array of characters. */
      public void write(int c) throws IOException {
         //servletOutputStream.write( c ) ;
         checkBuffer( 1 ) ;
         buffer[nextByte++] = (byte)c;
      }
      public void write( byte b[], int off, int len ) {
         //servletOutputStream.write ( b, off, len ) ;
         writeBuffer ( b, off, len ) ;
      }
      /** Write to the buffer **/
      final private void writeBuffer( byte b[], int off, int len ) {
         /* Ensure the capacity of the buffer */
         checkBuffer( len ) ;

         System.arraycopy(b, off, buffer, nextByte, len);
         nextByte += len;
      }
      /** Ensure the capacity of the buffer **/
      final private void checkBuffer( int len ) {
         if ( (nextByte + len) >= buffer.length) {
            /* exponentially increase capacity to meet the */
            /* requirements to capture the length.         */
            int newsize = buffer.length * ((nextByte + len) / buffer.length) * 2 ;
            byte[] oldBuffer = buffer ;
            /* Create new character buffer */
            buffer = new byte[newsize] ;
            /* Double the buffer size */
            System.arraycopy(oldBuffer, 0 , buffer, 0, newsize/2) ;
         }
      }
      byte[] getBuffer() {
         /* Trim the return data to the size of the data */
         byte returnBuffer[] = new byte[ nextByte ] ;
         System.arraycopy( buffer, 0, returnBuffer, 0, nextByte ) ;
         return ( returnBuffer ) ;
      }
      /*********************************************************
      * Returns the underlying output stream.
      * 
      *
      * Author:  
      * Created: 5/8/100  3:12pm
      * @param
      * @return
      *********************************************************/
      ServletOutputStream getOutputStream() {
         return ( servletOutputStream ) ;
      }
   }
   /*********************************************************
   * 
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/24/100  5:32pm
   * @author 
   * @version
   *********************************************************/
   class PrintWriterFilter extends PrintWriter {
      /* The underlying printwriter object */
      private PrintWriter printWriter = null ;
      private int nextChar = 0;
      private final int defaultCharBufferSize = 8192;
      private String lineSeparator = null ;

      /* The buffer we write into */
      private char cb[] = new char[defaultCharBufferSize];
      /* The size of the buffer */
      private int nChars = defaultCharBufferSize ;

      public PrintWriterFilter ( PrintWriter printWriter ) {
         super((Writer)printWriter) ;
         this.printWriter = (PrintWriter)printWriter ;
         lineSeparator = System.getProperty("line.separator");
      }
      /** Flush the stream. */
      public void flush() {
         /* Do not allow flushing of the buffer */
         /* We will need to filter the output   */
         /* before it is written to the underlying */
         /* printWriter.                        */
         //printWriter.flush() ;
      }
      /** Close the stream. */
      public void close() {
         /* Apply filters and write the stream */
         
         //printWriter.close() ;
      }
      /** Write a portion of an array of characters. */
      public void write(char buf[], int off, int len) {
         //printWriter.write( buf, off, len ) ;
         /* We will write to our buffer instead of writing   */
         /* to the printWriter.  This will allow us to apply */
         /* the filters to the buffer area                   */
         writeBuffer ( buf, off, len ) ;
      }
      /** Write a string */
      public void write( String s, int off, int len ) {
         int b = off, t = off + len;
         while (b < t) {
            int d = Math.min(nChars - nextChar, t - b);
            s.getChars(b, b + d, cb, nextChar);
            b += d;
            nextChar += d;
            checkBuffer( 1 ) ;
         } 
      }
      /** Write a character */
      public void write (int c) {
         checkBuffer( 1 ) ;
         cb[nextChar++] = (char) c;
      }
      /** newline character */
      public void newLine() {
         write( lineSeparator ) ;
         System.out.println("newline") ;
      }
      /** Print a String, and then finish the line. */
      public void println(String x) {
         print(x);
         newLine();
      }

      /** Write to the buffer **/
      final private void writeBuffer( char cbuf[], int off, int len ) {
         /* Ensure the capacity of the buffer */
         checkBuffer( len ) ;

         int b = off, t = off + len;
         while (b < t) {
            int d = Math.min(nChars - nextChar, t - b);
            System.arraycopy(cbuf, b, cb, nextChar, d);
            b += d;
            nextChar += d;
            //if (nextChar >= nChars)
            //flushBuffer();
         }
      }
      /** Ensure the capacity of the buffer **/
      final private void checkBuffer( int len ) {
         if ( (nextChar + len) >= nChars) {
            /* exponentially increase capacity to meet the */
            /* requirements to capture the length.         */
            nChars *= ((nextChar + len) / nChars) * 2 ;
            char[] oldBuffer = cb ;
            /* Create new character buffer */
            cb = new char[nChars] ;
            /* Double the buffer size */
            System.arraycopy(oldBuffer, 0 , cb, 0, nChars/2) ;
         }
      }
      char[] getBuffer() {
         char returnBuffer[] = new char[ nextChar ] ;
         System.arraycopy( cb, 0, returnBuffer, 0, nextChar ) ;
         return ( cb ) ;
         //return ( (new String("Hello")).toCharArray() ) ;
      }
      /*********************************************************
      * Returns the underlying print writer.
      * 
      *
      * Author:  
      * Created: 5/8/100  3:11pm
      * @param
      * @return
      *********************************************************/
      PrintWriter getWriter() {
         return ( printWriter ) ;
      }
   }
}

