package com.ibm.nzna.shared.servlet ;   
import java.io.Serializable ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.util.* ;
   /*********************************************************
   * Contains session state information about the web user.
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/19/100  1:03pm
   * @author Chad Meadows
   * @version
   *********************************************************/
public class SessionState implements Serializable {
   /* The request is the current issued request for this session */
   /* this contains all of the headers and parameters received   */
   /* from the last request issued to the web server.            */
   private HttpRequest  request              = null ;
   /* This is the persistent session state data which is kept    */
   /* from each transaction forward.                             */
   private Hashtable    sessionData          = new Hashtable() ;
   /* The time of the first issued request for the session       */
   private Date         sessionStartTime     = new Date() ;
   /* The time of the last issued request for the session        */
   private Date         lastTransactionTime  = null ;
   /* The unique session identifier for this web user.           */
   private String       sessionId            = null ;
   /* This is the transaction number of the request for the      */
   /* session.  This number is incremented for each request.     */
   /* The number is stored in the url's of the html so that      */
   /* If the user clicks back on the browser the transaction     */
   /* sequence number is effectively rolled back.                */
   private int          transactionSequence  = 0 ;

   /*********************************************************
   * Default Constructor
   * 
   *
   * Author:  
   * Created: 4/19/100  5:04pm
   * @param
   * @return
   *********************************************************/
   public SessionState ( String id ) {
      this.sessionId = id ;
   }
   /*********************************************************
   * Default Constructor
   * 
   *
   * Author:  
   * Created: 4/19/100  5:04pm
   * @param
   * @return
   *********************************************************/
   public SessionState ( String id, HttpServletRequest req ) {
      this.sessionId = id ;
      /* Create the request objet */
      request = new HttpRequest ( req ) ;
   }

   /*********************************************************
   * Sets http headers.  It is assumed this is done for each
   * web transaction and only once per web transaction.  This
   * method will keep a sequence count based on the number
   * of invocations.  The sequence count must be kept consistent
   * with the actual number of web transactions.
   * 
   *
   * Author:  
   * Created: 4/19/100  5:04pm
   * @param httpHeaders The http headers received in the request.
   * @return
   *********************************************************/
   public void setRequest ( HttpServletRequest req ) {
      /* Set our httpHeaders */
      /* Increment the sequence counter */
      /* FINISH CODE */
   }
   /*********************************************************
   * Returns the current request object for this session.
   *
   * Author:  
   * Created: 4/19/100  5:04pm
   * @return Current request object.
   *********************************************************/
   public HttpRequest getRequest () {
      return ( request ) ;
   }

   /*********************************************************
   * Returns the Id or index for this session.  This is the 
   * id value used to store and retrieve the session state.
   * 
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @param
   * @return The id value of this session.
   *********************************************************/
   public String getId () {
      return ( sessionId ) ;
   }
   /*********************************************************
   * Gets the sequence number for this session.
   * 
   *
   * Author:  
   * Created: 5/24/100  10:05pm
   * @param
   * @return
   *********************************************************/
   public int getSequence() {
      return (transactionSequence) ;
   }

   /*********************************************************
   * Gets the time this session was started 
   * 
   *
   * Author:  
   * Created: 5/24/100  9:19pm
   * @return The time the session was started.
   *********************************************************/
   public String getSessionStartTime() {
      return (sessionStartTime.toString()) ;
   }
   /*********************************************************
   * Sets the unique id for this session state.
   * 
   *
   * Author:  
   * Created: 5/2/100  2:11pm
   * @param
   * @return
   *********************************************************/
   public void setId ( String sessionId ) {
      this.sessionId = sessionId ;
   }
   /*********************************************************
   * Sets the sequence number for this session.
   *
   * Author:  
   * Created: 4/26/100  4:58pm
   * @param
   * @return 
   *********************************************************/
   void setSequence( int sequence ) { 
      /* Store the values in the request hashtable */
      transactionSequence = sequence ;
   }
   /*********************************************************
   * Gets the session data object
   * 
   *
   * Author:  
   * Created: 11/14/100  9:40pm
   * @param
   * @return Hashtable
   *********************************************************/
   public Hashtable getSessionData() {
      return ( sessionData ) ;
   }
}

