package com.ibm.nzna.shared.servlet ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.mq.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.lang.* ;
import com.ibm.nzna.shared.servlet.contentmodule.* ;
   /*********************************************************
   * ViewPort renders The HTML data received from the input
   * interface.  The data from the input interface is piped through
   * a formatting interface and finally to a rendering interface.
   * This output is HTML designed to be displayed in an encapsulated
   * area such as a table cell.
   * The ViewPortInput class is dynamically loaded.  This class is
   * specified in the properties file by the key 'viewPortInput.classname'.
   * The value should be the fully qualified classname of the class.
   * Import Filters may be specified in the properties file.  Multiple
   * input filters will be chained together. See ViewPortFilter.
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/14/100  12:58pm
   * @author 
   * @version
   *********************************************************/
public class ViewPort extends ServletBase {
   private final  String                  propertyFilename        = "ViewPort" ;
   private        ContentModule           contentModule           = null ;
   public  final  String                  contentModuleClassName  = "contentModule.classname" ;
   /*********************************************************
   * This initialization is called only on the first call to
   * the service method.  This allows for some initialization
   * which can not occur in the standard init because of 
   * some information which is only available on the first
   * call to the service method.
   * 
   *
   * Author:  Chad Meadows 
   * Created: 3/28/100  7:56pm
   * @param
   * @return
   *********************************************************/
   public boolean serviceInitialize () {
      super.serviceInitialize () ;
      try {
         /* Clone the properties object to pass to the interface */
         Properties viewPortProperties = (Properties) properties.clone() ;
         /* Initialize the contentModule class */
         String propertyValue = properties.getProperty( contentModuleClassName ) ;
         /* Parse the property value into the class name and the property file */
         StringTokenizer tokenizer = new StringTokenizer( propertyValue ) ;
         /* Get the classname fromt the string */
         String classname = tokenizer.nextToken() ;
         /* If there is another value it is the property file */
         if ( tokenizer.hasMoreElements() ) {
            String propertyFile = tokenizer.nextToken() ;
            /* Load the properties file for the view port */
            if (!StandardPropertyLoader.update( viewPortProperties, propertyFile )) {
               writeLog( "unable to load="+ propertyFile) ;
            }
         }

         if ( classname != null ) {
            /* Load the contentModule interface implementation class */
            Class contentModuleClass = Class.forName( classname ) ;
            /* Create an instance of the class */
            contentModule = (ContentModule) contentModuleClass.newInstance() ;
            writeLog("Loaded class: "+classname ) ;
            /* Initialize the contentModule */
            /* Initialize the view port Input interface */
            contentModule.init( viewPortProperties ) ;
         }
         else {
            /* Required property not found */
            writeLog ( "contentModule.classname is not defined") ;
            /* initialization has failed */
            return ( false ) ;
         }
      }
      catch ( Exception e ) {
         e.printStackTrace() ;
         return ( false );
      }
      return (true ) ;
   }

   /*********************************************************
   * Returns the propertyFile name which will be loaded in
   * the init method of the servlet.  The property file is 
   * located by searching the classpath of the vm in which
   * the servlet is running.  The filename should be returned
   * without any extension.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  1:37pm
   * @param
   * @return A String of the property file name without extension.
   *********************************************************/
   protected String getPropertyFileName() {
      return ( propertyFilename ) ;
   }
   /*********************************************************
   * All units of work for the servlet are performed in this
   * method.  This method is called by the service method of
   * this servlet.
   * 
   *
   * Author:  Chad Meadows
   * Created: 3/29/100  2:23pm
   * @param
   * @return
   *********************************************************/
   public void servletService ( HttpServletRequest req , HttpServletResponse resp )
                         throws ServletException, IOException, MalformedURLException {


      /* This is used to write our output to the WebServer */
      ServletOutputStream servletOutput = resp.getOutputStream() ;

      /* Collect the input from the input interface */
      writeLog( "Collecting Input", 5) ;
      String html = null ;
      try {
         html = contentModule.getHTML( getSessionState() ) ;
      } catch ( StandardException e ) {
         e.printStackTrace() ;
         html = StandardError.get( "IBM", "Template Loader Failed", "id:("+e.getErrorCode()+")") ;
      }

      /* End the HTML data tag */
      writeLog("Sending output", 5) ;

      servletOutput.print( html ) ;
   }
}
