
package com.ibm.nzna.shared.servlet.contentmodule ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.mq.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.lang.* ;
import java.util.* ;
   /*********************************************************
   * This class provides a data connection to a shared Data Manager.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/19/100  1:43pm
   * @author 
   * @version
   *********************************************************/
public abstract class SharedInterface implements ContentModule {
   /** The basename property including the servlet instance name **/
   SharedDataManager sharedDataManager = null ;
   Properties        properties        = new Properties() ;
   /* Logging module */
   LogModule         log               = null ;
   /*********************************************************
   * Initialize the input class.  This is provided for classes
   * which may need some prior initialization for efficiency.
   * This would be database connections or other similar tasks.
   * 
   *
   * Author:  
   * Created: 4/19/100  12:38pm
   * @param properties The properties used to initialize this class.
   * @return
   *********************************************************/
   public void init( Properties properties ) {
      this.properties = properties ;
      /* Get the logging module */
      log = LogManager.getLogModule ( properties ) ;

      /* Install shared data Manager          */
      String value = properties.getProperty( "sharedDataManager.classname" ) ;
      /* Load the shared data manager class */
      if ( value != null ) {
         String classname = value ;
         try {
            Class sharedDataManagerClass = Class.forName( classname ) ;
            /* Create an instance of the class */
            sharedDataManager = (SharedDataManager) sharedDataManagerClass.newInstance() ;
            log.writeLog("ViewPortSharedDataInput:  Loaded class: "+classname ) ;
            /* Get the property file name for the shared data manager */
            value = properties.getProperty( "sharedDataManager.properties") ;
            log.writeLog("Loading properties: " + value , 10) ;
            Properties sharedDataManagerProperties = (Properties) properties.clone() ;
            /* Load the properties */
            StandardPropertyLoader.update ( sharedDataManagerProperties, value ) ;
            /* Initialize the shared data manager */
            sharedDataManager.init( sharedDataManagerProperties ) ;
         }
         catch ( Exception e ) {
            System.err.println( e ) ;
         }
      }
      else {
         log.writeLog( "ViewPortSharedInterface: sharedDataManager.classname not defined") ;
      }
   }

   /*********************************************************
   * Collect the html to present to the page.  The session data
   * is passed to the sharedDataManager.  An application using
   * the sharedDataManager will collect our session data and
   * produce a html result to be displayed.  The result is then
   * returned to us in a Hashtable with the key.
   * 
   *
   * Author:  
   * Created: 4/18/100  1:36pm
   * @param sessionState the session state of the web user.
   * @return HTML string to be rendered on the web page.
   *********************************************************/
   public abstract String getHTML ( SessionState sessionState ) throws StandardException ;
   
}
