package com.ibm.nzna.shared.servlet.contentmodule ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.lang.* ;
import com.ibm.nzna.shared.servlet.template.* ;
import com.ibm.nzna.shared.servlet.filter.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.io.* ;
import HTTPClient.* ;
   /*********************************************************
   * This class is an input for receiving input in the viewPort.
   * The input is HTML format and is expected to be renderable
   * inside a table cell.
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/18/100  1:34pm
   * @author 
   * @version
   *********************************************************/
public class URLInput implements ContentModule {
   Properties properties = null ;
   public static final String contentUrl = "content.url" ;
   /* Logging module */
   LogModule               log                     = null ;

   /*********************************************************
   * Initialize the input class.  This is provided for classes
   * which may need some prior initialization for efficiency.
   * This would be database connections or other similar tasks.
   * 
   *
   * Author:  
   * Created: 4/19/100  12:38pm
   * @param propertyBasename The basename of the property file 
   * including the instance name of the servlet.
   * @return
   *********************************************************/
   public void init( Properties properties ) {
      this.properties = properties ;
      /* Get the logging module */
      log = LogManager.getLogModule ( properties ) ;
   }

   /*********************************************************
   * 
   * 
   *
   * Author:  
   * Created: 4/18/100  1:36pm
   * @param
   * @return
   *********************************************************/
   public String getHTML ( SessionState sessionState ) {
      URL                 urlContent             = null ;                                    
      InputStream         inputStream            = null ;                                    
      InputStreamReader   inputStreamReader      = null ;                                    
      BufferedReader      bufferedReader         = null ;
      HTTPConnection      httpConnection         = null ;
      String              contentURLProperty     = "" ;                                          
      String              inputLine              = "" ;
      String              content                = "" ;
      String              reqQueryString         = "" ;
      String              reqPathInfo            = "";
      String              reqParams              = "" ;
      String              http                   = "http://" ;

      HttpRequest         req                    = sessionState.getRequest() ;

      try {
         contentURLProperty   = properties.getProperty( contentUrl ) ;

         /* Determine if we should use post or get request when we */
         /* have parameters to send.                               */
         String requestMethod = properties.getProperty( "ViewPortURLInput.requestMethod", "post" ) ;

         /* If we are being used in a template module we will be assigned a template key     */
         /* We will use this key to determine the URL from which we must collect the content */
         String templateBindingKey = properties.getProperty ( TemplateModule.bindingKeyProperty ) ;
         if ( templateBindingKey != null) {
            /* Check for a request parameter using our key */
            String requestContentURLParameter = req.getParameter( UrlRedirectFilter.inputHostKeyPrefix + templateBindingKey ) ;
            if (requestContentURLParameter != null ) {
               /* Overide the property value with the value in the request if given*/
               contentURLProperty = requestContentURLParameter ;
            }
         }

         /* If relative url, we must fully qualify */
         contentURLProperty = ServletUtil.qualifyURL( contentURLProperty, sessionState ) ;

         // Get the request path information 
         reqPathInfo    = req.getPathInfo() ;
         if ( reqPathInfo == null ) 
               reqPathInfo = "" ;


         // Get the current request query string
         reqQueryString = req.getQueryString() ;

         /* Create the url path */
         String urlPath = contentURLProperty + reqPathInfo ;

         /* If the path info ends in slash we need to remove it */
         /* for the concatenation which follows                 */
         if ( urlPath.endsWith("/")) {
            urlPath = urlPath.substring(0, urlPath.length() - 1 ) ;
         }

         if ( reqQueryString == null ) 
            reqParams = urlPath ; 
         else
            reqParams = urlPath + "?" + reqQueryString ;

         // Up to this point the request params contain "http://" so we need to strip this string and add it back later
         reqParams = reqParams.substring( reqParams.indexOf(http)+http.length(), reqParams.length() ) ;

         // Form the URL with the request parameters and initiate the connection to it
         urlContent        = new URL( http + reqParams ) ;
         // Get the parameters to do the POST
         int e_len = req.getNumParameters() ;
         /* Create the array of parameters we will use to */
         /* create our http connection.                   */
         NVPair form_data[] = new NVPair[e_len] ;
         /* Enumerate the request values */
         Enumeration e = req.getParameterNames();
         for ( int i=0; e.hasMoreElements() ; i++ )  {
            String name = (String)e.nextElement();
            /* A name may have multiple values as from a html form */
            String[] values = req.getParameterValues( name ) ;
            for ( int valueCount = 0 ; valueCount < values.length ; valueCount++ ) {
               form_data[i] = new NVPair( name, values[ valueCount ] ) ;
            }
         }

         log.writeLog( "ViewPortURLInput: connecting to :"+urlContent) ;
         httpConnection    = new HTTPConnection( urlContent ) ;

         /* Turn off user interaction */
         httpConnection.setAllowUserInteraction( false ) ;

         // Set a Timeout for the response from the http connection
         httpConnection.setDefaultTimeout( (Integer.parseInt( properties.getProperty( "httpClient.httpWait" ) )) * 1000 ) ;

         /* Do a post if we have parameters to send, otherwise use a get request */
         HTTPResponse httpResponse = null ;
         if ( form_data.length == 0 ) {
            httpResponse = httpConnection.Get( urlContent.getFile() ) ;
         }
         else if ( requestMethod.equalsIgnoreCase("post")) {
            /* All of the parameters are posted */
            httpResponse = httpConnection.Post( urlContent.getFile(), form_data ) ;
         }
         else {
            /* The parameters must be passed on the query string */
            httpResponse = httpConnection.Get( urlContent.getFile() ) ;
         }

         // Open an input stream to read the contents from the url connection
         inputStream       = httpResponse.getInputStream() ;
         //inputStreamReader = new InputStreamReader( inputStream ) ;
         //bufferedReader    = new BufferedReader( inputStreamReader ) ;
         // Read the contents of the input stream into a string
         //while ( (inputLine = bufferedReader.readLine()) != null ) 
         //   content += inputLine + "\n" ;
         //bufferedReader.close() ;
         BufferedInputStream bufferedStream = new BufferedInputStream( inputStream ) ;
         byte streamBytes[] = InputStreamLoader.load( bufferedStream ) ;
         content = new String( streamBytes ) ;
         bufferedStream.close() ;

      } // try   
      catch (IOException ioe ) {
         ioe.printStackTrace() ;
         /* Set the content to an appropriate error message */
         try {
            content = StandardError.get( "IBM", "Service Temporarily Unavailable " + new Date() , 
                                         "ViewPortURLInput state -4 <!-- \n" + StandardException.getStackTrace( ioe ) + " -->" ) ;
            log.writeLog("ViewPortURLInput: unable to connect") ;
         }
         catch ( Exception e ) {
            e.printStackTrace() ;
         }
      }
      catch (ModuleException me ) {
         System.err.println( "Error handling request: " + me.getMessage() ) ;
      }
      catch (Exception e ) {
         e.printStackTrace() ;
      }
      /* Return the collected output from the URL */
      log.writeLog( "ViewPortURLInput:  Content: \n" + content + "\n--- ViewPortURLInput End Content --- \n\n\n", 11 ) ;

      return ( content ) ;
   }
}
