package com.ibm.nzna.shared.servlet.template ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.io.* ;
import com.ibm.nzna.shared.lang.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.servlet.viewport.filter.* ;
import com.ibm.nzna.shared.servlet.viewport.* ;

   /*********************************************************
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/14/100  5:50pm
   * @author 
   * @version
   *********************************************************/
public class DolphinScriptProcessor {
    /* Logging module */
    LogModule                log                     = null ;
    Properties               properties              = null ;

    public DolphinScriptProcessor ( Properties properties ) {
        init ( properties ) ;
    }
    /*********************************************************
    * Initializes the module based on the properties input.
    * servlet.template.module.bindings=The bindings property file.
    * servlet.template.module.filters=The filters property file.
    *
    * Author:  
    * Created: 4/14/100  5:52pm
    * @param
    * @return
    *********************************************************/
    public void init( Properties properties ) {
       /* Get the logging module */
       log = LogManager.getLogModule ( properties ) ;
       this.properties = properties ;
    }
    public String processScript( SessionState sessionState , String script ) {
        int            scriptIndex       = 0 ;
        int            lastscriptIndex   = 0 ;
        StringBuffer   outputBuffer         = new StringBuffer(script.length()) ;
        char[]         inputArray           = script.toCharArray() ;
        final String   dolphinTag           = "<!--:dolphin:" ;
        final String   srcTag               = "src=" ;
        final String   openTag              = "<" ;
        HttpRequest    req                  = sessionState.getRequest() ;
        URL            hostURL              = null ;

        /* We must find all hrefs in anchor tags */
        while ( (scriptIndex = script.indexOf( dolphinTag, scriptIndex)) != -1 ) {

           /* Log */
           log.writeLog( "Dolphin Script detected in template" ) ;

           /* Set the end position of the tag */
           int endOfScriptIndex = script.indexOf("-->", scriptIndex ) ;

           /* If there was no closing --> skip parsing */
           if ( endOfScriptIndex == -1 ) {
              break ;
           }

           /* Get the value of the dolphin tag */
           String tagValue = script.substring( scriptIndex + dolphinTag.length(), endOfScriptIndex ) ;
           int valuePos = tagValue.indexOf(':') ;
           String command  = tagValue.substring( 0, valuePos  ) ;
           String value    = tagValue.substring( valuePos + 1 ) ;

           log.writeLog("lastscriptIndex = " + lastscriptIndex + "scriptIndex = " + scriptIndex ) ;
           log.writeLog("context: " +  inputArray ) ;
           /* Add to new output buffer from the input up to the tag found */
           outputBuffer.append(inputArray, lastscriptIndex, scriptIndex - lastscriptIndex ) ;

           /* write the output of the command processed */
           outputBuffer.append( processCommand( sessionState, command , value ) ) ;

           /* set the last input data index to the end tag */
           scriptIndex = lastscriptIndex = endOfScriptIndex + 3 ;
        }

        /* Add the remaining of script after the last tag */
        outputBuffer.append( inputArray, lastscriptIndex, script.length() - lastscriptIndex ) ;

        return ( outputBuffer.toString() ) ;

    }
    public String processCommand ( SessionState sessionState, String command , String value) {
        log.writeLog("Processing Dolphin Script: " + command + ":" + value ) ;
        String processedResult = "" ;
        if ( command.equals("file")) {
            processedResult = processCommandFile ( sessionState, value ) ;
        }
        else if ( command.equals("class")) {
            processedResult = processCommandClass ( sessionState, value ) ;
        }
        else if ( command.equals("resource")) {
            processedResult = processCommandResource ( sessionState, value ) ;
        }
        else if ( command.equals("fill")) {
            processedResult = processCommandFill ( sessionState, value ) ;
        }
        else if ( command.equals("fillpath")) {
            processedResult = processCommandFillPath ( sessionState, value ) ;
        }
        return ( processedResult ) ;
    }
    private String processCommandFile ( SessionState sessionState, String value ) {
        File file = new File ( value ) ;
        if ( file.exists()) {
            try {
                FileInputStream fileInputStream = new FileInputStream( file ) ;
                String result = new String(InputStreamLoader.load( fileInputStream )) ;
                return ( result ) ;
            }
            catch ( Exception e ) {
                e.printStackTrace() ;
            }
        }
        return ("") ;
    }
    private String processCommandClass ( SessionState sessionState, String value ) {
        return ( null ) ;
    }
    private String processCommandResource ( SessionState sessionState, String value ) {
        return ( null ) ;
    }
    private String processCommandFill ( SessionState sessionState, String value ) {
        String      result  = "" ;
        HttpRequest request = sessionState.getRequest() ;
        String parameter = (String) request.getParameter( value ) ;
        if ( parameter != null ) {
            if ( parameter.indexOf("http") != -1) {
                try {
                    URL url = new URL ( parameter ) ;
                    result = new String( InputStreamLoader.load( url.openStream() )) ;
                }
                catch ( Exception e ) {
                    e.printStackTrace() ;
                }
            }
            else if ( parameter.indexOf( "file") != -1) {
            }
        }
        return ( result ) ;
    }
    private String processCommandFillPath ( SessionState sessionState, String value ) {
        String      result  = "" ;
        HttpRequest request = sessionState.getRequest() ;
        String pathInfo = request.getPathInfo() ;
        /* Does this path info match this command */
        if ( pathInfo.indexOf( value ) != -1 ) {
            try {
                String documentRoot = (String) properties.get( "document.root" ) ;
                String documentPath = documentRoot + pathInfo ;
                log.writeLog("DolphinScript Loading Document: " + documentPath ) ;
                File   file     = new File (documentPath) ;
                FileInputStream fileIO = new FileInputStream( file ) ;
                result = new String( InputStreamLoader.load( fileIO )) ;
            }
            catch ( Exception e ) {
                e.printStackTrace() ;
            }
        }
        return ( result ) ;
    }

}

