
package com.ibm.nzna.shared.servlet.template ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.io.* ;
import com.ibm.nzna.shared.lang.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.servlet.filter.* ;
import com.ibm.nzna.shared.servlet.contentmodule.* ;
import HTTPClient.* ;

   /*********************************************************
   * The template module contains the logic for loading the correct
   * template based on the session state data.  The PageGenTEmplateModule
   * does not properly conform to the design of template modules.
   * Template modules should not return content, but it was necessary
   * in this instance to interface with the Legacy PageGen code for
   * creating the navigation.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/14/100  5:50pm
   * @author 
   * @version
   *********************************************************/
public class PageGenTemplateModule extends TemplateModule {
   ContentFilter qualifyRelativeUrls = null ;
   public static final String pagegenUrl     = "servlet.template.module.pagegen.url" ;
   public static final String cacheSizeKey   = "servlet.template.module.pagegen.cache.size" ;
   public static final String qualifyUrl     = "servlet.template.module.pagegen.qualify.url" ;
   private Hashtable templateCache = new Hashtable() ;
   private Vector    historyVector = new Vector( ) ;
   private int       cacheSize     = 0 ;
   private boolean   qualify       = false ;

   /* Logging module */
   LogModule               log                     = null ;

   /*********************************************************
   * Initializes the module based on the properties input.
   * servlet.template.module.bindings=The bindings property file.
   * servlet.template.module.filters=The filters property file.
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   public void init( Properties properties , TemplateLoader templateLoader ) {
      super.init( properties , templateLoader ) ;

      /* Get the logging module */
      log = LogManager.getLogModule ( properties ) ;
      /* We need a filter to process the relative urls */
      qualifyRelativeUrls = (ContentFilter) new QualifyRelativeURL() ;
      /* clone the properties object */
      Properties filterProperties = (Properties) properties.clone() ;
      /* Set the content property so that unqualified url's will be */
      /* correctly qualified.                                       */
      properties.put( URLInput.contentUrl , properties.get( pagegenUrl ) ) ;
      /* Initialize the filter */
      qualifyRelativeUrls.init( properties ) ;
      /* Set the cache size */
      String cacheString   = properties.getProperty( cacheSizeKey , "0") ;
      cacheSize = Integer.parseInt( cacheString ) ;
      /* Set the qualify flag.  If set true this will fully qualify all */
      /* URL's loaded in the template.  This is usefull when pagegen is */
      /* not on the same server as the WebSphere server.                */
      String value   = properties.getProperty( qualifyUrl , "false") ;
      if ( value.equalsIgnoreCase("true")) {
         qualify = true ;
      }
   }


   /*********************************************************
   * Gets the PageGen template.  This method must get the correct
   * template depending on the the site. ( eSupport, info tips, business partner )
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   public String getTemplate ( SessionState sessionState ) {
      /* Get eSupport Template */
      return ( getPageGenTemplate ( sessionState ) ) ;
      /* Get info tips template */
      /* Get business partner template */
   }
   private String getPageGenTemplate( SessionState sessionState ) {
      //HttpServletRequest  req                   = navigationState.getRequest() ;
      HTTPConnection      httpConnection        = null ;
      HttpRequest         httpRequest           = sessionState.getRequest() ;
      String              htmlStr               = "" ;
      String              inputLine             = "" ;
      String              http                  = "http://" ;
      String              navigationURLProperty = "" ;
      String              navURL                = "" ;
      String              navContent            = "" ;
      URL                 navigationURL         = null ;
      InputStream         inputStream           = null ;                                     
      InputStreamReader   inputStreamReader     = null ;                                     
      BufferedReader      bufferedReader        = null ;
            
      String              queryString           = httpRequest.getQueryString() ;
         
      try {
         /* First attempt to lookup from cache */
         if ((cacheSize > 0 ) && ( templateCache.containsKey( sessionState.getId() ))) {
            log.writeLog("TemplateModule:  cache hit" ) ;
            navContent = (String) templateCache.get( sessionState.getId() ) ;
            return ( navContent ) ;
         }
         // Get the servlet initialization parameters and values
         navigationURLProperty   = properties.getProperty( pagegenUrl ) ;

         /* If the URL is relative, add the full path */
         navigationURLProperty = ServletUtil.qualifyURL ( navigationURLProperty, sessionState ) ;

         if ( navigationURLProperty == null ) {
            log.writeLog("TemplateModule:  property not found: "+ pagegenUrl ) ;
         }

         // Filter the URL property value and strip the "http://" substring
         navURL = navigationURLProperty.substring( navigationURLProperty.indexOf(http)+http.length(), navigationURLProperty.length() ) ;

         // Check for a query string
         /* We pass blank=yes to pageGen so pageGen will return the template */
         if ( queryString != null ) {
             navigationURL     = new URL( "http://" + navURL +"?"+"blank=yes&"+ queryString ) ;
         }
         else {
             // We do not have query string
         /* We pass blank=yes to pageGen so pageGen will return the template */
             navigationURL     = new URL( "http://" + navURL +"?"+"blank=yes" ) ;
         }

         log.writeLog("TemplateModule: opening connection: "+navigationURL) ;

         httpConnection    = new HTTPConnection( navigationURL ) ;

         /* Do not allow user interaction, pop up dlgs etc */
         httpConnection.setAllowUserInteraction( false ) ;

        
         httpConnection.setDefaultTimeout( (Integer.parseInt( properties.getProperty( "httpClient.httpWait" ) )) * 1000 ) ;
   
         HTTPResponse httpResponse = httpConnection.Get( navigationURL.getFile() ) ;
               
         
         // Open an input stream to read the contents from the url connection
         inputStream       = httpResponse.getInputStream() ;
         //inputStreamReader = new InputStreamReader( inputStream ) ;
         //bufferedReader    = new BufferedReader( inputStreamReader ) ;
         // Read the contents of the input stream into a string
         //while ( (inputLine = bufferedReader.readLine()) != null ) 
         //   navContent += inputLine + "\n" ;
         //bufferedReader.close() ;
         BufferedInputStream bufferedStream = new BufferedInputStream( inputStream ) ;
         byte streamBytes[] = InputStreamLoader.load( bufferedStream ) ;
         navContent = new String( streamBytes ) ;
         bufferedStream.close() ;

         /* We must qualify the URL's inside the content */
         if ( qualify ) {
            navContent = qualifyRelativeUrls.filter( navContent , sessionState ) ;
         }
         
         if ( cacheSize > 0) {
            /* Place in template cache */
            templateCache.put( sessionState.getId(), navContent ) ;
            historyVector.addElement(sessionState.getId()) ;

            /* Do not let cache grow too large */
            if ( historyVector.size() > cacheSize ) {
               /* Get the first item placed in history and remove it */
               String key = (String )historyVector.elementAt(0) ;
               historyVector.removeElementAt(0) ;
               /* Remove from the cache */
               templateCache.remove( key ) ;
               log.writeLog("TemplateModule: clearing cache") ;
            }
         }

      }
      catch( IOException e ) {
         e.printStackTrace() ;
            navContent = StandardError.get( "IBM", "Service Temporarily Unavailable " + new Date() , 
                                            "PageGenTemplateModule state -3 <!-- \n" + StandardException.getStackTrace( e ) + " -->" ) ;
      }
      catch( Exception e ) {
         e.printStackTrace() ;
         return (navContent) ;
      } /* try-catch */
      return ( navContent ) ;
   }
}
