package com.ibm.nzna.shared.servlet.template ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.servlet.filter.* ;
import com.ibm.nzna.shared.servlet.contentmodule.* ;
import com.ibm.nzna.shared.lang.* ;
   /*********************************************************
   * The template module contains the logic for loading the correct
   * template based on the session state data.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/14/100  5:50pm
   * @author 
   * @version
   *********************************************************/
public abstract class TemplateModule {
   /* Loaded properties for application */
   Properties              properties              = null ;
   /* The keys are the meta variables in the template */
   Hashtable               pluginBindingClasses    = new Hashtable() ;
   /* Filter managers for the plugins */
   Hashtable               pluginFilterManagers    = new Hashtable() ;
   /* The property files for the binding keys */
   Hashtable               propertyBindings        = new Hashtable() ;
   /* Cached plugin classes */
   Hashtable               cachedPlugins           = new Hashtable() ;
   /* The template loader */
   TemplateLoader          templateLoader          = null ;
   /* Logging module */
   LogModule               log                     = null ;
   /* The Dolphin Script Processor */
   DolphinScriptProcessor  dolphinScriptProcessor  = null ;

   /* The property key for loading a template module */
   public  static final  String   classnameProperty       = "servlet.template.module.classname" ;
   /* The property key for loading a template module initialization properties */
   public  static final  String   propertyFilename        = "servlet.template.module.properties" ;
   /* The meta tag key used to bind the class         */
   public  static final  String   bindingKeyProperty      = "servlet.template.module.key" ;
   /* The meta key bindings filename */
   public  static final  String   bindingsProperty        = "servlet.template.module.bindings" ;
   /*********************************************************
   * Initializes the module based on the properties input.
   * servlet.template.module.bindings=The bindings property file.
   * servlet.template.module.filters=The filters property file.
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   public void init( Properties properties , TemplateLoader templateLoader ) {
      /* Set the application properties */
      this.properties = properties ;
      /* Set the template loader*/
      this.templateLoader = templateLoader ;

      /* Get the logging module */
      log = LogManager.getLogModule ( properties ) ;

      /* Initialize the Dolphin Script Processor */
      dolphinScriptProcessor = new DolphinScriptProcessor ( properties ) ;

      String value = null ;
      value = properties.getProperty( bindingsProperty ) ;
      if ( value != null ) {
         /* The Bindings properties for plugins */
         AdvancedProperties      metaKeysBindings              = new AdvancedProperties() ;
         /* Allow the property keys to have spaces */
         metaKeysBindings.setAllowSpaceInKey( true ) ;
         /* Load the property file specified by this key */
         if ( StandardPropertyLoader.update( metaKeysBindings, value )!= true ) 
            log.writeLog("TemplateModule:  unable to load property file = "+value) ;
         /* Create the plugin and property hashtables from the property file */
         parseProperties( metaKeysBindings) ;
      }
   }
   /*********************************************************
   * Gets a template from the template loader based on data
   * store within the session state.
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   abstract public String getTemplate ( SessionState sessionState ) ;

   /*********************************************************
   * Fills in the template with the correct data using the 
   * bindings classes and filters.
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   public String fillTemplate ( SessionState sessionState, String template ) {
      /* Find bindings within the template */
      Enumeration plugins = pluginBindingClasses.keys() ;
      /* Parse all plugins for the bindings properties */
      while (plugins.hasMoreElements()) {
         String elementKey = (String) plugins.nextElement() ;
         ContentModule contentModule = null ;
         /* Is this class needed for this template */
         if ( (template.indexOf ( elementKey )) != -1 ) {
            /* Is the class already loaded */
            if ( !cachedPlugins.containsKey( elementKey )) {
               try {
                  String classname = (String) pluginBindingClasses.get ( elementKey ) ;
                  /* Get our class.  We may have been loaded with a custom */
                  /* class loader such as WebSphere 3.5                    */
                  Class thisClass = (Class) getClass() ;
                  /* We must load the plugin class */
                  Class pluginClass = thisClass.forName( classname ) ;
                  /* Create an instance of the class */
                  contentModule = (ContentModule) pluginClass.newInstance() ;
                  /* Initialize the View Port interface */
                  /* Load the properties file for the view port */
                  Properties viewPortProperties = (Properties) properties.clone() ;
                  if (!StandardPropertyLoader.update( viewPortProperties, (String) propertyBindings.get( elementKey ) )) {
                     log.writeLog( "TemplateModule:  unable to load="+(String) propertyBindings.get( elementKey )) ;
                  }
                  /* Add reference key to the properties to indicate */
                  /* The meta tag key used to bind the class         */
                  viewPortProperties.put( bindingKeyProperty , elementKey ) ;
                  /* Initialize the view port input class */
                  try {
                     contentModule.init( viewPortProperties ) ;
                  }
                  catch ( Exception e ) {
                     log.writeLog("TemplateModule:  initialization failure: "+classname ) ;
                     e.printStackTrace() ;
                  }
                  log.writeLog("TemplateModule:  loaded class="+ classname ) ;
                  /* Store the class in the cached table */
                  cachedPlugins.put( elementKey, contentModule ) ;
                  /* Create a filter manager for this plugin */
                  //Properties filterProperties = new Properties() ;
                  //StandardPropertyLoader.load ( filterProperties, (String) propertyBindings.get( elementKey ) ) ;
                  ContentFilterManager contentFilterManager = new ContentFilterManager( viewPortProperties ) ;
                  /* Store the filter manager in the filter table */
                  pluginFilterManagers.put( elementKey, contentFilterManager ) ;
               }
               catch ( Exception e ) {
                  e.printStackTrace() ;
               }
            }
            else {
               /* Set the view port input to the cached class for the key */
               contentModule = (ContentModule) cachedPlugins.get( elementKey ) ;
            }
            /* Get the input from the view port input */
            String input = null ;
            try {
               input = contentModule.getHTML( sessionState ) ;
            } catch ( StandardException e ) {
               log.writeLog("TemplateModule:  contentModule.getHTML failure: "+ contentModule.getClass().getName() ) ;
               e.printStackTrace() ;
               input = StandardError.get( "IBM", "Template Loader Failed", "id:("+e.getErrorCode()+")") ;
            }

            if ( (input != null) && (input.length() > 0 )) {
               /* The input may have embedded template meta tags    */
               /* Substitute any meta tags found if this is an      */
               /* embedded template.  we call ourselves recursively */
               /* to accomplish this.                               */
               log.writeLog( "TemplateModule:  Filling Template", 10) ;
               input = fillTemplate( sessionState, input ) ;
               log.writeLog( "TemplateModule:  Filling Template Complete", 10) ;

               /* Apply the filter manager to the input */
               ContentFilterManager filterManager = (ContentFilterManager) pluginFilterManagers.get( elementKey ) ;
               log.writeLog( "TemplateModule:  Applying filters", 10) ;
               input = filterManager.filter( input, sessionState ) ;
               log.writeLog( "TemplateModule:  Applying filters Complete", 10) ;
            } else {
               /* Set so we can return a template */
               input = "" ;
            }

            /* Replace the meta tag in the template with the input */
            template = RegularExpression.substitute( elementKey , input, template ) ;
         }
      }
      /* Perform all Dolphin Script commands */
      template = dolphinScriptProcessor.processScript ( sessionState, template ) ;

      return ( template ) ;
   }

   /*********************************************************
   * Not Implemented
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   private Hashtable getBindingClassesForTemplate( String template ) {
      return ( null ) ;
   }
   /*********************************************************
   * The bindings property file has two values per property.
   * This method parses the two values and places each of them
   * in their own hashtables.
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   private void parseProperties( Properties properties ) {
      Enumeration keys = properties.keys() ;
      while ( keys.hasMoreElements() ) {
         String keyValue = (String) keys.nextElement() ;
         String propertyValue = (String)properties.get ( keyValue ) ;
         StringTokenizer tokenizer = new StringTokenizer( propertyValue ) ;
         /* Get the classname fromt the string */
         String classname = tokenizer.nextToken() ;
         /* Store the plugin class for this meta key */
         pluginBindingClasses.put ( keyValue, classname ) ;
         /* If there is another value it is the property file */
         if ( tokenizer.hasMoreElements() ) {
            String propertyFile = tokenizer.nextToken() ;
            /* Store the property file for the metakey */
            propertyBindings.put( keyValue, propertyFile ) ;
         }
      }
   }
}
