package com.ibm.nzna.shared.servlet.viewport.filter ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.servlet.viewport.* ;
import com.ibm.nzna.shared.servlet.template.* ;
   /*********************************************************
   * Fully qualifies relative urls.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/14/100  5:50pm
   * @author 
   * @version
   * @see ViewPortFilterManager
   *********************************************************/
public class QualifyRelativeURL implements ViewPortFilter {
   Properties  properties     = null ;
   String      hostURLString  = null ;
   String      dynamicHost    = null ;
   /*********************************************************
   * Initialize the input class.  This is provided for classes
   * which may need some prior initialization for efficiency.
   * This would be database connections or other similar tasks.
   * 
   *
   * Author:  
   * Created: 4/19/100  12:38pm
   * @param The properties object.
   * 
   * @return
   *********************************************************/
   public void init( Properties properties ) {
      try {
         this.properties = properties ;
         /* Set the content url value */
         hostURLString =  properties.getProperty( ViewPortURLInput.contentUrl ) ;

      }
      catch ( Exception e ) {
         System.err.println( e ) ;
      }
   }

   /*********************************************************
   * Applies a filter to the input string and returns a String
   * result.
   * 
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   public String filter ( String inputData, SessionState sessionState ) {
      int            inputDataIndex       = 0 ;
      int            lastInputDataIndex   = 0 ;
      StringBuffer   outputBuffer         = new StringBuffer(inputData.length()) ;
      char[]         inputArray           = inputData.toCharArray() ;
      final String   hrefTag              = "a href=" ;
      final String   srcTag               = "src=" ;
      final String   openTag              = "<" ;
      HttpRequest    req                  = sessionState.getRequest() ;
      URL            hostURL              = null ;
      /* Set the host server value and port */
      try {
         hostURL = new URL( ServletUtil.qualifyURL( hostURLString, sessionState ) ) ;
      }
      catch ( Exception e ) {
         e.printStackTrace() ;
      }

      /* Set the default host server name */
      String hostServerName = hostURL.getHost() + ":" + hostURL.getPort() ; 

      /* If we are being used in a template module we will be assigned a template key */
      /* This is dynamically assigned by the template module and is not part of       */
      /* a property file definition.                                                  */
      String templateBindingKey = properties.getProperty ( TemplateModule.bindingKeyProperty ) ;
      if ( templateBindingKey != null) {
         /* Check for a request parameter using our key */
         /* This is the host name from which the input  */
         /* was collected.                              */
         /* If this value is set we dynamically change  */
         /* the unqualified url's to this parameter     */
         String requestHostParameter = req.getParameter( "viewportinput.host."+templateBindingKey ) ;
         if (requestHostParameter != null ) {
            /* The value of this key is a fully qualified url    */
            /* We will create an URL object to get the host name */
            try {
               hostURL = new URL( ServletUtil.qualifyURL(requestHostParameter, sessionState ) ) ;
               hostServerName = hostURL.getHost() + ":" + hostURL.getPort() ; 
            }
            catch ( Exception e ) {
               e.printStackTrace() ;
            }
         }
      }

      /* We must find all hrefs in anchor tags */
      while ( (inputDataIndex = inputData.indexOf( openTag, inputDataIndex)) != -1 ) {
         /* Move the index to next character after < */
         inputDataIndex++ ;

         /* Set the end position of the tag */
         int endOfHrefIndex = inputData.indexOf(">", inputDataIndex ) ;

         /* Used to locate src tags */
         int srcTagIndex = 0 ;

         /* determine if we are at an href anchor tag */
         /* or at a src parameter tag                 */
         if ( inputData.regionMatches(true, inputDataIndex, hrefTag, 0, hrefTag.length())) {
            /* move the input data index to first character after equals */
            inputDataIndex += hrefTag.length() ;

         } else if ( ((srcTagIndex = inputData.indexOf( srcTag, inputDataIndex )) < endOfHrefIndex) && srcTagIndex > 0) {
            /* FIX CODE, the above is very inefficient if the tag is not found */
            /* this tag contains a src parameter */
            /* move the input data index to first character after equals */
            inputDataIndex = srcTagIndex + srcTag.length() ;
         } else if ( ((srcTagIndex = inputData.indexOf( srcTag.toUpperCase() , inputDataIndex) ) < endOfHrefIndex) && srcTagIndex > 0) {
            /* FIX CODE, the above is very inefficient if the tag is not found */
            /* this tag contains a src parameter */
            /* move the input data index to first character after equals */
            inputDataIndex = srcTagIndex + srcTag.length() ;
         } else {
            /* No tags match we are searching for, we must continue */
            continue ;
         }

         /* Is the href embedded in quotes */
         boolean embeddedQuotes = false ;
         if ( inputArray[inputDataIndex] == '\"' ) {
            embeddedQuotes = true ;
            /* move input data index to character after quote */
            inputDataIndex++ ;
         }

         /* Add to new output buffer from the input up to the tag found */
         outputBuffer.append(inputArray, lastInputDataIndex, inputDataIndex - lastInputDataIndex ) ;

         /* Determine if this is a relative URL by searching for http in the tag */
         boolean relativeURL = !inputData.regionMatches(false, inputDataIndex, "http", 0, 4 ) ;

         /* Add the server name to qualify the URL */
         if ( relativeURL) {
            String urlPath = inputData.substring( inputDataIndex, endOfHrefIndex ) ;
            /* If the url path did not start with a / we must add a slash */
            if ( !urlPath.startsWith("/")) {
               /* Add the slash */
               urlPath = "/"+urlPath ;
            }
            outputBuffer.append( "http://"+hostServerName+""+ urlPath ) ;
         }
         /* URL is already qualified. write to buffer */
         else {
            /* write the current HREF without modification to the buffer */
            outputBuffer.append( inputArray, inputDataIndex, endOfHrefIndex - inputDataIndex ) ;
         }
         /* set the last input data index to the end tag */
         lastInputDataIndex = endOfHrefIndex ;
      }
      
      /* Add the remaining of inputData after the last tag */
      outputBuffer.append( inputArray, lastInputDataIndex, inputData.length() - lastInputDataIndex ) ;

      return ( outputBuffer.toString() ) ;
   }
}
