   
package com.ibm.nzna.shared.servlet.viewport.filter ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.servlet.template.* ;
   /*********************************************************
   * This is a special filter used to apply session tracking
   * information to the url's in the html.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/14/100  5:50pm
   * @author 
   * @version
   * @see ViewPortFilterManager
   *********************************************************/
public class SessionTrackingFilter implements ViewPortFilter {
   Properties  properties     = null ;
   /*********************************************************
   * Initialize the input class.  This is provided for classes
   * which may need some prior initialization for efficiency.
   * This would be database connections or other similar tasks.
   * 
   *
   * Author:  
   * Created: 4/19/100  12:38pm
   * @param The properties object.
   * 
   * @return
   *********************************************************/
   public void init( Properties properties ) {
      this.properties = properties ;
   }

   /*********************************************************
   * Applies a filter to the input string and returns a String
   * result.
   * 
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   public String filter ( String inputData, SessionState sessionState ) {
      int            inputDataIndex       = 0 ;
      int            lastInputDataIndex   = 0 ;
      StringBuffer   outputBuffer         = new StringBuffer(inputData.length()) ;
      char[]         inputArray           = inputData.toCharArray() ;
      final String   hrefTag              = "a href=" ;
      final String   openTag              = "<" ;
      HttpRequest    req                  = sessionState.getRequest() ;
      String         viewPortInputHost    = null ;

      /* If we are being used in a template module we will be assigned a template key */
      String templateBindingKey = properties.getProperty ( TemplateModule.bindingKeyProperty ) ;

      /* We must find all hrefs in anchor tags */
      while ( (inputDataIndex = inputData.indexOf( openTag, inputDataIndex)) != -1 ) {
         /* Move the index to next character after < */
         inputDataIndex++ ;

         /* determine if we are at an href anchor tag */
         if ( inputData.regionMatches(true, inputDataIndex, hrefTag, 0, hrefTag.length())) {
            /* move the input data index to first character after equals */
            inputDataIndex += hrefTag.length() ;
         } else {
            /* No tags match we are searching for, we must continue */
            continue ;
         }

         /* Set the end position of the tag */
         int endOfHrefIndex = inputData.indexOf(">", inputDataIndex ) ;
         /* Is the href embedded in quotes */
         boolean embeddedQuotes = false ;
         if ( inputArray[inputDataIndex] == '\"' ) {
            embeddedQuotes = true ;
            /* move input data index to character after quote */
            inputDataIndex++ ;
            /* move ending endex inside quote */
            endOfHrefIndex-- ;
         }

         /* Add to new output buffer from the input up to the tag found */
         outputBuffer.append(inputArray, lastInputDataIndex, inputDataIndex - lastInputDataIndex ) ;


         /* Get the value of the href tag */
         String hrefValue = inputData.substring( inputDataIndex, endOfHrefIndex ) ;
         /* Does this already contain a query string */
         boolean hasQueryString = (hrefValue.indexOf('?') > 0) ? true : false ;

         /* Set the value of the parameter we will add to the URL */
         String parameterValueOfTrackingKey = URLEncoder.encode(sessionState.getId() + "," + sessionState.getSequence()) ;
         if ( hasQueryString ) {
            /* Get the index of the query string */
            int queryStringIndex = inputData.indexOf('?', inputDataIndex ) ;
            /* write to the output buffer up to the query string */
            /* We add one to include the ? of the query string   */
            outputBuffer.append( inputArray, inputDataIndex, queryStringIndex - inputDataIndex + 1 ) ;
            /* write out to the buffer the value we will insert */
            String sessionTrackingKey = properties.getProperty("servlet.session.tracking.key" ) ;
            /* We add an & to the end as the original values will follow */
            outputBuffer.append( sessionTrackingKey + "=" + parameterValueOfTrackingKey +"&" ) ;
            /* write the remaining query string from the original href to the output */
            outputBuffer.append( inputArray, queryStringIndex + 1, endOfHrefIndex - queryStringIndex - 1 ) ;
         }
         /* There is no query string already part of this href */
         else {
            /* write to the output buffer up to the end of the url */
            outputBuffer.append( inputArray, inputDataIndex, endOfHrefIndex - inputDataIndex ) ;
            /* write out to the buffer the value we will insert */
            String sessionTrackingKey = properties.getProperty("servlet.session.tracking.key" ) ;
            /* Add the query string value to the url */
            outputBuffer.append( "?" +sessionTrackingKey + "=" + parameterValueOfTrackingKey ) ;
         }

         /* set the last input data index to the end tag */
         lastInputDataIndex = endOfHrefIndex ;
      }
      
      /* Add the remaining of inputData after the last tag */
      outputBuffer.append( inputArray, lastInputDataIndex, inputData.length() - lastInputDataIndex ) ;

      return ( outputBuffer.toString() ) ;
   }
}
      

