package com.ibm.nzna.shared.servlet.viewport.filter ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import java.io.* ;
import java.net.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.servlet.template.* ;
   /*********************************************************
   * This class rewrites urls with information needed by
   * the ViewPortURLInput to load web pages.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/14/100  5:50pm
   * @author 
   * @version
   * @see ViewPortFilterManager
   *********************************************************/
public class ViewPortUrlRedirectFilter implements ViewPortFilter {
   public static final String inputHostKeyPrefix = "viewportinput.host." ;
   Properties  properties     = null ;
   Vector      exceptionList  = null ;
   /*********************************************************
   * Initialize the input class.  This is provided for classes
   * which may need some prior initialization for efficiency.
   * This would be database connections or other similar tasks.
   * 
   *
   * Author:  
   * Created: 4/19/100  12:38pm
   * @param The properties object.
   * 
   * @return
   *********************************************************/
   public void init( Properties properties ) {
      this.properties = properties ;

      /* Get the URL exceptions which should not be rewritten */
      String exceptions = properties.getProperty("servlet.viewport.filter.stateinformation.noRedirectURL") ;
      if ( exceptions != null ) {
         exceptionList = new Vector() ;
         StringTokenizer tokenizer = new StringTokenizer( exceptions ) ;
         while ( tokenizer.hasMoreElements() ) {
            /* Get the classname from the string */
            String exceptionUrl = ((String) tokenizer.nextToken()).trim() ;
            exceptionList.addElement( exceptionUrl ) ;
         }
      }
   }

   /*********************************************************
   * Applies a filter to the input string and returns a String
   * result.
   * 
   *
   * Author:  
   * Created: 4/14/100  5:52pm
   * @param
   * @return
   *********************************************************/
   public String filter ( String inputData, SessionState sessionState ) {
      int            inputDataIndex       = 0 ;
      int            lastInputDataIndex   = 0 ;
      StringBuffer   outputBuffer         = new StringBuffer(inputData.length()) ;
      char[]         inputArray           = inputData.toCharArray() ;
      final String   hrefTag              = "a href=" ;
      final String   openTag              = "<" ;
      HttpRequest    req                  = sessionState.getRequest() ;
      String         viewPortInputHost    = null ;

      /* If we are being used in a template module we will be assigned a template key */
      String templateBindingKey = properties.getProperty ( TemplateModule.bindingKeyProperty ) ;

      /* We must find all hrefs in anchor tags */
      while ( (inputDataIndex = inputData.indexOf( openTag, inputDataIndex)) != -1 ) {
         /* Move the index to next character after < */
         inputDataIndex++ ;

         /* determine if we are at an href anchor tag */
         if ( inputData.regionMatches(true, inputDataIndex, hrefTag, 0, hrefTag.length())) {
            /* move the input data index to first character after equals */
            inputDataIndex += hrefTag.length() ;
         } else {
            /* No tags match we are searching for, we must continue */
            continue ;
         }

         /* Set the end position of the tag */
         int endOfHrefIndex = inputData.indexOf(">", inputDataIndex ) ;
         /* Is the href embedded in quotes */
         boolean embeddedQuotes = false ;
         if ( inputArray[inputDataIndex] == '\"' ) {
            embeddedQuotes = true ;
            /* move input data index to character after quote */
            inputDataIndex++ ;
            /* move ending index inside quote */
            endOfHrefIndex-- ;
         }

         /* Check for Exceptions.  These URL's should not be redirected */
         if ( exceptionList != null ) {
            boolean exception = checkException( inputData, inputDataIndex, endOfHrefIndex ) ;
            if ( exception ) {
               /* We continue searching for URL's to redirect */
               continue ;
            }
         }

         /* Add to new output buffer from the input up to the tag found */
         outputBuffer.append(inputArray, lastInputDataIndex, inputDataIndex - lastInputDataIndex ) ;

         /* Get the value of the href tag */
         String hrefValue = inputData.substring( inputDataIndex, endOfHrefIndex ) ;

         /* Add redirection url */
         String redirectURL = properties.getProperty("servlet.viewport.filter.stateinformation.redirectURL") ;

         /* Qualify the url with the server we are hosted on */
         redirectURL = ServletUtil.qualifyURL ( redirectURL,  sessionState ) ;

         /* Create the query string */
         /* The query string is dependant on the enablement of the session manager */
         /* FINSISH CODE */
         String queryString = URLEncoder.encode( inputHostKeyPrefix ) + URLEncoder.encode(templateBindingKey) + "=" + URLEncoder.encode(hrefValue) ;

         /* Create the new href to place in the buffer */
         String newHrefValue = redirectURL + "?" + queryString ;

         /* write the new HREF to the buffer */
         outputBuffer.append( newHrefValue ) ;

         /* set the last input data index to the end tag */
         lastInputDataIndex = endOfHrefIndex ;
      }
      
      /* Add the remaining of inputData after the last tag */
      outputBuffer.append( inputArray, lastInputDataIndex, inputData.length() - lastInputDataIndex ) ;

      return ( outputBuffer.toString() ) ;
   }
   /*********************************************************
   * Determines if this url should be rewritten
   * 
   *
   * Author:  
   * Created: 6/13/100  5:23pm
   * @param
   * @return
   *********************************************************/
   private boolean checkException ( String inputData, int start, int end ) {
      boolean found = false ;
      Enumeration enum = exceptionList.elements() ;
      while (enum.hasMoreElements()) {
         String exception = (String) enum.nextElement() ;
         int index = inputData.indexOf( exception, start ) ;
         if (( index > 0 ) && ( index < end )) {
            found = true ;
            break ;
         }
      }
      return ( found ) ;

   }
}
      

