package com.ibm.nzna.shared.sort ;
/*-----------------------------------------------------------*/
/*   QuickSort                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuickSort                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This is a very simple QuickSort algorithm. This algorithm can
 * automatically sort String and Integer objects. To sort any other
 * object, that object must implement the IQuickSort interface.
 *
 *
 * @version 1.0, 7/1/1998 
 * @author David Lentz
 */
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   QuickSorter                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuickSort extends Object {

   private IllegalArgumentException err1 =
   new IllegalArgumentException("Stack Overflow");

   public static final int COMP_LESS     = -1;
   public static final int COMP_EQUAL    =  0;
   public static final int COMP_GREATER  =  1;

   private class StackItem {
      public int left;
      public int right;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates the QuickSort
    *  object and sorts the array passed in. It will
    *  also sort ascending.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuickSort ( Object [] list ) {
      Sort(list,false);
   }




   /*------------------------------------------------------------*/
   /* Constructor(1)
    */
   /**
    *    This constructor will sort the vector passed
    *  in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuickSort ( Vector vec ) {
      Object[] list = null ;
      int      i    = 0 ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         list = new Object[vec.size()] ;
         vec.copyInto ( list ) ;
         Sort ( list, false ) ;

         /* Now, copy the values back into the vector  */
         vec.removeAllElements() ;
         while ( i < list.length ) {
            vec.addElement ( list[i] ) ;
            i++ ;
         }

         list=null ;
      }
   }




   /*------------------------------------------------------------*/
   /* sort
    */
   /**
    *    This method does the actual work of the
    *  QuickSort
    *  
    *
    *  @param list The array to sort
    *  @param descending TRUE = sort Descending, FALSE = sort Ascending
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void Sort ( Object [] list, boolean descending ) {
      final int stackSize = 32;
      StackItem [] stack = new StackItem [stackSize];

      for ( int n = 0; n < 32; ++n )
         stack[n] = new StackItem();

      int stackPtr = 0;

      // determine direction of sort
      int comp;

      if ( descending )
         comp = COMP_GREATER;
      else
         comp = COMP_LESS;

      // size of minimum partition to median-of-three
      final int Threshold = 7;

      // sizes of left and right partitions
      int lsize, rsize;

      // create working indexes
      int l, r, mid, scanl, scanr, pivot;

      // set initial values
      l = 0;
      r = list.length - 1;

      Object temp;

      // main loop
      while ( true ) {
         while ( r > l ) {
            if ( (r - l) > Threshold ) {
               // "median-of-three" partitioning
               mid = (l + r) / 2;

               // three-sort left, middle, and right elements
               if ( compare(list[mid],list[l]) == comp ) {
                  temp      = list[mid];
                  list[mid] = list[l];
                  list[l]   = temp;
               }

               if ( compare(list[r],list[l]) == comp ) {
                  temp    = list[r];
                  list[r] = list[l];
                  list[l] = temp;
               }

               // three-sort left, middle, and right elements
               if ( compare(list[r],list[mid]) == comp ) {
                  temp      = list[mid];
                  list[mid] = list[r];
                  list[r]   = temp;
               }

               // set-up for partitioning
               pivot = r - 1;

               temp        = list[mid];
               list[mid]   = list[pivot];
               list[pivot] = temp;

               scanl = l + 1;
               scanr = r - 2;
            } else {
               // set-up for partitioning
               pivot = r;
               scanl = l;
               scanr = r - 1;
            }

            for ( ;; ) {
               // scan from left for element >= to pivot
               while ( (compare(list[scanl],list[pivot]) == comp) && (scanl < r) )
                  ++scanl;

               // scan from right for element <= to pivot
               while ( (compare(list[pivot],list[scanr]) == comp) && (scanr > l) )
                  --scanr;

               // if scans have met, exit inner loop
               if ( scanl >= scanr )
                  break;

               // exchange elements
               temp        = list[scanl];
               list[scanl] = list[scanr];
               list[scanr] = temp;

               if ( scanl < r )
                  ++scanl;

               if ( scanr > l )
                  --scanr;
            }

            // exchange final element
            temp        = list[scanl];
            list[scanl] = list[pivot];
            list[pivot] = temp;

            // place largest partition on stack
            lsize = scanl - l;
            rsize = r - scanl;

            if ( lsize > rsize ) {
               if ( lsize != 1 ) {
                  ++stackPtr;

                  if ( stackPtr == stackSize )
                     throw err1;

                  stack[stackPtr].left  = l;
                  stack[stackPtr].right = scanl - 1;
               }

               if ( rsize != 0 )
                  l = scanl + 1;
               else
                  break;
            } else {
               if ( rsize != 1 ) {
                  ++stackPtr;

                  if ( stackPtr == stackSize )
                     throw err1;

                  stack[stackPtr].left  = scanl + 1;
                  stack[stackPtr].right = r;
               }

               if ( lsize != 0 )
                  r = scanl - 1;
               else
                  break;
            }
         }

         // iterate with values from stack
         if ( stackPtr != 0 ) {
            l = stack[stackPtr].left;
            r = stack[stackPtr].right;

            --stackPtr;
         } else
            break;
      }
   }

   int compare ( Object x1, Object x2 ) {
      if ( ( x1 != null ) && ( x2 != null ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   String Compares                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( (x1 instanceof String) &&  (x2 instanceof String) ) {
            int c = ((String)x1).compareTo((String)x2);

            if ( c < 0 )
               return COMP_LESS;
            else
               if ( c > 0 )
               return COMP_GREATER;
            else
               return COMP_EQUAL;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Integer Compares                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( (x1 instanceof Integer) &&  (x2 instanceof Integer) ) {
            int num1 = ((Integer)x1).intValue() ;
            int num2 = ((Integer)x2).intValue() ;

            if ( num1 < num2 )
               return COMP_LESS ;
            else if ( num1 > num2 )
               return COMP_GREATER ;
            else
               return COMP_EQUAL ;

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   IQuickSort Compares                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( (x1 instanceof IQuickSort ) && ( x2 instanceof IQuickSort ) )
            return( ((IQuickSort)x1).compareTo ( (IQuickSort)x2 ) ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Other Object Compares                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            int c = (x1.toString()).compareTo(x2.toString());

            if ( c < 0 )
               return COMP_LESS;
            else
               if ( c > 0 )
               return COMP_GREATER;
            else
               return COMP_EQUAL;
         }
      }
      return( COMP_EQUAL ) ;
   }

}

