package com.ibm.nzna.shared.spell ;
import java.awt.*;

public class ErrorHandler extends Dialog implements Runnable
{
	public TextInfo textInfo;
	public TextArea text;
	public String checkWord;
	public int errorPos;

	private int xPosition, yPosition;
	
	public Object arg;

	public TextField wordField;
	public Button replace;
	public Label label_2;
	public Button replaceAll;
	public List suggList;
	public Button ignore;
	public Button ignoreAll;
	public Button learnButton;
	public Button stopButton;

	public ErrorHandler(Frame f,
							  TextArea text,
							  WordBundle wordBundle,
							  String suggestions[],
							  TextInfo textInfo,
							  int xPos,
							  int yPos)
	{
		super(f,"Spell Checker - Error Detected",true);
		this.text=text;
		this.checkWord=wordBundle.word;
		this.textInfo=textInfo;
		this.errorPos=wordBundle.position;

		setBackground(Color.lightGray);
		GridBagLayout grid = new GridBagLayout();
		int rowHeights[] = {7,11,2,2,14,8,12,41,14};
		int columnWidths[] = {4,10,115,72,10,104,8,3};
		double rowWeights[] = {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
		double columnWeights[] = {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
		grid.rowHeights = rowHeights;
		grid.columnWidths = columnWidths;
		grid.rowWeights = rowWeights;
		grid.columnWeights = columnWeights;

		wordField = new TextField(checkWord,20);
		this.add(wordField);

		replace = new Button();
		replace.setLabel("Replace");
		this.add(replace);

		label_2 = new Label();
		label_2.setFont(new Font("Helvetica",Font.PLAIN , 12));
		label_2.setText("Suggestions:");
		this.add(label_2);

		replaceAll = new Button();
		replaceAll.setLabel("Replace All");
		this.add(replaceAll);

		suggList = new List(4,false);
		suggList.setBackground(Color.white);
		if(suggestions.length>0)
		{
			for(int j=0;j<suggestions.length && suggestions[j]!=null;j++)
			{
				suggList.addItem(suggestions[j]);
			}
		}
		this.add(suggList);

		ignore = new Button();
		ignore.setLabel("Ignore");
		this.add(ignore);

		ignoreAll = new Button();
		ignoreAll.setLabel("Ignore All");
		this.add(ignoreAll);

		learnButton = new Button();
		learnButton.setLabel("Learn");
		this.add(learnButton);

		stopButton = new Button();
		stopButton.setLabel("Stop");
		this.add(stopButton);

		// Geometry management
		GridBagConstraints con = new GridBagConstraints();

		reset(con);
		con.gridx = 2;
		con.gridy = 2;
		con.gridwidth = 2;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.HORIZONTAL;
		grid.setConstraints(wordField, con);

		reset(con);
		con.gridx = 5;
		con.gridy = 2;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.HORIZONTAL;
		grid.setConstraints(replace, con);

		reset(con);
		con.gridx = 1;
		con.gridy = 3;
		con.gridwidth = 2;
		con.anchor = GridBagConstraints.WEST;
		con.fill = GridBagConstraints.NONE;
		grid.setConstraints(label_2, con);

		reset(con);
		con.gridx = 5;
		con.gridy = 3;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.HORIZONTAL;
		grid.setConstraints(replaceAll, con);

		reset(con);
		con.gridx = 2;
		con.gridy = 4;
		con.gridwidth = 2;
		con.gridheight = 4;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.BOTH;
		grid.setConstraints(suggList, con);

		reset(con);
		con.gridx = 5;
		con.gridy = 4;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.HORIZONTAL;
		grid.setConstraints(ignore, con);

		reset(con);
		con.gridx = 5;
		con.gridy = 5;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.HORIZONTAL;
		grid.setConstraints(ignoreAll, con);

		reset(con);
		con.gridx = 5;
		con.gridy = 6;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.HORIZONTAL;
		grid.setConstraints(learnButton, con);

		reset(con);
		con.gridx = 5;
		con.gridy = 7;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.HORIZONTAL;
		grid.setConstraints(stopButton, con);


		// Resize behavior management and parent heirarchy
		setLayout(grid);

		pack();

		if(xPos==0 && yPos==0)	// uninitialized position, set to screen center
		{
			j_util.centerDialog(this,f);
			SpellEngine.setErrPos(this.location());
		} else
		{
			move(xPos,yPos);
		}
		xPosition=this.location().x;
		yPosition=this.location().y;
	}

	public void run()
	{
		text.select(errorPos,errorPos+checkWord.length());
		show();
	}
	
	public boolean handleEvent(Event evt)
	{
		switch(evt.id)
		{
			case Event.ACTION_EVENT:
				if(evt.target==ignore)
				{
					text.select(errorPos,errorPos);
					SpellEngine.resumeChecking(textInfo);
				} else
				if(evt.target==suggList)
				{
					// double click, replace text with selection
					wordField.setText(suggList.getSelectedItem());
					textInfo.curPos=errorPos;
					SpellEngine.replaceSingle(textInfo,checkWord,wordField.getText());
					// reset checker
					SpellEngine.resetChecker();
					// resume
					SpellEngine.resumeChecking(textInfo);
				
				} else
				if(evt.target==ignoreAll)
				{
					SpellEngine.addIgnore(checkWord);
					SpellEngine.resumeChecking(textInfo);
				} else
				if(evt.target==replace)
				{
					// Update text area position
					textInfo.curPos=errorPos;
					// Update contents of check String
					SpellEngine.replaceSingle(textInfo,checkWord,wordField.getText());
					// reset checker
					SpellEngine.resetChecker();
					// resume
					SpellEngine.resumeChecking(textInfo);
				} else
				if(evt.target==replaceAll)
				{
					textInfo.curPos=errorPos;
					//
					SpellEngine.replaceAll(textInfo,checkWord,wordField.getText());
					//
					SpellEngine.resetChecker();
					//
					SpellEngine.resumeChecking(textInfo);
				} else
				if(evt.target==learnButton)
				{
					SpellEngine.addLearned(checkWord);
					SpellEngine.resumeChecking(textInfo);
					// learn code goes here
				} else
				if(evt.target==stopButton)
				{
					SpellEngine.stopped=true;
					SpellEngine.resumeChecking(textInfo);
					// stop code goes here
				}
				break;
			case Event.LIST_SELECT:
				wordField.setText(suggList.getSelectedItem());
				// insert code for replace Single here
				break;
			case Event.WINDOW_MOVED:
				SpellEngine.setErrPos(this.location());
				break;
			default:
				if(this.location().x!=xPosition || this.location().y!=yPosition)
				{	// Alternate way of checking if window was moved but not event generated
					SpellEngine.setErrPos(this.location());
					xPosition=this.location().x;
					yPosition=this.location().y;
				}
				break;
		}
		return super.handleEvent(evt);
	}

	private void reset(GridBagConstraints con) {
		con.gridx = GridBagConstraints.RELATIVE;
		con.gridy = GridBagConstraints.RELATIVE;
		con.gridwidth = 1;
		con.gridheight = 1;

		con.weightx = 0;
		con.weighty = 0;
		con.anchor = GridBagConstraints.CENTER;
		con.fill = GridBagConstraints.NONE;

		con.insets = new Insets(0, 0, 0, 0);
		con.ipadx = 0;
		con.ipady = 0;
	}

	
}

