package com.ibm.nzna.shared.spell ;
import java.util.*;
import java.awt.*;
import java.io.*;

public class JSpellToolkit	implements JSpellChecker, JSpellMessageInt, JSpellProgressInt
{
	private SpellIndex x;				// Reference to spelling index
	private String[] suggestions;		// String array containing suggestions
	private Hashtable ignoreWords; 	// contains words to ignore (when user clicks Ignore All)
	private Hashtable validWords;		// Hashtable containing frequently used valid words
	private Container f;			      // used to facilitate creating dialogs
	private Runnable eh,ruProg;	 	// Threading interface for dialogs
	private Thread th=null,thProg;  	// Progress bar and Error handler Threads
	private int progxPos;				// progress bar location
	private int progyPos;
	private int errxPos;				   // JSpellError window position
	private int erryPos;
	private boolean stopped;			// spell checker has been stopped
	private WordBundle checkWord;
	private TextArea userTextArea;	// User text area to check
	private boolean ignoreUpper;		// ignore upper case words
	private int progressFreq;			// how often is progress Bar updated
	private int progressCounter;
	private boolean showProgress;		// show progress bar while checking
	private TextInfo textInfo;			// Handy class to contain text information
	
	private String nextWord;			// these are only necessary 
	private String nextWordUpper;		// for performance reasons 
   private static JSpellToolkit instance  = null ;
	
	public JSpellToolkit()
	{
		x=new SpellIndex();
		ignoreWords=new Hashtable();
		validWords=new Hashtable();
		addFrequentWords();
		setProgress(true);
		setIgnoreUpperCase(true);	// default to ignore words in upper case
		ruProg=null;
		progressCounter=0;
		progressFreq=25;			// report progress every 25 words
      instance = this ;
	}

	public final boolean isToolkitReady()
	{
		// Return status of Toolkit index
		return x.isIndexReady();
	}

	public final void setErrPos(Point p)
	{
		// Used by the error handler to remember position of error checker window
		errxPos=p.x;
		erryPos=p.y;
	}

	public final void addLearned(String word)
	{
		// Adds new word to dictionary hash table
		x.add(word,j_util.soundex(word),x.root);
	}

	public final void addIgnore(String word)
	{
		// Adds word to ignore to ignore hash table
		ignoreWords.put(word,"");
	}

	public final void setProgPos(Point p)
	{
		// Used by progress bar to remember position of progress bar window
		progxPos=p.x;
		progyPos=p.y;
	}

	public final void setStopped(boolean stopValue)
	{
		stopped=stopValue;
	}

	public final void setProgress(boolean progValue)
	{
		showProgress=progValue;
	}

	public final boolean getShowProgValue()
	{
		return showProgress;
	}
	
	public final void setIgnoreUpperCase(boolean ignoreValue)
	{
		// Toggle ignore words in upper case on/off
		ignoreUpper=ignoreValue;
	}

	public final void buildIndex()
	{
		// Build SpellIndex file using default 'dict.txt'
		buildIndex("dict.txt");
	}
	
	public final void buildIndex(String dictFile)
	{
		// Build SpellIndex file using String parameter of dictionary file
		x.build(dictFile);
	}

	public final void initRawCheck(String data)
	{
		setStopped(false);
		textInfo=new TextInfo(data);
		textInfo.setIgnore(ignoreUpper);
		suggestions=new String[10];
	}
	
	public final void check(TextArea ta)
	{
		// Developer calls this to specify a TextArea to check
		setStopped(false);

		textInfo=new TextInfo(ta.getText());
		textInfo.setIgnore(ignoreUpper);
		userTextArea=ta;

		initContainer();

		if(showProgress)
		{
			ruProg=new JSpellProgress((Frame)f,"Checking",100,progxPos,progyPos,this);
			thProg=new Thread(ruProg);
			thProg.start();
		}
		suggestions=new String[10];
		cleanCheck();
	}

	public final JSpellErrorInfo rawCheck()
	{
		try
		{
			while(textInfo.isWordsToCheck() && !stopped)
			{
				checkWord=textInfo.getNextWord();
				nextWord=checkWord.getWord();
				// change checkWord.getWord to checkWord.getUpperWord to make JSpell
				// ignore words whether in Upper or Lower case if user selected 'Ignore All' button
				if(!nextWord.equals("") &&
				   !ignoreWords.containsKey(nextWord) &&
				   !validWords.containsKey(nextWord))
				{
					nextWordUpper=checkWord.getUpperWord();
					for(int z=0;z<10;suggestions[z++]=null);
					if(!x.search(j_util.soundex(nextWordUpper),nextWordUpper,suggestions))
					{
						return new JSpellErrorInfo(checkWord,suggestions,textInfo);
					}
				}
			}
		} catch (IOException ioe)
		{
			System.out.println(ioe);
			ioe.printStackTrace();
			System.exit(0);
		}
		return null;
	}
	
	private final void cleanCheck()
	{
		try
		{
			while(textInfo.isWordsToCheck() && !stopped)
			{
				checkWord=textInfo.getNextWord();
				nextWord=checkWord.getWord();
				// change checkWord.getWord to checkWord.getUpperWord to make JSpell
				// ignore words whether in Upper or Lower case if user selected 'Ignore All' button
				if(!nextWord.equals("") &&
				   !ignoreWords.containsKey(nextWord) &&
				   !validWords.containsKey(nextWord))
				{
					nextWordUpper=checkWord.getUpperWord();
					for(int z=0;z<10;suggestions[z++]=null);
					if(!x.search(j_util.soundex(nextWordUpper),nextWordUpper,suggestions))
					{
							eh=new JSpellError((Frame)f,userTextArea,checkWord,suggestions,textInfo,errxPos,erryPos,this);
							th=new Thread(eh);		// spawn new thread.
							th.start();
							return;	// re enter this method from callback in error handler
					}
				}
				if(showProgress && progressCounter++>progressFreq)
				{
					((JSpellProgress)ruProg).setProgress((int)(textInfo.getCurPos()*100.0/textInfo.getTextLength()));
					progressCounter=0;
				}
			}
		} catch (IOException ioe)
		{
			System.out.println(ioe);
			ioe.printStackTrace();
			System.exit(0);
		}

		if(getShowProgValue())
		{
			((JSpellProgress)ruProg).setProgress(100);
		}

		if(stopped)
		{
			showErrorDialog("Notice","Spell Check Aborted");
		} else
		{
			showErrorDialog("Notice","Spelling Check Complete");
		}
	} 

	public final void killErrorDialog(Dialog x)
	{
		x.dispose();
		f.requestFocus();
	}		

	private final void showErrorDialog(String w, String m)
	{
		Runnable jsm=new JSpellMessage(w,m,(Frame)f,this);
		Thread th=new Thread(jsm);	
		th.start();
		stopped=true;

		//wordstocheck was HERE
		
		if(getShowProgValue())
		{
			if(thProg!=null)
			{
				thProg.stop();
				thProg=null;
				((JSpellProgress)ruProg).dispose();
			}
		}
	}

	public final void resumeChecking()
	{
		//wait for thread to die before constructing new one
		((JSpellError)eh).dispose();

		userTextArea.setText(textInfo.getTextString());
		userTextArea.select(textInfo.getCurPos(),textInfo.getCurPos());

		f.requestFocus();
		try {
			th.stop();
			th=null;
		}catch (Exception ie)
		{
			ie.printStackTrace();
			System.out.println("Trying to Stop Thread: "+ie);
		}
		try {
			cleanCheck();
		}
		catch (Exception e)
		{
			// Catch all errors
			showErrorDialog("Warning","ERROR: "+e);
		}
	}
	
	public final void replaceSingle(String orig, String repl)
	{
		// Copies StringBuffer up to TextInfo curpos + new word/phrase + remaining text
		StringBuffer newText;

		newText=new StringBuffer(textInfo.getTextString().substring(0,textInfo.getCurPos()));	// copy up to original word
		newText.append(repl);
		newText.append(textInfo.getTextString().substring(textInfo.getCurPos()+orig.length()));
		textInfo.setText(newText);
	}

	public final void replaceAll(String orig, String repl)
	{
		// Copies StringBuffer up to TextInfo curpos + new word/phrase + remaining text
		StringBuffer newText;

		int lastpos=textInfo.getCurPos();
		int x;
		
		newText=new StringBuffer(textInfo.getTextString().substring(0,lastpos));	// copy up to original word
		// copy text up to next occurrence of orig word, skip orig word, continue appending
		newText.append(repl);
		x=textInfo.getTextString().indexOf(orig,lastpos+1);
		while(x>0)	// means word found
		{
			newText.append(textInfo.getTextString().substring(lastpos+orig.length(),x));
			newText.append(repl);
			lastpos=x;
			x=textInfo.getTextString().indexOf(orig,lastpos+1);
		}
		newText.append(textInfo.getTextString().substring(lastpos+orig.length()));
		textInfo.setText(newText);
	}

	private final void initContainer()
	{
		f = userTextArea.getParent();
		while(f != null && !(f instanceof Frame)) {
			f = f.getParent();
		}
		if (f == null)
		{
			System.out.println("No Frame Container parent could be found.");
			return;
		}
	}		

	private final void addFrequentWords()
	{
		// Loads dictionary with most frequently used words, at a minimum
		// this method should add the words (and, of, the, for) to conserve space
		// on the client side while minimizing dictionary disk access.
		
		validWords.put("able","");
		validWords.put("about","");
		validWords.put("above","");
		validWords.put("according","");
		validWords.put("across","");
		validWords.put("act","");
		validWords.put("added","");
		validWords.put("after","");
		validWords.put("again","");
		validWords.put("against","");
		validWords.put("age","");
		validWords.put("agency","");
		validWords.put("ago","");
		validWords.put("agreed","");
		validWords.put("agreement","");
		validWords.put("air","");
		validWords.put("all","");
		validWords.put("almost","");
		validWords.put("alone","");
		validWords.put("along","");
		validWords.put("already","");
		validWords.put("also","");
		validWords.put("although","");
		validWords.put("always","");
		validWords.put("am","");
		validWords.put("among","");
		validWords.put("an","");
		validWords.put("analyst","");
		validWords.put("and","");
		validWords.put("announced","");
		validWords.put("annual","");
		validWords.put("another","");
		validWords.put("any","");
		validWords.put("anything","");
		validWords.put("are","");
		validWords.put("area","");
		validWords.put("around","");
		validWords.put("art","");
		validWords.put("as","");
		validWords.put("asked","");
		validWords.put("assets","");
		validWords.put("at","");
		validWords.put("august","");
		validWords.put("average","");
		validWords.put("away","");
		validWords.put("back","");
		validWords.put("bank","");
		validWords.put("banks","");
		validWords.put("based","");
		validWords.put("be","");
		validWords.put("became","");
		validWords.put("because","");
		validWords.put("become","");
		validWords.put("been","");
		validWords.put("before","");
		validWords.put("began","");
		validWords.put("behind","");
		validWords.put("being","");
		validWords.put("best","");
		validWords.put("better","");
		validWords.put("between","");
		validWords.put("bid","");
		validWords.put("big","");
		validWords.put("bill","");
		validWords.put("billion","");
		validWords.put("board","");
		validWords.put("body","");
		validWords.put("bond","");
		validWords.put("bonds","");
		validWords.put("book","");
		validWords.put("both","");
		validWords.put("brought","");
		validWords.put("business","");
		validWords.put("but","");
		validWords.put("buy","");
		validWords.put("buying","");
		validWords.put("by","");
		validWords.put("called","");
		validWords.put("came","");
		validWords.put("camera","");
		validWords.put("can","");
		validWords.put("capital","");
		validWords.put("car","");
		validWords.put("case","");
		validWords.put("cases","");
		validWords.put("cash","");
		validWords.put("cent","");
		validWords.put("cents","");
		validWords.put("century","");
		validWords.put("certain","");
		validWords.put("certainly","");
		validWords.put("chairman","");
		validWords.put("change","");
		validWords.put("chief","");
		validWords.put("child","");
		validWords.put("children","");
		validWords.put("clear","");
		validWords.put("client","");
		validWords.put("close","");
		validWords.put("closed","");
		validWords.put("come","");
		validWords.put("comment","");
		validWords.put("committee","");
		validWords.put("common","");
		validWords.put("companies","");
		validWords.put("company","");
		validWords.put("compared","");
		validWords.put("computer","");
		validWords.put("concern","");
		validWords.put("concerned","");
		validWords.put("congress","");
		validWords.put("continue","");
		validWords.put("contract","");
		validWords.put("control","");
		validWords.put("corp","");
		validWords.put("corporate","");
		validWords.put("cost","");
		validWords.put("costs","");
		validWords.put("could","");
		validWords.put("council","");
		validWords.put("countries","");
		validWords.put("country","");
		validWords.put("course","");
		validWords.put("court","");
		validWords.put("credit","");
		validWords.put("current","");
		validWords.put("cut","");
		validWords.put("day","");
		validWords.put("days","");
		validWords.put("death","");
		validWords.put("debt","");
		validWords.put("debug","");
		validWords.put("declined","");
		validWords.put("demand","");
		validWords.put("department","");
		validWords.put("development","");
		validWords.put("did","");
		validWords.put("different","");
		validWords.put("difficult","");
		validWords.put("director","");
		validWords.put("disk","");
		validWords.put("diskette","");
		validWords.put("do","");
		validWords.put("does","");
		validWords.put("doing","");
		validWords.put("dollar","");
		validWords.put("done","");
		validWords.put("door","");
		validWords.put("doubt","");
		validWords.put("dow","");
		validWords.put("down","");
		validWords.put("dr","");
		validWords.put("drive","");
		validWords.put("drug","");
		validWords.put("due","");
		validWords.put("during","");
		validWords.put("each","");
		validWords.put("earlier","");
		validWords.put("early","");
		validWords.put("earnings","");
		validWords.put("economic","");
		validWords.put("economy","");
		validWords.put("education","");
		validWords.put("effect","");
		validWords.put("either","");
		validWords.put("employees","");
		validWords.put("end","");
		validWords.put("ended","");
		validWords.put("england","");
		validWords.put("english","");
		validWords.put("enough","");
		validWords.put("european","");
		validWords.put("even","");
		validWords.put("ever","");
		validWords.put("every","");
		validWords.put("example","");
		validWords.put("except","");
		validWords.put("exchange","");
		validWords.put("executive","");
		validWords.put("expected","");
		validWords.put("expects","");
		validWords.put("experience","");
		validWords.put("eyes","");
		validWords.put("face","");
		validWords.put("fact","");
		validWords.put("family","");
		validWords.put("far","");
		validWords.put("father","");
		validWords.put("federal","");
		validWords.put("feet","");
		validWords.put("fell","");
		validWords.put("felt","");
		validWords.put("few","");
		validWords.put("field","");
		validWords.put("financial","");
		validWords.put("find","");
		validWords.put("firm","");
		validWords.put("firms","");
		validWords.put("first","");
		validWords.put("fiscal","");
		validWords.put("five","");
		validWords.put("floppy","");
		validWords.put("following","");
		validWords.put("food","");
		validWords.put("for","");
		validWords.put("foreign","");
		validWords.put("form","");
		validWords.put("former","");
		validWords.put("forward","");
		validWords.put("found","");
		validWords.put("four","");
		validWords.put("free","");
		validWords.put("french","");
		validWords.put("friday","");
		validWords.put("from","");
		validWords.put("full","");
		validWords.put("funds","");
		validWords.put("further","");
		validWords.put("futures","");
		validWords.put("gains","");
		validWords.put("gave","");
		validWords.put("general","");
		validWords.put("get","");
		validWords.put("girl","");
		validWords.put("give","");
		validWords.put("given","");
		validWords.put("go","");
		validWords.put("going","");
		validWords.put("gone","");
		validWords.put("good","");
		validWords.put("got","");
		validWords.put("government","");
		validWords.put("great","");
		validWords.put("group","");
		validWords.put("growth","");
		validWords.put("had","");
		validWords.put("half","");
		validWords.put("hand","");
		validWords.put("hands","");
		validWords.put("hard","");
		validWords.put("has","");
		validWords.put("have","");
		validWords.put("having","");
		validWords.put("he","");
		validWords.put("head","");
		validWords.put("heard","");
		validWords.put("held","");
		validWords.put("help","");
		validWords.put("her","");
		validWords.put("here","");
		validWords.put("herself","");
		validWords.put("high","");
		validWords.put("higher","");
		validWords.put("him","");
		validWords.put("himself","");
		validWords.put("his","");
		validWords.put("home","");
		validWords.put("hope","");
		validWords.put("house","");
		validWords.put("how","");
		validWords.put("however","");
		validWords.put("human","");
		validWords.put("idea","");
		validWords.put("if","");
		validWords.put("important","");
		validWords.put("in","");
		validWords.put("Inc","");
		validWords.put("including","");
		validWords.put("income","");
		validWords.put("increase","");
		validWords.put("increased","");
		validWords.put("indeed","");
		validWords.put("index","");
		validWords.put("industry","");
		validWords.put("information","");
		validWords.put("insurance","");
		validWords.put("interest","");
		validWords.put("International","");
		validWords.put("internet","");
		validWords.put("into","");
		validWords.put("intranet","");
		validWords.put("investment","");
		validWords.put("investors","");
		validWords.put("is","");
		validWords.put("issue","");
		validWords.put("issues","");
		validWords.put("it","");
		validWords.put("its","");
		validWords.put("itself","");
		validWords.put("japan","");
		validWords.put("japanese","");
		validWords.put("java","");
		validWords.put("john","");
		validWords.put("july","");
		validWords.put("june","");
		validWords.put("just","");
		validWords.put("keep","");
		validWords.put("kind","");
		validWords.put("knew","");
		validWords.put("know","");
		validWords.put("knowledge","");
		validWords.put("known","");
		validWords.put("labour","");
		validWords.put("land","");
		validWords.put("large","");
		validWords.put("largest","");
		validWords.put("last","");
		validWords.put("late","");
		validWords.put("later","");
		validWords.put("law","");
		validWords.put("least","");
		validWords.put("leave","");
		validWords.put("left","");
		validWords.put("less","");
		validWords.put("let","");
		validWords.put("level","");
		validWords.put("life","");
		validWords.put("light","");
		validWords.put("like","");
		validWords.put("likely","");
		validWords.put("line","");
		validWords.put("little","");
		validWords.put("living","");
		validWords.put("loans","");
		validWords.put("local","");
		validWords.put("London","");
		validWords.put("long","");
		validWords.put("look","");
		validWords.put("looked","");
		validWords.put("looking","");
		validWords.put("Lord","");
		validWords.put("loss","");
		validWords.put("love","");
		validWords.put("lower","");
		validWords.put("ltd","");
		validWords.put("made","");
		validWords.put("main","");
		validWords.put("major","");
		validWords.put("make","");
		validWords.put("maker","");
		validWords.put("making","");
		validWords.put("man","");
		validWords.put("management","");
		validWords.put("many","");
		validWords.put("market","");
		validWords.put("markets","");
		validWords.put("matter","");
		validWords.put("may","");
		validWords.put("me","");
		validWords.put("mean","");
		validWords.put("means","");
		validWords.put("meeting","");
		validWords.put("members","");
		validWords.put("men","");
		validWords.put("might","");
		validWords.put("million","");
		validWords.put("mind","");
		validWords.put("miss","");
		validWords.put("modern","");
		validWords.put("moment","");
		validWords.put("money","");
		validWords.put("monitor","");
		validWords.put("month","");
		validWords.put("months","");
		validWords.put("more","");
		validWords.put("morning","");
		validWords.put("most","");
		validWords.put("mother","");
		validWords.put("move","");
		validWords.put("mr","");
		validWords.put("mrs","");
		validWords.put("ms","");
		validWords.put("much","");
		validWords.put("music","");
		validWords.put("must","");
		validWords.put("my","");
		validWords.put("name","");
		validWords.put("named","");
		validWords.put("national","");
		validWords.put("near","");
		validWords.put("necessary","");
		validWords.put("need","");
		validWords.put("net","");
		validWords.put("never","");
		validWords.put("new","");
		validWords.put("news","");
		validWords.put("next","");
		validWords.put("night","");
		validWords.put("no","");
		validWords.put("nor","");
		validWords.put("not","");
		validWords.put("notes","");
		validWords.put("nothing","");
		validWords.put("now","");
		validWords.put("number","");
		validWords.put("oct","");
		validWords.put("october","");
		validWords.put("of","");
		validWords.put("off","");
		validWords.put("offer","");
		validWords.put("offering","");
		validWords.put("office","");
		validWords.put("officer","");
		validWords.put("official","");
		validWords.put("officials","");
		validWords.put("often","");
		validWords.put("oil","");
		validWords.put("old","");
		validWords.put("on","");
		validWords.put("once","");
		validWords.put("one","");
		validWords.put("only","");
		validWords.put("open","");
		validWords.put("operating","");
		validWords.put("operations","");
		validWords.put("or","");
		validWords.put("order","");
		validWords.put("other","");
		validWords.put("others","");
		validWords.put("our","");
		validWords.put("out","");
		validWords.put("outside","");
		validWords.put("over","");
		validWords.put("own","");
		validWords.put("own","");
		validWords.put("part","");
		validWords.put("particular","");
		validWords.put("particularly","");
		validWords.put("party","");
		validWords.put("past","");
		validWords.put("pay","");
		validWords.put("people","");
		validWords.put("per","");
		validWords.put("perhaps","");
		validWords.put("period","");
		validWords.put("place","");
		validWords.put("plan","");
		validWords.put("plans","");
		validWords.put("plant","");
		validWords.put("point","");
		validWords.put("points","");
		validWords.put("policy","");
		validWords.put("political","");
		validWords.put("position","");
		validWords.put("possible","");
		validWords.put("power","");
		validWords.put("present","");
		validWords.put("president","");
		validWords.put("price","");
		validWords.put("prices","");
		validWords.put("probably","");
		validWords.put("problem","");
		validWords.put("problems","");
		validWords.put("product","");
		validWords.put("production","");
		validWords.put("products","");
		validWords.put("profit","");
		validWords.put("program","");
		validWords.put("programming","");
		validWords.put("proposed","");
		validWords.put("public","");
		validWords.put("purchase","");
		validWords.put("put","");
		validWords.put("quarter","");
		validWords.put("question","");
		validWords.put("quite","");
		validWords.put("rate","");
		validWords.put("rates","");
		validWords.put("rather","");
		validWords.put("read","");
		validWords.put("real","");
		validWords.put("really","");
		validWords.put("reason","");
		validWords.put("recent","");
		validWords.put("recently","");
		validWords.put("record","");
		validWords.put("report","");
		validWords.put("reported","");
		validWords.put("research","");
		validWords.put("result","");
		validWords.put("results","");
		validWords.put("revenue","");
		validWords.put("right","");
		validWords.put("rise","");
		validWords.put("robert","");
		validWords.put("room","");
		validWords.put("rose","");
		validWords.put("round","");
		validWords.put("said","");
		validWords.put("sale","");
		validWords.put("sales","");
		validWords.put("same","");
		validWords.put("saw","");
		validWords.put("say","");
		validWords.put("says","");
		validWords.put("school","");
		validWords.put("second","");
		validWords.put("securities","");
		validWords.put("see","");
		validWords.put("seem","");
		validWords.put("seemed","");
		validWords.put("seems","");
		validWords.put("seen","");
		validWords.put("sell","");
		validWords.put("selling","");
		validWords.put("senior","");
		validWords.put("sense","");
		validWords.put("sept","");
		validWords.put("server","");
		validWords.put("service","");
		validWords.put("services","");
		validWords.put("set","");
		validWords.put("several","");
		validWords.put("shall","");
		validWords.put("share","");
		validWords.put("shares","");
		validWords.put("she","");
		validWords.put("short","");
		validWords.put("should","");
		validWords.put("show","");
		validWords.put("shown","");
		validWords.put("side","");
		validWords.put("since","");
		validWords.put("sir","");
		validWords.put("six","");
		validWords.put("small","");
		validWords.put("so","");
		validWords.put("social","");
		validWords.put("society","");
		validWords.put("sold","");
		validWords.put("some","");
		validWords.put("something","");
		validWords.put("sometimes","");
		validWords.put("soon","");
		validWords.put("source","");
		validWords.put("special","");
		validWords.put("spokesman","");
		validWords.put("stage","");
		validWords.put("stake","");
		validWords.put("state","");
		validWords.put("still","");
		validWords.put("stock","");
		validWords.put("stocks","");
		validWords.put("story","");
		validWords.put("strong","");
		validWords.put("such","");
		validWords.put("support","");
		validWords.put("sure","");
		validWords.put("system","");
		validWords.put("table","");
		validWords.put("take","");
		validWords.put("taken","");
		validWords.put("takeover","");
		validWords.put("taking","");
		validWords.put("talk","");
		validWords.put("tax","");
		validWords.put("tell","");
		validWords.put("terms","");
		validWords.put("text","");
		validWords.put("than","");
		validWords.put("that","");
		validWords.put("the","");
		validWords.put("their","");
		validWords.put("them","");
		validWords.put("themselves","");
		validWords.put("then","");
		validWords.put("there","");
		validWords.put("therefore","");
		validWords.put("these","");
		validWords.put("they","");
		validWords.put("thing","");
		validWords.put("things","");
		validWords.put("think","");
		validWords.put("third","");
		validWords.put("this","");
		validWords.put("those","");
		validWords.put("though","");
		validWords.put("thought","");
		validWords.put("three","");
		validWords.put("through","");
		validWords.put("thus","");
		validWords.put("time","");
		validWords.put("times","");
		validWords.put("to","");
		validWords.put("today","");
		validWords.put("together","");
		validWords.put("told","");
		validWords.put("too","");
		validWords.put("took","");
		validWords.put("top","");
		validWords.put("total","");
		validWords.put("towards","");
		validWords.put("town","");
		validWords.put("trade","");
		validWords.put("trading","");
		validWords.put("true","");
		validWords.put("turn","");
		validWords.put("turned","");
		validWords.put("two","");
		validWords.put("type","");
		validWords.put("under","");
		validWords.put("unit","");
		validWords.put("until","");
		validWords.put("up","");
		validWords.put("upon","");
		validWords.put("us","");
		validWords.put("use","");
		validWords.put("used","");
		validWords.put("usually","");
		validWords.put("value","");
		validWords.put("very","");
		validWords.put("vice","");
		validWords.put("view","");
		validWords.put("voice","");
		validWords.put("wall","");
		validWords.put("want","");
		validWords.put("wanted","");
		validWords.put("war","");
		validWords.put("was","");
		validWords.put("washington","");
		validWords.put("water","");
		validWords.put("way","");
		validWords.put("we","");
		validWords.put("week","");
		validWords.put("weeks","");
		validWords.put("well","");
		validWords.put("went","");
		validWords.put("were","");
		validWords.put("west","");
		validWords.put("what","");
		validWords.put("when","");
		validWords.put("where","");
		validWords.put("whether","");
		validWords.put("which","");
		validWords.put("while","");
		validWords.put("white","");
		validWords.put("who","");
		validWords.put("whole","");
		validWords.put("whom","");
		validWords.put("whose","");
		validWords.put("why","");
		validWords.put("wife","");
		validWords.put("will","");
		validWords.put("with","");
		validWords.put("within","");
		validWords.put("without","");
		validWords.put("without","");
		validWords.put("woman","");
		validWords.put("word","");
		validWords.put("words","");
		validWords.put("work","");
		validWords.put("working","");
		validWords.put("world","");
		validWords.put("would","");
		validWords.put("year","");
		validWords.put("years","");
		validWords.put("yen","");
		validWords.put("yes","");
		validWords.put("yesterday","");
		validWords.put("yet","");
		validWords.put("yield","");
		validWords.put("york","");
		validWords.put("you","");
		validWords.put("young","");
		validWords.put("your","");
		validWords.put("yours","");
	}


   public static JSpellToolkit getInstance() {
      return ( instance ) ;
   }


}
