package com.ibm.nzna.shared.spell;
import java.util.*;
import java.io.*;
import java.net.*;

class SpellHandler extends Thread
{
	protected Socket client;
	protected DataInputStream in;
	protected DataOutputStream out;

	Socket incoming;
	SpellIndex si;
	String suggestions[];

	public SpellHandler(Socket client_socket, SpellIndex si)
	{
		client=client_socket;
		this.si=si;
		// size of suggestions array should be configurable
		suggestions=new String[10];

		try
		{
			in=new DataInputStream(client.getInputStream());
			out=new DataOutputStream(client.getOutputStream());
		}
		catch (IOException e)
		{
			try{
				client.close();
			}
			catch (IOException e2)
			{
				System.err.println("Exception while getting socket streams: "+e);
			}
			return;
		}
		this.start();
	}

	public void run()
	{
		String str="";
		char curchar=' ',lastchar=' ';
		int wordPos;
		
		try
		{
			boolean done=false;
			while(!done)
			{
				curchar=(char)in.readByte();
				switch(curchar)
				{
					case '|':
						// word follows, read it
						str=in.readUTF();
						wordPos=in.readInt();
//						System.out.println(str+" "+wordPos);
						for(int i=10;i>0;i--)
						{
							suggestions[i-1]=null;		// after suggestion made clear it
						}
						if(!search(j_util.soundex(str.toUpperCase()),str.toUpperCase(),suggestions ) )
						{
							out.writeInt(wordPos);
							for(int i=0;i<suggestions.length && suggestions[i]!=null;i++)
							{
								out.writeUTF(suggestions[i]);
							}
							out.writeUTF(":");		// Terminates
							out.flush();
						}
						break;
					case '!':
						// new word to learn follows, read it
						str=in.readUTF();
						synchronized(si)
						{
							try {
							if(si.locked)
								si.wait();
							} catch (InterruptedException ie)
							{System.out.println(ie);
							}
							si.locked=true;
							si.add(str,j_util.soundex(str),si.root);
							System.out.println("Learned Word: "+str);
							si.locked=false;
							si.notify();
						}
						break;
					case ':':
						done=true;
						break;
					case '@':
						out.writeInt(-1);	// signals end of results
						out.flush();
						break;	
				}
			}
		}
		catch (Exception e)
		{
			System.out.println("Exception in Thread: "+e);
		}
		finally
		{
			try
			{
				client.close();
			}
			catch (Exception ex)
			{
				System.out.println(ex);
			}
		}
	}

	public boolean search(String soundex, String original, String suggestions[])
	{
		boolean result=false;
		synchronized(si)
		{
			try {
			if(si.locked)
				si.wait();
			} catch (InterruptedException ie)
			{System.out.println(ie);
			}
			si.locked=true;
			try
			{
				result=si.search(soundex,original,suggestions);
			}
			catch (IOException e)
			{
				System.out.println("Error in Thread (search method): "+e);
			}
			si.locked=false;
			si.notify();
		}
		return result;
	}
}