package com.ibm.nzna.shared.spell ;
import java.util.*;
import java.io.*;

public class SpellIndex
{
	boolean locked=false;
	File indexFile;
	RandomAccessFile jdx;
	boolean indexReady=false;;
	public static final String version="sndspell1.0"+"\u001a";
	public static final String defaultName="SNDSPELL.JDX";
	String indexName;
	byte nullNode[]=new byte[7*4];
	public static int baseOffset=version.length()+4;
	int logicalOffset;
	Hashtable nodeBuffer;
	Hashtable dataBuffer;
	int bufferCapacity=5;
	int dataBufferCap=3;
	int dataBufferLimit=1;
	float totalhits;
	float buffertime;
	int bufferNumber=0;
	Node_R root;

	// JSpell Version 1.1
	boolean verbose=true;		// display messages to console ( build() )
	// end JSpell Version 1.1 Enhancements
	
	public SpellIndex()
	{	this(defaultName);	// Call Constructor with default name
	}

	public SpellIndex(String name)
	{
		// Check for existence of dictionary
		// if exists, set indexReady to true,
		// otherwise set to false
		for(int i=0;i<7*4;i++)
		{
			nullNode[i]=0;
		}
		// Initialize buffer to default capacity
		nodeBuffer=new Hashtable(bufferCapacity);
		dataBuffer=new Hashtable(dataBufferCap);
		totalhits=0;
		buffertime=0;
		indexFile=new File(defaultName);
		if(indexFile.exists())
		{
			// Open as RandomAccessFile
			try
			{
				jdx=new RandomAccessFile(indexFile,"rw");
				jdx.seek(baseOffset-4);			// go to baseOffset
				logicalOffset=jdx.readInt();
				root=new Node_R();
				root.nodeOffset=0;
				try
				{
					jdx.seek(baseOffset);
					for(int i=0;i<26;i++)
					{
						root.child[i]=jdx.readInt();
					}
				} catch (Exception e)
				{
					System.out.println(e);
					e.printStackTrace();
					System.exit(-1);
				}
			}
			catch (Exception e)
			{
				System.out.println(e);
				e.printStackTrace();
			}
			indexReady=true;
		}
		else
		{
			indexReady=false;
		}
		if(indexReady==true)
			indexName=defaultName;
	}
	
	// JSpell Version 1.1 Enhancements
	public final boolean isIndexReady()
	{
		return indexReady;
	}

	public final void setVerbose(boolean newValue)
	{
		verbose=newValue;
	}

	public final boolean getVerbose()
	{
		return verbose;
	}

	// end JSpell Version 1.1 Enhancements
	
	public final boolean add(String origWord, String word, Node_S baseNode)
	{
		if(word.length()>0)
		{
			char curchar=word.charAt(0);		// first character
			int charval=curchar-'0';			// convert to an index 0-x
			if(baseNode.child[charval]==0)		// check if this index location is filled in
			{
				// update current node and add child node
				baseNode.child[charval]=logicalOffset; // set baseNode.child to point to next available block
				newNode(baseNode.child[charval]);
				updateNode(baseNode,charval);		// only update baseNode after child Node has been written
			}
			add(origWord, word.substring(1),readNode(baseNode.child[charval]));
		}
		else
		{
			addDataNode(baseNode,origWord);
		}
		return true;
	}

	public final boolean add(String origWord, String word, Node_R baseNode)
	{
		// pass index into key code (level) instead of using substring for speed
		if(word.length()>0)
		{
			char curchar=word.charAt(0);		// first character
			int charval=curchar-'A';			// convert to an index 0-x
			if(baseNode.child[charval]==0)		// check if this index location is filled in
			{
				// update current node and add child node
				baseNode.child[charval]=logicalOffset; // set baseNode.child to point to next available block
				newNode(baseNode.child[charval]);
				updateNode(baseNode,charval);						// only update baseNode after child Node has been written
			}
			add(origWord, word.substring(1),readNode(baseNode.child[charval]));
		}
		else
		{
			updateNode(baseNode);
		}
		return true;
	}

	private final void addNode_D(Node_D node) throws IOException
	{
		// buffer new node
		bufferNumber++;
		node.lastAccess=bufferNumber;
		addNode_DBuffer(node);
		jdx.seek(baseOffset+node.nodeOffset);
		jdx.writeUTF(node.dataElement);
		jdx.writeInt(node.nextNodeOffset);			// place holder for pointer to next data node in this case 0 since there are no more links from this point
	}

	private final Node_D getNode_D(int offset) throws IOException
	{
		Node_D resultNode=new Node_D();

		if(dataBuffer.containsKey((Object)(new Integer(offset))))
		{
			resultNode=(Node_D)(dataBuffer.get((Object)(new Integer(offset))));
		}
		else
		{
			jdx.seek(baseOffset+offset);					// go to the location in file where this data node is at

			resultNode=new Node_D();
			resultNode.nodeOffset=offset;
			resultNode.dataElement=jdx.readUTF();		// read word from this data node
			resultNode.nextNodeOffset=jdx.readInt();

			// since it was not in buffer, add it.
			bufferNumber++;
			resultNode.lastAccess=bufferNumber;
			addNode_DBuffer(resultNode);
		}


		return resultNode;
	}

	private final void addNode_DBuffer(Node_D node)
	{
		if(dataBuffer.size()<dataBufferCap)
		{
			dataBuffer.put((Object)(new Integer(node.nodeOffset)),(Object)node);
		} else
		{
			Node_D bufferNode;
			Enumeration enum=dataBuffer.keys();
			int newBufferNumber=0;
			int removed=0;
			while(enum.hasMoreElements())
			{
				Integer offsetkey=(Integer)enum.nextElement();
				bufferNode=(Node_D)(dataBuffer.get(offsetkey));
				if(bufferNode.lastAccess<bufferNumber-dataBufferLimit)
				{
					dataBuffer.remove(offsetkey);
					removed++;
				}
				else
				{
					newBufferNumber++;
					bufferNode.lastAccess=newBufferNumber;
				}
			}
			bufferNumber=newBufferNumber;
		}

	}

	public final boolean addDataNode(Node_S node, String origWord)
	{
		// passed Node_S which points to the node for this Soundex Code which contains
		// a pointer to the data offset for this node.
		//
		// scan through data offsets check if string data at node matches origWord,
		// if origWord is not found then add new dataNode and update previous node
		// in linked list to point to this new node.
		//
		//
		try
		{
			Node_D resultNode=null;
			int nextOffset=node.dataOffset;	// pointer to next data node
			boolean found=false;

			while(nextOffset!=0 && !found)
			{
				resultNode=getNode_D(nextOffset);

				if(resultNode.dataElement.equals(origWord))
					found=true;
				else
					nextOffset=resultNode.nextNodeOffset;
			}
			if(!found)										// word was not found
			{
				if(node.dataOffset==0)
				{
					// update Node_S				updating will also update buffer if it exists in buffer
					node.dataOffset=logicalOffset;
					node.lastAccess=System.currentTimeMillis();
					updateNode(node);
				}
				else
				{
					// update prior Node_D
					resultNode.nextNodeOffset=logicalOffset;		// update pointer to next data element
					addNode_D(resultNode);								// perform update of Node_D to point to next Node_D
				}

				Node_D newNode_D=new Node_D();

				newNode_D.nodeOffset=logicalOffset;		// here's the link...
				newNode_D.dataElement=origWord;
				newNode_D.nextNodeOffset=0;

				addNode_D(newNode_D);
				logicalOffset=(int)(jdx.getFilePointer()-baseOffset);
				writeOffset();
			}
		}
		catch (IOException i)
		{
			System.out.println(i);
		}
		return true;
	}

	public boolean remove(String word)
	{
		// unmark complete flag
		return true;
	}

	public boolean search(String soundex, String original, String suggestions[]) throws IOException
	{
		char curchar=soundex.charAt(0);		// first character
		int charval=curchar-'A';				// convert to an index 0-x
		int index=1;
		boolean foundFlag=false;
		if(root.child[charval]!=0)		// check if this index location is filled in
		{
			// read pointer to first phonetic code off of root node
			Node_S phonNode=readNode(root.child[charval]);
			while(index<soundex.length() && phonNode.child[soundex.charAt(index)-'0']!=0)
			{
				phonNode=readNode(phonNode.child[soundex.charAt(index)-'0']);
				index++;
			}
			if(phonNode.dataOffset!=0)		// Found word with same phonetic as original
			{
				Node_D dataNode;
				int found=0;
				dataNode=getNode_D(phonNode.dataOffset);
				if(dataNode.dataElement.toUpperCase().equals(original.toUpperCase()))
					foundFlag=true;
				else
				{
					if(j_util.eqscore(dataNode.dataElement,original)>=0.6)
						suggestions[0]=dataNode.dataElement;
				}
				while(dataNode.nextNodeOffset!=0 && !foundFlag)	// traverse linked list of words
				{
					dataNode=getNode_D(dataNode.nextNodeOffset);
					if(dataNode.dataElement.toUpperCase().equals(original.toUpperCase()))
						foundFlag=true;
					else
					{
						if(j_util.eqscore(dataNode.dataElement,original)>=0.6 && found<suggestions.length)
						{
							suggestions[found]=dataNode.dataElement;
							found++;
						}
					}
				}
			}
		}

		return foundFlag;
	}

	public final boolean build(String dict)
	{
		// Builds index by opening dictionary file and repeatedly calling add method.
		indexFile.delete();	// Remove file

		int i=0;	// Incremental counter
		int j=0;	// Total counter

		initIndex();	// opens jdx
		try
		{
			String word;
			DataInputStream d = new DataInputStream((new BufferedInputStream(new FileInputStream(dict))));
			while (d.available()>0)
			{
				word=d.readLine();
				add(word,j_util.soundex(word),root);
				if(verbose)
				{
					j++;
					i++;
					if(i==100)
					{
						System.out.println("Word "+j+" "+word);
						i=0;
					}
				}
			}
			d.close();
			indexReady=true;
		}
		catch (Exception e)
		{
			System.out.println(e);
			e.printStackTrace();
			System.exit(0);
		}
		return true;
	}

	public final void initIndex()
	{
		// Called by build
		// Write signature String, initalize root node.
		try
		{
			jdx=new RandomAccessFile(indexFile,"rw");
			jdx.seek(0);
			jdx.writeBytes(version);
			// set next file offset to zero
			logicalOffset=0;
			writeOffset();
			root=new Node_R();
			root.nodeOffset=0;
			try
			{
				jdx.seek(baseOffset);
				for(int i=0;i<26;i++)
				{
					jdx.writeInt(0);
				}
				logicalOffset+=4*26;
				writeOffset();
			} catch (Exception e)
			{
				System.out.println(e);
				e.printStackTrace();
				System.exit(-1);
			}
		}
		catch (Exception e)
		{
			System.out.println(e);
			e.printStackTrace();
			System.exit(-1);
		}
	}

	public final boolean updateNode(Node_R node,int charval)
	{
		try
		{
			jdx.seek(baseOffset+node.nodeOffset+charval*4);
			jdx.writeInt(node.child[charval]);
		} catch (Exception e)
		{
			System.out.println("updateNode: "+e);
			System.out.println(node.toString());
			e.printStackTrace();
			System.exit(-1);
		}
		return true;
	}

	public final boolean updateNode(Node_R node)
	{
		try
		{
			jdx.seek(baseOffset+node.nodeOffset);
			for(int i=0;i<26;i++)
			{
				jdx.writeInt(node.child[i]);	// Write empty pointers to new nodes
			}
		} catch (Exception e)
		{
			System.out.println("updateNode: "+e);
			System.out.println(node.toString());
			e.printStackTrace();
			System.exit(-1);
		}
		return true;
	}

	public final boolean updateNode(Node_S node, int charval)
	{
		try
		{
			jdx.seek(baseOffset+node.nodeOffset+charval*4);
			jdx.writeInt(node.child[charval]);
		} catch (Exception e)
		{
			System.out.println("updateNode: "+e);
			System.out.println(node.toString());
			e.printStackTrace();
			System.exit(-1);
		}
		return true;
	}

	public final boolean updateNode(Node_S node)
	{
		try
		{
			jdx.seek(baseOffset+node.nodeOffset);
			for(int i=0;i<7;i++)
			{
				jdx.writeInt(node.child[i]);	// Write empty pointers to new nodes
			}
			jdx.writeInt(node.dataOffset);
		} catch (Exception e)
		{
			System.out.println(e);
			e.printStackTrace();
			System.exit(-1);
		}
		return true;
	}

	public final boolean newNode(int offset)
	{
		try
		{
			jdx.seek(baseOffset+offset);
			jdx.write(nullNode);				// write byte Array
			jdx.writeInt(0);					// empty data offset
			logicalOffset+=4*7+4;
			writeOffset();
		} catch (Exception e)
		{
			System.out.println("NEW NODE: "+e);
			e.printStackTrace();
			System.exit(-1);
		}
		return true;
	}

	public Node_S readNode(int offset)
	{
		Node_S resultNode;
		if(nodeBuffer.containsKey((Object)(new Integer(offset))))
		{
			// Node exists in buffer
			resultNode=(Node_S)(nodeBuffer.get((Object)(new Integer(offset))));
			totalhits++;
			buffertime+=(System.currentTimeMillis()-resultNode.lastAccess);
			resultNode.lastAccess=System.currentTimeMillis();
		}
		else
		{	// Node does not exist in buffer read from disk
			resultNode=new Node_S();
			try
			{
				jdx.seek(baseOffset+offset);
				for(int i=0;i<7;i++)
				{
					resultNode.child[i]=jdx.readInt();
				}
				resultNode.dataOffset=jdx.readInt();
				resultNode.nodeOffset=offset;
			}
			catch (IOException e)
			{
				System.out.println("READ NODE: "+e);
				System.out.println("Trying to read logical offset "+Integer.toHexString(offset));
				System.out.println("Trying to read actual offset "+Integer.toHexString((baseOffset+offset)));

				e.printStackTrace();
				System.exit(-1);
			}
			resultNode.lastAccess=System.currentTimeMillis();
			// If buffer is not at capacity, add node to buffer
			if(nodeBuffer.size()<bufferCapacity)
			{
				nodeBuffer.put((Object)(new Integer(offset)),(Object)resultNode);
			} else
			{
				// Clean up buffer--
				// Time how long it took to fill buffer
				// remove items with access times older than
				// the time it took to fill buffer.
				Enumeration enum=nodeBuffer.keys();
				int removed=0;
				while(enum.hasMoreElements())
				{
					Integer offsetkey=(Integer)enum.nextElement();
					if(System.currentTimeMillis()-((Node_S)((nodeBuffer.get(offsetkey)))).lastAccess>(buffertime/totalhits)*2)
					{	//The last time this element was accessed is older than
						//the time it takes to fill the buffer, therefore it is
						//inefficient to keep it around.
						nodeBuffer.remove(offsetkey);
						removed++;
					}
				}
				buffertime=0;
				totalhits=0;
			}
      }
		return resultNode;
   }

	public final void writeOffset()
	{
		try
		{
			jdx.seek(baseOffset-4);
			jdx.writeInt(logicalOffset);
		} catch (Exception e)
		{
			System.out.println(e);
			e.printStackTrace();
			System.exit(-1);
		}
	}
}
