package com.ibm.nzna.shared.spell ;  
/*-----------------------------------------------------------*/
/*   SpellPanel                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SpellPanel                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import javax.swing.border.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This is a simple JPanel that interfaces with the JSpellToolkit 
 * to provide user feedback while spell checking.
 *
 * @version 1.0, 10/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SpellPanel                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class SpellPanel extends JPanel implements Runnable, ListSelectionListener, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_SPELL          = null ;
   private JLabel      st_SPELL_CHECKING = null ;
   private JLabel      st_REPLACE_WITH   = null ;
   private DButton     pb_REPLACE        = null ;
   private DButton     pb_IGNORE         = null ;
   private DButton     pb_IGNORE_ALL     = null ;
   private DButton     pb_LEARN          = null ;
   private DButton     pb_STOP           = null ;
   private JList       lb_SUGGESTIONS    = null ;
   private JScrollPane scr_SUGGESTIONS   = null ;
   private JTextField  ef_REPLACE_WITH   = null ;
   private JPanel      buttonPanel       = null ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JSpellToolkit   spellToolkit    = null ;
   private Dimension       prefSize        = new Dimension ( 200, (18*3)+4 ) ;
   private Vector          listeners       = null ;
   private String          currentText     = null ;
   private boolean         controlsEnabled = false ;
   private JSpellErrorInfo errorInfo       = null ;
   private boolean         foundErrors     = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    The default constructor creates a blank,
    *  disabled panel. The panel can be re-enabled
    *  by calling 'showErrorInfo'
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SpellPanel () {         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our GUI                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      initializeGUI () ;


   } /* spellPanel  */





   /*------------------------------------------------------------*/
   /* initializeGUI
    */
   /**
    *    This method will initialize our GUI controls
    *  and will add them to our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeGUI () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SPELL          = new JLabel ( "Spell Checker" ) ;
      st_SPELL_CHECKING = new JLabel ( "Checking the spelling" ) ;
      st_REPLACE_WITH   = new JLabel ( "Replace With:" ) ;
      ef_REPLACE_WITH   = new JTextField ( "" ) ;
      pb_REPLACE        = new DButton ( "Replace" ) ;
      pb_IGNORE         = new DButton ( "Ignore" ) ;
      pb_IGNORE_ALL     = new DButton ( "Ignore All" ) ;
      pb_LEARN          = new DButton ( "Learn" ) ;
      pb_STOP           = new DButton ( "Stop" ) ;
      lb_SUGGESTIONS    = new JList ( ) ;
      scr_SUGGESTIONS   = new JScrollPane ( lb_SUGGESTIONS ) ;
      buttonPanel       = new JPanel() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_LEARN.addActionListener              ( this ) ;
      pb_IGNORE_ALL.addActionListener         ( this ) ;
      pb_REPLACE.addActionListener            ( this ) ;
      pb_IGNORE.addActionListener             ( this ) ;          
      pb_STOP.addActionListener               ( this ) ;
      lb_SUGGESTIONS.addListSelectionListener ( this ) ;
      ef_REPLACE_WITH.addActionListener       ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SPELL.setFont ( new Font ( "Helv", Font.BOLD, 12 ) ) ;
      buttonPanel.setBorder ( new BevelBorder ( BevelBorder.LOWERED ) ) ;
      buttonPanel.setBackground ( new Color ( 150, 150, 150 ) ) ;
      setOpaque ( false ) ;




      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADd the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_SPELL ) ;
      add ( ef_REPLACE_WITH ) ;
      add ( st_REPLACE_WITH ) ;
      add ( scr_SUGGESTIONS ) ;
      add ( st_SPELL_CHECKING ) ;
      add ( buttonPanel ) ;
      buttonPanel.setLayout ( new GridLayout ( 3, 2, 2, 2 ) ) ;
      buttonPanel.add ( pb_REPLACE ) ;
      buttonPanel.add ( pb_LEARN ) ;
      buttonPanel.add ( pb_IGNORE ) ;
      buttonPanel.add ( pb_IGNORE_ALL ) ;
      buttonPanel.add ( pb_STOP ) ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The first time we show, disable everything              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      enableControls ( false );

   } /* initializeGUI  */





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all 
    *  of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;
      int       x    = 2 ;
      int       y    = 2 ;

      super.doLayout () ;

      st_SPELL.setBounds ( x, y, 150, 18 ) ;
      st_REPLACE_WITH.setBounds ( x, y+18, 150, 18 ) ;
      ef_REPLACE_WITH.setBounds ( x, y+36, 150, 18 ) ;
      x+=155 ;

      scr_SUGGESTIONS.setBounds ( x, y, 200, (18*3) ) ;
      x+=210 ;

      buttonPanel.setBounds ( x, y, 200, (18*3) ) ;

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REPLACE                                              */ 
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == ef_REPLACE_WITH ) || ( e.getSource() == pb_REPLACE ) ) {
         notifyReplaceWord ( errorInfo.getWord(), ef_REPLACE_WITH.getText(), errorInfo.getPosition() ) ;
         currentText = currentText.substring ( 0, errorInfo.getPosition() )+ef_REPLACE_WITH.getText()+currentText.substring ( errorInfo.getPosition()+errorInfo.getWord().length() ) ;
         spellToolkit.initRawCheck ( currentText ) ;
         new Thread ( this ).start() ; // Kick off the checker again
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_LEARN                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_LEARN ) {
         if ( errorInfo.getWord() != null )
            spellToolkit.addLearned ( errorInfo.getWord() ) ;
         new Thread ( this ).start() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_STOP                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_STOP ) 
         stopSpellCheck() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_IGNORE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_IGNORE ) ||
                ( e.getSource() == pb_IGNORE_ALL ) ) {
         if ( errorInfo.getWord() != null )
            spellToolkit.addIgnore ( errorInfo.getWord() ) ;
         new Thread ( this ).start() ;
      }
   } /* actionPerformed  */





   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    Returns our preferred size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( prefSize ) ;
   }


   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    Returns our preferred size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* enableControls
    */
   /**
    *    This method will either enable or disable the
    *  controls
    *  
    *
    *  @param enableControls TRUE = Enable the controls, FALSE = disable
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void enableControls ( boolean enable ) {
      st_SPELL.setEnabled ( enable ) ;
      st_SPELL_CHECKING.setEnabled ( enable ) ;
      st_REPLACE_WITH.setEnabled ( enable ) ;
      pb_REPLACE.setEnabled ( enable ) ;
      pb_IGNORE.setEnabled ( enable ) ;
      pb_IGNORE_ALL.setEnabled ( enable ) ;
      pb_LEARN.setEnabled ( enable ) ;
      pb_STOP.setEnabled ( enable ) ;
      lb_SUGGESTIONS.setEnabled ( enable ) ;
      scr_SUGGESTIONS.setEnabled ( enable ) ;
      ef_REPLACE_WITH.setEnabled ( enable ) ;
      buttonPanel.setEnabled ( enable ) ;
      controlsEnabled = enable ;
   }





   /*------------------------------------------------------------*/
   /* addSpellPanelListener
    */
   /**
    *    This method will add a listener to the list
    *  of listeners that will be notified of events
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addSpellPanelListener ( SpellPanelListener listener ) {
      if ( listeners == null )
         listeners = new Vector ( 1 ) ;

      listeners.addElement ( listener ) ;
   }






   /*------------------------------------------------------------*/
   /* removeSpellPanelListener
    */
   /**
    *    This method will remove a SpellPanelListener from
    *  the list of notified SpellPanelListeners
    *  
    *
    *  @param listener The SpellPanelListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeSpellPanelListener ( SpellPanelListener listener ) {
      if ( listeners != null ) {
         listeners.removeElement ( listener ) ;
         if ( listeners.size() == 0 )
            listeners = null ;
      }

   }

   /*------------------------------------------------------------*/
   /* spellCheck
    */
   /**
    *    This method will spell check the string passed in, stopping
    *  at each word and asking for user intervention. This method will return
    *  immediatly. The caller will not get notification of an event
    *  until the user presses 'stop' or the spell check is complete. This in
    *  turn will fire the 'spellCheckStopped' notification to all register SpellPanelListeners
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void spellCheck ( String text ) {                                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are currently spell checking, stop that one       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( currentText != null )
         stopSpellCheck() ;

      this.currentText = text ;              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Start the spell checker                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      startSpellCheck () ;

      new Thread ( this ).start() ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread actually does the looping work
    *  of the spell checked
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Disable the controls if need be                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( controlsEnabled )
         enableControls ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initiate the Raw Check                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      errorInfo = spellToolkit.rawCheck() ;                                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If errorInfo is not null, there were misspelled words,  */
      /* otherwise the spell check was clean                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      
      if ( errorInfo != null ) {

         foundErrors = true ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notify of the error found                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         notifySpellCheckWord ( errorInfo.getWord(), errorInfo.getPosition(), errorInfo.getEndPosition() ) ;        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Populate our data                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         lb_SUGGESTIONS.setListData ( errorInfo.getSuggestions() ) ;
         ef_REPLACE_WITH.setText ( errorInfo.getWord() ) ;
         //ef_REPLACE_WITH.requestFocus() ;


         enableControls ( true ) ;


      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Notify that we are done                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else 
         stopSpellCheck() ;


   }





   /*------------------------------------------------------------*/
   /* stopSpellCheck
    */
   /**
    *    This method will clear out all of our controls
    *  and memory to stop the Spell Checker
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void stopSpellCheck () {
      
      lb_SUGGESTIONS.setListData ( new Vector ( 1 ) ) ;
      ef_REPLACE_WITH.setText ( "" ) ;
      enableControls ( false ) ;
      spellToolkit = null ;
      currentText = null ;

      notifySpellCheckComplete ( foundErrors ) ;

   }




   /*------------------------------------------------------------*/
   /* startSpellCheck
    */
   /**
    *    This method will create and start the
    *  spell checker.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void startSpellCheck () {
      spellToolkit = new SpellChecker().getToolkit() ;

      foundErrors = false ;
      spellToolkit.initRawCheck ( currentText ) ;

   }




   /*------------------------------------------------------------*/
   /* notifySpellCheckWord
    */
   /**
    *    This method will call the spellCheckWord
    *  method on all registered listeners
    *  
    *
    *  @param word The word that was mispelled
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void notifySpellCheckWord ( String word, int startPos , int endPos ) {
      if ( listeners != null ) {
         int i       = 0 ;
         int size    = listeners.size() ;

         while ( i < size ) {
            ((SpellPanelListener)listeners.elementAt(i)).spellCheckWord ( word, startPos, endPos ) ;
            i++ ;
         }

      }
   }



   /*------------------------------------------------------------*/
   /* notifyReplaceWord
    */
   /**
    *    This method will call the replaceWord
    *  method on all registered listeners
    *  
    *
    *  @param oldWord The old, misspelled word
    *  @param newWord The new, correct word
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void notifyReplaceWord ( String oldWord, String newWord, int startPos ) {
      if ( listeners != null ) {
         int i       = 0 ;
         int size    = listeners.size() ;

         while ( i < size ) {
            ((SpellPanelListener)listeners.elementAt(i)).replaceWord ( oldWord, newWord, startPos ) ;
            i++ ;
         }

      }
   }


   /*------------------------------------------------------------*/
   /* notifySpellCheckComplete
    */
   /**
    *    This method will calle the spellCheckComplete
    *  method on all registered listeners
    *  
    *
    *  @param foundErrors TRUE if we found errors, false if not
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void notifySpellCheckComplete ( boolean foundErrors ) {
      if ( listeners != null ) {
         int i       = 0 ;
         int size    = listeners.size() ;

         while ( i < size ) {
            ((SpellPanelListener)listeners.elementAt(i)).spellCheckComplete ( foundErrors ) ;
            i++ ;
         }

      }
   }




   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when an item in
    *  our list box has changed it's state
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( !e.getValueIsAdjusting() ) {
         ef_REPLACE_WITH.setText ( (String)lb_SUGGESTIONS.getSelectedValue() ) ;
      }
   }

} /* SpellPanel */
