package com.ibm.nzna.shared.spell ;
import java.util.*;
import java.io.*;
import java.net.*;

public class SpellServer extends Thread
{
	public final static int DEFAULT_PORT=5317;
	protected int port;
	protected ServerSocket listen_socket;
	protected static SpellIndex x;

	public static void fail(Exception e, String msg)
	{
		System.err.println(msg+": "+e);
		System.exit(1);
	}

	public SpellServer(int port)
	{
		if(port==0) port=DEFAULT_PORT;
		this.port=port;
		try
		{
			listen_socket=new ServerSocket(port);
		}
		catch (IOException e)
		{
			fail(e,"Exception creating server socket");
		}
		System.out.println("Server: listening on port " + port);
		this.start();
	}

	public void run()
	{
		try
		{
			while(true)
			{
				Socket client_socket=listen_socket.accept();
				SpellHandler sh=new SpellHandler(client_socket,this.x);
			}
		}
		catch (IOException e)
		{
				fail(e,"Exception while listening for connections");
		}
	}

	public static void main(String args[])
	{
		//Spelling class stub loader

		SpellIndex x=new SpellIndex();
		SpellServer.x=x;
		if(!x.indexReady)
		{	System.out.println("Building Dictionary Using 'dict.txt'");
			x.build("dict.txt");
		}

		int port=0;
		if(args.length==1)
		{
			try
			{
				port=Integer.parseInt(args[0]);
			}
			catch(NumberFormatException e)
					{port=0;}
		}
		if(port==99999)
		{
			// debug port to test server
			try
			{
				String word;
				int i=0;
				int j=0;
				String[] suggestions=new String[10];
				DataInputStream d = new DataInputStream((new BufferedInputStream(new FileInputStream("dict.txt"))));
				while (d.available()>0)
				{
					word=d.readLine();
					if(x.search(j_util.soundex(word.toUpperCase()),word.toUpperCase(),suggestions ) )
						System.out.print(".");
					j++;
					i++;
 /*					if(i==100)
					{
						System.out.println("Word "+j+" "+word);
						i=0;
					}*/
				}
				d.close();
			}
			catch (Exception e)
			{
				System.out.println(e);
				e.printStackTrace();
				System.exit(0);
			}
			
		}
		else
			new SpellServer(port);
	}
}

class SpellHandler extends Thread
{
	protected Socket client;
	protected DataInputStream in;
	protected DataOutputStream out;

	Socket incoming;
	SpellIndex si;
	String suggestions[];

	public SpellHandler(Socket client_socket, SpellIndex si)
	{
		client=client_socket;
		this.si=si;
		// size of suggestions array should be configurable
		suggestions=new String[10];

		try
		{
			in=new DataInputStream(client.getInputStream());
			out=new DataOutputStream(client.getOutputStream());
		}
		catch (IOException e)
		{
			try{
				client.close();
			}
			catch (IOException e2)
			{
				System.err.println("Exception while getting socket streams: "+e);
			}
			return;
		}
		this.start();
	}

	public void run()
	{
		String str="";
		char curchar=' ',lastchar=' ';
		int wordPos;
		
		try
		{
			boolean done=false;
			while(!done)
			{
				curchar=(char)in.readByte();
				switch(curchar)
				{
					case '|':
						// word follows, read it
						str=in.readUTF();
						wordPos=in.readInt();
//						System.out.println(str+" "+wordPos);
						for(int i=10;i>0;i--)
						{
							suggestions[i-1]=null;		// after suggestion made clear it
						}
						if(!search(j_util.soundex(str.toUpperCase()),str.toUpperCase(),suggestions ) )
						{
							out.writeInt(wordPos);
							for(int i=0;i<suggestions.length && suggestions[i]!=null;i++)
							{
								out.writeUTF(suggestions[i]);
							}
							out.writeUTF(":");		// Terminates
							out.flush();
						}
						break;
					case '!':
						// new word to learn follows, read it
						str=in.readUTF();
						synchronized(si)
						{
							try {
							if(si.locked)
								si.wait();
							} catch (InterruptedException ie)
							{System.out.println(ie);
							}
							si.locked=true;
							si.add(str,j_util.soundex(str),si.root);
							System.out.println("Learned Word: "+str);
							si.locked=false;
							si.notify();
						}
						break;
					case ':':
						done=true;
						break;
					case '@':
						out.writeInt(-1);	// signals end of results
						out.flush();
						break;	
				}
			}
		}
		catch (Exception e)
		{
			System.out.println("Exception in Thread: "+e);
		}
		finally
		{
			try
			{
				client.close();
			}
			catch (Exception ex)
			{
				System.out.println(ex);
			}
		}
	}

	public boolean search(String soundex, String original, String suggestions[])
	{
		boolean result=false;
		synchronized(si)
		{
			try {
			if(si.locked)
				si.wait();
			} catch (InterruptedException ie)
			{System.out.println(ie);
			}
			si.locked=true;
			try
			{
				result=si.search(soundex,original,suggestions);
			}
			catch (IOException e)
			{
				System.out.println("Error in Thread (search method): "+e);
			}
			si.locked=false;
			si.notify();
		}
		return result;
	}
}
