package com.ibm.nzna.shared.util;
/*-----------------------------------------------------------*/
/*   AccessRec                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AccessRec                           */
/*                                                           */
/*                                                           */
/*         @Copyright 2000 IBM                               */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 *  The Access Rec is used to interface with the SecureProperties
 * class to get the User Id and Password for production systems. The
 * secure properties file must follow this naming convention for the
 * keys:<br><br>
 *
 * For Specifying User Ids:<br>
 * Host.access.Type.uid<br><br>
 *
 * For Specifying Passwords:<br>
 * Host.access.Type.pwd<br><br><br>
 *
 * This example specifies the User Id and Password for the production database on webdb01<br>
 * webdb01.access.db.uid=userid<br>
 * webdb01.access.db.pwd=password<br>
 *
 *
 *
 * @version 1.0, 12/12/2000
 * @author  Brian Bokor
 */
import com.ibm.nzna.shared.util.*;
import java.util.*;

/*-----------------------------------------------------------*/
/*   AccessRec                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Keymaster                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 *
 *
 *
 * @version 1.0, 12/12/2000
 * @author  Brian Bokor
 */

class AccessRec {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String                  uid              = null ;
   private String                  pwd              = null ;
   private Object                  objPwd           = null ;
   private String                  host             = null ;
   private String                  type             = null ;
   private SecureProperties secureProperties        = null ;


   /*------------------------------------------------------------*/
   /* AccessRec
	*/
   /**
	*    This constructor creates the default
	*  object w/o a property file loaded
	*
	*
	*  @param host The host of the server to retrieve User Ids/Passwords from
	*  @param accessType One of the predefined access types defined in this class (ACCESS_FTP, ACCESS_DATABASE etc..)
	*  @param None
	*
	*  @return Nothing
	*//*---------------------------------------------------------*/

   public AccessRec(String host, String type ) {

	  secureProperties = KeyMaster.getSecureProperties() ;

	  if ( secureProperties != null ) {

		 this.host = host ;
		 this.type = type ;
		 /*-----------------------------------------------------------*/
		 /*                                                           */
		 /*   Get the user and password information                   */
		 /*   Will return a null value if nothing is found            */
		 /*                                                           */
		 /*-----------------------------------------------------------*/

		 uid = (String) secureProperties.get(this.host + "." + "access" + "." + this.type + "." + "uid" );
		 pwd = (String) secureProperties.get(this.host + "." + "access" + "." + this.type + "." + "pwd" );

	  } else
		 System.err.println ( "Cannot access SecureProperties instance." ) ;
   }   
   /*------------------------------------------------------------*/
   /* AccessRec
	*/
   /**
	*    This constructor creates and object with the properties object sent
	*    to  it from the calling program.
	*
	*  @param host The host of the server to retrieve User Ids/Passwords from
	*  @param accessType One of the predefined access types defined in this class (ACCESS_FTP, ACCESS_DATABASE etc..)
	*  @param secureProperties Property file already loaded by calling program
	*
	*  @return Nothing
	*//*---------------------------------------------------------*/

   public AccessRec(String host, String type, SecureProperties secureProperties ) {
		 /*-----------------------------------------------------------*/
		 /*                                                           */
		 /*   Get the user and password information                   */
		 /*                                                           */
		 /*-----------------------------------------------------------*/
		 this.host = host ;
		 this.type = type ;
		 this.secureProperties = secureProperties ;
		 //enter info in to change type to string
		 uid = (String) secureProperties.get(this.host + "." + "access" + "." + this.type + "." + "uid" );
		 pwd = (String) secureProperties.get(this.host + "." + "access" + "." + this.type + "." + "pwd" );
   }   
   /*------------------------------------------------------------*/
   /* getUID
	*/
   /**
	*
	*  This method will return a UserId value contained in
	*  AccessRec
	*
	*  @param None
	*  @param None
	*  @param None
	*
	*  @return String
	*//*---------------------------------------------------------*/
   public String getUID() {
	  return(uid);
   }   
   /*------------------------------------------------------------*/
   /* getPWD
	*/
   /**
	*
	*  This method will return a Password value contained in
	*  AccessRec
	*
	*  @param None
	*  @param None
	*  @param None
	*
	*  @return String
	*//*---------------------------------------------------------*/
   public String getPWD() {
	  return(pwd);
   }   
}