package com.ibm.nzna.shared.util ;
/*-----------------------------------------------------------*/
/*   FontUtil                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FontUtil                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 Cogent                           */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class holds static methods to help extend the functionality
 * of fonts
 *
 *
 * @version 1.0, 5/5/1999 
 * @author  David Lentz
 */

import java.awt.Font ;
import java.awt.FontMetrics ;
import java.awt.Graphics ;
import java.awt.Toolkit ;
import java.awt.Rectangle ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   FontUtil                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class FontUtil extends Object implements java.io.Serializable {

   private int             currentCacheSize        = 128 ;
   private int[]           fontCharWidths          = new int[ currentCacheSize ] ;
   private Font            font                    = null ;
   private FontMetrics     fontMetrics             = null ;
   private int             fontHeight              = 0 ;
   private int             fontAscent              = 0 ;
   private int             fontDescent             = 0 ;
   private int             fontAdvance             = 0 ;
   private byte[]          byteChars               = new byte[1024] ;
   private int             fontLeading             = 0 ;

   public FontUtil () {
   }
   public FontUtil (Font font) {
      setFont(font) ;
   }

   public Font getFont () {
      return( font ) ;
   }

   public void setFont (Font font) {
      this.font            = font ;
      this.fontMetrics     = Toolkit.getDefaultToolkit().getFontMetrics (font) ;
      this.fontHeight      = fontMetrics.getHeight() ;
      this.fontAscent      = fontMetrics.getAscent() ;
      this.fontDescent     = fontMetrics.getDescent() ;
      this.fontAdvance     = fontMetrics.getMaxAdvance() ;
      this.fontLeading     = fontMetrics.getLeading() ;
      /* clear the character width array for this new font */
      for ( int index = 0 ; index < currentCacheSize ; index++ ) {
         fontCharWidths[ index ] = 0 ;
      }
   }
   public int getYCentered (int height) {
      int         space    = (height-fontHeight)/2 ;
      int         fontY    = height-space-fontDescent-(fontLeading/2) ;

      return(fontY) ;
   }

   public static int getYCentered (Font font, int height) {
      FontMetrics fontMetrics     = Toolkit.getDefaultToolkit().getFontMetrics (font) ;
      int fontHeight  = fontMetrics.getHeight() ;
      int fontLeading = fontMetrics.getLeading() ;
      int fontDescent = fontMetrics.getDescent() ;
      int space       = (height-fontHeight)/2 ;
      int fontY       = height-space-fontDescent-(fontLeading/2) ;

      return(fontY) ;
   }


   /* This is static so it can be compiled inline */
   public static int getCharWidth (FontUtil fGraphics, char c ) {
      /* Check to see if we need to use a large array */
      if ( c > fGraphics.currentCacheSize - 1 ) {
         /* This will allow for a unicode character set */
         fGraphics.currentCacheSize = 65535 ;
         /* Create a new array for holding the data */
         fGraphics.fontCharWidths = new int [ fGraphics.currentCacheSize ] ;
      }
      if ( fGraphics.fontCharWidths[c] == 0 ) {
         return(fGraphics.fontCharWidths[c] = fGraphics.fontMetrics.charWidth(c)) ;
      }
      return( fGraphics.fontCharWidths[c] ) ;
   }

   public int getCharsWidth ( char chars[], int start, int end ) {
      int charsLengthSum = 0 ;
      for ( int charIndex = start; charIndex <= end; charIndex++ ) {
         charsLengthSum += getCharWidth( this, chars[ charIndex ] ) ;
      }
      return( charsLengthSum ) ;
   }
   public void drawChars ( Graphics graphics, char chars[], int length , int x, int y, Rectangle clipRect, int xLimit) {
      /* make sure length is not 0 */
      if ( length == 0 )
         return ;
      /* make sure we do not go beyond the array boundary */
      else
         validateArrayLength ( this , length ) ;

      int xOffSet = clipRect.x - x ;
      if ( xOffSet < 0 )
         xOffSet = 0 ;
      int widthBounds = (clipRect.x + clipRect.width) - x ;
      int width = xLimit ;
      if ( widthBounds < width )
         width = widthBounds ;

      boolean  unicode = false ;
      /* we must determine which chararcters to draw */
      int charPosition        = 0 ;
      int charPositionStart   = 0 ;
      int charPositionEnd     = 0 ;
      int xPosition           = 0 ;
      int xStartCharPosition  = 0 ;
      int xEnding             = xLimit ;
      /* Find the starting character to draw */
      while ( (charPosition < length - 1) && ((xOffSet - xPosition) > getCharWidth( this, chars[charPosition])) ) {
         /* Set x position of the starting character to draw */
         xStartCharPosition = xPosition ;
         /* move our search x position the width of the current character */
         xPosition += getCharWidth( this, chars[charPosition]) ;
         /* We will fill our byte buffer for optimized drawing */
         /* If we encounter a unicode character we can not use */
         /* the optimized byte buffer.                         */
         if ( chars[charPosition] > 255 )
            unicode = true ;
         byteChars[charPosition] = (byte) chars[charPosition] ;
         /* set the index of the starting character */
         charPositionStart = ++charPosition ;
      }
      /* Set the xOffSet to the starting character position */
      /* We must move the offset because we can not draw a  */
      /* partial character.                                 */
      //xOffSet = xStartCharPosition ;
      xOffSet = xPosition ;

      charPosition = charPositionEnd = charPositionStart ;
      /* Find the ending character to draw */
      /* Note, Fix Code, need to go 1 char past width so will paint correct when clipping, but never past xLimit */
      while ( (charPosition < length ) && 
              ((xPosition + getCharWidth( this, chars[charPosition])) < xLimit ) &&
              (xPosition < width) ) {
         /* move our search x position the width of the current character */
         xPosition += getCharWidth( this, chars[charPosition]) ;
         /* We will fill our byte buffer for optimized drawing */
         /* If we encounter a unicode character we can not use */
         /* the optimized byte buffer.                         */
         if ( chars[charPosition] > 255 )
            unicode = true ;
         byteChars[charPosition] = (byte) chars[charPosition] ;
         /* set the index of the ending character */
         charPositionEnd = charPosition++ ;
      }
      //System.out.println("xPosition "+xPosition) ;
      //System.out.println("charPositionStart "+charPositionStart+" charPositionEnd "+charPositionEnd+ "width "+width+" xOffSet "+xOffSet) ;
      //System.out.println("drawing character "+chars[charPositionStart]+" at "+(xOffSet+x)) ;
      //System.out.println("Starting position "+charPositionStart+" endingPosition "+charPositionEnd) ;
      /* draw the characters into the graphics context */
      if ( unicode )
         graphics.drawChars ( chars    , charPositionStart, charPositionEnd - charPositionStart + 1, xOffSet + x, y) ;
      else
         graphics.drawBytes ( byteChars, charPositionStart, charPositionEnd - charPositionStart + 1, xOffSet + x, y) ;
   }
   public void drawChars ( Graphics graphics, char chars[], int charOffSet , int length, int x, int y) {
      /* make sure length is not 0 */
      if ( length == 0 )
         return ;

      boolean  unicode        = false ;
      int      charPosition   = 0 ;
      while ( charPosition < length ) {
         if ( chars[charPosition] > 255 )
            unicode = true ;
         byteChars[charPosition] = (byte) chars[charPosition] ;
         charPosition++ ;
      }
      if ( unicode )
         graphics.drawChars ( chars    , charOffSet, length,  x, y) ;
      else
         graphics.drawBytes ( byteChars, charOffSet, length,  x, y) ;
   }
   /*********************************************************
   * <b><font color="maroon">Description:</b></font>
   * Draws characters within a region.  If characters go beyond limit
   * ... are drawn at the end.
   * This function is not currently unicode compliant.
   *
   * <p><b><font color="teal">Author:</b></font> <i>Chad Meadows</i>
   * <p><b><font color="teal">Created:</b></font> 6/9/1998  8:18am
   * @param
   * @return
   *********************************************************/
   public void drawCharsEtc ( Graphics graphics, char chars[], int length , int x, int y, Rectangle clipRect, int xLimit ) {
      boolean drawEtc = false ;
      /* make sure length is not 0 */
      if ( length == 0 )
         return ;
      /* make sure we do not go beyond the array boundary */
      else
         validateArrayLength ( this , length ) ;

      int xOffSet = -x;
      if ( clipRect != null )
         xOffSet = clipRect.x - x ;
      if ( xOffSet < 0 )
         xOffSet = 0 ;
      int widthBounds = 0 ;
      if ( clipRect != null )
         widthBounds = (clipRect.x + clipRect.width) - x ;
      else
         widthBounds = xLimit+100 - x ;
      int width = xLimit ;
      if ( widthBounds < width )
         width = widthBounds ;

      boolean  unicode = false ;
      /* we must determine which chararcters to draw */
      int charPosition        = 0 ;
      int charPositionStart   = 0 ;
      int charPositionEnd     = 0 ;
      int xPosition           = 0 ;
      int xStartCharPosition  = 0 ;
      int xEnding             = xLimit ;
      /* Find the starting character to draw */
      while ( (charPosition < length - 1) && ((xOffSet - xPosition) > getCharWidth( this, chars[charPosition])) ) {
         /* Set x position of the starting character to draw */
         xStartCharPosition = xPosition ;
         /* move our search x position the width of the current character */
         xPosition += getCharWidth( this, chars[charPosition]) ;
         /* We will fill our byte buffer for optimized drawing */
         /* If we encounter a unicode character we can not use */
         /* the optimized byte buffer.                         */
         if ( chars[charPosition] > 255 )
            unicode = true ;
         byteChars[charPosition] = (byte) chars[charPosition] ;
         /* set the index of the starting character */
         charPositionStart = ++charPosition ;
      }
      /* Set the xOffSet to the starting character position */
      /* We must move the offset because we can not draw a  */
      /* partial character.                                 */
      //xOffSet = xStartCharPosition ;
      xOffSet = xPosition ;

      charPosition = charPositionEnd = charPositionStart ;
      /* Find the ending character to draw */
      /* Note, Fix Code, need to go 1 char past width so will paint correct when clipping, but never past xLimit */
      while ( (charPosition < length ) && 
              ((xPosition + getCharWidth( this, chars[charPosition])) < xLimit ) &&
              (xPosition < width) ) {
         /* move our search x position the width of the current character */
         xPosition += getCharWidth( this, chars[charPosition]) ;
         /* We will fill our byte buffer for optimized drawing */
         /* If we encounter a unicode character we can not use */
         /* the optimized byte buffer.                         */
         if ( chars[charPosition] > 255 )
            unicode = true ;
         byteChars[charPosition] = (byte) chars[charPosition] ;
         /* set the index of the ending character */
         charPositionEnd = charPosition++ ;
      }

      /* Will remaining characters go beyond xLimit */
      int charPositionRemaining = charPosition ;
      while ( charPositionRemaining < length ) {
         /* move our search x position the width of the current character */
         xPosition += getCharWidth( this, chars[charPositionRemaining]) ;
         /* We will fill our byte buffer for optimized drawing */
         /* If we encounter a unicode character we can not use */
         /* the optimized byte buffer.                         */
         if ( chars[charPositionRemaining] > 255 )
            unicode = true ;
         byteChars[charPositionRemaining] = (byte) chars[charPositionRemaining] ;
         /* Have we reached the limit of our allowed drawing area? */
         /* We must replace the last characters with ...           */
         if ( (xPosition >= xLimit) && ( charPositionRemaining > 1 ) ) {
            drawEtc = true ;
            byteChars[ charPositionRemaining ]     = (int)'.' ;
            byteChars[ charPositionRemaining - 1 ] = (int)'.' ;
            byteChars[ charPositionRemaining - 2 ] = (int)'.' ;
            break ;
         }
         /* set the index of the ending character */
         charPositionRemaining++ ;
      }

      /* If we replaced characters with ... we must now recalculate */
      /* the number of characters to draw since the widths will be  */
      /* different of the characters we replaced.                   */
      if ( drawEtc ) {
         charPosition        = 0 ;
         charPositionStart   = 0 ;
         charPositionEnd     = 0 ;
         xPosition           = 0 ;
         xStartCharPosition  = 0 ;
         xEnding             = xLimit ;
         /* Find the starting character to draw */
         while ( (charPosition < length - 1) && ((xOffSet - xPosition) > getCharWidth( this, (char) byteChars[charPosition])) ) {
            /* Set x position of the starting character to draw */
            xStartCharPosition = xPosition ;
            /* move our search x position the width of the current character */
            xPosition += getCharWidth( this, (char) byteChars[charPosition]) ;
            /* We will fill our byte buffer for optimized drawing */
            /* If we encounter a unicode character we can not use */
            /* the optimized byte buffer.                         */
            //if (chars[charPosition] > 255) 
            //   unicode = true ;
            //byteChars[charPosition] = (byte) chars[charPosition] ;
            /* set the index of the starting character */
            charPositionStart = ++charPosition ;
         }
         /* Set the xOffSet to the starting character position */
         /* We must move the offset because we can not draw a  */
         /* partial character.                                 */
         //xOffSet = xStartCharPosition ;
         xOffSet = xPosition ;
         charPosition = charPositionEnd = charPositionStart ;
         /* Find the ending character to draw */
         /* Note, Fix Code, need to go 1 char past width so will paint correct when clipping, but never past xLimit */
         while ( (charPosition <= charPositionRemaining ) && 
                 ((xPosition + getCharWidth( this, (char)byteChars[charPosition])) < xLimit ) &&
                 (xPosition < width) ) {
            /* move our search x position the width of the current character */
            xPosition += getCharWidth( this, (char)byteChars[charPosition]) ;
            /* We will fill our byte buffer for optimized drawing */
            /* If we encounter a unicode character we can not use */
            /* the optimized byte buffer.                         */
            //if (chars[charPosition] > 255) 
            //   unicode = true ;
            /* set the index of the ending character */
            charPositionEnd = charPosition++ ;
         }
      }


      /* draw the characters into the graphics context */
      if ( unicode )
         graphics.drawChars ( chars    , charPositionStart, charPositionEnd - charPositionStart + 1, xOffSet + x, y) ;
      else
         graphics.drawBytes ( byteChars, charPositionStart, charPositionEnd - charPositionStart + 1, xOffSet + x, y) ;
   }

   /*********************************************************
   * <b><font color="maroon">Description:</b></font>
   * 
   *
   * <p><b><font color="teal">Author:</b></font> <i>Chad Meadows</i>
   * <p><b><font color="teal">Created:</b></font> 10/22/1998  11:47am
   * @param
   * @return
   *********************************************************/
   private final static void validateArrayLength ( FontUtil fGraphics , int length ) {
      if ( length >= ( fGraphics.byteChars.length - 4 ) ) {
         fGraphics.byteChars = new byte [ length * 2 ] ;
      }
   }

   /*------------------------------------------------------------*/
   /* drawHotLinkString
    */
   /**
    *    This method will draw a string and automatically
    *  underline the string to make it look like a 
    *  URL Hot Link
    *
    *  @param g The Graphics Context to draw into
    *  @param string The String to draw
    *  @param x,y The x and y coordinates
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void drawHotLinkString ( Graphics g, String string, int x, int y ) {
      FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics ( g.getFont() ) ;

      g.drawString ( string, x, y ) ;
      g.drawLine ( x, y+1, x+fm.stringWidth ( string ), y+1 ) ;
   }


}
