package com.ibm.nzna.shared.util;
/*-----------------------------------------------------------*/
/*   KeyMaster                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Keymaster                           */
/*                                                           */
/*                                                           */
/*         @Copyright 2000 IBM                               */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 *
 *
 *
 * @version 1.0, 12/12/2000
 * @author  Brian Bokor
 */
import com.ibm.nzna.shared.util.*;
import java.util.*;

/*-----------------------------------------------------------*/
/*                                                           */
/*   KeyMaster Class                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class KeyMaster extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public static final members                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String ACCESS_FTP      = "ftp" ;
   public static final String ACCESS_DATABASE = "database" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static SecureProperties       secureProperties = null ;
   private static StandardPropertyLoader loader           = null ;
   private        AccessRec              accessRec        = null ;
   private        boolean                accessValid      = true ;

   /*------------------------------------------------------------*/
   /* KeyMaster
    */
   /**
    *    This constructor creates the default
    *  object
    *
    *
    *  @param host The host of the server to retrieve User Ids/Passwords from
    *  @param accessType One of the predefined access types defined in this class (ACCESS_FTP, ACCESS_DATABASE etc..)
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public KeyMaster ( String host, String accessType ) throws UnknownAccessTypeException {

      if ( !isAccessTypeValid( accessType ) )
        throw new UnknownAccessTypeException() ;

        accessRec = new AccessRec( host, accessType );
   }

   public KeyMaster ( String host, String accessType, Properties p ) throws UnknownAccessTypeException {

      if ( !isAccessTypeValid( accessType ) )
        throw new UnknownAccessTypeException() ;

      secureProperties = ( SecureProperties )p ;
      accessRec = new AccessRec( host, accessType, secureProperties );
   }

   /*------------------------------------------------------------*/
   /* isAccessTypeValid
    */
   /**
    *   This method will check to see if the AccessType sent in
    * is one of the allowable types
    *
    *
    *
    *  @param String
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean isAccessTypeValid( String accessType ){

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Method for checking the Access Type                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
          if( accessType.equals( ACCESS_FTP ) || accessType.equals( ACCESS_DATABASE ) )
            return true;
          else
            return false;
   }

   /*------------------------------------------------------------*/
   /* getUserId
    */
   /**
    *    This method will return the User Id
    *  for the access type requested
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getUserId ( ) {
      return( accessRec.getUID () ) ;
   }


   /*------------------------------------------------------------*/
   /* getPasswordId
    */
   /**
    *    This method will return the Password
    *  for the access type requested
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getPassword() {
      return( accessRec.getPWD() );
   }

   /*------------------------------------------------------------*/
   /* getSecureProperties
    */
   /**
    *    This method will return the one instance of the
    *  secure properties that the KeyMaster uses. If the
    *  instance has not been created yet, it will create the instance now
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return SecureProperties
    *//*---------------------------------------------------------*/
   protected static SecureProperties getSecureProperties() {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the instance is null, create a new one now           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( secureProperties == null ) {
         try {
            loader = new StandardPropertyLoader() ;
            secureProperties = ( SecureProperties )loader.getSecureProperties() ;
         } catch ( Exception e ) {
            e.printStackTrace() ;
            secureProperties = null ;
         }
      } /* secureProperties */
      return( secureProperties ) ;
   }
}

/*-----------------------------------------------------------*/
/*   AccessRec                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Keymaster                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 *
 *
 *
 * @version 1.0, 12/12/2000
 * @author  Brian Bokor
 */

class AccessRec {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String                  uid              = null ;
   private String                  pwd              = null ;
   private Object                  objPwd           = null ;
   private String                  host             = null ;
   private String                  type             = null ;
   private SecureProperties secureProperties        = null ;


   /*------------------------------------------------------------*/
   /* AccessRec
    */
   /**
    *    This constructor creates the default
    *  object w/o a property file loaded
    *
    *
    *  @param host The host of the server to retrieve User Ids/Passwords from
    *  @param accessType One of the predefined access types defined in this class (ACCESS_FTP, ACCESS_DATABASE etc..)
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/

   public AccessRec(String host, String type ) {

      secureProperties = KeyMaster.getSecureProperties() ;

      if ( secureProperties != null ) {

         this.host = host ;
         this.type = type ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the user and password information                   */
         /*   Will return a null value if nothing is found            */
         /*                                                           */
         /*-----------------------------------------------------------*/

         uid = (String) secureProperties.get(this.host + "." + "access" + "." + this.type + "." + "uid" );
         pwd = (String) secureProperties.get(this.host + "." + "access" + "." + this.type + "." + "pwd" );

      } else
         System.err.println ( "Cannot access SecureProperties instance." ) ;
   }
   /*------------------------------------------------------------*/
   /* AccessRec
    */
   /**
    *    This constructor creates and object with the properties object sent
    *    to  it from the calling program.
    *
    *  @param host The host of the server to retrieve User Ids/Passwords from
    *  @param accessType One of the predefined access types defined in this class (ACCESS_FTP, ACCESS_DATABASE etc..)
    *  @param secureProperties Property file already loaded by calling program
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/

   public AccessRec(String host, String type, SecureProperties secureProperties ) {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the user and password information                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         this.host = host ;
         this.type = type ;
         this.secureProperties = secureProperties ;
         //enter info in to change type to string
         uid = (String) secureProperties.get(this.host + "." + "access" + "." + this.type + "." + "uid" );
         pwd = (String) secureProperties.get(this.host + "." + "access" + "." + this.type + "." + "pwd" );
   }
   /*------------------------------------------------------------*/
   /* getUID
    */
   /**
    *
    *  This method will return a UserId value contained in
    *  AccessRec
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getUID() {
      return(uid);
   }
   /*------------------------------------------------------------*/
   /* getPWD
    */
   /**
    *
    *  This method will return a Password value contained in
    *  AccessRec
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getPWD() {
      return(pwd);
   }
}

