package com.ibm.nzna.shared.util ;
/*-----------------------------------------------------------*/
/*   Log                                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Log                                 */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class holds static methods useful for creating and logging messages 
 *
 *
 * @version 1.0, 8/18/1999 
 */
import java.io.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   Log                                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class Log extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Variables                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static boolean showOutput       = false ;
   

   /*------------------------------------------------------------*/
   /* clearLogFile
    */
   /**
    *    This method will clear a log file of all
    *  messages. If the file does not exist, a 
    *  new one will be created
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void clearLogFile ( String logFilename ) {
      try {
         File               debugFile = new File ( logFilename ) ;
         FileOutputStream   out       = new FileOutputStream ( debugFile ) ;

         out.write ( new String ( "" ).getBytes() ) ;

         out.flush() ;

         out.close() ;
      } catch ( Exception e ) {
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "--     Error opening debug file: "+logFilename ) ;
         System.out.println ( "-----------------------------------------------" ) ;
         e.printStackTrace() ;
      }
   }



   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will append the log message
    *  passed to the log file passed
    *  
    *
    *  @param logFile The name of the log file
    *  @param logMessage The name of the message to log
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String log ( String logFilename, Exception ex ) {
      String retStr = "" ;

      try {
         StringWriter    sw      = new StringWriter () ;
         PrintWriter     pw      = new PrintWriter ( sw ) ;

         /* Get the stackTrace */
         ex.printStackTrace ( pw ) ;

         retStr = sw.toString() ;

         FileUtil.appendToFile ( logFilename, retStr ) ;
         if ( showOutput )
            System.out.print ( retStr ) ;

      } catch ( Exception e ) {
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "--     Error opening debug file: "+logFilename ) ;
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "Exception while opening:" ) ;
         e.printStackTrace() ;
         System.out.println ( "\n\nLogging Exception:" ) ;
         ex.printStackTrace() ;

      }

      return ( retStr ) ;
   }


   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will append the log message
    *  passed to the log file passed
    *  
    *
    *  @param logFile The name of the log file
    *  @param logMessage The name of the message to log
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void log ( String logFilename, String logMessage ) {
      try {
         FileUtil.appendToFile ( logFilename, logMessage ) ;
         if ( showOutput )
            System.out.print ( logMessage ) ;
      } catch ( Exception e ) {
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "--     Error opening debug file: "+logFilename ) ;
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "Exception while opening:" ) ;
         e.printStackTrace() ;
         System.out.println ( "\n\nLogging message:\n"+logMessage ) ;

      }
   }



   /*------------------------------------------------------------*/
   /* endLogSection
    */
   /**
    *    This method will end a section by putting a line of asterisks
    *
    *  @param logFilename The name of the log file
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String endLogSection ( String logFilename ) {
      String retStr = "" ;

      try {
         String sectionHeader = "*------------------------------------------------\n" ;
         FileUtil.appendToFile ( logFilename, sectionHeader ) ;
         retStr = sectionHeader ;
      } catch ( Exception e ) {
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "--     Error opening debug file: "+logFilename ) ;
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "Exception while opening:" ) ;
         e.printStackTrace() ;
      }

      return ( retStr ) ;
   }


   /*------------------------------------------------------------*/
   /* beginLogSection
    */
   /**
    *    This method will write out a box of
    *  asterisks with a section name in the
    *  middle of the box.
    *
    *  @param logFilename The name of the log file
    *  @param sectionName The name of the section
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String beginLogSection ( String logFilename, String sectionName ) {
      String retStr = "" ;

      try {
         String sectionHeader = "*------------------------------------------------\n"+
                                "*\n*  "+sectionName+"\n*\n"+
                                "*------------------------------------------------\n" ;
         FileUtil.appendToFile ( logFilename, sectionHeader ) ;
         retStr = sectionHeader ;

      } catch ( Exception e ) {
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "--     Error opening debug file: "+logFilename ) ;
         System.out.println ( "-----------------------------------------------" ) ;
         System.out.println ( "Exception while opening:" ) ;
         e.printStackTrace() ;

      }

      return ( retStr ) ;
   }

} /* Log */
