package com.ibm.nzna.shared.util ;
/*-----------------------------------------------------------*/
/*   LogSystem                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LogSystem                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 *   The Log system is used to log to the different output files from
 * Java applications. It provides a consistant look to all log files.
 * You can also use the 'setErr' and 'setOut' methods to redirect stdout and stderr
 * to the log file being maintained.
 *
 * @version 1.0, 1/7/2000 
 * @author  David Lentz
 */
import java.io.* ;
import java.util.* ;
//import com.ibm.nzna.shared.gui.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   LogSystem                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class LogSystem extends Object {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String       defaultLogFile       = null ;
   private static int          logDebugLevel        = 0 ;
   private static LogSystem    instance             = null ;
   private static boolean      provideGUI           = true ;
   private        boolean      errRedirect          = false ;
   private        boolean      outRedirect          = false ;
   //private static OutputWindow logWindow            = null ;
   private static Vector       alias                = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the Log System.
    *  It will create and clear the log file passed
    *  in the filename param.Any subsequent calls to any static
    *  method in any LogSystem class will write to the filename
    *  passed in.
    *
    *  @param filename The main Log file to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LogSystem ( int debugLevel, String defaultLogFile ) {
      setLogFile ( debugLevel, defaultLogFile ) ;

      instance = this ;
   }





   /*------------------------------------------------------------*/
   /* LogSystem
    */
   /**
    *    This is the default constructor. It will
    *  not create a log file for use. You must use
    *  the 'setLogFile' method to begin logging
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LogSystem () {
   }





   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will log a String to the
    *  log file.
    *  
    *
    *  @param debugLevel The Debug Level that this message should appear in
    *  @param message The message to log
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void log ( int debugLevel, String message ) {

      if ( getInstance() == null )
         System.out.println ( message ) ;
      else {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have our window up, append to it                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         //if ( logWindow != null )
         //   logWindow.appendText ( message+"\n" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the debugLevel passed is 0, write it out             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( debugLevel == 0 ) {
            Log.log ( defaultLogFile, message+"\n" ) ;
            logAlias ( message+"\n" ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, decide if we need to write it out            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            if ( logDebugLevel <= debugLevel ) {
               Log.log ( defaultLogFile, message+"\n" ) ;
               logAlias ( message+"\n" ) ;
            }
         }
      }
   }





   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will log an Exception
    *  to the log file. It will also display the
    *  Exception to the user
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void log ( int debugLevel, Exception e ) {
      String  outStr = "" ;

      if ( getInstance() == null )
         e.printStackTrace() ;
      else {
         try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we are to show the exception, get the string         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( logDebugLevel <= debugLevel ) {


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Log the error                                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               outStr = Log.log ( defaultLogFile, e ) ;
               logAlias ( e ) ;
               Log.log ( defaultLogFile, "-----------------------------------------\n" ) ;                 
               logAlias ( "-----------------------------------------\n" ) ;
               //if ( logWindow != null )
               //   logWindow.appendText ( outStr+"\n-----------------------------------------\n" ) ;
            }

         } catch ( Exception e1 ) {
            System.out.println ( "Could not log Message:\n----------------------------------" ) ;
            e.printStackTrace() ;
            System.out.println ( "\n\nReceived Exception during log:\n" ) ;
            e1.printStackTrace() ;
         }
      }

   }


   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will log an Exception
    *  to the log file. It will also display the
    *  Exception to the user
    *
    *  @param debugLevel The level of debug that the debug information should write with
    *  @param e The Exception to write
    *  @param showErrorDlg TRUE = Show the 'Error in last action took' window.
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void log ( int debugLevel, Exception e, boolean showErrorDlg ) {
      String  outStr = "" ;

      if ( getInstance() == null )
         e.printStackTrace() ;
      else {
         try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we are to show the exception, get the string         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( logDebugLevel <= debugLevel ) {


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Log the error                                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               outStr = Log.log ( defaultLogFile, e ) ;
               logAlias ( e ) ;
               Log.log ( defaultLogFile, "-----------------------------------------\n" ) ;                 
               logAlias ( "-----------------------------------------\n" ) ;
               //if ( logWindow != null )
               //   logWindow.appendText ( outStr+"\n-----------------------------------------\n" ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Show the error to the user                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               //if ( ( showErrorDlg ) && ( provideGUI ) )
               //   new ErrorDlg ( e ) ;

            }

         } catch ( Exception e1 ) {
            System.out.println ( "Could not log Message:\n----------------------------------" ) ;
            e.printStackTrace() ;
            System.out.println ( "\n\nReceived Exception during log:\n" ) ;
            e1.printStackTrace() ;
         }
      }

   }






   /*------------------------------------------------------------*/
   /* beginLogSection
    */
   /**
    *    This method will begin a new section
    *  in the log file. Each new section will have a full
    *  block heading with the description.
    *
    *  @param name The name of the section
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void beginLogSection ( String name ) {
      if ( getInstance() != null ) {
         String outStr = Log.beginLogSection ( defaultLogFile, name ) ;

         logAlias ( outStr ) ;

         //if ( logWindow != null )
         //   logWindow.appendText ( outStr+"\n" ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* endLogSection
    */
   /**
    *    This method will end the current 
    *  log section.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void endLogSection () {
      if ( getInstance() != null ) {
         String outStr = Log.endLogSection ( defaultLogFile ) ;

         Log.log ( defaultLogFile, "\n\n\n" ) ;
         logAlias ( outStr+"\n\n\n" ) ;

         //if ( logWindow != null )
         //   logWindow.appendText ( outStr+"\n\n\n" ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return the instance of the
    *  LogSystem that has been created
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static LogSystem  getInstance () {
      return( instance ) ;
   }





   /*------------------------------------------------------------*/
   /* setDebugLevel
    */
   /**
    *    This method will set the level of debugging
    *  
    *  
    *
    *  @param debugLevel The debug level
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setDebugLevel ( int debugLevel ) {
      logDebugLevel = debugLevel ;
   }





   /*------------------------------------------------------------*/
   /* setLogFile
    */
   /**
    *    THis methos will set what log file that the
    *  LogSystem is using.
    *  
    *
    *  @param debugLevel The level of debugging information to log
    *  @param defaultLogFile The name of the file
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLogFile ( int debugLevel, String defaultLogFile ) {
      this.defaultLogFile = defaultLogFile ;
      Log.clearLogFile ( defaultLogFile ) ;
      instance = this ;
      if ( errRedirect )
         setErr() ;
      if ( outRedirect )
         setOut() ;
   }





   /*------------------------------------------------------------*/
   /* setProvideGUI
    */
   /**
    *    This method will set the 'provideGUI' property
    *  which determines if any errors are shown
    *  in a GUI window.
    *
    *  @param provideGUI TRUE or FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setProvideGUI ( boolean newProvideGUI ) {
      provideGUI = newProvideGUI ;
   }





   /*------------------------------------------------------------*/
   /* showLogWindow
    */
   /**
    *    This method will show the log window or
    *  hide it.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void showLogWindow ( boolean show ) {
      //if ( ( getInstance() != null ) && (  show ) ) {
      //   logWindow = new OutputWindow () ;
      //   logWindow.setTitle ( defaultLogFile ) ;
      //   try {
      //      logWindow.appendText ( FileUtil.readFile ( defaultLogFile ) ) ;
      //   } catch ( Exception e ) {
      //      logWindow.appendText ( "Cannot load initial Debug File." ) ;
      //   }
      //} else
      //   logWindow.dispose() ;
   }




   /*------------------------------------------------------------*/
   /* getDebugLevel
    */
   /**
    *    This method will return the level of debugging
    *  the system is set for
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public static int getDebugLevel () {
      return( logDebugLevel ) ;
   }





   /*------------------------------------------------------------*/
   /* setErr
    */
   /**
    *    This method will redirect stderr
    *  to the log file being written to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setErr ( ) {
      System.setErr ( new LogSystemStream ( this ) ) ;
      errRedirect = true ;
   }




   /*------------------------------------------------------------*/
   /* setOut
    */
   /**
    *    This method will redirect stdout to the
    *  log file that is being maintained
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOut ( ) {
      System.setOut ( new LogSystemStream ( this ) ) ;
      outRedirect = true ;
   }




   /*------------------------------------------------------------*/
   /* addAlias
    */
   /**
    *    This method will add an alias to the LogSystem.
    *  The alias is a separate file that is written
    *  along with the main Log file.
    *
    *  @param alias The name of the log file
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addAlias ( String aliasFilename ) {
      if ( getInstance() != null ) {
         if ( alias == null )
            alias = new Vector ( 1 ) ;

         alias.addElement ( aliasFilename ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* removeAlias
    */
   /**
    *    This method will remove an alias from
    *  the LogSystem. Please note that this does
    *  not delete the file, it simply prevents any more logging to the file
    *
    *  @param alias The name of the file
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeAlias ( String aliasFilename ) {
      if ( getInstance() != null ) {
         if ( alias != null ) {
            alias.removeElement ( aliasFilename ) ;
            if ( alias.size() == 0 )
               alias = null ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* logAlias
    */
   /**
    *    This method will log this string in all
    *  alias log files
    *  
    *
    *  @param string The string to log
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void logAlias ( String string ) {
      if ( getInstance() != null ) {
         if ( alias != null ) {
            int i    = 0 ;
            int size = alias.size() ;

            while ( i < size ) {
               Log.log ( (String)alias.elementAt(i), string ) ;
               i++ ;
            }
         }
      }
   }



   /*------------------------------------------------------------*/
   /* logAlias
    */
   /**
    *    This method will log this exception in all
    *  alias log files
    *  
    *
    *  @param exception The exception to log
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void logAlias ( Exception exception ) {
      if ( getInstance() != null ) {
         if ( alias != null ) {
            int i    = 0 ;
            int size = alias.size() ;

            while ( i < size ) {
               Log.log ( (String)alias.elementAt(i), exception ) ;
               i++ ;
            }
         }
      }
   }



   /**
     *  This method will simply write an exception to a file
     *
     * @param e The throwable exception
     * @param filename The name of the file to write out to
     **/
   public static void logExceptionToFile ( Throwable e, String filename ) {
      StringWriter    sw      = new StringWriter () ;
      PrintWriter     pw      = new PrintWriter ( sw ) ;

      /* Get the stackTrace */
      e.printStackTrace ( pw ) ;

      String retStr = sw.toString() ;
      FileUtil.saveFile ( filename, retStr ) ;
   }
} /* LogSystem */
