package com.ibm.nzna.shared.util ;         
/*-----------------------------------------------------------*/
/*   LogSystemStream                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LogSystemStream                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
/**
 * This subclasses PrintStream so that the LogSystem can redirect stdout and stderr 
 *
 *
 * @version 1.0, 5/7/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LogSystemStream                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class LogSystemStream extends PrintStream {      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private LogSystem log    = null ;
   private String    stream = "" ;



   /*------------------------------------------------------------*/
   /* LogSystemStream
    */
   /**
    *    This method will create our stream so that
    *  we write to the log specfiied
    *  
    *
    *  @param log The LogSystem to use for writing
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LogSystemStream ( LogSystem log ) {
      super ( null ) ;
      this.log = log ;
   }


   public void flush() {
      log.log ( 1, stream ) ;
      stream="" ;
   }

   public void close() {
      stream="" ;
   }

   public boolean checkError() {
      flush() ;
      return( false ) ;
   }

   protected void setError(){
   }

   public void write(int b) {
      stream+=""+b ;
   }


   public void write(byte buf[],
                     int off,
                     int len) {
      for ( int i = off; i < len; i++ ) {
         stream+=""+buf[i] ;
      }
   }

   public void print(boolean b) {
      stream+=""+b ;
   }

   public void print(char c) {
      stream+=""+c ;
   }

   public void print(int i) {
      stream+=""+i ;
   }

   public void print(long l) {
      stream+=""+l ;
   }

   public void print(float f) {
      stream+=""+f ;
   }

   public void print(double d) {
      stream+=""+d ;
   }

   public void print(char s[]) {
      for ( int i = 0; i < s.length; i++ )
         stream+=""+s[i] ;
   }

   public void print(String s) {
      stream+=s ;
   }

   public void print(Object obj) {
      stream+=obj.toString() ;
   }

   public void println() {
      stream+=System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(boolean x) {
      stream+=x+System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(char x) {
      stream+=x+System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(int x) {
      stream+=x+System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(long x) {
      stream+=x+System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(float x) {
      stream+=x+System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(double x) {
      stream+=x+System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(char x[]) {
      for ( int i = 0; i < x.length; i++ )
         stream+=x[i] ;
      stream+=System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(String x) {
      stream+=x+System.getProperty ( "line.separator" ) ;
      flush() ;
   }

   public void println(Object x) {
      stream+=x.toString()+System.getProperty ( "line.separator" ) ;
      flush() ;
   }

} /* LogSystemStream */
