package com.ibm.nzna.shared.util ;
import java.util.* ;
   /*********************************************************
   * Allows encryption in a property file.  The get method
   * automatically decrypts and put encrypts
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 10/30/100  4:37pm
   * @author 
   * @version
   *********************************************************/
public class SecureProperties extends Properties {
   private boolean encryptActive = false ;
   private boolean decryptActive = true ;
   /*********************************************************
   * Overrides the standard get to provide decryption.
   * 
   *
   * Author:  
   * Created: 10/30/100  4:38pm
   * @param
   * @return
   *********************************************************/
   public Object get ( Object key ) {
      String value = (String) super.get ( key ) ;
      if ( decryptActive && value != null) {
         value = decrypt ( value ) ;
      }
      return ( value ) ;
   }
   /*********************************************************
   * Overrides the standard put to provide encryption
   * 
   *
   * Author:  
   * Created: 10/30/100  4:39pm
   * @param
   * @return
   *********************************************************/
   public Object put ( Object key, Object value ) {
      Object returnValue = null ;
      /* We only can encrypt strings */
      if (( encryptActive ) && ( value instanceof String )) {
         value = encrypt ( (String) value ) ;
      }
      returnValue = super.put ( key, value ) ;
      return ( returnValue ) ;
   }
   /*********************************************************
   * Encrypts a String.
   * 
   *
   * Author:  
   * Created: 10/30/100  4:46pm
   * @param
   * @return
   *********************************************************/
   public String encrypt ( String value ) {
      byte bytes[] = value.getBytes() ;
      for ( int i = 0 ; i < bytes.length; i++) {
         bytes[i] = (byte)(bytes[i] - i - 128 ) ;
      }
      value = new String( bytes ) ;
      return ( value ) ;
   }
   /*********************************************************
   * Decrypts a String.
   * 
   *
   * Author:  
   * Created: 10/30/100  4:50pm
   * @param
   * @return
   *********************************************************/
   public String decrypt ( String value ) {
      byte bytes[] = value.getBytes() ;
      for ( int i = 0 ; i < bytes.length; i++) {
         bytes[i] = (byte)(bytes[i] + i + 128 ) ;
      }
      value = new String( bytes ) ;
      return ( value ) ;
   }
   /*********************************************************
   * Sets encryption on or off for putting values
   * 
   *
   * Author:  
   * Created: 11/8/100  9:11pm
   * @param
   * @return
   *********************************************************/
   public void setEncrypt ( boolean encrypt ) {
      this.encryptActive = encrypt ;
   }
   /*********************************************************
   * Sets the descryption on or off for getting values
   * 
   *
   * Author:  
   * Created: 11/8/100  9:13pm
   * @param
   * @return
   *********************************************************/
   public void setDecrypt ( boolean decrypt ) {
      this.decryptActive = decrypt ;
   }
}
