package com.ibm.nzna.shared.util ;
/*-----------------------------------------------------------*/
/*   WinUtil                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WinUtil                             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This class holds simple static methods useful in a windowed environment 
 *
 *
 * @version 1.0, 2/8/1999 
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WinUtil                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class WinUtil extends Object {





   /*------------------------------------------------------------*/
   /* centerWindow
    */
   /**
    *    This method will center the given window
    *  on the screen
    *  
    *
    *  @param Component The Component to center
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void centerWindow ( Component component ) {
      try {
         if ( component != null ) {
            Dimension winSize = component.getSize () ;
            Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize () ;

            component.setLocation ( (scrSize.width/2)-(winSize.width/2), 
                                    (scrSize.height/2)-(winSize.height/2) ) ;
         } /* if frame is not null */
      } catch ( Exception e ) {
         e.printStackTrace() ;
         component.setLocation ( 0, 0 ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setJTextAreaAsLabel
    */
   /**
    *    This method will configure a JTextArea
    *  to appear like a multi line JLabel.
    *  
    *
    *  @param textArea The JTextArea to configure
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setJTextAreaAsLabel ( JTextArea textArea ) {
      textArea.setLineWrap ( true ) ;
      textArea.setWrapStyleWord ( true ) ;
      textArea.setEditable ( false ) ;
      textArea.setOpaque ( false ) ;
   }





   /*------------------------------------------------------------*/
   /* centerChildInParent
    */
   /**
    *    This method will center a component inside
    *  of it's parent. 
    *  
    *
    *  @param child The child component to center
    *  @param parent The parent component to center inside of
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void centerChildInParent ( Component child, Component parent ) {
      Dimension size  = parent.getSize() ;
      Dimension cSize = child.getSize() ;
      Point     loc   = parent.getLocation() ;

      child.setBounds ( loc.x+ ((size.width/2)-(cSize.width/2)), loc.y+((size.height/2)-(cSize.height/2)), cSize.width, cSize.height ) ;
   }


   /*-----------------------------------------------------------*/
   /* wallPaper
    */
   /**
    *    Tiles an image across a component
    *  
    *  
    *
    *  @param Component the Component to tile across
    *  @param g The Graphics context to use
    *  @param image The image to tile
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void wallPaper(Component component, 
                                Graphics  g, 
                                Image     image) {
      if ( image != null ) {
         Dimension compsize = component.getSize();

         int patchW = image.getWidth(component);
         int patchH = image.getHeight(component);

         if ( ( patchW > 0 ) && ( patchH > 0 ) ) {
            for ( int r=0; r < compsize.width; r += patchW ) {
               for ( int c=0; c < compsize.height; c += patchH ) {
                  g.drawImage(image, r, c, component);
               }
            }
         } /* if */
      }
   }




   /*-----------------------------------------------------------*/
   /* wallPaper
    */
   /**
    *    Tiles an image inside of the rectangle passed
    *  
    *  
    *
    *  @param rect The bounds of the tiling
    *  @param component The component to tile on
    *  @param g The Graphics context
    *  @param image The image
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void wallPaper(Rectangle rect,
                                Component component, 
                                Graphics  g, 
                                Image     image) {
      if ( image != null ) {

         int patchW = image.getWidth(component);
         int patchH = image.getHeight(component);

         if ( ( patchW > 0 ) && ( patchH > 0 ) ) {
            for ( int r=rect.x; r < rect.x+rect.width; r += patchW ) {
               for ( int c=rect.y; c < rect.y+rect.height; c += patchH ) {
                  g.drawImage(image, r, c, component);
               }
            }
         }
      }
   }

   public static void wallPaper(int x, int y, int width, int height,
                                Component component, 
                                Graphics  g, 
                                Image     image) {
      if ( image != null ) {

         int patchW = image.getWidth(component);
         int patchH = image.getHeight(component);

         if ( ( patchW > 0 ) && ( patchH > 0 ) ) {
            for ( int r=x; r < x+width; r += patchW ) {
               for ( int c=y; c < y+height; c += patchH ) {
                  g.drawImage(image, r, c, component);
               }
            }
         }
      }
   }


   /*------------------------------------------------------------*/
   /* wallPaper
    */
   /**
    *    Tiles an image on a window. The offset
    *  values are the distances from the top
    *  left corner of the tilable region.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void wallPaper(int x, int y, int width, int height,
                                int xOffset, 
                                int yOffset,
                                Component component, 
                                Graphics  g, 
                                Image     image) {
      if ( image != null ) {

         int patchW = image.getWidth(component);
         int patchH = image.getHeight(component);

         /* set the starting drawing position using */
         /* the offset values.  This makes sure the */
         /* tile is aligned even if we were given   */
         /* a region to draw within the tiled area  */
         int xDiff = ((x + xOffset) % patchW) ;
         int yDiff = ((y + yOffset) % patchH) ;
         x        -= xDiff;
         width    += xDiff ;
         y        -= yDiff ;
         height   += yDiff ;

         if ( ( patchW > 0 ) && ( patchH > 0 ) ) {
            for ( int r=x; r < x+width; r += patchW ) {
               for ( int c=y; c < y+height; c += patchH ) {
                  g.drawImage(image, r, c, component);
               }
            }
         }
      }
   }


   /*------------------------------------------------------------*/
   /* wallPaper
    */
   /**
    *    Tiles an image on a window. The offset
    *  values are the distances from the top
    *  left corner of the tilable region.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void wallPaper(int x, int y, int width, int height,
                                int xOffset, 
                                int yOffset,
                                Component component, 
                                Graphics  g, 
                                Image     image,
                                int imageWidth,
                                int imageHeight) {
      if ( image != null ) {

         int patchW = imageWidth ;
         int patchH = imageHeight;

         /* set the starting drawing position using */
         /* the offset values.  This makes sure the */
         /* tile is aligned even if we were given   */
         /* a region to draw within the tiled area  */
         int xDiff = ((x + xOffset) % patchW) ;
         int yDiff = ((y + yOffset) % patchH) ;
         x        -= xDiff;
         width    += xDiff ;
         y        -= yDiff ;
         height   += yDiff ;

         if ( ( patchW > 0 ) && ( patchH > 0 ) ) {
            for ( int r=x; r < x+width; r += patchW ) {
               for ( int c=y; c < y+height; c += patchH ) {
                  g.drawImage(image, r, c, imageWidth, imageHeight, component);
               }
            }
         }
      }
   }



   /*-----------------------------------------------------------*/
   /* wallPaper
    */
   /**
    *    Starts tiling at a specified x,y coordinate
    *  
    *  
    *
    *  @param x,y The coordinate to start tiling
    *  @param component The component to tile
    *  @param g The Graphics context
    *  @param image The image
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void wallPaper(int x, int y,
                                Component component, 
                                Graphics  g, 
                                Image     image) {
      if ( image != null ) {


         int patchW = image.getWidth(component);
         int patchH = image.getHeight(component);


         if ( ( patchW > 0 ) && ( patchH > 0 ) ) {
            for ( int r=0; r < x; r += patchW ) {
               for ( int c=0; c < y; c += patchH ) {
                  g.drawImage(image, r, c, component);
               }
            }
         }
      }
   }





   /*------------------------------------------------------------*/
   /* getParentFrame
    */
   /**
    *    This method will will look up a component's
    *  Frame parent if there is one.
    *  
    *
    *  @param component The component to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Frame getParentFrame ( Component c ) {
      Frame ret = null ;

      while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
         if ( c instanceof Frame )
            ret = (Frame)c ;
      }

      return( ret ) ;
   }

   /*------------------------------------------------------------*/
   /* getParentJFrame
    */
   /**
    *    This method will will look up a component's
    *  JFrame parent if there is one.
    *  
    *
    *  @param component The component to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static JFrame getParentJFrame ( Component c ) {
      JFrame ret = null ;

      while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
         if ( c instanceof JFrame )
            ret = (JFrame)c ;
      }

      return( ret ) ;
   }


   /*------------------------------------------------------------*/
   /* getParentDialog
    */
   /**
    *    This method will will look up a component's
    *  Dialog parent if there is one.
    *  
    *
    *  @param component The component to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Dialog getParentDialog ( Component c ) {
      Dialog ret = null ;

      while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
         if ( c instanceof Dialog )
            ret = (Dialog)c ;
      }

      return( ret ) ;

   }
   /*------------------------------------------------------------*/
   /* getParentJDialog
    */
   /**
    *    This method will will look up a component's
    *  JDialog parent if there is one.
    *  
    *
    *  @param component The component to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static JDialog getParentJDialog ( Component c ) {
      JDialog ret = null ;

      while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
         if ( c instanceof JDialog )
            ret = (JDialog)c ;
      }

      return( ret ) ;
   }

   /*------------------------------------------------------------*/
   /* getParentJScrollPane
    */
   /**
    *    This method will will look up a component's
    *  JScrollPane parent if there is one.
    *  
    *
    *  @param component The component to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static JScrollPane getParentJScrollPane ( Component c ) {
      JScrollPane ret = null ;

      while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
         if ( c instanceof JScrollPane )
            ret = (JScrollPane)c ;
      }

      return( ret ) ;
   }


   /*------------------------------------------------------------*/
   /* selectTreeNode
    */
   /**
    *    This method will make sure that the tree
    *  node passed is both selected and viewable. This
    *  means that all parent's of the node will be expanded
    *  and the JTree passed will scroll to the row
    *
    *  @param tree The JTree that contains the Node passed
    *  @param node The DefaultMutableTreeNode to select
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void selectTreeNode ( JTree tree, DefaultMutableTreeNode node ) {
      if ( ( tree != null ) && ( node != null ) ) {
         TreeNode[] nodePath   = node.getPath() ;
         TreePath   expandPath = new TreePath ( nodePath ) ;

         tree.scrollPathToVisible ( expandPath ) ;
         tree.setSelectionPath ( new TreePath ( nodePath ) ) ;


      }
   }


   /*------------------------------------------------------------*/
   /* expandTreeNode
    */
   /**
    *    This method will make sure that the tree
    *  node passed is both expanded and viewable. This
    *  means that all parent's of the node will be expanded
    *  and the JTree passed will scroll to the row
    *
    *  @param tree The JTree that contains the Node passed
    *  @param node The DefaultMutableTreeNode to expand
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void expandTreeNode ( JTree tree, DefaultMutableTreeNode node ) {

      if ( ( tree != null ) && ( node != null ) ) {
         TreeNode[] nodePath   = node.getPath() ;
         TreePath   expandPath = new TreePath ( nodePath ) ;

         tree.scrollPathToVisible ( expandPath ) ;
         tree.expandPath ( new TreePath ( nodePath ) ) ;


      }
   }


   /*------------------------------------------------------------*/
   /* expandTreeNode
    */
   /**
    *    This method will make sure that the tree
    *  node passed is both expanded and viewable. This
    *  means that all parent's of the node will be expanded
    *  and the JTree passed will scroll to the row
    *
    *  @param tree The DetailJTree that contains the Node passed
    *  @param node The DefaultMutableTreeNode to expand
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void expandTreeNode ( DetailJTree tree, DefaultMutableTreeNode node ) {

      if ( ( tree != null ) && ( node != null ) ) {
         TreeNode[] nodePath   = node.getPath() ;
         TreePath   expandPath = new TreePath ( nodePath ) ;

         tree.getJTree().expandPath ( new TreePath ( nodePath ) ) ;
         //tree.getJTree().scrollPathToVisible ( expandPath ) ;

      }
   }


   /**
     *  This method will simply expand all items in a JTree
     *
     * @param tree The Tree to expand
     **/
   public static void expandTree ( JTree tree ) {
      int rowCount = tree.getRowCount() ;
      int i        = 0 ;

      while ( i < rowCount ) {
         tree.expandRow ( i ) ;
         rowCount = tree.getRowCount() ;
         i++ ;
      }

   }

   /**
     * This method will select an entire Vector of objects in a JList. Please
     * note that this method will only work if the JList has a DataModel of DefaultListModel
     *
     * @param JList the JList to select the objects in
     * @param Vector The Vector of data to select.
     **/
   public static void selectItems ( JList jList, Vector data ) {
      int[] indices = new int [ data.size() ] ;

      if ( jList.getModel() instanceof DefaultListModel ) {
         DefaultListModel model = (DefaultListModel) jList.getModel() ;

         for ( int i = 0; i < indices.length; i++ ) {
            indices[i] = model.indexOf ( data.elementAt(i) ) ;
         }

         jList.setSelectedIndices ( indices ) ;
      }
   }

   /**
     * This method will select an entire Vector of objects in a JList. Please
     * note that this method will only work if the JList has a DataModel of DefaultListModel
     *
     * @param JList the JList to select the objects in
     * @param Vector The Vector of data to select.
     **/
   public static void selectItems ( JList jList, Object[] data ) {
      int[] indices = new int [ data.length ] ;

      if ( jList.getModel() instanceof DefaultListModel ) {
         DefaultListModel model = (DefaultListModel) jList.getModel() ;

         for ( int i = 0; i < indices.length; i++ ) {
            indices[i] = model.indexOf ( data[i] ) ;
         }

         jList.setSelectedIndices ( indices ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* finalizeComponents
    */
   /**
    *    This method will finalize all components contained
    *  in the Container passed in. This method is recursive. All
    *  components contained in components etc.. will be finalized. Please
    *  note that this method will throw the Throwable exception since the
    *  'finalize' method on Object does.<br><br>
    *
    * <b>Please Note!</b> This method does not finalize the container passed in!
    *
    *  @param container The container to finalize all components in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void finalizeComponents ( Container container ) throws Throwable {
      Object[] components = container.getComponents() ;
      int      i          = 0 ;

      while ( i < components.length ) {
         if ( components[i] instanceof Container )
            finalizeComponents ( (Container)components[i] ) ;
         //components[i].finalize() ;
         i++ ;
      }

   } /* finalizeComponents  */


   /*------------------------------------------------------------*/
   /* setBackgroundAll
    */
   /**
    *    This method will set the background on the container passed
    *  and all children of that container.
    *
    *  @param color The Color to set the background to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setBackgroundAll ( Container container, Color color ) {
      Object[] components = container.getComponents() ;
      int      i          = 0 ;

      while ( i < components.length ) {
         if ( components[i] instanceof Container )
            setBackgroundAll ( (Container)components[i], color ) ;
         if ( ( components[i] instanceof Component ) || ( components[i] instanceof Container ) )
            ((Component)components[i]).setBackground ( color ) ;
         i++ ;
      }

   } /* finalizeComponents  */

   /*------------------------------------------------------------*/
   /* setBackgroundAllContainer
    */
   /**
    *    This method will set the background on the container passed
    *  and all container children of that container.
    *
    *  @param color The Color to set the background to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setBackgroundAllContainer ( Container container, Color color ) {
      Object[] components = container.getComponents() ;
      int      i          = 0 ;

      while ( i < components.length ) {
         if ( components[i] instanceof Container )
            setBackgroundAllContainer ( (Container)components[i], color ) ;
         if ( components[i] instanceof Container )
            ((Component)components[i]).setBackground ( color ) ;
         i++ ;
      }

   } /* finalizeComponents  */



   /**
     *  This method will move an item in a JList model up
     *
     * @param object The item to move
     * @param listModel The DefaultListModel that the JList has as it's model
     **/
   public static void moveItemUp ( Object object, DefaultListModel listModel ) {
      try {
         int currentIndex = listModel.indexOf ( object ) ;

         if ( currentIndex > 0 ) {
            listModel.removeElement ( object ) ;
            listModel.insertElementAt ( object, currentIndex-1 ) ;
         }
      } catch ( NullPointerException e ) {
      }
   }


   /**
     *  This method will move an item in a JList model up
     *
     * @param object The item to move
     * @param listModel The DefaultListModel that the JList has as it's model
     **/
   public static void moveItemDown ( Object object, DefaultListModel listModel ) {
      try {
         int currentIndex = listModel.indexOf ( object ) ;
         int size         = listModel.size()-1 ;

         if ( currentIndex < size ) {
            listModel.removeElement ( object ) ;
            listModel.insertElementAt ( object, currentIndex+1 ) ;
         }
      } catch ( NullPointerException e ) {
      }
   }


   /**
     *  This method will simple move the selected item in a JList Up. It will
     * then reselect the item. Please note that the JList MUST have a Model
     * of DefaultListModel. This method will do nothing if it does not
     *
     * @param list The JList to alter
     **/
   public static void moveItemUp ( JList list ) {
      try {
         DefaultListModel listModel     = (DefaultListModel)list.getModel() ;
         Object           selectedItem  = list.getSelectedValue() ;

         if ( selectedItem != null ) {
            moveItemUp ( selectedItem, listModel ) ;
            list.setSelectedValue ( selectedItem, true ) ;
         }

      } catch ( ClassCastException e ) {
      }
   }


   /**
     *  This method will simple move the selected item in a JList Down. It will
     * then reselect the item. Please note that the JList MUST have a Model
     * of DefaultListModel. This method will do nothing if it does not
     *
     * @param list The JList to alter
     **/
   public static void moveItemDown ( JList list ) {
      try {
         DefaultListModel listModel     = (DefaultListModel)list.getModel() ;
         Object           selectedItem  = list.getSelectedValue() ;

         if ( selectedItem != null ) {
            moveItemDown ( selectedItem, listModel ) ;
            list.setSelectedValue ( selectedItem, true ) ;
         }

      } catch ( ClassCastException e ) {
      }
   }



   /**
     *  This method will copy all elements from the Vector passed into the
     * DefaultListModel passed. If the records extend PersistentRec, the
     * 'valid' method will be called on each one before inserting into the list.
     *
     * @param dataVector The Vector containing data objects
     * @param listModel The DefaultListModel
     **/
   public static void setListData ( Vector dataVector, DefaultListModel listModel ) {

      if ( listModel != null )
         listModel.removeAllElements() ;

      if ( ( dataVector != null ) && ( listModel != null ) && ( dataVector.size() > 0 ) ) {
         int      i    = 0 ;
         Object   data = null ;
         int      size = dataVector.size() ;

         while ( i < size ) {
            data = dataVector.elementAt(i) ;

            if ( data instanceof PersistentRec ) {
               if ( ((PersistentRec)data).valid() )
                  listModel.addElement ( data ) ;
            } else
               listModel.addElement ( data  ) ;

            i++ ;
         }
      }
   }

   /**
     *  This method will return a Vector from a JList which has a model of DefaultListModel
     *
     * @param JList the JList to return data from
     *
     * @returns Vector
     **/
   public static Vector getListData ( JList list ) {
      DefaultListModel listModel = (DefaultListModel) list.getModel() ;
      Vector           data      = new Vector ( 1 ) ;
      int              i         = 0 ;
      int              size      = listModel.getSize() ;

      while ( i < size ) {
         data.addElement ( listModel.elementAt(i) ) ;
         i++ ;
      }

      return( data ) ;
   }


   /**
     *  This method will set the data in a JComboBox. Please note that
     * you must have previously set the CombBox model to a new DefaultComboBoxModel
     * for this method to function. (JComboBox.setModel ( new DefaultComboBoxModel ) ;
     *
     * @param comboBox The JComboBox to put data in
     * @param data The data to insert
     **/
   public static void setJComboBoxData ( JComboBox comboBox, Vector data ) {
      DefaultComboBoxModel model = (DefaultComboBoxModel) comboBox.getModel() ;

      if ( model != null ) {

         model.removeAllElements() ;

         if ( data != null ) {
            int i    = 0 ;
            int size = data.size() ;

            while ( i < size ) {
               if ( data.elementAt(i) != null )
                  model.addElement ( data.elementAt(i++ ) ) ;
               else
                  i++ ;
            }
         }
      }

   }


   /**
  *  This method will set the data in a JComboBox. Please note that
  * you must have previously set the CombBox model to a new DefaultComboBoxModel
  * for this method to function. (JComboBox.setModel ( new DefaultComboBoxModel ) ;
  *
  * @param comboBox The JComboBox to put data in
  * @param data The data to insert
  **/
   public static void setJComboBoxData ( JComboBox comboBox, Object[] data ) {
      DefaultComboBoxModel model = (DefaultComboBoxModel) comboBox.getModel() ;

      if ( model != null ) {

         model.removeAllElements() ;

         if ( data != null ) {
            int i    = 0 ;
            int size = data.length ;

            while ( i < size ) {
               if ( data[i] instanceof String ) {
                  if ( ((String)data[i]).length() > 0 )
                     model.addElement ( data[i++] ) ;
                  else
                     i++ ;
               } else
                  model.addElement ( data[i++] ) ;
            }
         }
      }

   }

} /* WinUtil */

