package com.ibm.nzna.tools.classpath ;
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
import java.util.zip.* ;
import java.io.* ;
   /*********************************************************
   * This samples class shows the system and api properties
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 6/22/100  4:22pm
   * @author 
   * @version
   *********************************************************/
public class CheckClasspath {
   final static String fileSeparator  = System.getProperty( "file.separator" ) ;
   public static void main (String args[] ) {
      Properties systemProperties = System.getProperties() ;
      String          pathSep     = System.getProperty ( "path.separator" ) ;
      String          classpath   = System.getProperty ( "java.class.path" ) ;
      StringTokenizer strtok      = new StringTokenizer ( classpath, pathSep ) ;
      String          filename    = null ;

      Hashtable classpathTable = new Hashtable() ;
      while ( strtok.hasMoreTokens() ) {
         filename             = strtok.nextToken() ;
         File file            = new File( filename ) ;
         String canonicalPath = null ;
         try {
            canonicalPath =  file.getCanonicalPath() ;
         }
         catch ( Exception e ) {
            e.printStackTrace() ;
            canonicalPath = file.getAbsolutePath() ;
         }
         
         /* Ignore duplicate entries but log as info */
         if ( classpathTable.containsKey( canonicalPath )) {
            System.out.println("classpath:(duplicate) - "+ filename ) ;
            continue ;
         }
         if ( file.canRead() ) {
            System.out.print( "classpath:(passed) - " ) ;
            if ( file.isDirectory() ) {
               Hashtable filesTable = getFilenames( filename , false ) ;
               System.out.println(filesTable.size() + " valid entries: " + filename ) ;
               /* must do this before prefix removed */
               testRead( filesTable ) ;
               /* remove classpath prefix */
               filesTable = removeClasspathPrefix ( file.getAbsolutePath(), filesTable ) ;
               checkConflict ( classpathTable, filesTable ) ;
               classpathTable.put( canonicalPath, filesTable ) ;
            }
            else if ( (filename.endsWith(".jar")) || (filename.endsWith(".zip" ))) {
               Hashtable filesTable = getZipContents( filename  ) ;
               System.out.println(filesTable.size() + " valid entries: " + filename ) ;
               checkConflict ( classpathTable, filesTable ) ;
               classpathTable.put( canonicalPath, filesTable ) ;
            }
            else {
               System.out.println( filename ) ;
            }
         }
         else if ( file.exists()) {
            System.out.println( "classpath:(failed read) "+filename ) ;
         }
         else {
            System.out.println( "classpath:(not valid) "+filename ) ;
         }
      }
   }
   public static Hashtable getZipContents ( String filename ) {
      Hashtable zipHash = new Hashtable() ;
      try {
         ZipFile zipFile  = new ZipFile ( filename ) ;
         Enumeration enum = zipFile.entries() ;
         while ( enum.hasMoreElements() ) {
            ZipEntry zipEntry = (ZipEntry) enum.nextElement() ;
            if ( !zipEntry.isDirectory() ) {
               /* Add the zipEntry to the hash using the filename as the key */
               zipHash.put( zipEntry.getName() , zipEntry ) ;
            }
         }
      }
      catch ( Exception e ) {
         e.printStackTrace() ;
      }
      return ( zipHash ) ;
   }
   public static Hashtable getFilenames( String path , boolean allFiles) {
      Hashtable fileHash = new Hashtable () ;
      File file = new File ( path ) ;

      String files[] = file.list() ;
      if ( files != null ) {
         for ( int fileNumber = 0; fileNumber < files.length ; fileNumber++ ) {
            File fileInPath = new File( file.getPath() + fileSeparator + files[ fileNumber ] ) ;
            if ( fileInPath.isDirectory() ) {
               Hashtable subFilelist = getFilenames( fileInPath.getAbsolutePath() , allFiles) ;
               Enumeration enum = subFilelist.keys() ;
               while ( enum.hasMoreElements() ) {
                  Object key     = enum.nextElement() ;
                  Object value   = subFilelist.get( key ) ;
                  fileHash.put ( key, value ) ;
               }
            }
            /* Only add files and not directories */
            else {
               if (( allFiles ) || ( files[ fileNumber ].indexOf(".class") > -1 )) {
                  fileHash.put( fileInPath.getAbsolutePath() , fileInPath ) ;
               }
            }
         }
      }
      return ( fileHash ) ;
   }
   public static boolean checkConflict ( Hashtable classpaths, Hashtable files ) {
      boolean conflict = false ;
      Enumeration filesEnum = files.keys() ;
      while ( filesEnum.hasMoreElements()) {
         Object fileKey = filesEnum.nextElement() ;
         /* Ignore manifest files which are in jar files */
         if ( !fileKey.equals("META-INF/MANIFEST.MF")) {
            /* Check for conflicts in classpath entries */
            Enumeration classpathEnum = classpaths.keys() ;
            while ( classpathEnum.hasMoreElements() ) {
               String      key      = (String ) classpathEnum.nextElement() ;
               Hashtable   filelist = (Hashtable) classpaths.get( key ) ;
               if ( filelist.containsKey( fileKey )) {
                  System.out.println( "   Overridden " + fileKey + " ---> " + key ) ;
                  conflict = true ;
               }
            }
         }
      }
      return ( conflict ) ;
   }
   public static Hashtable removeClasspathPrefix ( String classpathEntry, Hashtable fileList ) {
      Hashtable fixedHash = new Hashtable() ;
      int classpathEntryLength = classpathEntry.length() + 1 ;
      Enumeration enum = fileList.keys() ;
      while ( enum.hasMoreElements() ) {
         String filename = (String) enum.nextElement() ;
         Object value    = fileList.get( filename ) ;
         filename = filename.substring( classpathEntryLength, filename.length() ) ;
         /* So these key will match the keys from jar files */
         filename = filename.replace('\\', '/' ) ;
         fixedHash.put( filename, value ) ;
      }
      return ( fixedHash ) ;
   }
   public static void testRead ( Hashtable fileList ) {
      Enumeration enum = fileList.keys() ;
      while ( enum.hasMoreElements() ) {
         String filename = (String) enum.nextElement() ;
         File   file    = (File)fileList.get( filename ) ;
         if ( !file.canRead() ) {
            System.out.println( "    cannot read: "+ filename ) ;
         }
      }
   }
}

