package com.ibm.nzna.tools.cleanCodeDir ;
/*-----------------------------------------------------------*/
/*   CleanCodeDir                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CleanCodeDir                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.util.* ;
/**
 * CleanCodeDir is a small utility that will remove all non
 * code related files from all directories recursivly. Currently,
 * code related files are files that end in the following extensions:<br><br>
 *
 * .java<br>
 * .gif<br>
 * .jpg<br>
 * .bat<br>
 * .cmd<br>
 * .bmp<br>
 * .exe<br>
 * .lng<br>
 * .property<br>
 * .properties<br>
 * .jdx<br>
 * .vsb<br>
 * .c<br>
 * .rc<br>
 * .h<br>
 * .def<br>
 * .ico<br><br>
 *
 * or filenames that match<br><br>
 *
 * makefile  (Not case sensitive)<br> 
 *
 * @version 1.0, 1/18/100 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   CleanCodeDir                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class CleanCodeDir {


    /**
      * This is the main starting point for the application.
      * currently this application takes no params
      */
    public static void main ( String args[] ) {
        Properties p       = System.getProperties () ;
        String     homeDir = p.getProperty ( "user.dir" ) ;

        cleanDir ( new File ( homeDir ) ) ;

    }


    /**
      * This method will clean all files from the directory passed. If
      * the directory passed has any directories, this method will call
      * itself recursivly until all directories are cleaned
      *
      * @param homeDir The home directory that the cleaning started in
      * @param dirName The directory to clean
      */
    private static void cleanDir ( File dir ) {

        /* Only process this if it is a directory  */
        if ( dir.isDirectory() ) {
            String files[] = dir.list ( new filter() ) ;

            if ( ( files != null ) && ( files.length > 0 ) ) {
                int  i           = 0 ;
                File currentFile = null ;

                while ( i < files.length ) {
                    currentFile = new File ( dir.getAbsolutePath()+File.separator+files[i] ) ;

                    /* If this is a directory, clean it  */
                    if ( currentFile.isDirectory() )
                        cleanDir ( currentFile ) ;

                    /* Otherwise, delete the file  */
                    else {
                        currentFile.delete() ;
                        System.out.println ( "Deleted ["+files[i]+"]" ) ;
                    }
                    
                    i++ ;
                } /* while  */
            } /* if  */
        } /* if  */
    } /* cleanDir  */
} /* CleanCodeDir  */



/**
  * This is the filename filter that the cleanCodeDir uses.
  */
class filter implements FilenameFilter {

    public boolean accept ( File dir, String name ) {
        name = name.toLowerCase() ;

        if ( ( name.endsWith ( ".java" ) ) ||
             ( name.endsWith ( ".gif" ) ) ||
             ( name.endsWith ( ".jpg" ) ) ||
             ( name.endsWith ( ".bat" ) ) ||
             ( name.endsWith ( ".cmd" ) ) ||
             ( name.endsWith ( ".bmp" ) ) ||
             ( name.endsWith ( ".exe" ) ) ||
             ( name.endsWith ( ".lng" ) ) ||
             ( name.endsWith ( ".property" ) ) ||
             ( name.endsWith ( ".properties" ) ) ||
             ( name.endsWith ( ".jdx" ) ) ||
             ( name.endsWith ( ".vsb" ) ) ||
             ( name.endsWith ( ".c" ) ) ||
             ( name.endsWith ( ".rc" ) ) ||
             ( name.endsWith ( ".h" ) ) ||
             ( name.endsWith ( ".def" ) ) ||
             ( name.endsWith ( "html" ) ) ||
             ( name.endsWith ( "xml" ) ) ||
             ( name.endsWith ( ".ico" ) ) ||
             ( name.equals   ( "makefile" ) ) ||
             ( name.equals   ( "cleancodedir.class" ) ) )
            return( false ) ;

        return( true ) ;
    }
}
