package com.ibm.nzna.tools.lvUpdate ;
/*-----------------------------------------------------------*/
/*   LVUpdate                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LVUpdate                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import java.sql.* ;
import java.io.* ;
/**
 * This program updates/inserts a Long Var Char field into DB/2 from Command Line 
 *
 *
 * @version 1.0, 1/23/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LVUpdate                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class LVUpdate {


   private String databaseName   = null ;
   private String databaseServer = null ;
   private String filename       = null ;
   private String sql            = null ;
   private String userId         = null ;
   private String password       = null ;
   private String longVarChar    = null ;
   private String portNumber     = null ;


   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This is the execution starting point of the program
    *  
    *  
    *
    *  @param database name
    *  @param database server
    *  @param database port number (JDBC)
    *  @param filename to import
    *  @param insert String and Where Clause (Needs to be enclosed in double quotes
    *  @param user The User Id to connect as
    *  @param password The password to connect with
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String args[] ) {
      try {
         new LVUpdate ( args[0],
                        args[1],
                        args[2],
                        args[3],
                        args[4],
                        args[5],
                        args[6] ) ;
      } catch ( Exception e ) {
         printUsage() ;
      }
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This creates and runs the LVUpdate utility. 
    *  
    *  
    *
    *  @param databaseName The name of the database to contact
    *  @param databaseServer The name of the server to contact
    *  @param filename The name of the file to read as the long var char
    *  @param sql The SQL String to run
    *  @param user The User Id to connect as
    *  @param password The password to connect with
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LVUpdate ( String databaseName, String databaseServer, String portNumber,
                     String filename,     String sql,
                     String userId,       String password ) {

      this.databaseName   = databaseName ;
      this.databaseServer = databaseServer ;
      this.filename       = filename ;
      this.sql            = sql ;
      this.userId         = userId ;
      this.password       = password ;
      this.portNumber     = portNumber ;

      if ( connectToDatabase() ) {
         if ( readFile() ) {
            if ( updateLongVarChar() )
               System.out.println ( "Row Updated Succesfully" ) ;
         }

         System.out.println ( "Disconnecting" ) ;
         DatabaseSystem.shutdown() ;
      } else
         System.out.println ( "Could not connect to database! Please check Database Name, Server, User Id and Password" ) ;

      System.exit ( 0 ) ;

   }







   /*------------------------------------------------------------*/
   /* connectToDatabase
    */
   /**
    *  This method will attempt to create a connection to the database
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if connected
    *//*---------------------------------------------------------*/
   private boolean connectToDatabase () {

      try {
         new DatabaseSystem ( false, null, false ) ;

         System.out.println ( "Connecting to "+databaseName+" on "+databaseServer ) ;
         return( DatabaseSystem.createConnection ( 1,
                                                   databaseName,
                                                   databaseServer,
                                                   portNumber,
                                                   userId,
                                                   password ) ) ;
      } catch ( Exception e ) {
         System.out.println ( "Cannot connect to database.\nException:" ) ;
         e.printStackTrace() ;
      }

      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* readFile
    */
   /**
    *    This method will attempt to read the file
    *  holding the text for the long var char
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean readFile () {
      try {
         longVarChar = FileUtil.readFile ( filename ) ;

         return( ( longVarChar != null ) && ( longVarChar.length() > 0 ) ) ;
      } catch ( FileNotFoundException e1 ) {
         System.out.println ( "File not found :"+filename ) ;
      } catch ( Exception e ) {
         System.out.println ( "Could not read :"+filename ) ;
         e.printStackTrace() ;
      }

      return( false ) ;
   }






   /*------------------------------------------------------------*/
   /* updateLongVarChar
    */
   /**
    *    This method does the actual SQL to update
    *  the Long Var Char
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean updateLongVarChar () {
      SQLMethod sqlMethod = new SQLMethod ( 1, "updateLongVarChar", 5 ) ;
      boolean   rc        = false ;

      if ( longVarChar.length() < 32700 ) {
         try {
            PreparedStatement stmt = sqlMethod.createPreparedStatement ( sql ) ;

            System.out.println ( "Updating Long Var Char Field with String of "+longVarChar.length()+" bytes" ) ;
            stmt.setString ( 1, longVarChar ) ;

            stmt.execute() ;

            rc = true ;
         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            System.out.println ( "\nError, cannot update Long Var Char. Full Error:" ) ;
            e.printStackTrace() ;
         }
      } else
         System.out.println ( "File too long to fit in Long Var Char!" ) ;

      sqlMethod.close() ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* printUsage
    */
   /**
    *    This method will print to the command line
    *  the usage of this program and it's params
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void printUsage ( ) {
      System.out.println ( "Usage: java com.ibm.nzna.tools.lvUpdate.LVUpdate <databaseName> <serverName> <filename to import> <sql> <userId> <password>" ) ;
      System.out.println ( "\nDescription\n---------------------" ) ;
      System.out.println ( "This program updates a Long Var Char field in DB/2" ) ;
      System.out.println ( "\nParameter Descriptions\n-----------------------" ) ;
      System.out.println ( "databaseName - The database name that is cataloged on your system" ) ;
      System.out.println ( "serverName   - The I/P Address or full host name of the server " ) ;
      System.out.println ( "               the database resides on" ) ;
      System.out.println ( "fileName     - The name of the file holding the text to put into" ) ;
      System.out.println ( "               the long var char" ) ;
      System.out.println ( "sql          - The SQL String used to insert/update the Long Var Char." ) ;
      System.out.println ( "               An example of this SQL is as follows. If you have a table" ) ;
      System.out.println ( "               called MY.DOCUMENTS with the following columns: ") ;
      System.out.println ( "               DOCID       integer" ) ;
      System.out.println ( "               BODY        Long Var Char" ) ;
      System.out.println ( "               DBUSER      char(8)" ) ;
      System.out.println ( "               CHANGEDTIME timestamp\n" ) ;
      System.out.println ( "               And you wanted to insert a row, you would have an SQL" ) ;
      System.out.println ( "               like this:\n" ) ;
      System.out.println ( "               INSERT INTO MY.DOCUMENTS (DOCID, BODY, DBUSER, CHANGEDTIME )" ) ;
      System.out.println ( "               values ( 1, ?, 'ME', CURRENT TIMESTAMP )\n" ) ;
      System.out.println ( "               Please note that the column in 'values' for the long var char" ) ;
      System.out.println ( "               is a Question mark. This must be present.\n\n" ) ;
      System.out.println ( "               If you wanted to update a row, you would have an SQL" ) ;
      System.out.println ( "               like this:\n" ) ;
      System.out.println ( "               UPDATE MY.DOCUMENTS SET BODY = ? WHERE DOCIND = 1" ) ;
      System.out.println ( "userId       - The User Id to connect with" ) ;
      System.out.println ( "password     - The password to connect with" ) ;
   }


} /* LVUpdate */
