package com.ibm.nzna.tools.makeMakeAll ;
/*-----------------------------------------------------------*/
/*   CleanCodeDir                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MakeMakeAll                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.util.* ;
/**
 * MakeMakeAll is a small utility that will create a command
 * file called 'makeall.cmd'. The command file will compile all java files 
 * in the current directory and all directories under the current 
 * directory recursivly.<br><br>
 *
 * The make file will first erase the .class file associated with a .java
 * file and will then use jikes to recompile the .java file.
 *
 * @author David Lentz
 **/


public class MakeMakeAll {


   /**
     * This is the main starting point for the application
     */
   /**
     * This is the main starting point for the application.
     * currently this application takes no params
     */
   public static void main ( String args[] ) {
      Properties        p        = System.getProperties () ;
      String            homeDir  = p.getProperty ( "user.dir" ) ;
      File              makeFile = new File ( "makeall.cmd" ) ;
      try {
         FileOutputStream  out      = new FileOutputStream ( makeFile ) ;

         out.write ( new String ( "@echo off\n@cls\n" ).getBytes() ) ;
         makeDir ( out, new File ( homeDir ) ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

   }


   /**
     * This method will gather all .java files and will make an entry
     * in the makeAll file passed in for each.
     *
     * @param out The makeall.cmd file OutputStream to add lines to
     * @param dir The current directory to process
     */
   private static void makeDir ( FileOutputStream out, File dir ) {

      /* Only process this if it is a directory  */
      if ( dir.isDirectory() ) {
         String files[] = dir.list ( new makeFilter() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Process Each Java File                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            if ( ( files != null ) && ( files.length > 0 ) ) {
               out.write ( new String ( "@echo Compiling "+dir.toString()+"\n" ).getBytes() ) ;
               out.write ( new String ( "@jikes "+dir.toString()+File.separator+"*.java\n" ).getBytes() ) ;
               System.out.println ( "Found ["+dir.toString()+"]" ) ;

            } /* if  */
         } catch ( Exception e ) {
            e.printStackTrace() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, process each directory                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         files = dir.list() ;

         try {
            if ( ( files != null ) && ( files.length > 0 ) ) {
               int  i           = 0 ;
               File currentFile = null ;

               while ( i < files.length ) {
                  currentFile = new File ( dir.getAbsolutePath()+File.separator+files[i] ) ;

                  if ( currentFile.isDirectory() )
                     makeDir (out, currentFile ) ;
                  i++ ;
               } /* while  */

            } /* if  */

         } catch ( Exception e1 ) {
            e1.printStackTrace() ;
         } /* catch  */

      } /* if  */
   } /* cleanDir  */
} /* CleanCodeDir  */



/**
  * This is the filename filter that the cleanCodeDir uses.
  */
class makeFilter implements FilenameFilter {

   public boolean accept ( File dir, String name ) {
      name = name.toLowerCase() ;

      if ( name.endsWith ( ".java" ) )
         return( true ) ;

      return( false ) ;
   }
}
